/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: CompositeBlockImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.core.impl;

import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.EObjectContainmentEList;
import org.eclipse.emf.ecore.util.EObjectContainmentWithInverseEList;
import org.eclipse.emf.ecore.util.InternalEList;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.CorePackage;
import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.InstantValue;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Composite Block</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.core.impl.CompositeBlockImpl#getRelations <em>Relations</em>}</li>
 *   <li>{@link fr.supelec.modhelx.core.impl.CompositeBlockImpl#getBlocks <em>Blocks</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class CompositeBlockImpl extends BlockImpl implements CompositeBlock {
	/**
	 * The cached value of the '{@link #getRelations() <em>Relations</em>}' containment reference list.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getRelations()
	 * @generated
	 * @ordered
	 */
	protected EList<Relation> relations;

	/**
	 * The cached value of the '{@link #getBlocks() <em>Blocks</em>}' containment reference list.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getBlocks()
	 * @generated
	 * @ordered
	 */
	protected EList<Block> blocks;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected CompositeBlockImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return CorePackage.Literals.COMPOSITE_BLOCK;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EList<Relation> getRelations() {
		if (relations == null) {
			relations = new EObjectContainmentEList<Relation>(Relation.class, this, CorePackage.COMPOSITE_BLOCK__RELATIONS);
		}
		return relations;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EList<Block> getBlocks() {
		if (blocks == null) {
			blocks = new EObjectContainmentWithInverseEList<Block>(Block.class, this, CorePackage.COMPOSITE_BLOCK__BLOCKS, CorePackage.BLOCK__CONTAINER);
		}
		return blocks;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		super.setup(moc, initialTime);
		
		for(Block b : getBlocks()) {
			b.setup(moc, initialTime);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void startOfSnapshot(ModelOfComputation moc) {
		super.startOfSnapshot(moc);
		
		for(Block b : getBlocks()) {
			b.startOfSnapshot(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void reset(ModelOfComputation moc, InstantValue time) {
		super.reset(moc, time);
		
		for(Block b : getBlocks()) {
			b.reset(moc, time);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc) {
		super.update(moc);
		
		for(Block b : getBlocks()) {
			b.update(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean validate(ModelOfComputation moc) {
		boolean res = true;
		
		for(Block b : getBlocks()) {
			res = res && b.validate(moc);
		}
		return super.validate(moc, res);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfSnapshot(ModelOfComputation moc) {
		super.endOfSnapshot(moc);
		
		for(Block b : getBlocks()) {
			b.endOfSnapshot(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void wrapup(ModelOfComputation moc) {
		super.wrapup(moc);
		
		for(Block b : getBlocks()) {
			b.wrapup(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean snapDet(ModelOfComputation moc) {
		boolean res = true;
		
		for(Block b : getBlocks()) {
			res = res && b.snapDet(moc);
		}
		return super.snapDet(moc, res);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void collectTimeConstraints(ModelOfComputation moc, EList<InstantValue> timeConstraints) {
		super.collectTimeConstraints(moc, timeConstraints);
		
		for(Block b : getBlocks()) {
			b.collectTimeConstraints(moc, timeConstraints);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void initialSchedule(ModelOfComputation moc) {
		super.initialSchedule(moc);

		for(Block b : getBlocks()) {
			b.initialSchedule(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void prePropagate(ModelOfComputation moc) {
		super.prePropagate(moc);
		
		for(Block b : getBlocks()) {
			b.prePropagate(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void preSchedule(ModelOfComputation moc) {
		super.preSchedule(moc);

		for(Block b : getBlocks()) {
			b.preSchedule(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void postSchedule(ModelOfComputation moc) {
		super.postSchedule(moc);

		for(Block b : getBlocks()) {
			b.postSchedule(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void postPropagate(ModelOfComputation moc) {
		super.postPropagate(moc);
		
		for(Block b : getBlocks()) {
			b.postPropagate(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void finalSchedule(ModelOfComputation moc) {
		super.finalSchedule(moc);

		for(Block b : getBlocks()) {
			b.finalSchedule(moc);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean newSnap(ModelOfComputation moc) {
		boolean newsnap = super.newSnap(moc); // default value of newsnap for a block is false
		
		for(Block b : getBlocks()) {
			if(b.newSnap(moc)) return true; // in the point of view of a composite block, only one true answer of a block makes the new snapshot necessary
		}
		return newsnap;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public NotificationChain eInverseAdd(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case CorePackage.COMPOSITE_BLOCK__BLOCKS:
				return ((InternalEList<InternalEObject>)(InternalEList<?>)getBlocks()).basicAdd(otherEnd, msgs);
		}
		return super.eInverseAdd(otherEnd, featureID, msgs);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs) {
		switch (featureID) {
			case CorePackage.COMPOSITE_BLOCK__RELATIONS:
				return ((InternalEList<?>)getRelations()).basicRemove(otherEnd, msgs);
			case CorePackage.COMPOSITE_BLOCK__BLOCKS:
				return ((InternalEList<?>)getBlocks()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case CorePackage.COMPOSITE_BLOCK__RELATIONS:
				return getRelations();
			case CorePackage.COMPOSITE_BLOCK__BLOCKS:
				return getBlocks();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case CorePackage.COMPOSITE_BLOCK__RELATIONS:
				getRelations().clear();
				getRelations().addAll((Collection<? extends Relation>)newValue);
				return;
			case CorePackage.COMPOSITE_BLOCK__BLOCKS:
				getBlocks().clear();
				getBlocks().addAll((Collection<? extends Block>)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case CorePackage.COMPOSITE_BLOCK__RELATIONS:
				getRelations().clear();
				return;
			case CorePackage.COMPOSITE_BLOCK__BLOCKS:
				getBlocks().clear();
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case CorePackage.COMPOSITE_BLOCK__RELATIONS:
				return relations != null && !relations.isEmpty();
			case CorePackage.COMPOSITE_BLOCK__BLOCKS:
				return blocks != null && !blocks.isEmpty();
		}
		return super.eIsSet(featureID);
	}

	
	@Override
	public String toString() {
		return "Composite(" + getName() + ")";
	}
	
	/**
	 * <!-- begin-user-doc -->
	 * Connect pin "from" to pin "to" in this composite by creating 
	 * a relation and adding it to the compsite block.
	 * @param from source of the relation
	 * @param to target of the relation
	 * @return the newly created relation
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Relation connect(Pin from, Pin to) {
		Relation r = CoreFactory.eINSTANCE.createRelation();
		r.setSource(from);
		r.setTarget(to);
		getRelations().add(r);
		return r;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Block addBlock(Block block, String name) {
		block.setName(name);
		getBlocks().add(block);
		return block;
	}
} //CompositeBlockImpl
