/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TimeIntervalValue.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.core.time;

import org.eclipse.emf.ecore.EObject;

/**
 * <!-- begin-user-doc -->
 * A representation of the model object '<em><b>Interval Value</b></em>'.
 * <!-- end-user-doc -->
 *
 * <p>
 * The following features are supported:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.core.time.TimeIntervalValue#getMin <em>Min</em>}</li>
 *   <li>{@link fr.supelec.modhelx.core.time.TimeIntervalValue#getMax <em>Max</em>}</li>
 *   <li>{@link fr.supelec.modhelx.core.time.TimeIntervalValue#isMinOpen <em>Is Min Open</em>}</li>
 *   <li>{@link fr.supelec.modhelx.core.time.TimeIntervalValue#isMaxOpen <em>Is Max Open</em>}</li>
 * </ul>
 * </p>
 *
 * @see fr.supelec.modhelx.core.time.TimePackage#getTimeIntervalValue()
 * @model annotation="GenModel documentation='From MARTE:\rA time interval value is a set of instants values specified by a pair of instant values. A time interval value denotes 0 or many time intervals in the time base associated with the onClock clock of its bounds. When the onClock clock has a maximal value, due to clock roll-over, a time interval value may denote many time intervals.\rWhen used in a time value specification, a time interval value designates any time value of the interval (including or not the bound values according to the isMinOpen et isMaxOpen attribute values).'"
 * @generated
 */
public interface TimeIntervalValue extends EObject {
	/**
	 * Returns the value of the '<em><b>Min</b></em>' reference.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Min</em>' reference isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @return the value of the '<em>Min</em>' reference.
	 * @see #setMin(InstantValue)
	 * @see fr.supelec.modhelx.core.time.TimePackage#getTimeIntervalValue_Min()
	 * @model required="true"
	 * @generated
	 */
	InstantValue getMin();

	/**
	 * Sets the value of the '{@link fr.supelec.modhelx.core.time.TimeIntervalValue#getMin <em>Min</em>}' reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the new value of the '<em>Min</em>' reference.
	 * @see #getMin()
	 * @generated
	 */
	void setMin(InstantValue value);

	/**
	 * Returns the value of the '<em><b>Max</b></em>' reference.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Max</em>' reference isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @return the value of the '<em>Max</em>' reference.
	 * @see #setMax(InstantValue)
	 * @see fr.supelec.modhelx.core.time.TimePackage#getTimeIntervalValue_Max()
	 * @model required="true"
	 * @generated
	 */
	InstantValue getMax();

	/**
	 * Sets the value of the '{@link fr.supelec.modhelx.core.time.TimeIntervalValue#getMax <em>Max</em>}' reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the new value of the '<em>Max</em>' reference.
	 * @see #getMax()
	 * @generated
	 */
	void setMax(InstantValue value);

	/**
	 * Returns the value of the '<em><b>Is Min Open</b></em>' attribute.
	 * The default value is <code>"false"</code>.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Is Min Open</em>' attribute isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @return the value of the '<em>Is Min Open</em>' attribute.
	 * @see #setIsMinOpen(boolean)
	 * @see fr.supelec.modhelx.core.time.TimePackage#getTimeIntervalValue_IsMinOpen()
	 * @model default="false" required="true"
	 * @generated
	 */
	boolean isMinOpen();

	/**
	 * Sets the value of the '{@link fr.supelec.modhelx.core.time.TimeIntervalValue#isMinOpen <em>Is Min Open</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the new value of the '<em>Is Min Open</em>' attribute.
	 * @see #isMinOpen()
	 * @generated
	 */
	void setIsMinOpen(boolean value);

	/**
	 * Returns the value of the '<em><b>Is Max Open</b></em>' attribute.
	 * The default value is <code>"false"</code>.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Is Max Open</em>' attribute isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @return the value of the '<em>Is Max Open</em>' attribute.
	 * @see #setIsMaxOpen(boolean)
	 * @see fr.supelec.modhelx.core.time.TimePackage#getTimeIntervalValue_IsMaxOpen()
	 * @model default="false" required="true"
	 * @generated
	 */
	boolean isMaxOpen();

	/**
	 * Sets the value of the '{@link fr.supelec.modhelx.core.time.TimeIntervalValue#isMaxOpen <em>Is Max Open</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the new value of the '<em>Is Max Open</em>' attribute.
	 * @see #isMaxOpen()
	 * @generated
	 */
	void setIsMaxOpen(boolean value);

} // TimeIntervalValue
