/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TimeAdapterFactory.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.core.time.util;

import fr.supelec.modhelx.core.time.*;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.notify.impl.AdapterFactoryImpl;
import org.eclipse.emf.ecore.EObject;

import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.DurationValue;
import fr.supelec.modhelx.core.time.Instant;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.core.time.JunctionInstant;
import fr.supelec.modhelx.core.time.LogicalClock;
import fr.supelec.modhelx.core.time.TimeIntervalValue;
import fr.supelec.modhelx.core.time.TimePackage;
import fr.supelec.modhelx.core.time.TimeValue;

/**
 * <!-- begin-user-doc -->
 * The <b>Adapter Factory</b> for the model.
 * It provides an adapter <code>createXXX</code> method for each class of the model.
 * <!-- end-user-doc -->
 * @see fr.supelec.modhelx.core.time.TimePackage
 * @generated
 */
public class TimeAdapterFactory extends AdapterFactoryImpl {
	/**
	 * The cached model package.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static TimePackage modelPackage;

	/**
	 * Creates an instance of the adapter factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public TimeAdapterFactory() {
		if (modelPackage == null) {
			modelPackage = TimePackage.eINSTANCE;
		}
	}

	/**
	 * Returns whether this factory is applicable for the type of the object.
	 * <!-- begin-user-doc -->
	 * This implementation returns <code>true</code> if the object is either the model's package or is an instance object of the model.
	 * <!-- end-user-doc -->
	 * @return whether this factory is applicable for the type of the object.
	 * @generated
	 */
	@Override
	public boolean isFactoryForType(Object object) {
		if (object == modelPackage) {
			return true;
		}
		if (object instanceof EObject) {
			return ((EObject)object).eClass().getEPackage() == modelPackage;
		}
		return false;
	}

	/**
	 * The switch that delegates to the <code>createXXX</code> methods.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected TimeSwitch<Adapter> modelSwitch =
		new TimeSwitch<Adapter>() {
			@Override
			public Adapter caseTimeValue(TimeValue object) {
				return createTimeValueAdapter();
			}
			@Override
			public Adapter caseInstantValue(InstantValue object) {
				return createInstantValueAdapter();
			}
			@Override
			public Adapter caseClock(Clock object) {
				return createClockAdapter();
			}
			@Override
			public Adapter caseInstant(Instant object) {
				return createInstantAdapter();
			}
			@Override
			public Adapter caseLogicalClock(LogicalClock object) {
				return createLogicalClockAdapter();
			}
			@Override
			public Adapter caseDurationValue(DurationValue object) {
				return createDurationValueAdapter();
			}
			@Override
			public Adapter caseTimeIntervalValue(TimeIntervalValue object) {
				return createTimeIntervalValueAdapter();
			}
			@Override
			public Adapter caseJunctionInstant(JunctionInstant object) {
				return createJunctionInstantAdapter();
			}
			@Override
			public Adapter defaultCase(EObject object) {
				return createEObjectAdapter();
			}
		};

	/**
	 * Creates an adapter for the <code>target</code>.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param target the object to adapt.
	 * @return the adapter for the <code>target</code>.
	 * @generated
	 */
	@Override
	public Adapter createAdapter(Notifier target) {
		return modelSwitch.doSwitch((EObject)target);
	}


	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.time.TimeValue <em>Value</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.time.TimeValue
	 * @generated
	 */
	public Adapter createTimeValueAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.time.InstantValue <em>Instant Value</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.time.InstantValue
	 * @generated
	 */
	public Adapter createInstantValueAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.time.Clock <em>Clock</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.time.Clock
	 * @generated
	 */
	public Adapter createClockAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.time.Instant <em>Instant</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.time.Instant
	 * @generated
	 */
	public Adapter createInstantAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.time.LogicalClock <em>Logical Clock</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.time.LogicalClock
	 * @generated
	 */
	public Adapter createLogicalClockAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.time.DurationValue <em>Duration Value</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.time.DurationValue
	 * @generated
	 */
	public Adapter createDurationValueAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.time.TimeIntervalValue <em>Interval Value</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.time.TimeIntervalValue
	 * @generated
	 */
	public Adapter createTimeIntervalValueAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.time.JunctionInstant <em>Junction Instant</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.time.JunctionInstant
	 * @generated
	 */
	public Adapter createJunctionInstantAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for the default case.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @generated
	 */
	public Adapter createEObjectAdapter() {
		return null;
	}

} //TimeAdapterFactory
