/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: PluginBlockImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.corelib.impl;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.corelib.CorelibPackage;
import fr.supelec.modhelx.corelib.PluginBlock;
import fr.supelec.modhelx.util.PluggableBlock;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Plugin Block</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.corelib.impl.PluginBlockImpl#getPluggedInClassName <em>Plugged In Class Name</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class PluginBlockImpl extends AtomicBlockImpl implements PluginBlock {
	private ModelOfComputation myMoC_ = null;
	private PluggableBlock pluggedIn_ = null;
	/**
	 * The default value of the '{@link #getPluggedInClassName() <em>Plugged In Class Name</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getPluggedInClassName()
	 * @generated
	 * @ordered
	 */
	protected static final String PLUGGED_IN_CLASS_NAME_EDEFAULT = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected PluginBlockImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return CorelibPackage.Literals.PLUGIN_BLOCK;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public String getPluggedInClassName() {
		return (String) getValueOfParameter("pluggedInClassName");
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setPluggedInClassName(String newPluggedInClassName) {
		setValueOfParameter("pluggedInClassName", newPluggedInClassName);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		super.setup(moc, initialTime);
		myMoC_ = moc;
		String pluggedInClassName = getPluggedInClassName();
		if (pluggedInClassName == null) {
			throw new Error(getClass().getName()+".setup(): plugged in class name not specified");
		}
		try {
			pluggedIn_ = (PluggableBlock) getClass().getClassLoader().loadClass(pluggedInClassName).newInstance();
		} catch (InstantiationException e) {
			e.printStackTrace();
			throw new Error(getClass().getName()+".setup(): plugged in class name cannot be instanciated");
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new Error(getClass().getName()+".setup(): plugged in class name cannot be accessed");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			throw new Error(getClass().getName()+".setup(): plugged in class name \"" + pluggedInClassName + "\" cannot be found");
		}
		pluggedIn_.handledBy(this);
		pluggedIn_.setup(moc, initialTime);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void startOfSnapshot(ModelOfComputation moc) {
		super.startOfSnapshot(moc);
		pluggedIn_.startOfSnapshot(moc);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void reset(ModelOfComputation moc, InstantValue time) {
		super.reset(moc, time);
		pluggedIn_.reset(moc, time);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc) {
		super.update(moc);
		pluggedIn_.update(moc);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean validate(ModelOfComputation moc) {
		return (super.validate(moc) && pluggedIn_.validate(moc));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfSnapshot(ModelOfComputation moc) {
		super.endOfSnapshot(moc);
		pluggedIn_.endOfSnapshot(moc);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void wrapup(ModelOfComputation moc) {
		super.wrapup(moc);
		pluggedIn_.wrapup(moc);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean snapDet(ModelOfComputation moc) {
		return (super.snapDet(moc) && pluggedIn_.snapDet(moc));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void collectTimeConstraints(ModelOfComputation moc, EList<InstantValue> timeConstraints) {
		super.collectTimeConstraints(moc, timeConstraints);
		pluggedIn_.collectTimeConstraints(moc, timeConstraints);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean newSnap(ModelOfComputation moc) {
		return (super.newSnap(moc) || pluggedIn_.newSnap(moc));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public ModelOfComputation getMoC() {
		return myMoC_;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case CorelibPackage.PLUGIN_BLOCK__PLUGGED_IN_CLASS_NAME:
				return getPluggedInClassName();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case CorelibPackage.PLUGIN_BLOCK__PLUGGED_IN_CLASS_NAME:
				setPluggedInClassName((String)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case CorelibPackage.PLUGIN_BLOCK__PLUGGED_IN_CLASS_NAME:
				setPluggedInClassName(PLUGGED_IN_CLASS_NAME_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case CorelibPackage.PLUGIN_BLOCK__PLUGGED_IN_CLASS_NAME:
				return PLUGGED_IN_CLASS_NAME_EDEFAULT == null ? getPluggedInClassName() != null : !PLUGGED_IN_CLASS_NAME_EDEFAULT.equals(getPluggedInClassName());
		}
		return super.eIsSet(featureID);
	}

} //PluginBlockImpl
