/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DeFsmAdapterImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.adapterlib.impl;

import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.InterfaceBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.AdapterlibPackage;
import fr.supelec.modhelx.mocs.adapterlib.DeFsmAdapter;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.fsm.FsmModelOfComputation;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>De Fsm Adapter</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.adapterlib.impl.DeFsmAdapterImpl#getPreparationTime <em>Preparation Time</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class DeFsmAdapterImpl extends InterfaceBlockImpl implements DeFsmAdapter {
	/**
	 * The default value of the '{@link #getPreparationTime() <em>Preparation Time</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getPreparationTime()
	 * @generated
	 * @ordered
	 */
	protected static final int PREPARATION_TIME_EDEFAULT = 0;

    /**
     * Variable permettant de stocker la date du dernier événement "coffee" reçu
     */
    private double tLastDEevt;

    /**
     * Private fields to store the different pins we need to recognize in setup and update
     */
	private Pin coffeePin;
	private Pin servedPin;
	private Pin coinPin;
	
	private DeModelOfComputation externalmoc;
	private FsmModelOfComputation internalmoc;

    /**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	protected DeFsmAdapterImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return AdapterlibPackage.Literals.DE_FSM_ADAPTER;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public int getPreparationTime() {
		Object value = getValueOfParameter("preparationTime");
		if (value == null) {
			return PREPARATION_TIME_EDEFAULT;
		}
		try {
			Integer v = (Integer) value;
			return v.intValue();
		}
		catch (ClassCastException ex) {
			throw new Error("getPreparationTime : getPreparationTime is not an Integer");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setPreparationTime(int newPreparationTime) {
		setValueOfParameter("preparationTime", new Integer(newPreparationTime));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void adaptIn(ModelOfComputation moc) {
		super.adaptIn(moc);

		if(coinPin.getStoredTokens().size()==1){
			Token extevt = coinPin.getToken(true);
			Platform.getLog().info("****** DeFsmAdapterImpl.adapIn : coin event received and consumed ("+extevt+")");
			
			Pin intCoinPin = coinPin.getOutgoingRelations().get(0).getTarget();
			Token intevt = internalmoc.createEvent();
			intCoinPin.addToken(intevt);
			Platform.getLog().info("****** DeFsmAdapterImpl.adapIn : coin event put on internal model ("+intevt+")");
		}
		if(coffeePin.getStoredTokens().size()==1){ // TODO We assume that the events can be received at the same time ?
			Token extevt = coffeePin.getToken(false);
			InstantValue evttime = externalmoc.getTimestampOfEvent(extevt);
			this.tLastDEevt = evttime.getDenotedInstant().get(0).getDate();
			coffeePin.getStoredTokens().remove(extevt);
			Platform.getLog().info("****** DeFsmAdapterImpl.adapIn : coffee event received ("+extevt+") with time "+this.tLastDEevt);
			
			Pin intCoffeePin = coffeePin.getOutgoingRelations().get(0).getTarget();
			Token intevt = internalmoc.createEvent();
			intCoffeePin.addToken(intevt);
			Platform.getLog().info("****** DeFsmAdapterImpl.adapIn : coffee event put on internal model ("+intevt+")");
			
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void adaptOut(ModelOfComputation moc) {
		super.adaptOut(moc);

		if(servedPin.getIncomingRelations().get(0).getSource().getStoredTokens().size()==1){
			
			Token intevt = servedPin.getIncomingRelations().get(0).getSource().getToken(true);
			Platform.getLog().info("****** DeFsmAdapterImpl.adapIn : internal served event received and consumed ("+intevt+")");
			
			InstantValue timeServed = externalmoc.getClock().createInstantValue(this.tLastDEevt+getPreparationTime());
			Token extevt = externalmoc.createEvent(timeServed, new Integer(1));
			this.servedPin.addToken(extevt);
			
			Platform.getLog().info("****** DeFsmAdapterImpl.adapIn : served event put on external served pin ("+intevt+") with time "+timeServed);
		}

	}
	
	@Override
	protected boolean doUpdateInternalModel(ModelOfComputation moc) {
		return true;
	}


	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		super.setup(moc, initialTime);
		
		//Check that external MoC is DE
		externalmoc = getExternalMoc(moc);

		//Check that internal MoC is FSM
		try {
			internalmoc = (FsmModelOfComputation)this.getInternalModel().getMoc();
		}
		catch (ClassCastException ex) {
			throw new Error("DeFsmAdapter : external moc is not FSM");
		}
		
		//Initial check on the number of pins
		if(getPinsIn().size() < 2) throw new Error("DeFsmAdapter : missing input pins");
		if(getPinsOut().size() < 1) throw new Error("DeFsmAdapter : missing output pins");

		//Check that the different pins have been created using the appropriate methods so that we can recognize them
		if(coffeePin==null) throw new Error("DeFsmAdapter : coffee pin not recognized, use the createCoffeePin method to create this pin");
		if(coinPin==null) throw new Error("DeFsmAdapter : coin pin not recognized, use the createCoinPin method to create this pin");
		if(servedPin==null) throw new Error("DeFsmAdapter : served pin not recognized, use the createServedPin method to create this pin");
		
		//Check the relations from external pins to pins of the internal model (and vice versa)
		if(coffeePin.getOutgoingRelations().size()<1) throw new Error("DeFsmAdapter : coffee pin not connected to the internal model");
		if(coinPin.getOutgoingRelations().size()<1) throw new Error("DeFsmAdapter : coin pin not connected to the internal model");
		if(servedPin.getIncomingRelations().size()<1) throw new Error("DeFsmAdapter : internal model not connected to the served pin");
		
		//Initialize the date of the last received event (not really necessary)
		tLastDEevt = 0;
	}
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Pin createCoinPin() {
		coinPin = CoreFactory.eINSTANCE.createPin();
		coinPin.setName("coin");
		this.getPinsIn().add(coinPin);
		return coinPin;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Pin createCoffeePin() {
		coffeePin = CoreFactory.eINSTANCE.createPin();
		coffeePin.setName("coffee");
		this.getPinsIn().add(coffeePin);
		return coffeePin;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Pin createServedPin() {
		servedPin = CoreFactory.eINSTANCE.createPin();
		servedPin.setName("served");
		this.getPinsOut().add(servedPin);
		return servedPin;
	}

	private DeModelOfComputation getExternalMoc(ModelOfComputation moc) {
		try {
			DeModelOfComputation m = (DeModelOfComputation)moc;
			return m;
		}
		catch (ClassCastException ex) {
			throw new Error("DeFsmAdapter : external moc is not DE");
		}
	}


	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case AdapterlibPackage.DE_FSM_ADAPTER__PREPARATION_TIME:
				return getPreparationTime();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case AdapterlibPackage.DE_FSM_ADAPTER__PREPARATION_TIME:
				setPreparationTime((Integer)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case AdapterlibPackage.DE_FSM_ADAPTER__PREPARATION_TIME:
				setPreparationTime(PREPARATION_TIME_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case AdapterlibPackage.DE_FSM_ADAPTER__PREPARATION_TIME:
				return getPreparationTime() != PREPARATION_TIME_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}

} //DeFsmAdapterImpl
