/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DeSdfAdapterImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.adapterlib.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.InterfaceBlockImpl;
import fr.supelec.modhelx.core.time.DurationValue;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.AdapterlibPackage;
import fr.supelec.modhelx.mocs.adapterlib.DeSdfAdapter;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>De Sdf Adapter</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.adapterlib.impl.DeSdfAdapterImpl#getSampleTime <em>Sample Time</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class DeSdfAdapterImpl extends InterfaceBlockImpl implements DeSdfAdapter {
	/**
	 * The default value of the '{@link #getSampleTime() <em>Sample Time</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getSampleTime()
	 * @generated
	 * @ordered
	 */
	protected static final int SAMPLE_TIME_EDEFAULT = -1;

	// parameter name for the token rate
	private final static String	SDF_RATE_PARAM_NAME = "SDFTokenRate";
	
	// parameter name for the sample time
	private final static String SAMPLE_TIME_PARAM_NAME = "SampleTime";
	
	// parameter name for the initial value of an inner pin
	private final static String INITIAL_VALUE_PARAM_NAME = "InitialValue";
	
	// time of the next sample
	private InstantValue timeOfNextSample = null;
	
	private InstantValue timeOfCurrentSnapshot = null;
	
	// sampled/blocked value on I/O pins
	private final Map<Pin, Object> sampledValues = new HashMap<Pin, Object>();

	private boolean doUpdateInternalModel;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DeSdfAdapterImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return AdapterlibPackage.Literals.DE_SDF_ADAPTER;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public int getSampleTime() {
		Object o = getValueOfParameter(SAMPLE_TIME_PARAM_NAME);
		if(o instanceof Integer) return (Integer)o;
		else return SAMPLE_TIME_EDEFAULT;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setSampleTime(int newSampleTime) {
		setValueOfParameter(SAMPLE_TIME_PARAM_NAME, newSampleTime);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public void adaptIn(ModelOfComputation moc) {
		super.adaptIn(moc);
		
		DeModelOfComputation deMoc = (DeModelOfComputation) moc;
		SdfModelOfComputation sdfMoc = (SdfModelOfComputation) getInternalModel().getMoc();
		
		for(Pin pIn : getPinsIn()) {
			if(pIn.hasTokens()) {
				Object val = deMoc.getValueOfToken( pIn.getToken(true) );
				sampledValues.put(pIn, val);
			}

			// TODO: hack en attendant le retour de Cécile sur ce qui est conditionné par doUpdateInternalModel()
			if(doUpdateInternalModel(moc)) {
				for(Relation r : pIn.getOutgoingRelations()) {
					Pin pInner = r.getTarget();
					Token t = sdfMoc.createDataToken(sampledValues.get(pIn));
					Platform.getLog().finer(getName() + ": adding SDF token [val=" + sdfMoc.getValueOfToken(t) + "] onto [" + pInner + "]");
					pInner.addToken(t);
					Platform.getLog().finest("now on " + pInner + ": " + pInner.getStoredTokens());
				}
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public void adaptOut(ModelOfComputation moc) {
		super.adaptOut(moc);
		
		// TODO: hack en attendant le retour de Cécile sur ce qui est conditionné par doUpdateInternalModel()
		if(! doUpdateInternalModel(moc)) return;
		
		DeModelOfComputation deMoc = (DeModelOfComputation) moc;
		SdfModelOfComputation sdfMoc = (SdfModelOfComputation) getInternalModel().getMoc();
		
		for(Pin pOut : getInternalModel().getStructure().getPinsOut()) {
			while(pOut.hasTokens()) {  // TODO instead of removing all tokens, maybe we should be removing only the token count corresponding to the rate
				Object val = sdfMoc.getValueOfToken(pOut.getToken(true));
				Object lastValue = sampledValues.get(pOut);
				if(lastValue == null || !lastValue.equals(val)) {
					sampledValues.put(pOut, val);
					
					for(Relation r : pOut.getOutgoingRelations()) {
						Token t = deMoc.createEvent(timeOfCurrentSnapshot, val);
						Pin pOuter = r.getTarget();
						//System.out.println(getName() + ": adding DE event [val=" + val + "] onto [" + pOuter + "], time=" + timeOfCurrentSnapshot);
						Platform.getLog().finer(getName() + ": adding DE event [val=" + val + "] onto [" + pOuter + "], time=" + timeOfCurrentSnapshot);
						pOuter.addToken(t);
					}
					
				}
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public void setup(ModelOfComputation extMoc, InstantValue initialTime) {
		super.setup(extMoc, initialTime);
		
		Platform.getLog().finer("** Adapter: setup at " + initialTime);
		
		// check that the external MoC is DE
		if(! (extMoc instanceof DeModelOfComputation))
			throw new Error("DeSdfAdapter: external MoC must be DE.");
		
		// check that the internal MoC is SDF
		if(! (getInternalModel().getMoc() instanceof SdfModelOfComputation))
			throw new Error("DeSdfAdapter: internal MoC must be SDF.");
		
		// check if every input and output pin of the inner model has a sample rate
		if(!checkPinsHaveSampleRate(getInternalModel().getStructure().getPinsIn()) ||
				!checkPinsHaveSampleRate(getInternalModel().getStructure().getPinsOut())) {
			throw new Error("DeSdfAdapter: all pins of internal model must have the " + SDF_RATE_PARAM_NAME + " parameter.");
		}
		
		// check if the sample time has been explicitly set, and is strictly positive
		if(getSampleTime() <= 0)
			throw new Error("DeSdfAdapter: the sample time must be set explicitly, at a stricly positive value.");
		
		// check if all inner input pins have the InitialValue parameter
		for(Pin pIn : getInternalModel().getStructure().getPinsIn()) {
			Object ini = getInitialValue(pIn);
			if(ini == null)
				throw new Error("DeSdfAdapter: every inner input pin must have an explicit initial value. " + pIn + " doesn't have one.");
			else {
				// the pin must have only one incoming relation, coming from the outer pin
				List<Relation> rels = pIn.getIncomingRelations();
				if(rels.size() != 1)
					throw new Error("DeSdfAdapter: every inner input pin must have exaclty one incoming relation.");
				
				Pin pInOuter = rels.get(0).getSource();
				
				// put this value in the sample value map
				sampledValues.put(pInOuter, ini);
			}
		}
		
	}
	

	private boolean checkPinsHaveSampleRate(EList<Pin> l) {
		for(Pin p : l)
			if(p.getParameters().get(SDF_RATE_PARAM_NAME) == null) return false;
		return true;
	}
	
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public void collectTimeConstraints(ModelOfComputation moc, EList<InstantValue> timeConstraints) {
		DeModelOfComputation deMoc = (DeModelOfComputation) moc;
		
		if(timeOfNextSample == null) {
			timeOfNextSample = deMoc.getClock().createInstantValue(0.);
			//Platform.getLog().finest("First snapshot. Constraint at default instant 0.");
		}
		
		Platform.getLog().finer("Adding constraint: " + timeOfNextSample);
		
		//Platform.getLog().finer("Before: " + timeConstraints);
		timeConstraints.add(timeOfNextSample);
		//Platform.getLog().finer("After: " + timeConstraints);
		
		super.collectTimeConstraints(deMoc, timeConstraints, timeOfNextSample);
	}
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public void setInitialValue(Pin pin, Object value) {
		pin.setValueOfParameter(INITIAL_VALUE_PARAM_NAME, value);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public Object getInitialValue(Pin pin) {
		return pin.getValueOfParameter(INITIAL_VALUE_PARAM_NAME);
	}

	/*
	 * TODO: delete this, since we don't adapt time at reset.
	 * 
	@Override
	public void reset(ModelOfComputation moc, InstantValue time) {
		InstantValue sdfTime = ((SdfModelOfComputation)getInternalModel().getMoc()).getClock().createInstantValue(
				time.getDenotedInstant().get(0).getDate() / sampleTime);
		Platform.getLog().finer("Adapting time: DE time=" + time + "; SDF time=" + sdfTime);
		super.reset(moc, sdfTime);
	}
	*/
	
	
	@Override
	protected boolean doUpdateInternalModel(ModelOfComputation moc) {
		doUpdateInternalModel = timeOfCurrentSnapshot.compareTo(timeOfNextSample) == 0;
		Platform.getLog().finer("Time_current_snapshot=" + timeOfCurrentSnapshot + ",  Time_next_sample=" + timeOfNextSample + " => " + (doUpdateInternalModel ? "do!" : "skip"));
		return doUpdateInternalModel;
	}
	
	@Override
	public void reset(ModelOfComputation moc, InstantValue time) {
		super.reset(moc, time);
		
		timeOfCurrentSnapshot = time;
	}
	
	@Override
	public void endOfSnapshot(ModelOfComputation moc) {
		super.endOfSnapshot(moc);
		
		if(doUpdateInternalModel) {
			DeModelOfComputation deMoc = (DeModelOfComputation) moc;
			
			DurationValue d = deMoc.getClock().createDurationValue(
					deMoc.getClock().createInstantValue(0),
					deMoc.getClock().createInstantValue(getSampleTime()));
			timeOfNextSample = timeOfNextSample.add(d);
		}
	}
	
	@Override
	public boolean newSnap(ModelOfComputation moc) {
		return getInternalModel().newSnap();
	}
// TODO supprimer (mis au niveau générique de IBImpl)	

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case AdapterlibPackage.DE_SDF_ADAPTER__SAMPLE_TIME:
				return getSampleTime();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case AdapterlibPackage.DE_SDF_ADAPTER__SAMPLE_TIME:
				setSampleTime((Integer)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case AdapterlibPackage.DE_SDF_ADAPTER__SAMPLE_TIME:
				setSampleTime(SAMPLE_TIME_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case AdapterlibPackage.DE_SDF_ADAPTER__SAMPLE_TIME:
				return getSampleTime() != SAMPLE_TIME_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}

} //DeSdfAdapterImpl
