/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DeTfsmAdapterImpl.java 334 2011-03-14 16:47:41Z hardebolle $
 */
package fr.supelec.modhelx.mocs.adapterlib.impl;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.InterfaceBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.AdapterlibPackage;
import fr.supelec.modhelx.mocs.adapterlib.DeTfsmAdapter;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.tfsm.TFsmModelOfComputation;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>De Tfsm Adapter</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class DeTfsmAdapterImpl extends InterfaceBlockImpl implements DeTfsmAdapter {
	
	private final static String ASSOCIATED_VALUE_PARAM_NAME = "AssociatedValue";
	private final static String INGORED_VALUES_PARAM_NAME = "IgnoredValues";
	
	private DeModelOfComputation externalMoc;
	private TFsmModelOfComputation internalMoc;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DeTfsmAdapterImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return AdapterlibPackage.Literals.DE_TFSM_ADAPTER;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void adaptIn(ModelOfComputation moc) {
		super.adaptIn(moc);
		
		// is there a token on one of the input pins?
		for(Pin pIn : getPinsIn()) {
			while(pIn.hasTokens()){
			//if(pIn.getStoredTokens().size() > 0) {
			// 2011-03-14 : added a loop on the tokens in order to take into account more that one token on each pin of the interface block
				
				// retrieve the token and the associated value
				Token evt = pIn.getToken(true);
				
				Object evtValue = externalMoc.getValueOfToken(evt);
				if(evtValue == null) throw new Error("Incoming event carrying no value: " + evt);
				
				Platform.getLog().finer("DeTfsmAdapter: got token " + evt + " on pin " + pIn + " with value " + evtValue + " at time " + externalMoc.getTimeOfCurrentSnapshot());
				
				// is the value referenced on one of the relations to the TFSM?
				boolean relationFound = false;
				for(Relation r : pIn.getOutgoingRelations()) {
					if(evtValue.equals(r.getValueOfParameter(ASSOCIATED_VALUE_PARAM_NAME))) {
						// then add a pure event to the internal model pin
						Token pureEvent = internalMoc.createEvent();
						r.getTarget().addToken(pureEvent);
						
						Platform.getLog().finer("External event " + evtValue + " on " + pIn + " transformed into an internal token onto " + r.getTarget());
						
						relationFound = true;
						break;
					}
				}

				// Relation was not found. Either this is a normal situation,
				// in which case the value is declared to be ignored, or this
				// is abnormal
				if(!relationFound) {
					Object ignoredValues = pIn.getValueOfParameter(INGORED_VALUES_PARAM_NAME);
					if(! (ignoredValues instanceof EList<?> && ((EList<?>)ignoredValues).contains(evtValue)))
						System.err.println("On pin " + pIn + ": no outgoing relation nor ignore list entry found for event value " + evtValue);
				}
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void adaptOut(ModelOfComputation moc) {
		// is there a token on one of the internal output pins?
		for(Pin pOut : getInternalModel().getStructure().getPinsOut()) {
			while(pOut.hasTokens()){
//			if(pOut.getStoredTokens().size() > 0) {
			// 2011-03-14 : added a loop on the tokens in order to take into account more that one token on each pin of the internal model
				// consume one token
				pOut.getToken(true);
				
				for(Relation r : pOut.getOutgoingRelations()) {
					// determine the value to associate with it in DE
					Object val = r.getValueOfParameter(ASSOCIATED_VALUE_PARAM_NAME);
					Token deEvent = externalMoc.createEvent(externalMoc.getTimeOfCurrentSnapshot(), val);
					r.getTarget().addToken(deEvent);
					
					Platform.getLog().finer("Internal token transformed into a DE event " + val + " onto " + r.getTarget());
				}
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		super.setup(moc, initialTime);
		
		// check if the internal and external MoCs are correct
		if(moc instanceof DeModelOfComputation) {
			externalMoc = (DeModelOfComputation) moc;
		} else {
			throw new Error("The external MoC should be DE, not " + moc);
		}
		
		if(getInternalModel().getMoc() instanceof TFsmModelOfComputation) {
			internalMoc = (TFsmModelOfComputation) getInternalModel().getMoc();
		} else {
			throw new Error("The internal MoC should be TFSM, not " + getInternalModel().getMoc());
		}
		
		// check if all relations gave the parameter 'AssociatedValue'
		for(Relation r : getRelations()) {
			if(r.getValueOfParameter(ASSOCIATED_VALUE_PARAM_NAME) == null)
				throw new Error("Relation lacking the " + ASSOCIATED_VALUE_PARAM_NAME + " parameter: " + r);
		}
		
		// TODO: check if all pins are connected => do it generically for all interface blocks? 
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void collectTimeConstraints(ModelOfComputation moc, EList<InstantValue> timeConstraints) {
		super.collectTimeConstraints(moc, timeConstraints);
		getInternalModel().collectTimeConstraints(timeConstraints);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setValueAssociatedWithInterfaceRelation(Relation relation, Object value) {
		relation.setValueOfParameter(ASSOCIATED_VALUE_PARAM_NAME, value);
	}
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setIgnoredValuesOnPin(Pin pin, EList<Object> values) {
		pin.setValueOfParameter(INGORED_VALUES_PARAM_NAME, values);
	}

	@Override
	protected boolean doUpdateInternalModel(ModelOfComputation moc) {
		return true;
	}

} //DeTfsmAdapterImpl
