/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DeModelOfComputationImpl.java 334 2011-03-14 16:47:41Z hardebolle $
 */
package fr.supelec.modhelx.mocs.de.impl;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;
import java.util.logging.Level;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.ModelOfComputationImpl;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.de.DePackage;
import fr.supelec.modhelx.util.DEBlockDependencyComparator;
import fr.supelec.modhelx.util.Selector;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>DE Model Of Computation</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.de.impl.DeModelOfComputationImpl#getClock <em>Clock</em>}</li>
 *   <li>{@link fr.supelec.modhelx.mocs.de.impl.DeModelOfComputationImpl#getTimeOfCurrentSnapshot <em>Time Of Current Snapshot</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class DeModelOfComputationImpl extends ModelOfComputationImpl implements DeModelOfComputation {
	/**
	 * The cached value of the '{@link #getClock() <em>Clock</em>}' reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getClock()
	 * @generated
	 * @ordered
	 */
	protected Clock clock;

	/**
	 * The cached value of the '{@link #getTimeOfCurrentSnapshot() <em>Time Of Current Snapshot</em>}' reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getTimeOfCurrentSnapshot()
	 * @generated
	 * @ordered
	 */
	protected InstantValue timeOfCurrentSnapshot;

	/**
	 * The global list of events representing the state of the model.
	 */
	private ArrayList<Token> globalEventList;
	
	/**
	 * Two lists with the events to process in the current snapshot.
	 * initialEventList : is used when re-computing a snapshot which has not been validated
	 * currentEventList : is used during the computation of the snapshot
	 */
	private ArrayList<Token> initialEventList;
	private ArrayList<Token> currentEventList;
	
	/**
	 * Three lists to maintain DE-specific data on all the tokens which represent DE events during computation.
	 * valueOfEvents : keeps the value of each event
	 * timestampOfEvents : keeps the timestamp of each event
	 * destinationPinOfEvents :keeps the destination pin of each event
	 */
	//Le MoC maintient les donnees specifiques DE correspondant aux tokens
	//ATTENTION contient TOUTES les donnees (globales, initiales, courantes) au cours du snapshot
	//donc ajouts au cours du snapshot mais nettoyage necessaire au reset et au eos
	private WeakHashMap<Token, Object> valueOfEvents;
	private WeakHashMap<Token, InstantValue> timestampOfEvents;
	private WeakHashMap<Token, Pin> destinationPinOfEvents;
	//TODO question : dupliquer initial/courant ?
	private WeakHashMap<Token, Object> initialValueOfEvents;
	private WeakHashMap<Token, InstantValue> initialTimestampOfEvents;
	private WeakHashMap<Token, Pin> initialDestinationPinOfEvents;
	//CAUTION use of weakhashmaps to get rid of the deletion of tokens in the three lists... but may generate bugs 
	
	/**
	 * Two lists with the blocks that have to be updated (either because of a constraint or of an event) for the current snapshot
	 * initialBlocksWithMinTimeConstraint : is used when re-computing a snapshot which has not been validated
	 * currentBlocksWithMinTimeConstraint : is used during the computation of the snapshot
	 */
//	private ArrayList<Block> initialBlocksWithMinTimeConstraint;
//	private ArrayList<Block> currentBlocksWithMinTimeConstraint;
	private HashSet<Block> initialBlocksWithMinTimeConstraint;
	private HashSet<Block> currentBlocksWithMinTimeConstraint;
	
	

	/**
	 * Minimum date of all the constraints for the current snapshot
	 */
	private InstantValue minTimeConstraint;

	/**
	 * The block to update in the current step of computation. 
	 */
	private Block currentBlock;

	/**
	 * A map assigning a rank to the blocks according to their dependencies
	 * Two independent block have the same rank.
	 * A block which depends on another has a higher rank.
	 */
	private Map<Block, Integer> blockRanks_ = null;
	
	/**
	 * The list of all the blocks of the model, in a topological order.
	 */
//	private ArrayList<Block> blockSorted;
	
//	private boolean someSourceBlocks;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	protected DeModelOfComputationImpl() {
		super();
		
		valueOfEvents = new WeakHashMap<Token, Object>();
		timestampOfEvents = new WeakHashMap<Token, InstantValue>();
		destinationPinOfEvents = new WeakHashMap<Token, Pin>();
		initialValueOfEvents = new WeakHashMap<Token, Object>();
		initialTimestampOfEvents = new WeakHashMap<Token, InstantValue>();
		initialDestinationPinOfEvents = new WeakHashMap<Token, Pin>();

		globalEventList = new ArrayList<Token>();
		initialEventList = new ArrayList<Token>();
		currentEventList = new ArrayList<Token>();
//		initialBlocksWithMinTimeConstraint = new ArrayList<Block>();
//		currentBlocksWithMinTimeConstraint = new ArrayList<Block>();
		initialBlocksWithMinTimeConstraint = new HashSet<Block>();
		currentBlocksWithMinTimeConstraint = new HashSet<Block>();
		minTimeConstraint = null;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return DePackage.Literals.DE_MODEL_OF_COMPUTATION;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Clock getClock() {
		if (clock != null && clock.eIsProxy()) {
			InternalEObject oldClock = (InternalEObject)clock;
			clock = (Clock)eResolveProxy(oldClock);
			if (clock != oldClock) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, DePackage.DE_MODEL_OF_COMPUTATION__CLOCK, oldClock, clock));
			}
		}
		return clock;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Clock basicGetClock() {
		return clock;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setClock(Clock newClock) {
		Clock oldClock = clock;
		clock = newClock;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, DePackage.DE_MODEL_OF_COMPUTATION__CLOCK, oldClock, clock));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public InstantValue getTimeOfCurrentSnapshot() {
		if (timeOfCurrentSnapshot != null && timeOfCurrentSnapshot.eIsProxy()) {
			InternalEObject oldTimeOfCurrentSnapshot = (InternalEObject)timeOfCurrentSnapshot;
			timeOfCurrentSnapshot = (InstantValue)eResolveProxy(oldTimeOfCurrentSnapshot);
			if (timeOfCurrentSnapshot != oldTimeOfCurrentSnapshot) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, DePackage.DE_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT, oldTimeOfCurrentSnapshot, timeOfCurrentSnapshot));
			}
		}
		return timeOfCurrentSnapshot;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public InstantValue basicGetTimeOfCurrentSnapshot() {
		return timeOfCurrentSnapshot;
	}

	/**
	 * collectTimeConstraints is called by the platform when it computes the time of the next snapshot.
	 * Constraints are collected in the timeConstraints argument.
	 * <!-- begin-user-doc -->
	 * In DE, the time of the next snapshot depends not only on the constraints emitted by the blocks but also
	 * on the events which are present in the global even list.
	 * In this operation :
	 * <ul>
	 * <li>we constitue the list of blocks which emit a time constraint, in initialBlocksWithMinTimeConstraint ;</li>
	 * <li>we list, in the EList timeConstraints, the time constraints emitted by the blocks and the time constraints which can be deduced from the events remaining in the global event list ;</li>
	 * <li>we compute the minimum time of all these contraints, in minTimeConstraint ;</li>
	 * </ul>
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void collectTimeConstraints(Model model, EList<InstantValue> timeConstraints) {
		
    	//super.collectTimeConstraints(model, timeConstraints);
		Platform.getLog().entering(getClass().getName(), "collectTimeConstraints", model.getName());
		
		// We need to keep the set of blocks which have a minimal time constraint
		initialBlocksWithMinTimeConstraint.clear();

		// We need to keep the minimum date of the time constraints 
		minTimeConstraint = null;
		
		// First we collect the constraints which are emitted by the blocks of the model
		// And we list the blocks which emit a constraint
		InstantValue minConstraintTime = null;
    	
    	for (Block b : model.getStructure().getBlocks()) {
        	EList<InstantValue> localTimeConstraints = new BasicEList<InstantValue>();
        	b.collectTimeConstraints(this, localTimeConstraints); // In localTimeConstraints we have the contraints emitted by the block b
        	
        	Platform.getLog().finest("Time constraints collected for block " + b.getName() + ": " + localTimeConstraints); 
        	
        	if (localTimeConstraints.isEmpty()) continue; // If b has not emitted any constraint, we collect the constraints of the next block
        	
        	InstantValue i = localTimeConstraints.get(0); // Else, we initialize i with the date of the first contraint
        	
        	if (minConstraintTime == null) {
        		minConstraintTime = i;
        		initialBlocksWithMinTimeConstraint.add(b);
        	} else if (i.getDenotedInstant().get(0).getDate() < minConstraintTime.getDenotedInstant().get(0).getDate()) {
        		minConstraintTime = i;
        		initialBlocksWithMinTimeConstraint.clear();
        		initialBlocksWithMinTimeConstraint.add(b);
        	} else if (i.getDenotedInstant().get(0).getDate() == minConstraintTime.getDenotedInstant().get(0).getDate()) {
        		initialBlocksWithMinTimeConstraint.add(b);
        	}
    	}
    	
    	// Then we check the dates of the events in the global event list
    	// We do not need to keep the set of blocks which are destination of the events, this will be computed dynamically during the initial schedule
		InstantValue minEvtsTime = null;
		// First we initialize minEvtsTime
		if (minConstraintTime != null) {
			minEvtsTime = minConstraintTime;//
		} else if (globalEventList.size() > 0) {
			minEvtsTime = timestampOfEvents.get(globalEventList.get(0));
		}
		
		if (minEvtsTime != null) {
			// Then we compute the minimum date of the events in the list
			for (Token t : globalEventList) {
				InstantValue ttimestamp = timestampOfEvents.get(t);
				if(ttimestamp.getDenotedInstant().get(0).getDate() <= minEvtsTime.getDenotedInstant().get(0).getDate()){
					minEvtsTime = ttimestamp;
				}
			}
		}
		
    	// At last we take the minimum date and post a constraint on it
		if (minConstraintTime != null && minEvtsTime != null ) {
			if(minConstraintTime.getDenotedInstant().get(0).getDate() > minEvtsTime.getDenotedInstant().get(0).getDate()){
				initialBlocksWithMinTimeConstraint.clear(); // if the minimum time is < to the minimum time of all the contraints
															// then we don't take the contraints into account
				minTimeConstraint = minEvtsTime;
			}else{
	    		minTimeConstraint = minConstraintTime;
			}
    		timeConstraints.add(minTimeConstraint);
    	} else if (minConstraintTime != null) {
    		minTimeConstraint = minConstraintTime;
    		timeConstraints.add(minTimeConstraint);
    	} else if (minEvtsTime != null) {
    		minTimeConstraint = minEvtsTime;
    		timeConstraints.add(minEvtsTime);
    	}
    	
		Platform.getLog().finer("final time constraints: " + timeConstraints);
	}

	/**
	 * <!-- begin-user-doc -->
	 * In DE, the startOfSnapshot is responsible for taking into account the events which may be present on the inputs of the model.
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void startOfSnapshot(Model model) {
		
		super.startOfSnapshot(model);
		
		// We put the events from the inputs of the model into the global event list
		for (Pin pIn : model.getStructure().getPinsIn()) {
			while (pIn.hasTokens()) {
				moveEventsToList(pIn, globalEventList);
			}
		}
		if (Platform.getLog().isLoggable(Level.INFO)) {
			Platform.getLog().info("DeModelOfComputation.startOfSnapshot : **** global event list = " + globalEventList);
			Platform.getLog().info("DeModelOfComputation.startOfSnapshot : **** global event list timestamps = " +timestampOfEvents);
		}
		
		// We save the content of the lists of events, for the reset operation (if the snapshot is not validated)
		initialValueOfEvents.clear();
		initialValueOfEvents.putAll(valueOfEvents);
		initialTimestampOfEvents.clear();
		initialTimestampOfEvents.putAll(timestampOfEvents);
		initialDestinationPinOfEvents.clear();
		initialDestinationPinOfEvents.putAll(destinationPinOfEvents);
		
	}

	/**
	 * <!-- begin-user-doc -->
	 * In DE, a new snapshot is necessary when :
	 * <ul>
	 * <li>the global event list is not empty ;</li>
	 * <li>the model contains blocks which are sources of events (they emit time constraints)</li>
	 * <li>or specified by execution parameters set by a user.</li>
	 * </ul>
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean newSnap(Model model) {
		super.newSnap(model);
		
		Platform.getLog().info("**** globalEventList = " + globalEventList);

		// A new snapshot is necessary :
		//1 - if the global event list is not empty
		if (globalEventList.size() > 0) {
			Platform.getLog().finer("DE.newSnap returns true because there are events to process");
			return true;
		}
		
//		if(someSourceBlocks) newSnap = true;
//		Platform.getLog().info("**** globalEventList = " + globalEventList + ", someSourceBlocks = " + someSourceBlocks + " => newSnap=" + newSnap);
		
		//or 
		//2 - if at least one block requires it (a source of events)
		for (Block b : model.getStructure().getBlocks()) {
			boolean newSnap = b.newSnap(this);
			Platform.getLog().finest("DE.newSpan: newSnap on " + b + " returns " + newSnap);
			if (newSnap) {
				Platform.getLog().finer("DE.newSnap returns true because block " + b + " requests more snapshots");
				return true;
			}
		}
		
		Platform.getLog().finer("DE.newSnap return false");
		return false;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	@SuppressWarnings ("unchecked")
	public void reset(Model model, InstantValue time) {
		super.reset(model, time);
		
		// Reset of the time of the current snapshot
		timeOfCurrentSnapshot = time;
		
		// Selection of the events to process in the current snapshot...
		DeEventTimeSelector sel = new DeEventTimeSelector(getTimeOfCurrentSnapshot());
		initialEventList = sel.selectAll(globalEventList);
		
		// ...And copy of the list in case of non validation of the snapshot
		currentEventList = (ArrayList<Token>)initialEventList.clone();
		
		// Copy of the different lists in case of non validation of the snapshot
		valueOfEvents.clear();
		valueOfEvents.putAll(initialValueOfEvents);
		timestampOfEvents.clear();
		timestampOfEvents.putAll(initialTimestampOfEvents);
		destinationPinOfEvents.clear();
		destinationPinOfEvents.putAll(initialDestinationPinOfEvents);

		if (Platform.getLog().isLoggable(Level.INFO)) {
			Platform.getLog().info("DeModelOfComputation.reset : **** current event list = " + currentEventList);
		}


		// Copy of the list of the blocks which have emitted a time constraint for this snapshot
		currentBlocksWithMinTimeConstraint = (HashSet<Block>) initialBlocksWithMinTimeConstraint.clone();
		
		Platform.getLog().info("**** current blocks with min time constr = " + currentBlocksWithMinTimeConstraint);

	}

    /**
     * Methode privee pour deplacer les evenements presents sur les sorties d'un bloc
     * dans une liste d'evenements
     */
    private void moveEventsToList(Pin psource, Collection<Token> listtarget){
		while (psource.hasTokens()) {
    		//Les evenements produits doivent etre transmis a tous les blocs connectes
    		// au point d'interface sur lequel ils ont ete emis
			//Donc, pour chaque evenement produit
			Token producedEvent = psource.getToken(true);
			
			//Et pour chaque relation partant de ce point d'interface, on creee une copie
			//Le destinataire de la copie est le point d'interface cible de la relation
    		for (Relation r : psource.getOutgoingRelations()) {
    			Token ev = CoreFactory.eINSTANCE.createToken();
    			valueOfEvents.put(ev, getValueOfToken(producedEvent));
    			timestampOfEvents.put(ev, getTimestampOfEvent(producedEvent));
    			destinationPinOfEvents.put(ev, r.getTarget());

    			//Chaque copie est ajoutee a la liste fournie
    			listtarget.add(ev);
    			//Remarque : si aucune relation ne part du point d'interface, l'evenement n'est pas deplace dans la liste
    			// car il n'est alors pas a traiter (il n'est destine a aucun bloc)
    		}
			this.timestampOfEvents.remove(producedEvent);
			this.valueOfEvents.remove(producedEvent);
			this.destinationPinOfEvents.remove(producedEvent);
    	}
    }

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void update(Model model) {
		// Only current block is updated
		//super.update(model);
		Platform.getLog().entering(getClass().getName(), "update", model.getName());
		Platform.getController().enterMethod(this, "update", model);
	
		Platform.getLog().info("**** update of = " + currentBlock);

		currentBlock.update(this);

		Platform.getController().leaveMethod();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public boolean snapDet(Model model) {
		super.snapDet(model);

		//Le snapshot est determine si :
		// 1- tous les evenements au temps courant ont ete traites
		//TODO modif sauf les evenements destines aux sorties du modele
//ATTENTION pose pb quand ces evenements sont a remettre dans un snapshot suivant dans lequel il n'y aura pas d'autres blocs a updater...
//SOLUTION choisie : ces evenements seront rremis sur les pins du modele dans le EOS, quelle que soit leur date
//Il ne faut donc pas en tenir compte pour determiner si le snapshot est fini

		ArrayList<Token> events = new ArrayList<Token>();
		events.addAll(currentEventList);
		boolean allevtstreated = false;

		//On evacue les tokens destines aux sorties du modele
		for(Token t : currentEventList){
			//Si on est dans le cas ou le token est destine a la sortie d'un bloc, et que ce bloc est la structure du modele
			if (  (destinationPinOfEvents.get(t).getOutputOf()!=null) 
			   && (model.getStructure().getPinsOut().contains(destinationPinOfEvents.get(t)))
			){
				//alors on peut enlever ce token de la liste des tokens restants
				events.remove(t);
			}
		}
		
		//Ensuite on evacue les tokens qui ne sont pas a traiter au temps courant
		DeEventTimeSelector sel = new DeEventTimeSelector(getTimeOfCurrentSnapshot());
		ArrayList<Token> remainingevents = sel.selectAll(events);
		allevtstreated = remainingevents.size() == 0; // vrai si il n'y a pas de tokens a traiter au temps courant

		// 2- toutes les contraintes au temps courant ont ete traitees
		boolean allconsttreated = currentBlocksWithMinTimeConstraint.size() == 0;

		Platform.getLog().info("**** snapdet DONE");

		return allevtstreated && allconsttreated;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void endOfSnapshot(Model model) {
		super.endOfSnapshot(model);

		Platform.getLog().info("DeModelOfComputation.endOfSnapshot before : **** current event list = " + currentEventList);

		//ATTENTION modif : plus deplacement des tokens restant sur les sorties du modele
		for (Pin structModelOUT : model.getStructure().getPinsOut()) {
//			//selection des tokens restants au temps courant
//			DeEventTimeSelector selTime = new DeEventTimeSelector(getTimeOfCurrentSnapshot());
//			ArrayList<Token> remainingevents = selTime.selectAll(currentEventList);
//			
//			//selection des tokens restants au temps courant pour la sortie du modele courante
//			DeEventTargetSelector selTarget = new DeEventTargetSelector(structModelOUT);
//			ArrayList<Token> evttoprocess = selTarget.selectAll(remainingevents);
//ATTENTION pose pb quand tokens a remettre dans un snapshot dans lequel il n'y a pas d'autres blocs a updater...
//SOLUTION choisie : remettre les evts sur les pins tout de suite, quelle que soit leur date

			//selection des tokens a mettre sur la sortie du modele courante
			DeEventTargetSelector selTarget = new DeEventTargetSelector(structModelOUT);
			ArrayList<Token> evttoprocess = selTarget.selectAll(currentEventList);

			//On depose ces evenements sur la sortie du modele
			structModelOUT.getStoredTokens().addAll(evttoprocess);

		    //On les enleve de la liste des evenements a traiter
		    currentEventList.removeAll(evttoprocess);
		}

		Platform.getLog().info("DeModelOfComputation.endOfSnapshot after : **** current event list = " + currentEventList);

		//On met a jour la liste des evenements du modele :
		// 1- en enlevant les evenements traites pendant le snapshot
		globalEventList.removeAll(initialEventList);
		// Nettoyage des listes de donnees du MoC
	    for(Token t : initialEventList) destinationPinOfEvents.remove(t); 
	    for(Token t : initialEventList) timestampOfEvents.remove(t);
	    for(Token t : initialEventList) valueOfEvents.remove(t);

		// 2- et en ajout les evenements produits pendant le snapshot
		globalEventList.addAll(currentEventList);
		//pas besoin de les ajouter aux listes de donnees du MoC car elles sont ajoutees au fur et a mesure du calcul
		
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void setup(Model model, InstantValue initialTime) {
		super.setup(model, initialTime);
		
		List<List<Pin>> loops = DEBlockDependencyComparator.findDependencyLoops(model.getStructure());
		if (loops.size() != 0) {
			System.err.println("DE model " + getName() + " contains non-causal loop(s):");
			for (List<Pin> loop : loops) {
				boolean notfirst = false;
				for (Pin p : loop) {
					if (notfirst) {
						System.err.print(" -> ");
					} else {
						notfirst = true;
					}
					System.err.print(DEBlockDependencyComparator.getOwner(p).getName() + "." + p.getName());
				}
				System.err.println();
			}
			throw new Error("DE model " + getName() + " contains at least one non-causal loop");
		}
		blockRanks_ = DEBlockDependencyComparator.assignDependencyRank(model.getStructure());
		
		// TODO make something with initial time ?

		// Are there any source blocks?
//		someSourceBlocks = false;
//		for(Block b : model.getStructure().getBlocks()) {
//			if(b.getPinsIn().size() == 0) {
//				someSourceBlocks = true;
//				break;
//			}
//		}
		
//		// We have to make a topological sort of blocks !
//		// This one works only on acyclic graphs
//		blockSorted = new ArrayList<Block>();
//		ArrayList<Block> blocklist = new ArrayList<Block>();
//		blocklist.addAll(model.getStructure().getBlocks());
//		BlockComparator compare = new BlockComparator();
//		
//		boolean ok = true;
//		while (ok) {
//			HashSet<Block> biggers = new HashSet<Block>();
//			for (Block b1 : blocklist) {
//				for (Block b2 : blocklist) {
//					if (b1 == b2) continue;
//					if (compare.compare(b1, b2) < 0) {
//						biggers.add(b2);
//					}
//					if (compare.compare(b1, b2) > 0) {
//						biggers.add(b1);
//					}
//				}
//			}
//			if (biggers.isEmpty()) {
//				// No more bigger block
//				blockSorted.addAll(blocklist);
//				break;
//			}
//			blocklist.removeAll(biggers);
//
//			// When the graph to sort is cyclic, biggers contains all the blocks of the graph
//			// therefore blocklist is empty at this time and the loop never end
//			// So we add a test to interrupt the loop in this case
//			if(blocklist.isEmpty()){
//				throw new Error("CAUTION (setup) :  try to sort a cyclic graph !");
//			}
//			blockSorted.addAll(blocklist);
//			blocklist.clear();
//			blocklist.addAll(biggers);
//		}				
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Token createEvent(InstantValue timestamp, Object value) {
		Token t = CoreFactory.eINSTANCE.createToken();
		valueOfEvents.put(t, value);
		timestampOfEvents.put(t, timestamp);
		return t;
	}

	@Override
	public Object getValueOfToken(Token t) {
		return valueOfEvents.get(t);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void initialSchedule(Model model) {
		super.initialSchedule(model);

		//On va lister les blocs qu'il faut mettre a jour dans le snapshot courant
		//  2011-03-14 : blocklist has been changed for a set because each block must be updated only one time in DE)
		HashSet<Block> blocklist = new HashSet<Block>();

		//Ces blocs sont soit des destinataires d'evenements a traiter au temps courant,
		//ATTENTION traitement special quand les evenements sont destines aux sorties du modele
		for (Token evt : currentEventList) {
			Block b = getDestinationPinOfEvent(evt).getInputOf();
			if(b!=null && this.getTimestampOfEvent(evt).equals(getTimeOfCurrentSnapshot())) blocklist.add(b);
			// 2011-03-14 : added a check on the time of the event since currentEventList contains all the events produced during the snapshot, including future events...
		}

		//Soit des blocs ayant emis une contrainte au temps courant
		// Or a block has emitted a "-1" time constraint in order to be updated ASAP
		if (minTimeConstraint.equals(getTimeOfCurrentSnapshot()) || minTimeConstraint.getDenotedInstant().get(0).getDate() < 0) {
			blocklist.addAll(currentBlocksWithMinTimeConstraint);
		}

		//Si la liste ainsi obtenue n'est pas vide
		if (!blocklist.isEmpty()) {

			//Alors le premier bloc a mettre a jour est le premier de la liste selon l'ordre topologique
//			BlockComparator compare = new BlockComparator(blockSorted);
//		    currentBlock = (Block) Collections.min(blocklist, compare);

			int minrank = Integer.MAX_VALUE;
			for (Block b : blocklist) {
				int r = blockRanks_.get(b);
				if (r < minrank) {
					minrank = r;
					currentBlock = b;
				}
			}
//		    currentBlock = Collections.min(blocklist, new DEBlockDependencyComparator());
			Platform.getLog().info("**** current bloc = " +currentBlock);

		    //S'il s'agit d'un bloc ayant emis une contrainte, il faut retirer celle-ci de la liste des contraintes
		    currentBlocksWithMinTimeConstraint.remove(currentBlock);

		} else {
			throw new Error("DeModelOfComputation.schedule : no block to schedule");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void prePropagate(Model model) {
		super.prePropagate(model);
		
		if (Platform.getLog().isLoggable(Level.INFO)) {
			Platform.getLog().info("**** current event list BEFORE = " + currentEventList);
			Platform.getLog().info("**** total dest pin list BEFORE = " + destinationPinOfEvents);
		}

		//On va chercher dans la file des evenements les evenements qui sont destines au block courant
		// et qui portent la date courante
		for (Pin currCompPinIN : currentBlock.getPinsIn()) {
			DeEventTargetSelector seldest = new DeEventTargetSelector(currCompPinIN);
			ArrayList<Token> evtdest = seldest.selectAll(currentEventList);

			//TODO ajout d'une selection sur la date courante
			DeEventTimeSelector sel = new DeEventTimeSelector(this.getTimeOfCurrentSnapshot());
			ArrayList<Token> evttoprocess = sel.selectAll(evtdest);

			//On depose ces evenements sur les entrees du bloc
		    currCompPinIN.getStoredTokens().addAll(evttoprocess);

		    //On les enleve de la liste des evenements a traiter
		    currentEventList.removeAll(evttoprocess);
		    //ATTENTION ne surtout pas les enlever des listes car infos peuvent etre demandees par autres blocs
//		    for(Token t : evttoprocess){
//		    	destinationPinOfEvents.remove(t);
//		    	timestampOfEvents.remove(t);
//		    	valueOfEvents.remove(t);
//		    }

		}
		if (Platform.getLog().isLoggable(Level.INFO)) {
			Platform.getLog().info("**** current event list AFTER = " + currentEventList);
			Platform.getLog().info("**** total dest pin list AFTER = " + destinationPinOfEvents);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void preSchedule(Model model) {
		super.preSchedule(model);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void postSchedule(Model model) {
		super.postSchedule(model);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void postPropagate(Model model) {
		super.postPropagate(model);
		
		if (Platform.getLog().isLoggable(Level.INFO)) {
			Platform.getLog().info("postPropagate at time " + timeOfCurrentSnapshot);
			Platform.getLog().info("**** current event list BEFORE = " + currentEventList);
			Platform.getLog().info("**** total dest pin list BEFORE = " + destinationPinOfEvents);
		}

		//Les evenements produits sur chacun des points d'interface du bloc mis a jour
		// sont deplaces dans la liste des evenements a traiter,
		for (Pin currentCompPin : currentBlock.getPinsOut()) {
			if (Platform.getLog().isLoggable(Level.INFO)) {
				Platform.getLog().info("look on " + currentCompPin);
			}
		    if (currentCompPin.hasTokens()) {
		    	// Tous les tokens sont mis dans la file, meme si le pin est en relation avec des sorties du modele (ex alias)
				moveEventsToList(currentCompPin, currentEventList);
		    }
		}
		
		if (Platform.getLog().isLoggable(Level.INFO)) {
			Platform.getLog().info("**** current event list AFTER = " + currentEventList);
			Platform.getLog().info("**** total dest pin list AFTER = " + destinationPinOfEvents);
			Platform.getLog().info("**** total timestamp list AFTER = " + timestampOfEvents);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void finalSchedule(Model model) {
		super.finalSchedule(model);
	}

	public InstantValue getTimestampOfEvent(Token t) {
		return timestampOfEvents.get(t);
	}

	public Pin getDestinationPinOfEvent(Token t) {
		return destinationPinOfEvents.get(t);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case DePackage.DE_MODEL_OF_COMPUTATION__CLOCK:
				if (resolve) return getClock();
				return basicGetClock();
			case DePackage.DE_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT:
				if (resolve) return getTimeOfCurrentSnapshot();
				return basicGetTimeOfCurrentSnapshot();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case DePackage.DE_MODEL_OF_COMPUTATION__CLOCK:
				setClock((Clock)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case DePackage.DE_MODEL_OF_COMPUTATION__CLOCK:
				setClock((Clock)null);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case DePackage.DE_MODEL_OF_COMPUTATION__CLOCK:
				return clock != null;
			case DePackage.DE_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT:
				return timeOfCurrentSnapshot != null;
		}
		return super.eIsSet(featureID);
	}

	private class DeEventTimeSelector extends Selector<Token> {
    	
        private final InstantValue time;
       
        public DeEventTimeSelector(InstantValue time) {
        	this.time = time;
        }    
        public boolean select(Token ev) {
          return time.equals(getTimestampOfEvent(ev));
        }
    }

    private class DeEventTargetSelector extends Selector<Token> {
    	
        private final Pin target;
       
        public DeEventTargetSelector(Pin target) {
        	this.target = target;
        }    
        public boolean select(Token ev) {
          return getDestinationPinOfEvent(ev) == target;
        }
    }

} //DeModelOfComputationImpl
