/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DeCoffeeMachineUserImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.delib.impl;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.core.time.TimeValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUser;
import fr.supelec.modhelx.mocs.delib.DelibPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>De Coffee Machine User</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.delib.impl.DeCoffeeMachineUserImpl#getFirstTimeOfEmit <em>First Time Of Emit</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class DeCoffeeMachineUserImpl extends AtomicBlockImpl implements DeCoffeeMachineUser {
	/**
	 * The default value of the '{@link #getFirstTimeOfEmit() <em>First Time Of Emit</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getFirstTimeOfEmit()
	 * @generated
	 * @ordered
	 */
	protected static final int FIRST_TIME_OF_EMIT_EDEFAULT = 0;

	private boolean isFirstTimeOfEmit = true;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DeCoffeeMachineUserImpl() {
		super();
	}

	private DeModelOfComputation getMoc(ModelOfComputation moc) {
		try {
			DeModelOfComputation m = (DeModelOfComputation)moc;
			return m;
		}
		catch (ClassCastException ex) {
			throw new Error("DeEventPrinter : moc is not DE");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return DelibPackage.Literals.DE_COFFEE_MACHINE_USER;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public int getFirstTimeOfEmit() {
		Object value = getValueOfParameter("firstTimeOfEmit");
		if (value == null) {
			return FIRST_TIME_OF_EMIT_EDEFAULT;
		}
		try {
			Integer v = (Integer) value;
			return v.intValue();
		}
		catch (ClassCastException ex) {
			throw new Error("DeCoffeeMachineUser.getFirstTimeOfEmit : firstTimeOfEmit is not an Integer");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setFirstTimeOfEmit(int newFirstTimeOfEmit) {
		setValueOfParameter("firstTimeOfEmit", new Integer(newFirstTimeOfEmit));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc_) {
		DeModelOfComputation moc = getMoc(moc_);
		super.update(moc);
		
		TimeValue date = moc.getClock().createInstantValue(getFirstTimeOfEmit());
		
		Pin input = getPinsIn().get(0);
		if (! input.hasTokens() && moc.getTimeOfCurrentSnapshot().compareTo(date)!=0) {
			throw new Error("DeCoffeeMachineUser.update : no event on input token and time of current snapshot different from fist time of emit");
			
		} else if (! input.hasTokens() && moc.getTimeOfCurrentSnapshot().compareTo(date)==0){
			for (Pin p : getPinsOut()) {
				Token evOut = moc.createEvent(moc.getTimeOfCurrentSnapshot(), new String("first token"));
				System.err.println("put token on " + p);
				p.addToken(evOut);
			}
			
		} else {
			while (input.hasTokens()) {
				Token evIn = input.getToken(true);
				if (! moc.getTimestampOfEvent(evIn).equals(moc.getTimeOfCurrentSnapshot())) {
					throw new Error("DeCoffeeMachineUser.update : event has invalid date");
				}
				for (Pin p : getPinsOut()) {
					Token evOut = moc.createEvent(moc.getTimestampOfEvent(evIn), moc.getValueOfToken(evIn));
					p.addToken(evOut);
				}
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc_, InstantValue initialTime) {
		DeModelOfComputation moc = getMoc(moc_);
		super.setup(moc, initialTime);
		
		if (getPinsIn().size() != 1) {
			throw new Error("DeCoffeeMachineUser.setup : one and only one input pin is expected");
		}
		if (getPinsOut().size() == 0) {
			throw new Error("DeCoffeeMachineUser.setup : at least one output pin is expected");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void collectTimeConstraints(ModelOfComputation moc, EList<InstantValue> timeConstraints) {
		DeModelOfComputation moc_ = getMoc(moc);
		//super.collectTimeConstraints(moc, timeConstraints);
		if(isFirstTimeOfEmit){
			Platform.getLog().info("*********** DeCoffeeMachineUserImpl : production de contrainte a t = 0");
			InstantValue firstTimeOfEmit = moc_.getClock().createInstantValue(getFirstTimeOfEmit());
			timeConstraints.add(firstTimeOfEmit);
			super.collectTimeConstraints(moc_, timeConstraints, firstTimeOfEmit);
			isFirstTimeOfEmit = false;
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case DelibPackage.DE_COFFEE_MACHINE_USER__FIRST_TIME_OF_EMIT:
				return getFirstTimeOfEmit();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case DelibPackage.DE_COFFEE_MACHINE_USER__FIRST_TIME_OF_EMIT:
				setFirstTimeOfEmit((Integer)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_COFFEE_MACHINE_USER__FIRST_TIME_OF_EMIT:
				setFirstTimeOfEmit(FIRST_TIME_OF_EMIT_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_COFFEE_MACHINE_USER__FIRST_TIME_OF_EMIT:
				return getFirstTimeOfEmit() != FIRST_TIME_OF_EMIT_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}

} //DeCoffeeMachineUserImpl
