/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DeCoffeeMachineUserWithoutInputImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.delib.impl;

import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;

import fr.supelec.modhelx.core.impl.AtomicBlockImpl;

import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;

import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUserWithoutInput;
import fr.supelec.modhelx.mocs.delib.DelibPackage;

import org.eclipse.emf.common.util.EList;

import org.eclipse.emf.ecore.EClass;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>De Coffee Machine User Without Input</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.delib.impl.DeCoffeeMachineUserWithoutInputImpl#getTimeBetweenCoinAndCoffee <em>Time Between Coin And Coffee</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class DeCoffeeMachineUserWithoutInputImpl extends AtomicBlockImpl implements DeCoffeeMachineUserWithoutInput {

	/**
	 * The default value of the '{@link #getTimeBetweenCoinAndCoffee() <em>Time Between Coin And Coffee</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getTimeBetweenCoinAndCoffee()
	 * @generated
	 * @ordered
	 */
	protected static final int TIME_BETWEEN_COIN_AND_COFFEE_EDEFAULT = 1;

	
	private boolean isFirstTimeOfEmit;
	
	private Pin coinPin;
	private Pin coffeePin;
	
	private DeModelOfComputation getMoc(ModelOfComputation moc) {
		try {
			DeModelOfComputation m = (DeModelOfComputation)moc;
			return m;
		}
		catch (ClassCastException ex) {
			throw new Error("DeEventPrinter : moc is not DE");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	protected DeCoffeeMachineUserWithoutInputImpl() {
		super();
		isFirstTimeOfEmit = true;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return DelibPackage.Literals.DE_COFFEE_MACHINE_USER_WITHOUT_INPUT;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public int getTimeBetweenCoinAndCoffee() {
		Object value = getValueOfParameter("timeBetweenCoinAndCoffee");
		if (value == null) {
			return TIME_BETWEEN_COIN_AND_COFFEE_EDEFAULT;
		}
		try {
			Integer v = (Integer) value;
			return v.intValue();
		}
		catch (ClassCastException ex) {
			throw new Error("DeCoffeeMachineUserWithoutInputImpl.getTimeBetweenCoinAndCoffeey : time is not an Integer");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setTimeBetweenCoinAndCoffee(int newTimeBetweenCoinAndCoffee) {
		setValueOfParameter("timeBetweenCoinAndCoffee", new Integer(newTimeBetweenCoinAndCoffee));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void collectTimeConstraints(ModelOfComputation moc, EList<InstantValue> timeConstraints) {
		DeModelOfComputation moc_ = getMoc(moc);
		if(isFirstTimeOfEmit){
			Platform.getLog().info("*********** DeCoffeeMachineUserImpl : production de contrainte a t = "+ moc_.getClock().getCurrentTime());
			//Computation of the time for the constraint
			//TODO should it be independant from the moc ?
			InstantValue firstTimeOfEmit = moc_.getClock().createInstantValue(moc_.getClock().getCurrentTime());
			timeConstraints.add(firstTimeOfEmit);
			super.collectTimeConstraints(moc_, timeConstraints, firstTimeOfEmit);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc_) {
		DeModelOfComputation moc = getMoc(moc_);
		super.update(moc);
		
		if(isFirstTimeOfEmit){
			Token evOut = moc.createEvent(moc.getTimeOfCurrentSnapshot(), new String("coin"));
			coinPin.addToken(evOut);
			Platform.getLog().info("put token coin with date "+moc.getTimeOfCurrentSnapshot().getDenotedInstant().get(0).getDate()+" on " + coinPin);
			
			double time = moc.getTimeOfCurrentSnapshot().getDenotedInstant().get(0).getDate() + this.getTimeBetweenCoinAndCoffee();
			evOut = moc.createEvent(moc.getClock().createInstantValue(time), new String("coffee"));
			coffeePin.addToken(evOut);
			Platform.getLog().info("put token coffee with date "+time+" on " + coffeePin);
			isFirstTimeOfEmit = false;
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc_, InstantValue initialTime) {
		DeModelOfComputation moc = getMoc(moc_);
		super.setup(moc, initialTime);
		
		//Initial check of the input and output pins
		if (getPinsIn().size() != 0) {
			throw new Error("DeCoffeeMachineUserWithoutInputImpl.setup : no input pin expected");
		}
		if (getPinsOut().size() < 1) {
			throw new Error("DeCoffeeMachineUserWithoutInputImpl.setup : at least one output pin is expected");
		}

		//Check that the different pins have been created using the appropriate methods so that we can recognize them
		if(coinPin==null) throw new Error("DeCoffeeMachineUserWithoutInputImpl.setup : coin pin not recognized, use the createCoinPin method to create this pin");
		if(coffeePin==null) throw new Error("DeCoffeeMachineUserWithoutInputImpl.setup : coffee pin not recognized, use the createCoffeePin method to create this pin");
}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Pin createCoinPin() {
		coinPin = CoreFactory.eINSTANCE.createPin();
		coinPin.setName("coin");
		this.getPinsOut().add(coinPin);
		return coinPin;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Pin createCoffeePin() {
		coffeePin = CoreFactory.eINSTANCE.createPin();
		coffeePin.setName("coffee");
		this.getPinsOut().add(coffeePin);
		return coffeePin;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case DelibPackage.DE_COFFEE_MACHINE_USER_WITHOUT_INPUT__TIME_BETWEEN_COIN_AND_COFFEE:
				return getTimeBetweenCoinAndCoffee();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case DelibPackage.DE_COFFEE_MACHINE_USER_WITHOUT_INPUT__TIME_BETWEEN_COIN_AND_COFFEE:
				setTimeBetweenCoinAndCoffee((Integer)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_COFFEE_MACHINE_USER_WITHOUT_INPUT__TIME_BETWEEN_COIN_AND_COFFEE:
				setTimeBetweenCoinAndCoffee(TIME_BETWEEN_COIN_AND_COFFEE_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_COFFEE_MACHINE_USER_WITHOUT_INPUT__TIME_BETWEEN_COIN_AND_COFFEE:
				return getTimeBetweenCoinAndCoffee() != TIME_BETWEEN_COIN_AND_COFFEE_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}

} //DeCoffeeMachineUserWithoutInputImpl
