/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DeDelayImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.delib.impl;

import java.util.ArrayList;

import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.DurationValue;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeDelay;
import fr.supelec.modhelx.mocs.delib.DelibPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>De Delay</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.delib.impl.DeDelayImpl#getTimeDelay <em>Time Delay</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class DeDelayImpl extends AtomicBlockImpl implements DeDelay {
	/**
	 * The default value of the '{@link #getTimeDelay() <em>Time Delay</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getTimeDelay()
	 * @generated
	 * @ordered
	 */
	protected static final int TIME_DELAY_EDEFAULT = 1;
	
	private ArrayList<Token> currentTokens = new ArrayList<Token>();

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DeDelayImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return DelibPackage.Literals.DE_DELAY;
	}

	private DeModelOfComputation getMoc(ModelOfComputation moc) {
		try {
			DeModelOfComputation m = (DeModelOfComputation)moc;
			return m;
		}
		catch (ClassCastException ex) {
			throw new Error("DeConstantEmitter : moc is not DE");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public int getTimeDelay() {
		Object value = getValueOfParameter("timeDelay");
		if (value == null) {
			return TIME_DELAY_EDEFAULT;
		}
		try {
			Integer v = (Integer) value;
			return v.intValue();
		}
		catch (ClassCastException ex) {
			throw new Error("DeDelay.getTimeDelay : timeDelay is not an Integer");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setTimeDelay(int newTimeDelay) {
		setValueOfParameter("timeDelay", new Integer(newTimeDelay));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc_) {
		DeModelOfComputation moc = getMoc(moc_);
		super.update(moc);
		
		//TODO : mais POURQUOI ne prendre en compte pour l'update QUE les tokens qui etaient presents au sos ?
//		for (Token t : currentTokens) {
		for (Token t : getPinsIn().get(0).getStoredTokens()) {
			DurationValue d = moc.getClock().createDurationValue(
					moc.getClock().createInstantValue(0),
					moc.getClock().createInstantValue(getTimeDelay()));
			Token newToken = moc.createEvent(moc.getTimestampOfEvent(t).add(d), moc.getValueOfToken(t));
			getPinsOut().get(0).addToken(newToken);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc_, InstantValue initialTime) {
		DeModelOfComputation moc = getMoc(moc_);
		super.setup(moc, initialTime);
		
		if (getPinsIn().size() != 1) {
			throw new Error("DeDelay.setup : one input pin is expected");
		}
		if (getPinsOut().size() == 0) {
			throw new Error("DeDelay.setup : at least one output pin is expected");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void startOfSnapshot(ModelOfComputation moc_) {
		DeModelOfComputation moc = getMoc(moc_);
		super.startOfSnapshot(moc);
		
		while (getPinsIn().get(0).hasTokens()) {
			currentTokens.add(getPinsIn().get(0).getToken(true));
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfSnapshot(ModelOfComputation moc_) {
		DeModelOfComputation moc = getMoc(moc_);
		super.endOfSnapshot(moc);
		
		currentTokens.clear();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case DelibPackage.DE_DELAY__TIME_DELAY:
				return getTimeDelay();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case DelibPackage.DE_DELAY__TIME_DELAY:
				setTimeDelay((Integer)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_DELAY__TIME_DELAY:
				setTimeDelay(TIME_DELAY_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_DELAY__TIME_DELAY:
				return getTimeDelay() != TIME_DELAY_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}

} //DeDelayImpl
