/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DeEventPrinterImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.delib.impl;

import java.io.PrintStream;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DelibPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>De Event Printer</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.delib.impl.DeEventPrinterImpl#getPrintStream <em>Print Stream</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class DeEventPrinterImpl extends AtomicBlockImpl implements DeEventPrinter {
	/**
	 * The default value of the '{@link #getPrintStream() <em>Print Stream</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getPrintStream()
	 * @generated
	 * @ordered
	 */
	protected static final Object PRINT_STREAM_EDEFAULT = null;

	/**
	 * The cached value of the '{@link #getPrintStream() <em>Print Stream</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getPrintStream()
	 * @generated
	 * @ordered
	 */
	protected Object printStream = PRINT_STREAM_EDEFAULT;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DeEventPrinterImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return DelibPackage.Literals.DE_EVENT_PRINTER;
	}

	private DeModelOfComputation getMoc(ModelOfComputation moc) {
		try {
			DeModelOfComputation m = (DeModelOfComputation)moc;
			return m;
		}
		catch (ClassCastException ex) {
			throw new Error("DeEventPrinter : moc is not DE");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Object getPrintStream() {
		return printStream;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setPrintStream(Object newPrintStream) {
		if (! (newPrintStream instanceof PrintStream)) {
			throw new Error("DeEventPrinter.setPrintStream : the argument must be a PrintStream");
		}
		Object oldPrintStream = printStream;
		printStream = newPrintStream;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, DelibPackage.DE_EVENT_PRINTER__PRINT_STREAM, oldPrintStream, printStream));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void update(ModelOfComputation moc_) {
		DeModelOfComputation moc = getMoc(moc_);
		super.update(moc);
		
		if (getPrintStream() == null) return;
		PrintStream out = (PrintStream) getPrintStream();
		
		int nbEventsReceived = 0;
		for (Pin p : getPinsIn()) {
			nbEventsReceived += p.getStoredTokens().size();
		}
		
		//if (nbEventsReceived == 0) {
		//	out.println("No event received");
		//	return;
		//}
		
		for (Pin p : getPinsIn()) {
			while (p.hasTokens()) {
				Token ev = p.getToken(true);
				String s = "Event received on " + p.getName()
				+ " at " + moc.getTimestampOfEvent(ev).getDenotedInstant().get(0).getDate()
				+ " with value " + moc.getValueOfToken(ev);
				//Platform.getLog().info("#### " + s);
				out.println(s);
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@Override
	public void setup(ModelOfComputation moc_, InstantValue initialTime) {
		DeModelOfComputation moc = getMoc(moc_);
		super.setup(moc, initialTime);
		
		if (getPinsIn().size() == 0) {
			throw new Error("DeEventPrinter.setup : input pins are expected");
		}
		if (getPinsOut().size() != 0) {
			throw new Error("DeEventPrinter.setup : output pins are not expected");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case DelibPackage.DE_EVENT_PRINTER__PRINT_STREAM:
				return getPrintStream();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case DelibPackage.DE_EVENT_PRINTER__PRINT_STREAM:
				setPrintStream(newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_EVENT_PRINTER__PRINT_STREAM:
				setPrintStream(PRINT_STREAM_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_EVENT_PRINTER__PRINT_STREAM:
				return PRINT_STREAM_EDEFAULT == null ? printStream != null : !PRINT_STREAM_EDEFAULT.equals(printStream);
		}
		return super.eIsSet(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy()) return super.toString();

		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (printStream: ");
		result.append(printStream);
		result.append(')');
		return result.toString();
	}

} //DeEventPrinterImpl
