/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DeScenarioPlayerImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.delib.impl;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.util.EObjectResolvingEList;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeScenarioEvent;
import fr.supelec.modhelx.mocs.delib.DeScenarioPlayer;
import fr.supelec.modhelx.mocs.delib.DelibPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>De Scenario Player</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.delib.impl.DeScenarioPlayerImpl#getEvents <em>Events</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class DeScenarioPlayerImpl extends AtomicBlockImpl implements DeScenarioPlayer {
	private DeModelOfComputation deMoc;
	private LinkedList<DeScenarioEvent> sortedEvents = new LinkedList<DeScenarioEvent>();
	private DeScenarioEvent nextEvent;
	private InstantValue timeOfNextEvent;
	private boolean needToPickNextEvent;   // true when a "next event" needs to be picked at the next startOfSnapshot. It means that the current "next event" has just been emitted.
	private boolean eventEmitted;   // true when an event has been emitted in the current snapshot. Set to false at reset, set to true at update if necessary
	
	/**
	 * The cached value of the '{@link #getEvents() <em>Events</em>}' reference list.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getEvents()
	 * @generated
	 * @ordered
	 */
	protected EList<DeScenarioEvent> events;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DeScenarioPlayerImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return DelibPackage.Literals.DE_SCENARIO_PLAYER;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public EList<DeScenarioEvent> getEvents() {
		if (events == null) {
			events = new EObjectResolvingEList<DeScenarioEvent>(DeScenarioEvent.class, this, DelibPackage.DE_SCENARIO_PLAYER__EVENTS);
		}
		return events;
	}

	@Override
	public void startOfSnapshot(ModelOfComputation moc) {
		if(needToPickNextEvent) {
			if(sortedEvents.size() > 0) {
				nextEvent = sortedEvents.removeFirst();
			} else nextEvent = null;
			needToPickNextEvent = false;
		}
	}
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void collectTimeConstraints(ModelOfComputation moc, EList<InstantValue> timeConstraints) {
		if(nextEvent != null) {
			timeOfNextEvent = deMoc.getClock().createInstantValue(nextEvent.getTime());
			timeConstraints.add(timeOfNextEvent);
		} else timeOfNextEvent = null;
		
		super.collectTimeConstraints(moc, timeConstraints);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc) {
		if(deMoc.getTimeOfCurrentSnapshot().compareTo(timeOfNextEvent) == 0) {
			nextEvent.getPin().addToken(deMoc.createEvent(timeOfNextEvent, nextEvent.getValue()));
			eventEmitted = true;
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		// check if the MoC is DE
		if(moc instanceof DeModelOfComputation) {
			deMoc = (DeModelOfComputation) moc;
		} else throw new Error("DeScenarioPlayer works only in DE");
		
		// sort the events to emit
		sortedEvents.clear();
		sortedEvents.addAll(events);
		Collections.sort(sortedEvents, new Comparator<DeScenarioEvent>() {
			public int compare(DeScenarioEvent evt1, DeScenarioEvent evt2) {
				return evt1.getTime() - evt2.getTime(); 
			}
		});
		
		Platform.getLog().finest("Ordered events: " + sortedEvents);
		
		// initially, a "next event" will need to be picked at the first startOfSnapshot
		needToPickNextEvent = true;
	}
	
	@Override
	public boolean newSnap(ModelOfComputation moc) {
		return nextEvent != null;
	}
	
	@Override
	public void endOfSnapshot(ModelOfComputation moc) {
		super.endOfSnapshot(moc);
		
		if(eventEmitted) needToPickNextEvent = true;
	}
	
	@Override
	public void reset(ModelOfComputation moc, InstantValue time) {
		super.reset(moc, time);
		
		eventEmitted = false;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case DelibPackage.DE_SCENARIO_PLAYER__EVENTS:
				return getEvents();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case DelibPackage.DE_SCENARIO_PLAYER__EVENTS:
				getEvents().clear();
				getEvents().addAll((Collection<? extends DeScenarioEvent>)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_SCENARIO_PLAYER__EVENTS:
				getEvents().clear();
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case DelibPackage.DE_SCENARIO_PLAYER__EVENTS:
				return events != null && !events.isEmpty();
		}
		return super.eIsSet(featureID);
	}

} //DeScenarioPlayerImpl
