/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DelibAdapterFactory.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.delib.util;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.notify.impl.AdapterFactoryImpl;
import org.eclipse.emf.ecore.EObject;

import fr.supelec.modhelx.core.AtomicBlock;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.ModHelXObject;
import fr.supelec.modhelx.mocs.delib.*;
import fr.supelec.modhelx.mocs.delib.DeAdder;
import fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUser;
import fr.supelec.modhelx.mocs.delib.DeConstantEmitter;
import fr.supelec.modhelx.mocs.delib.DeDelay;
import fr.supelec.modhelx.mocs.delib.DeDuplicator;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DelibPackage;

/**
 * <!-- begin-user-doc -->
 * The <b>Adapter Factory</b> for the model.
 * It provides an adapter <code>createXXX</code> method for each class of the model.
 * <!-- end-user-doc -->
 * @see fr.supelec.modhelx.mocs.delib.DelibPackage
 * @generated
 */
public class DelibAdapterFactory extends AdapterFactoryImpl {
	/**
	 * The cached model package.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static DelibPackage modelPackage;

	/**
	 * Creates an instance of the adapter factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public DelibAdapterFactory() {
		if (modelPackage == null) {
			modelPackage = DelibPackage.eINSTANCE;
		}
	}

	/**
	 * Returns whether this factory is applicable for the type of the object.
	 * <!-- begin-user-doc -->
	 * This implementation returns <code>true</code> if the object is either the model's package or is an instance object of the model.
	 * <!-- end-user-doc -->
	 * @return whether this factory is applicable for the type of the object.
	 * @generated
	 */
	@Override
	public boolean isFactoryForType(Object object) {
		if (object == modelPackage) {
			return true;
		}
		if (object instanceof EObject) {
			return ((EObject)object).eClass().getEPackage() == modelPackage;
		}
		return false;
	}

	/**
	 * The switch that delegates to the <code>createXXX</code> methods.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DelibSwitch<Adapter> modelSwitch =
		new DelibSwitch<Adapter>() {
			@Override
			public Adapter caseDeRampEmitter(DeRampEmitter object) {
				return createDeRampEmitterAdapter();
			}
			@Override
			public Adapter caseDeDuplicator(DeDuplicator object) {
				return createDeDuplicatorAdapter();
			}
			@Override
			public Adapter caseDeAdder(DeAdder object) {
				return createDeAdderAdapter();
			}
			@Override
			public Adapter caseDeEventPrinter(DeEventPrinter object) {
				return createDeEventPrinterAdapter();
			}
			@Override
			public Adapter caseDeDelay(DeDelay object) {
				return createDeDelayAdapter();
			}
			@Override
			public Adapter caseDeCoffeeMachineUser(DeCoffeeMachineUser object) {
				return createDeCoffeeMachineUserAdapter();
			}
			@Override
			public Adapter caseDeCoffeeMachineUserWithoutInput(DeCoffeeMachineUserWithoutInput object) {
				return createDeCoffeeMachineUserWithoutInputAdapter();
			}
			@Override
			public Adapter caseDeConstantEmitter(DeConstantEmitter object) {
				return createDeConstantEmitterAdapter();
			}
			@Override
			public Adapter caseDeScenarioPlayer(DeScenarioPlayer object) {
				return createDeScenarioPlayerAdapter();
			}
			@Override
			public Adapter caseDeScenarioEvent(DeScenarioEvent object) {
				return createDeScenarioEventAdapter();
			}
			@Override
			public Adapter caseModHelXObject(ModHelXObject object) {
				return createModHelXObjectAdapter();
			}
			@Override
			public Adapter caseBlock(Block object) {
				return createBlockAdapter();
			}
			@Override
			public Adapter caseAtomicBlock(AtomicBlock object) {
				return createAtomicBlockAdapter();
			}
			@Override
			public Adapter defaultCase(EObject object) {
				return createEObjectAdapter();
			}
		};

	/**
	 * Creates an adapter for the <code>target</code>.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param target the object to adapt.
	 * @return the adapter for the <code>target</code>.
	 * @generated
	 */
	@Override
	public Adapter createAdapter(Notifier target) {
		return modelSwitch.doSwitch((EObject)target);
	}


	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeRampEmitter <em>De Ramp Emitter</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeRampEmitter
	 * @generated
	 */
	public Adapter createDeRampEmitterAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeDuplicator <em>De Duplicator</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeDuplicator
	 * @generated
	 */
	public Adapter createDeDuplicatorAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeAdder <em>De Adder</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeAdder
	 * @generated
	 */
	public Adapter createDeAdderAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeEventPrinter <em>De Event Printer</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeEventPrinter
	 * @generated
	 */
	public Adapter createDeEventPrinterAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeDelay <em>De Delay</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeDelay
	 * @generated
	 */
	public Adapter createDeDelayAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUser <em>De Coffee Machine User</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUser
	 * @generated
	 */
	public Adapter createDeCoffeeMachineUserAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUserWithoutInput <em>De Coffee Machine User Without Input</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUserWithoutInput
	 * @generated
	 */
	public Adapter createDeCoffeeMachineUserWithoutInputAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeConstantEmitter <em>De Constant Emitter</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeConstantEmitter
	 * @generated
	 */
	public Adapter createDeConstantEmitterAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeScenarioPlayer <em>De Scenario Player</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeScenarioPlayer
	 * @generated
	 */
	public Adapter createDeScenarioPlayerAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.delib.DeScenarioEvent <em>De Scenario Event</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.delib.DeScenarioEvent
	 * @generated
	 */
	public Adapter createDeScenarioEventAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.ModHelXObject <em>Mod Hel XObject</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.ModHelXObject
	 * @generated
	 */
	public Adapter createModHelXObjectAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.Block <em>Block</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.Block
	 * @generated
	 */
	public Adapter createBlockAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.AtomicBlock <em>Atomic Block</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.AtomicBlock
	 * @generated
	 */
	public Adapter createAtomicBlockAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for the default case.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @generated
	 */
	public Adapter createEObjectAdapter() {
		return null;
	}

} //DelibAdapterFactory
