/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: DelibSwitch.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.delib.util;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

import fr.supelec.modhelx.core.AtomicBlock;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.ModHelXObject;
import fr.supelec.modhelx.mocs.delib.*;
import fr.supelec.modhelx.mocs.delib.DeAdder;
import fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUser;
import fr.supelec.modhelx.mocs.delib.DeConstantEmitter;
import fr.supelec.modhelx.mocs.delib.DeDelay;
import fr.supelec.modhelx.mocs.delib.DeDuplicator;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DelibPackage;

/**
 * <!-- begin-user-doc -->
 * The <b>Switch</b> for the model's inheritance hierarchy.
 * It supports the call {@link #doSwitch(EObject) doSwitch(object)}
 * to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object
 * and proceeding up the inheritance hierarchy
 * until a non-null result is returned,
 * which is the result of the switch.
 * <!-- end-user-doc -->
 * @see fr.supelec.modhelx.mocs.delib.DelibPackage
 * @generated
 */
public class DelibSwitch<T> {
	/**
	 * The cached model package
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static DelibPackage modelPackage;

	/**
	 * Creates an instance of the switch.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public DelibSwitch() {
		if (modelPackage == null) {
			modelPackage = DelibPackage.eINSTANCE;
		}
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	public T doSwitch(EObject theEObject) {
		return doSwitch(theEObject.eClass(), theEObject);
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected T doSwitch(EClass theEClass, EObject theEObject) {
		if (theEClass.eContainer() == modelPackage) {
			return doSwitch(theEClass.getClassifierID(), theEObject);
		}
		else {
			List<EClass> eSuperTypes = theEClass.getESuperTypes();
			return
				eSuperTypes.isEmpty() ?
					defaultCase(theEObject) :
					doSwitch(eSuperTypes.get(0), theEObject);
		}
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected T doSwitch(int classifierID, EObject theEObject) {
		switch (classifierID) {
			case DelibPackage.DE_RAMP_EMITTER: {
				DeRampEmitter deRampEmitter = (DeRampEmitter)theEObject;
				T result = caseDeRampEmitter(deRampEmitter);
				if (result == null) result = caseAtomicBlock(deRampEmitter);
				if (result == null) result = caseBlock(deRampEmitter);
				if (result == null) result = caseModHelXObject(deRampEmitter);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case DelibPackage.DE_DUPLICATOR: {
				DeDuplicator deDuplicator = (DeDuplicator)theEObject;
				T result = caseDeDuplicator(deDuplicator);
				if (result == null) result = caseAtomicBlock(deDuplicator);
				if (result == null) result = caseBlock(deDuplicator);
				if (result == null) result = caseModHelXObject(deDuplicator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case DelibPackage.DE_ADDER: {
				DeAdder deAdder = (DeAdder)theEObject;
				T result = caseDeAdder(deAdder);
				if (result == null) result = caseAtomicBlock(deAdder);
				if (result == null) result = caseBlock(deAdder);
				if (result == null) result = caseModHelXObject(deAdder);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case DelibPackage.DE_EVENT_PRINTER: {
				DeEventPrinter deEventPrinter = (DeEventPrinter)theEObject;
				T result = caseDeEventPrinter(deEventPrinter);
				if (result == null) result = caseAtomicBlock(deEventPrinter);
				if (result == null) result = caseBlock(deEventPrinter);
				if (result == null) result = caseModHelXObject(deEventPrinter);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case DelibPackage.DE_DELAY: {
				DeDelay deDelay = (DeDelay)theEObject;
				T result = caseDeDelay(deDelay);
				if (result == null) result = caseAtomicBlock(deDelay);
				if (result == null) result = caseBlock(deDelay);
				if (result == null) result = caseModHelXObject(deDelay);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case DelibPackage.DE_COFFEE_MACHINE_USER: {
				DeCoffeeMachineUser deCoffeeMachineUser = (DeCoffeeMachineUser)theEObject;
				T result = caseDeCoffeeMachineUser(deCoffeeMachineUser);
				if (result == null) result = caseAtomicBlock(deCoffeeMachineUser);
				if (result == null) result = caseBlock(deCoffeeMachineUser);
				if (result == null) result = caseModHelXObject(deCoffeeMachineUser);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case DelibPackage.DE_COFFEE_MACHINE_USER_WITHOUT_INPUT: {
				DeCoffeeMachineUserWithoutInput deCoffeeMachineUserWithoutInput = (DeCoffeeMachineUserWithoutInput)theEObject;
				T result = caseDeCoffeeMachineUserWithoutInput(deCoffeeMachineUserWithoutInput);
				if (result == null) result = caseAtomicBlock(deCoffeeMachineUserWithoutInput);
				if (result == null) result = caseBlock(deCoffeeMachineUserWithoutInput);
				if (result == null) result = caseModHelXObject(deCoffeeMachineUserWithoutInput);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case DelibPackage.DE_CONSTANT_EMITTER: {
				DeConstantEmitter deConstantEmitter = (DeConstantEmitter)theEObject;
				T result = caseDeConstantEmitter(deConstantEmitter);
				if (result == null) result = caseDeRampEmitter(deConstantEmitter);
				if (result == null) result = caseAtomicBlock(deConstantEmitter);
				if (result == null) result = caseBlock(deConstantEmitter);
				if (result == null) result = caseModHelXObject(deConstantEmitter);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case DelibPackage.DE_SCENARIO_PLAYER: {
				DeScenarioPlayer deScenarioPlayer = (DeScenarioPlayer)theEObject;
				T result = caseDeScenarioPlayer(deScenarioPlayer);
				if (result == null) result = caseAtomicBlock(deScenarioPlayer);
				if (result == null) result = caseBlock(deScenarioPlayer);
				if (result == null) result = caseModHelXObject(deScenarioPlayer);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case DelibPackage.DE_SCENARIO_EVENT: {
				DeScenarioEvent deScenarioEvent = (DeScenarioEvent)theEObject;
				T result = caseDeScenarioEvent(deScenarioEvent);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			default: return defaultCase(theEObject);
		}
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Ramp Emitter</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Ramp Emitter</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeRampEmitter(DeRampEmitter object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Duplicator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Duplicator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeDuplicator(DeDuplicator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Adder</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Adder</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeAdder(DeAdder object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Event Printer</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Event Printer</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeEventPrinter(DeEventPrinter object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Delay</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Delay</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeDelay(DeDelay object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Coffee Machine User</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Coffee Machine User</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeCoffeeMachineUser(DeCoffeeMachineUser object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Coffee Machine User Without Input</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Coffee Machine User Without Input</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeCoffeeMachineUserWithoutInput(DeCoffeeMachineUserWithoutInput object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Constant Emitter</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Constant Emitter</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeConstantEmitter(DeConstantEmitter object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Scenario Player</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Scenario Player</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeScenarioPlayer(DeScenarioPlayer object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>De Scenario Event</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>De Scenario Event</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseDeScenarioEvent(DeScenarioEvent object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Mod Hel XObject</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Mod Hel XObject</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModHelXObject(ModHelXObject object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Block</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Block</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseBlock(Block object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Atomic Block</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Atomic Block</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAtomicBlock(AtomicBlock object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>EObject</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch, but this is the last case anyway.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>EObject</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject)
	 * @generated
	 */
	public T defaultCase(EObject object) {
		return null;
	}

} //DelibSwitch
