/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: PetriNetModelOfComputationImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.petrinet.impl;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.ModelOfComputationImpl;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.DurationValue;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.petrinet.PetriNetModelOfComputation;
import fr.supelec.modhelx.mocs.petrinet.PetrinetPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Petri Net Model Of Computation</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.petrinet.impl.PetriNetModelOfComputationImpl#getClock <em>Clock</em>}</li>
 *   <li>{@link fr.supelec.modhelx.mocs.petrinet.impl.PetriNetModelOfComputationImpl#getTimeOfCurrentSnapshot <em>Time Of Current Snapshot</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class PetriNetModelOfComputationImpl extends ModelOfComputationImpl implements PetriNetModelOfComputation {
	/**
	 * The cached value of the '{@link #getClock() <em>Clock</em>}' reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getClock()
	 * @generated
	 * @ordered
	 */
	protected Clock clock;

	/**
	 * The cached value of the '{@link #getTimeOfCurrentSnapshot() <em>Time Of Current Snapshot</em>}' reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getTimeOfCurrentSnapshot()
	 * @generated
	 * @ordered
	 */
	protected InstantValue timeOfCurrentSnapshot;
	
	/**
	 * The computed date of the next snapshot.
	 * This date is computed at the end of a snapshot by adding 1 unit to the time of the current snapshot.
	 * It is used to initialize the time of the next snapshot (in the reset operation). 
	 */
	private InstantValue nextTimeOfSnapshot;
	
	/**
	 * The block to update in the current step of computation. 
	 */
	private Block currentBlock;
	
	/**
	 * The list of blocks to update in the snapshot. Initially, it is the set
	 * of transitions that may fire.
	 */
	private LinkedList<Block> blocksToUpdate = new LinkedList<Block>();
		
	/**
	 * We need to save the lists of tokens present on all the pins of the blocks in case of reset.
	 * Only one table called "initial" is needed, the current lists are in fact the "storedTokens" lists on each pin.
	 */
	private Map<Token,Pin> initialTokenList;



	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	protected PetriNetModelOfComputationImpl() {
		super();
		
		currentBlock = null;
		
		initialTokenList = new HashMap<Token,Pin>();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return PetrinetPackage.Literals.PETRI_NET_MODEL_OF_COMPUTATION;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Clock getClock() {
		if (clock != null && clock.eIsProxy()) {
			InternalEObject oldClock = (InternalEObject)clock;
			clock = (Clock)eResolveProxy(oldClock);
			if (clock != oldClock) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, PetrinetPackage.PETRI_NET_MODEL_OF_COMPUTATION__CLOCK, oldClock, clock));
			}
		}
		return clock;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Clock basicGetClock() {
		return clock;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setClock(Clock newClock) {
		Clock oldClock = clock;
		clock = newClock;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, PetrinetPackage.PETRI_NET_MODEL_OF_COMPUTATION__CLOCK, oldClock, clock));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public InstantValue getTimeOfCurrentSnapshot() {
		if (timeOfCurrentSnapshot != null && timeOfCurrentSnapshot.eIsProxy()) {
			InternalEObject oldTimeOfCurrentSnapshot = (InternalEObject)timeOfCurrentSnapshot;
			timeOfCurrentSnapshot = (InstantValue)eResolveProxy(oldTimeOfCurrentSnapshot);
			if (timeOfCurrentSnapshot != oldTimeOfCurrentSnapshot) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, PetrinetPackage.PETRI_NET_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT, oldTimeOfCurrentSnapshot, timeOfCurrentSnapshot));
			}
		}
		return timeOfCurrentSnapshot;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public InstantValue basicGetTimeOfCurrentSnapshot() {
		return timeOfCurrentSnapshot;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(Model model, InstantValue initialTime) {
		super.setup(model, initialTime);
		
		// In Petri Nets, the logical time is always initialized at 0.
		// Then it will progress regularly of 1 unit (computed in the endOfSnapshot)
		// TODO uuuuh, this was copied from SDF. Discuss it with the others.
		timeOfCurrentSnapshot = this.getClock().createInstantValue(0);
		nextTimeOfSnapshot = this.getClock().createInstantValue(0);
		
		// Blocks may only be Places and Transitions
		// A Place may only be connected to Transitions
		// A Transition may only be connected to Places
		for(Block block : model.getStructure().getBlocks()) {
			// Check if the blocks have the correct number of input/output pins
			if(block.getPinsIn().size() != 1)
				throw new Error("Ill-formed model for PetriNet: block " +
						block.getName() + " has " + block.getPinsIn().size() +
						" input pins, this should be 1.");
			
			if(block.getPinsOut().size() != 1)
				throw new Error("Ill-formed model for PetriNet: block " +
						block.getName() + " has " + block.getPinsIn().size() +
						" output pins, this should be 1.");
			
			if(blockIsPlace(block)) {
				for(Pin p : block.getPinsIn())
					for(Relation r : p.getIncomingRelations())
						if(! blockIsTransition(r.getSource().getOutputOf()))
							throw new Error("Ill-formed model for PetriNet.");
				for(Pin p : block.getPinsOut())
					for(Relation r : p.getOutgoingRelations())
						if(! blockIsTransition(r.getTarget().getInputOf()) )
							throw new Error("Ill-formed model for PetriNet.");
			} else if(blockIsTransition(block)) {
				for(Pin p : block.getPinsIn())
					for(Relation r : p.getIncomingRelations())
						if(! blockIsPlace(r.getSource().getOutputOf()) )
							throw new Error("Ill-formed model for PetriNet.");
				for(Pin p : block.getPinsOut())
					for(Relation r : p.getOutgoingRelations())
						if(! blockIsPlace(r.getTarget().getInputOf()) )
							throw new Error("Ill-formed model for PetriNet.");				
			} else {
				StringBuffer msg = new StringBuffer("Ill-formed model for PetriNet: block ");
				msg.append(block.getName()).append(" is neither a place nor a transition. ");
				msg.append("Class: ").append(block.getClass()).append(". Parameters: ");
				msg.append(block.getParameters());
				throw new Error(msg.toString());
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void startOfSnapshot(Model model) {
		super.startOfSnapshot(model);
		
		Platform.getLog().info("startOfSnapshot: " + petriNetStatus(model));
		
		//Copy all the tokens present on the pins of the blocks into the initial table of tokens in case of reset
    	initialTokenList.clear();
		for (Block b : model.getStructure().getBlocks()) {
    		for(Pin p : b.getPinsIn()){
    			for(Token t : p.getStoredTokens()){
    				initialTokenList.put(t, p);
    			}
    		}
    		for(Pin p : b.getPinsOut()){
    			for(Token t : p.getStoredTokens()){
    				initialTokenList.put(t, p);
    			}
    		}
    	}

		//Copy all the tokens present on the pins of the model into the initial table of tokens in case of reset
		for(Pin p : model.getStructure().getPinsIn()){
			for(Token t : p.getStoredTokens()){
				initialTokenList.put(t, p);
			}
		}
		for(Pin p : model.getStructure().getPinsOut()){
			for(Token t : p.getStoredTokens()){
				initialTokenList.put(t, p);
			}
		}

		//Initialization of the current time with the time computed at the end of the previous snapshot
		timeOfCurrentSnapshot = nextTimeOfSnapshot;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfSnapshot(Model model) {
		super.endOfSnapshot(model);
		
		Platform.getLog().info("endOfSnapshot: " + petriNetStatus(model));
		
		//Compute the time of the next snapshot
		nextTimeOfSnapshot = this.getClock().createInstantValue(this.getClock().getCurrentTime()+1);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(Model model) {
		if(currentBlock == null) return;
		
		// Only the current block is updated
		Platform.getLog().entering(getClass().getName(), "update", model.getName());
		
		currentBlock.update(this);
		
		// apply Petrinet's token simple propagation rule
		// it is the same for places and transitions
		Pin pinIn = currentBlock.getPinsIn().get(0);
		Pin pinOut = currentBlock.getPinsOut().get(0);
		
		if(pinIn.getStoredTokens().size() > 0) {
			pinIn.getToken(true);  // remove the token in input
			pinOut.addToken(CoreFactory.eINSTANCE.createToken()); // produce a token in output
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void reset(Model model, InstantValue time) {
		super.reset(model, time);
		
		Platform.getLog().info("reset1: " + petriNetStatus(model));
		
		//Check if the time provided by the platform is equal to the computed time of the next snapshot
		if(time.compareTo(timeOfCurrentSnapshot)!=0){
			//If not, SDF should do nothing (i.e. do not compute anything)
			//TODO
			//But for the moment, we print an error message if SDF is asked a snapshot at a date which is different from the computed
			throw new Error("SdfModelOfComputation.reset : time provided by the platform is not the time computed");
		}else{
			double timeValue = time.getDenotedInstant().get(0).getDate();
			timeOfCurrentSnapshot = this.getClock().createInstantValue(timeValue);
		}
		
		//Reinitialize all the "storedTokens" lists of the pins with the initial table of tokens
		for(Pin p : initialTokenList.values()){
			p.getStoredTokens().clear();
		}
		for(Token t : initialTokenList.keySet()){
			initialTokenList.get(t).getStoredTokens().add(t);
		}
		
		
		Platform.getLog().info("reset2: " + petriNetStatus(model));

		//Move ALL the tokens present on the input pins of the model onto the input
		//pins of the connected blocks (it is checked at setup that the pins have
		//consistent numbers of consumed tokens)
		for(Pin pin : model.getStructure().getPinsIn()){
			for(Relation r : pin.getOutgoingRelations()){
				//Copy the tokens on the target pin
				r.getTarget().getStoredTokens().addAll(pin.getStoredTokens());
			}
			//Remove the tokens from the inputs of the model
			pin.getStoredTokens().clear();
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean snapDet(Model model) {
		return blocksToUpdate.size() == 0;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean newSnap(Model model) {
		// For now suppose that a new snapshot is always necessary
		DurationValue d = this.getClock().createDurationValue(
				this.getClock().createInstantValue(0),
				this.getClock().createInstantValue(1));
		timeOfCurrentSnapshot = timeOfCurrentSnapshot.add(d);
		
		return true;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void collectTimeConstraints(Model model, EList<InstantValue> timeConstraints) {
		//We post a constraint to indicate the time of the current snapshot
		//(which is computed at the endOfSnapshot of the previous snapshot and initialized in the startOfSnapshot)
		timeConstraints.add(timeOfCurrentSnapshot);

		super.collectTimeConstraints(model, timeConstraints); //CollectTimeConstraints delegated to the model structure
		//Equivalent to a collectTimeConstraints on all the blocks of the model
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void initialSchedule(Model model) {
		// display places and tokens
		Platform.getLog().info("initialSchedule: " + petriNetStatus(model));
		
		Platform.getLog().info("initialSchedule 1: blocksToUpdate=" + blocksToUpdate);
		
		if(blocksToUpdate.size() == 0) {
			// first pass of the snapshot, let's determine the list of
			// transitions that may fire
			for(Block block : model.getStructure().getBlocks()) {
				///Platform.getLog().info("initialSchedule: block " + block + "  -> isTransition=" + blockIsTransition(block));
				if(blockIsTransition(block) && transitionMayFire(block))
					blocksToUpdate.addLast(block);
			}
		}
		
		Platform.getLog().info("initialSchedule 2: blocksToUpdate=" + blocksToUpdate);
		
		// In any case, let's find a block to update. 
		// We look for it in the "blocksToUpdate" list:
		//  - If we find a place, then update it.
		//  - If we find a transition, then update it. Indeed, we are sure
		// that it may fire because finalSchedule ensures that the head of the
		// list, if it is a transition, may fire.
		if(blocksToUpdate.size() > 0)
			currentBlock = blocksToUpdate.removeFirst();
		else currentBlock = null;
		
		Platform.getLog().info("initialSchedule 3: blocksToUpdate=" + blocksToUpdate + ", currentBlock=" + currentBlock);
		
		// update internal data
		Platform.getController().changeMoCInternalData(this, "blocksToUpdate", blocksToUpdate);
		Platform.getController().changeMoCInternalData(this, "currentBlock", currentBlock);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void prePropagate(Model model) {
		if(currentBlock == null) return;
		
		// for transitions
		if(blockIsTransition(currentBlock)) {
			Pin pinIn = currentBlock.getPinsIn().get(0);
			
			// remove one token in each upstream place
			for(Relation r : pinIn.getIncomingRelations())
				r.getSource().getStoredTokens().remove(0);
			
			// add one token in input of the current block
			pinIn.addToken(CoreFactory.eINSTANCE.createToken());
		}
		
		// for places: nothing to do
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void postPropagate(Model model) {
		if(currentBlock == null) return;
		
		Pin pinOut = currentBlock.getPinsOut().get(0);
		
		// we postpropagate for transitions only
		if(blockIsTransition(currentBlock)) {
			// the block MUST have produced one token => remove it
			if(pinOut.getStoredTokens().size() < 1)
				throw new Error("Block " + currentBlock.getName() + " should have 1 token in output.");
				
			pinOut.getStoredTokens().remove(0);
			
			// put one token in each downstream place
			for(Relation r : pinOut.getOutgoingRelations()) {
				r.getTarget().addToken(CoreFactory.eINSTANCE.createToken());
				Platform.getLog().info("postpropagate: added token to " + r.getTarget());
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void finalSchedule(Model model) {
		if(currentBlock == null) return;
		
		// if we have just updated a transition, then any downstream place
		// must be updated before we do anything else
		if(blockIsTransition(currentBlock)) {
			Pin pinOut = currentBlock.getPinsOut().get(0);
			
			Platform.getLog().info("finalSchedule: currentBlock.pinOut=" + pinOut);
			for(Relation r : pinOut.getOutgoingRelations()) {
				Platform.getLog().info("finalSchedule: outrel=" + r);
				blocksToUpdate.addFirst(r.getTarget().getInputOf());
			}
		}
		
		Platform.getLog().info("finalSchedule 1: blocksToUpdate=" + blocksToUpdate);
		
		// if the first element of blocksToUpdate is a transition,
		// check if it may fire... If not, remove the transition and
		// use the next block for the next update
		boolean nextBlockOk = false;
		while((! nextBlockOk) && blocksToUpdate.size() > 0) {
			Block nextBlock = blocksToUpdate.getFirst();
			if(blockIsPlace(nextBlock) || 
					blockIsTransition(nextBlock) && transitionMayFire(nextBlock)) nextBlockOk = true;
			else blocksToUpdate.removeFirst();
		}
		
		Platform.getLog().info("finalSchedule 2: blocksToUpdate=" + blocksToUpdate);
		
		Platform.getController().changeMoCInternalData(this, "blocksToUpdate", blocksToUpdate);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setBlockIsPlace(Block block) {
		block.getParameters().put(ROLE_ATTRIBUTE, ROLE_PLACE);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setBlockIsTransition(Block block) {
		block.getParameters().put(ROLE_ATTRIBUTE, ROLE_TRANSITION);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case PetrinetPackage.PETRI_NET_MODEL_OF_COMPUTATION__CLOCK:
				if (resolve) return getClock();
				return basicGetClock();
			case PetrinetPackage.PETRI_NET_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT:
				if (resolve) return getTimeOfCurrentSnapshot();
				return basicGetTimeOfCurrentSnapshot();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case PetrinetPackage.PETRI_NET_MODEL_OF_COMPUTATION__CLOCK:
				setClock((Clock)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case PetrinetPackage.PETRI_NET_MODEL_OF_COMPUTATION__CLOCK:
				setClock((Clock)null);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case PetrinetPackage.PETRI_NET_MODEL_OF_COMPUTATION__CLOCK:
				return clock != null;
			case PetrinetPackage.PETRI_NET_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT:
				return timeOfCurrentSnapshot != null;
		}
		return super.eIsSet(featureID);
	}
	
	
	/**
	 * Tells if a given block in a Petri net model is a place.
	 * 
	 * @param block a block
	 * @return {@code true} if the block is a place
	 */
	private boolean blockIsPlace(Block block) {
		return ROLE_PLACE.equals(block.getParameters().get(ROLE_ATTRIBUTE));
	}
	
	/**
	 * Tells if a given block in a Petri net model is a transition.
	 * 
	 * @param block a block
	 * @return {@code true} if the block is a transition
	 */
	private boolean blockIsTransition(Block block) {
		return ROLE_TRANSITION.equals(block.getParameters().get(ROLE_ATTRIBUTE));
	}
	
	/**
	 * Tells whether a transition may fire. To fire, all "incoming places" must
	 * hold at least one token.
	 * 
	 * @param transition the transition
	 * @return {@code true} iff the transition may fire
	 */
	private boolean transitionMayFire(Block transition) {
		assert blockIsTransition(transition);
		
		// we have only one input pin
		Pin pIn = transition.getPinsIn().get(0);
		
		for(Relation r : pIn.getIncomingRelations()) {
			Pin p = r.getSource();
			if(p.getStoredTokens().size() == 0) return false;
		}
		return true;
	}
	
	private String petriNetStatus(Model model) {
		String res = "*** Petri Net Status: ";
		for(Block block : model.getStructure().getBlocks()) {
			if(blockIsPlace(block)) {
				res += block.getName() + "(" +
					block.getPinsIn().get(0).getStoredTokens().size() + "+" +
					block.getPinsOut().get(0).getStoredTokens().size() + ")  ";
			}
		}
		return res + "***";
	}

} //PetriNetModelOfComputationImpl
