/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfModelOfComputationImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdf.impl;

import java.util.HashMap;
import java.util.WeakHashMap;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.ModelOfComputationImpl;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.DurationValue;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdf.SdfPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the Synchronous DataFlow '<em><b>Model Of Computation</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.sdf.impl.SdfModelOfComputationImpl#getClock <em>Clock</em>}</li>
 *   <li>{@link fr.supelec.modhelx.mocs.sdf.impl.SdfModelOfComputationImpl#getTimeOfCurrentSnapshot <em>Time Of Current Snapshot</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class SdfModelOfComputationImpl extends ModelOfComputationImpl implements SdfModelOfComputation {
	
	/**
	 * The cached value of the '{@link #getClock() <em>Clock</em>}' reference.
	 * <!-- begin-user-doc -->
	 * SDF is an "untimed" MoC, but still, a logical clock is needed. This clock
	 * is synchronized on the snapshots and progresses of one unit at each snapshot
	 * (only if the snapshot has a meaning for SDF).
	 * <!-- end-user-doc -->
	 * @see #getClock()
	 * @generated
	 * @ordered
	 */
	protected Clock clock;
	
	/**
	 * The cached value of the '{@link #getTimeOfCurrentSnapshot() <em>Time Of Current Snapshot</em>}' reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getTimeOfCurrentSnapshot()
	 * @generated
	 * @ordered
	 */
	protected InstantValue timeOfCurrentSnapshot;
	
	/**
	 * The computed date of the next snapshot.
	 * This date is computed at the end of a snapshot by adding 1 unit to the time of the current snapshot.
	 * It is used to initialize the time of the next snapshot (in the reset operation). 
	 */
	private InstantValue nextTimeOfSnapshot;
	
	/**
	 * The list of all the blocks of the model, sorted according to the reverse topological order.
	 * Does not change during the computation (sorted during the setup).
	 */
	//private ArrayList<Block> sortedBlockList;
	
	/**
	 * The block to update in the current step of computation. 
	 */
	private Block currentBlock;
		
	/**
	 * We need to save the lists of tokens present on all the pins of the blocks in case of reset.
	 * Only one table called "initial" is needed, the current lists are in fact the "storedTokens" lists on each pin.
	 */
	private HashMap <Token,Pin> initialTokenList;
	
	/**
	 * A list to maintain the value of all the tokens which represent SDF data during computation.
	 */
	private WeakHashMap<Token, Object> valueOfDataTokens;
	private WeakHashMap<Token, Object> initialValueOfDataTokens;
	
	/**
	 * Number of activation of each block during the current snapshot
	 */
	private HashMap<Block, Integer> activationCounts_ = null;

	/**
	 * Label of the parameter specifying the production/consumption rate of the pins of SDF blocks
	 */
	private final static String	SDF_RATE_PARAM_NAME = "SDFTokenRate";
	private final static String	SDF_NO_INSTANTANEOUS_DEPENDENCY = "SDFNoInstantaneousDependency";
		
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	protected SdfModelOfComputationImpl() {
		super();
				
		currentBlock = null;
		
		initialTokenList = new HashMap<Token,Pin>();

		valueOfDataTokens = new WeakHashMap<Token,Object>();
		initialValueOfDataTokens = new WeakHashMap<Token,Object>();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdfPackage.Literals.SDF_MODEL_OF_COMPUTATION;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Clock getClock() {
		if (clock != null && clock.eIsProxy()) {
			InternalEObject oldClock = (InternalEObject)clock;
			clock = (Clock)eResolveProxy(oldClock);
			if (clock != oldClock) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SdfPackage.SDF_MODEL_OF_COMPUTATION__CLOCK, oldClock, clock));
			}
		}
		return clock;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Clock basicGetClock() {
		return clock;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setClock(Clock newClock) {
		Clock oldClock = clock;
		clock = newClock;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SdfPackage.SDF_MODEL_OF_COMPUTATION__CLOCK, oldClock, clock));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public InstantValue getTimeOfCurrentSnapshot() {
		if (timeOfCurrentSnapshot != null && timeOfCurrentSnapshot.eIsProxy()) {
			InternalEObject oldTimeOfCurrentSnapshot = (InternalEObject)timeOfCurrentSnapshot;
			timeOfCurrentSnapshot = (InstantValue)eResolveProxy(oldTimeOfCurrentSnapshot);
			if (timeOfCurrentSnapshot != oldTimeOfCurrentSnapshot) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, SdfPackage.SDF_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT, oldTimeOfCurrentSnapshot, timeOfCurrentSnapshot));
			}
		}
		return timeOfCurrentSnapshot;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public InstantValue basicGetTimeOfCurrentSnapshot() {
		return timeOfCurrentSnapshot;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(Model model, InstantValue initialTime) {
		super.setup(model, initialTime);
				
		//Check that the consumed/produced token numbers are consistent for the input and the output pins of the model
		//and the connected blocks
		for(Pin pInModel : model.getStructure().getPinsIn()){
			int reqNumbTokModel = this.getPinRate(pInModel);
			for(Relation r : pInModel.getOutgoingRelations()){
				int reqNumbTokBlock = this.getPinRate(r.getTarget());
				if(reqNumbTokModel != reqNumbTokBlock){
					throw new Error("Ill-formed model for SDF : the token rates are not consistent on the input pins of the model and on the connected blocks");
				}
			}
		}
		for(Pin pOutModel : model.getStructure().getPinsOut()){
			int prodNumbTokModel =  this.getPinRate(pOutModel);
			for(Relation r : pOutModel.getIncomingRelations()){
				int prodNumbTokBlock =  this.getPinRate(r.getSource());
				if(prodNumbTokModel != prodNumbTokBlock){
					throw new Error("Ill-formed model for SDF : the token rates are not consistent on the output pins of the model and on the connected blocks");
				}
			}
		}

		//In SDF, the logical time is always initialized at 0.
		//Then it will progress regularly of 1 unit (computed in the endOfSnapshot)
		timeOfCurrentSnapshot = this.getClock().createInstantValue(0);
		nextTimeOfSnapshot = this.getClock().createInstantValue(0);		
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void startOfSnapshot(Model model) {
		super.startOfSnapshot(model);
		
		//Copy all the tokens present on the pins of the blocks into the initial table of tokens in case of reset
    	initialTokenList.clear();
		for (Block b : model.getStructure().getBlocks()) {
    		for(Pin p : b.getPinsIn()){
    			for(Token t : p.getStoredTokens()){
    				initialTokenList.put(t, p);
    			}
    		}
    		for(Pin p : b.getPinsOut()){
    			for(Token t : p.getStoredTokens()){
    				initialTokenList.put(t, p);
    			}
    		}
    	}

		//Copy all the tokens present on the pins of the model into the initial table of tokens in case of reset
		for(Pin p : model.getStructure().getPinsIn()){
			for(Token t : p.getStoredTokens()){
				initialTokenList.put(t, p);
			}
		}
		for(Pin p : model.getStructure().getPinsOut()){
			for(Token t : p.getStoredTokens()){
				initialTokenList.put(t, p);
			}
		}
		
		//Copy the values of the tokens
		initialValueOfDataTokens.clear();
		initialValueOfDataTokens.putAll(valueOfDataTokens);

		Platform.getLog().info("**** " + valueOfDataTokens.size() + " initial values of tokens = " +initialValueOfDataTokens);
		
		//Initialization of the current time with the time computed at the end of the previous snapshot
		timeOfCurrentSnapshot = nextTimeOfSnapshot;
		
	}
	
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfSnapshot(Model model) {
		super.endOfSnapshot(model);
		
		Platform.getLog().info("**** " + valueOfDataTokens.size() + " values of tokens = " +valueOfDataTokens);
		
		//Compute the time of the next snapshot
//		nextTimeOfSnapshot = this.getClock().createInstantValue(this.getClock().getCurrentTime()+1);
		nextTimeOfSnapshot = timeOfCurrentSnapshot.add(
				getClock().createDurationValue(
						getClock().createInstantValue(0), 
						getClock().createInstantValue(1)
				)
		);
		
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(Model model) {
		// Only the current block is updated
		Platform.getLog().entering(getClass().getName(), "update", model.getName());
		
		currentBlock.update(this);
		//We assume that the block consumes the tokens present on its input pins
		//(depending on the number of tokens which is consumed to be present on each pin)
		//and produces the specified number of tokens on its output pins
		//CAUTION the tokens must be removed/replaced in the table containing the values...
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void reset(Model model, InstantValue time) {
		super.reset(model, time);
		
		Platform.getLog().finer("Time=" + time + ", expected time=" + timeOfCurrentSnapshot);
		
		/*
		 * TODO: ChJ -> I have disabled the time of snapshot check. We would
		 * need to do nothing is the date is not the date expected for the next
		 * snapshot.
		 * 
		 * 
		//Check if the time provided by the platform is equal to the computed time of the next snapshot
		if(time.compareTo(timeOfCurrentSnapshot)!=0){
			//If not, SDF should do nothing (i.e. do not compute anything)
			//TODO
			//But for the moment, we print an error message if SDF is asked a snapshot at a date which is different from the computed
			throw new Error("SdfModelOfComputation.reset: time provided by the platform is not the time computed (expected " + timeOfCurrentSnapshot + ", got " + time + ")");
		}else */  {
			double timeValue = time.getDenotedInstant().get(0).getDate();
			timeOfCurrentSnapshot = this.getClock().createInstantValue(timeValue);
		}
		
		//Reinitialize all the "storedTokens" lists of the pins with the initial table of tokens
		for(Pin p : initialTokenList.values()){
			p.getStoredTokens().clear(); //Not really necessary since super.reset(...) removes all the tokens from the pins
		}
		for(Token t : initialTokenList.keySet()){
			initialTokenList.get(t).getStoredTokens().add(t);
		}
		
		//Reinitialize the values of data tokens with the initial table of values
		valueOfDataTokens.clear();
		valueOfDataTokens.putAll(initialValueOfDataTokens);
		
		//propagateTokensFromInputPins(model);
		for(Pin pin : model.getStructure().getPinsIn()){
			propagateTokensFromPin(pin);
		}
		
		// Reset activation counts
		activationCounts_ = new HashMap<Block, Integer>();
	}
	
//	private void propagateTokensFromInputPins(Model model) {
//		//Move ALL the tokens present on the input pins of the model onto the input
//		//pins of the connected blocks (it is checked at setup that the pins have
//		//consistent numbers of consumed tokens)
//		for(Pin pin : model.getStructure().getPinsIn()){
//			Platform.getLog().finest("propagating all tokens located on " + pin + ": " + pin.getStoredTokens());
//			for(Relation r : pin.getOutgoingRelations()){
//				//Copy the tokens on the target pin
//				r.getTarget().getStoredTokens().addAll(pin.getStoredTokens());
//			}
//			//Remove the tokens from the inputs of the model
//			pin.getStoredTokens().clear();
//		}
//	}
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean snapDet(Model model) {
		super.snapDet(model);

		//The snapshot is determined when the configuration of the model is back to the inital config
		//as saved in the start of snapshot
		//i.e. all pins have the same number of tokens as in the start of snapshot, except the output pins of the model
		
		//Check the input pins of the model
		boolean inputPinsOK = true;
		java.util.Iterator<Pin> inputpinsit = model.getStructure().getPinsIn().iterator();
		while(inputpinsit.hasNext() && inputPinsOK){
			Pin pInModel = inputpinsit.next();
			int currentNb = pInModel.getStoredTokens().size();
			int initialNb = 0;
			for(Token t : this.initialTokenList.keySet()){
				if(this.initialTokenList.get(t).equals(pInModel)) initialNb++;
			}
			if(currentNb != initialNb) inputPinsOK = false;
		}
		
		//If the input pins of the model are OK then we check the pins of the blocks of the model
		boolean blocksOK = true;
		if(inputPinsOK){
			java.util.Iterator<Block> blockit = model.getStructure().getBlocks().iterator();
			while(blockit.hasNext() && blocksOK){
				Block b = blockit.next();
				//Check the input pins
				java.util.Iterator<Pin> binputpinsit = b.getPinsIn().iterator();
				while(binputpinsit.hasNext() && blocksOK){
					Pin pInBlock = binputpinsit.next();
					int currentNb = pInBlock.getStoredTokens().size();
					int initialNb = 0;
					for(Token t : this.initialTokenList.keySet()){
						if(this.initialTokenList.get(t).equals(pInBlock)) initialNb++;
					}
					if(currentNb != initialNb) blocksOK = false;
				}
				
				//Check the output pins
				java.util.Iterator<Pin> boutputpinsit = b.getPinsOut().iterator();
				while(boutputpinsit.hasNext() && blocksOK){
					Pin pOutBlock = boutputpinsit.next();
					int currentNb = pOutBlock.getStoredTokens().size();
					int initialNb = 0;
					for(Token t : this.initialTokenList.keySet()){
						if(this.initialTokenList.get(t).equals(pOutBlock)) initialNb++;
					}
					if(currentNb != initialNb) blocksOK = false;
				}
				
			}
		}
		
		return inputPinsOK && blocksOK;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean newSnap(Model model) {
		super.newSnap(model);

		//A new snapshot is always necessary in SDF
		//("Source" blocks produce data each time a snapshot is made)
		//Exceptions : a new snapshot cannot be done
		//1) if the model has inputs and no data on the inputs
		//2) if the user has specified a finite number of snapshots

//		//1) We check if data is present on the input pins of the model (if it has input pins)
// 		for(Pin p : model.getStructure().getPinsIn()){
////			if(p.getStoredTokens().isEmpty())
////				return false;//TODO verifier le nb de tokens presents par rapport au nb de tokens consumed par pin ?
//			if(p.getStoredTokens().size() < getPinRate(p)) {
//				return false;
//			}
//		}
		//TODO commenté car plante les simus quand SDF est embarqué dans un IB (puisque les tokens ne sont pas présents sur les entrees du modele au moment du newsnap)
 		//TODO mettre au clair les conditions du newsnap
 
     	//2) is handled by the platform
 		
 		//If the next snapshot is going to happen, 
		//We initiate the time for the next snapshot = time of the current snapshot + 1 (logical time)
		DurationValue d = this.getClock().createDurationValue(
				this.getClock().createInstantValue(0),
				this.getClock().createInstantValue(1));
		timeOfCurrentSnapshot = timeOfCurrentSnapshot.add(d);
		
		Platform.getLog().finer("Next SDF update will be done at time " + timeOfCurrentSnapshot);
		
		return true;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void collectTimeConstraints(Model model, EList<InstantValue> timeConstraints) {
		//We post a constraint to indicate the time of the current snapshot
		//(which is computed at the endOfSnapshot of the previous snapshot and initialized in the startOfSnapshot)
		timeConstraints.add(timeOfCurrentSnapshot);

		super.collectTimeConstraints(model, timeConstraints); //CollectTimeConstraints delegated to the model structure
		//Equivalent to a collectTimeConstraints on all the blocks of the model
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void initialSchedule(Model model) {
		super.initialSchedule(model);
		// Choose the next block to update. Only blocks which have all their inputs available can be scheduled.
		// However, to avoid endless production of unconsumed tokens, a schedulable block should be scheduled
		// only if it has not yet produced enough data for the blocks which depend on it.
		// Sinks (blocks which produce nothing) should be scheduled as soon as their can be in order
		// to avoid the storage of numerous token on pins.
		// Among schedulable blocks, the one which has already been activated the least number of times
		// in the current snapshot is scheduled.
		this.currentBlock = null;
		for (Block b : model.getStructure().getBlocks()) {
			// A block can be scheduled only if it has enough data on all its inputs
			boolean allpinsok = true;
			java.util.Iterator<Pin> pinit = b.getPinsIn().iterator();
			while(pinit.hasNext() && allpinsok){
				Pin pIn = pinit.next();
				int requiredTokenNb = this.getPinRate(pIn);
				int receivedTokenNb = pIn.getStoredTokens().size();
				if(receivedTokenNb < requiredTokenNb){
					allpinsok = false;
				}
			}
			if (!allpinsok) {
				continue; // look at next block
			}
//			System.out.println("Block "+b.getName()+" has all its input data available");
			// block b has all its required inputs
			if (b.getPinsOut().size() == 0) {
				// This is a sink block, so it should be scheduled to consume available data
				currentBlock = b;
				break; // update this block NOW
			}
			// This block is not a sink.
			// It should NOT be scheduled if it has already produced enough data for its consumer blocks ?
			for (Pin pOut : b.getPinsOut()) {
				// If at least one target pin has not enough data, we should produce some
				for (Relation r : pOut.getOutgoingRelations()) {
					Pin targetPin = r.getTarget();
					if (getPinRate(targetPin) > targetPin.getStoredTokens().size()) {
						if (currentBlock == null) {
							currentBlock = b;
						} else {
							// Choose this block only if we do not have a current block
							// or if its activation count is smaller than the current block's
							if (getActivationCount(b) < getActivationCount(currentBlock)) {
								currentBlock = b;
							}
						}
						break;
					}
//					System.out.println(targetPin+" has "+targetPin.getStoredTokens().size()+" and needs "+getPinRate(targetPin));
				}
				if (currentBlock != null) {
					break;
				}
			}
			// Here, we may have found a schedulable block, but with must still looks
			// for other blocks in case there is a sink further in the model.
		}
		if (currentBlock == null) {
			// No schedulable block => there is a loop in the model
			// so there should be a block with an output that does not
			// depend instantaneously on its inputs
			for (Block b : model.getStructure().getBlocks()) {
				if ((getActivationCount(b) % 2) == 0) {
					// If the block has not already been partially 
					// updated for producing not instantaneously dependent
					// outputs, check for such outputs.
					for (Pin p : b.getPinsOut()) {
						if (!isPinInstantaneouslyDependent(p)) {
							// we can update b because it can produce
							// an output on p even if it does not have 
							// all its inputs available
							if (currentBlock == null) {
								currentBlock = b;
							} else {
								if (getActivationCount(b) < getActivationCount(currentBlock)) {
									currentBlock = b;
								}
							}
						}
					}
				}
			}
			// Should not happen !
			if (currentBlock == null) {
				throw new Error("SDF Scheduling problem: no block to update (loop without delay?)");
			}
			halfStepActivationCount(currentBlock);
		} else {
			if ((getActivationCount(currentBlock) % 2) == 1) {
				// If block has already been partially updated to produce
				// non instantaneously dependent outputs, this upadte is 
				// the second halve of its activation
				halfStepActivationCount(currentBlock);
			} else {
				stepActivationCount(currentBlock);
			}
		}
		Platform.getLog().info("**** block to update = " +currentBlock);

	}

	/**
	 * Return the number of activations of this block during the current snapshot.
	 * For regular blocks, the returned value is twice the number of calls to update.
	 * For blocks with outputs which do not depend instantaneously on their inputs,
	 * the returned value may be odd if the block has been updated when its inputs
	 * were not available, and not yet updated again with its inputs available.
	 * @param b
	 * @return
	 */
	private int getActivationCount(Block b) {
		Integer count = activationCounts_.get(b);
		if (count == null) {
			return 0;
		} else {
			return count;
		}
	}
	
	/**
	 * Register a full update for block b (adds 2 to its activation count).
	 * @param b
	 */
	private void stepActivationCount(Block b) {
		activationCounts_.put(b, getActivationCount(b) + 2) ;
	}

	/**
	 * Register a partial update for block b (adds 1 to its activation count).
	 * This is used when a block is updated without having its inputs available.
	 * It can happen only to blocks with outputs which do not depend instantaneously
	 * on their inputs.
	 * @param b
	 */
	private void halfStepActivationCount(Block b) {
		activationCounts_.put(b, getActivationCount(b) + 1) ;
	}
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void postPropagate(Model model) {
		super.postPropagate(model);
		//We have to copy the tokens from the output pins of the current block 
		//on each of the pins which are connected through the relations
		// (even in the case of a block which is connected to the output pins of the model)
		for(Pin pOut : currentBlock.getPinsOut()){
			propagateTokensFromPin(pOut);
		}
		
	}
	
	private void propagateTokensFromPin(Pin p){
		//for all the tokens present on the pin p
		//TODO ATTENTION : we suppose that the token rates are equals on both pins pOut et pIn
		//therefore we do not count the tokens
		while(p.hasTokens()){
			//get the value of the token and remove the token from p
			Token token = p.getToken(true);
			Object value = this.getValueOfToken(token);
			//For each block connected through a relation leaving p
			for(Relation r : p.getOutgoingRelations()){
				Pin pIn = r.getTarget();
				//Create a copy of the token
				Token t = this.createDataToken(value);
				Platform.getLog().finer("put token with value "+value+" on pin "+pIn);
				//and put the token on the input pin
				pIn.addToken(t);
			}
		}
	}


	
	@Override
	public Object getValueOfToken(Token token) {
		return valueOfDataTokens.get(token);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Token createDataToken(Object value) {
		Token t = CoreFactory.eINSTANCE.createToken();
		valueOfDataTokens.put(t, value);
		return t;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setPinRate(Pin pin, int rate) {
		pin.setValueOfParameter(SDF_RATE_PARAM_NAME, new Integer(rate));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public int getPinRate(Pin pin) {
		if(pin.getParameters().containsKey(SDF_RATE_PARAM_NAME)){
			return ((Integer)pin.getValueOfParameter(SDF_RATE_PARAM_NAME)).intValue();
		}else{
			// Use default value when no rate is specified
			return 1;
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setPinInstantaneousDependency(Pin pin, boolean depends) {
		pin.setValueOfParameter(SDF_NO_INSTANTANEOUS_DEPENDENCY, !depends);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean isPinInstantaneouslyDependent(Pin pin) {
		Object nodep = pin.getValueOfParameter(SDF_NO_INSTANTANEOUS_DEPENDENCY);
		if (nodep == null) {
			return true;
		} else {
			return !(Boolean)nodep;
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void startOfUpdate(Model model) {
		super.startOfUpdate(model);
		
		for(Pin pmodel : model.getStructure().getPinsIn()){
			propagateTokensFromPin(pmodel);
		}
		//propagateTokensFromInputPins(model);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfUpdate(Model model) {
		super.endOfUpdate(model);
		//TODO verifier si eou réllement nécessaire (postPropagate fait le job)
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean further(Model model) {
		super.further(model); 
		//TODO voir si on ne peut pas mettre ça par défaut dans MoCImpl
		return !this.snapDet(model);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SdfPackage.SDF_MODEL_OF_COMPUTATION__CLOCK:
				if (resolve) return getClock();
				return basicGetClock();
			case SdfPackage.SDF_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT:
				if (resolve) return getTimeOfCurrentSnapshot();
				return basicGetTimeOfCurrentSnapshot();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SdfPackage.SDF_MODEL_OF_COMPUTATION__CLOCK:
				setClock((Clock)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SdfPackage.SDF_MODEL_OF_COMPUTATION__CLOCK:
				setClock((Clock)null);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SdfPackage.SDF_MODEL_OF_COMPUTATION__CLOCK:
				return clock != null;
			case SdfPackage.SDF_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT:
				return timeOfCurrentSnapshot != null;
		}
		return super.eIsSet(featureID);
	}

} //SdfModelOfComputationImpl
