/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfAccumulatorImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.impl;

import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfAccumulator;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;
import fr.supelec.modhelx.util.Numeric;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Sdf Accumulator</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.sdflib.impl.SdfAccumulatorImpl#getInitialAccumulatorValue <em>Initial Accumulator Value</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class SdfAccumulatorImpl extends AtomicBlockImpl implements SdfAccumulator {
	/**
	 * The default value of the '{@link #getInitialAccumulatorValue() <em>Initial Accumulator Value</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getInitialAccumulatorValue()
	 * @generated
	 * @ordered
	 */
	protected static final double INITIAL_ACCUMULATOR_VALUE_EDEFAULT = 0.0;
	// accumulator value
	private double accumulator = 0;
	

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SdfAccumulatorImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdflibPackage.Literals.SDF_ACCUMULATOR;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public double getInitialAccumulatorValue() {
		Object value = getValueOfParameter("initialAccumulatorValue");
		if (value == null) {
			return INITIAL_ACCUMULATOR_VALUE_EDEFAULT;
		}
		try {
			Double v = (Double) value;
			return v.doubleValue();
		}
		catch (ClassCastException ex) {
			throw new Error("SdfAccumulator.getInitialAccumulatorValue : initialAccumulatorValue is not a Double");
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setInitialAccumulatorValue(double newInitialAccumulatorValue) {
		setValueOfParameter("initialAccumulatorValue", new Double(newInitialAccumulatorValue));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public void update(ModelOfComputation moc) {
		SdfModelOfComputation sdfMoc = (SdfModelOfComputation) moc;
		
		Pin pOut = getPinsOut().get(0);
		Pin pIn = getPinsIn().get(0);
		
		// DEBUG: print input pin contents
		{
			Platform.getLog().finest("Tokens in input: " + pIn.getStoredTokens());
		}
		
		for(int tokenCtr = 0; tokenCtr < sdfMoc.getPinRate(pIn); tokenCtr++) {
			Token tIn = pIn.getToken(true);
			double before = accumulator;
			accumulator += Numeric.doubleValueOfObject(sdfMoc.getValueOfToken(tIn));
			
			Platform.getLog().finer("acc was " + before + ", emitting token [val=" + accumulator + "]");
			Token tOut = sdfMoc.createDataToken(accumulator);
			pOut.addToken(tOut);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		if(getPinsIn().size() != 1) throw new Error("SdfAccumulator: must have 1 input.");
		if(getPinsOut().size() < 1) throw new Error("SdfAccumulator: wrong number of outputs.");
		
		accumulator = getInitialAccumulatorValue();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public double getCurrentValue() {
		return accumulator;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SdflibPackage.SDF_ACCUMULATOR__INITIAL_ACCUMULATOR_VALUE:
				return getInitialAccumulatorValue();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SdflibPackage.SDF_ACCUMULATOR__INITIAL_ACCUMULATOR_VALUE:
				setInitialAccumulatorValue((Double)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_ACCUMULATOR__INITIAL_ACCUMULATOR_VALUE:
				setInitialAccumulatorValue(INITIAL_ACCUMULATOR_VALUE_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_ACCUMULATOR__INITIAL_ACCUMULATOR_VALUE:
				return getInitialAccumulatorValue() != INITIAL_ACCUMULATOR_VALUE_EDEFAULT;
		}
		return super.eIsSet(featureID);
	}

} //SdfAccumulatorImpl
