/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfAdderImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.impl;

import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfAdder;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;
import fr.supelec.modhelx.util.Numeric;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Sdf Adder</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SdfAdderImpl extends AtomicBlockImpl implements SdfAdder {
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SdfAdderImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdflibPackage.Literals.SDF_ADDER;
	}
	
	private final String MULTFACTOR_PARAM_NAME = "MultFactor";
	private SdfModelOfComputation myMoC_;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc_) {
		super.update(moc_);
		
		// Compute the sum of all inputs, weighted by the multiplication factor of each input
		// If the rate of an input if n, the values of n tokens is added for this input
		double output = 0.0;
		for (Pin p : getPinsIn()) {
			double fact = getMultFactor(p);
			int rate = myMoC_.getPinRate(p);
			double pval = 0.0;
			for (int i = 0; i < rate; i++) {
				pval += Numeric.doubleValueOfObject(myMoC_.getValueOfToken(p.getToken(true)));
			}
			output += pval * fact;
		}
		// Produce tokens with value the sum on each output.
		// If the rate of an input is n, n tokens are put on it.
		for (Pin p : getPinsOut()) {
			int rate = myMoC_.getPinRate(p);
			for (int i = 0; i < rate; i++) {
				p.addToken(myMoC_.createDataToken(output));
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		super.setup(moc, initialTime);
		
		if (moc instanceof SdfModelOfComputation) {
			myMoC_ = (SdfModelOfComputation)moc;
		} else {
			throw new Error(getClass().getName() + ".setup : moc is not SDF");
		}
		for (Pin p : getPinsIn()) {
			if (!hasMultFactor(p)) {
				setMultFactor(p, 1);
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setMultFactor(Pin pin, double multFactor) {
		pin.setValueOfParameter(MULTFACTOR_PARAM_NAME, multFactor);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public double getMultFactor(Pin pin) {
		Object val = pin.getValueOfParameter(MULTFACTOR_PARAM_NAME);
		if(val instanceof Double) {
			return (Double)val;
		} else {
			throw new Error("Pin " + pin + " does not have a parameter " + MULTFACTOR_PARAM_NAME);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean hasMultFactor(Pin pin) {
		return pin.getValueOfParameter(MULTFACTOR_PARAM_NAME) != null;
	}

} //SdfAdderImpl
