/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfConstantEmitterImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.impl;

import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfConstantEmitter;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Sdf Constant Emitter</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.sdflib.impl.SdfConstantEmitterImpl#getValueToEmit <em>Value To Emit</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class SdfConstantEmitterImpl extends AtomicBlockImpl implements SdfConstantEmitter {
	/**
	 * The default value of the '{@link #getValueToEmit() <em>Value To Emit</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getValueToEmit()
	 * @generated NOT
	 * @ordered
	 */
	protected static final Object VALUE_TO_EMIT_EDEFAULT = new Integer(0);
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SdfConstantEmitterImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdflibPackage.Literals.SDF_CONSTANT_EMITTER;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Object getValueToEmit() {
		//The value to emit is stored as a parameter of the block
		Object value = getValueOfParameter("valueToEmit");
		if (value == null) {
			return VALUE_TO_EMIT_EDEFAULT;
		}
		return value;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setValueToEmit(Object newValueToEmit) {
		setValueOfParameter("valueToEmit", newValueToEmit);
	}

	private SdfModelOfComputation getMoc(ModelOfComputation moc) {
		try {
			SdfModelOfComputation m = (SdfModelOfComputation)moc;
			return m;
		}
		catch (ClassCastException ex) {
			throw new Error("SdfConstantEmitter : moc is not SDF");
		}
	}


	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc_) {
		SdfModelOfComputation moc = getMoc(moc_);
		super.update(moc);
		
		//For each output pin, the block must produce the specified number of tokens
		//The values of the tokens are stored by the MoC
		for(Pin pOut : getPinsOut()){
			int nbOfTokensToProduce = moc.getPinRate(pOut);
			for(int i = 0 ; i < nbOfTokensToProduce ; i++){
				Token t = moc.createDataToken(getValueToEmit());
				pOut.addToken(t);
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc_, InstantValue initialTime) {
		SdfModelOfComputation moc = getMoc(moc_);
		super.setup(moc, initialTime);
		
		//Check that the block has no input pin and at least one output pin
		if (getPinsIn().size() != 0) {
			throw new Error("SdfConstantEmitter.setup : input pins are not expected");
		}
		if (getPinsOut().size() == 0) {
			throw new Error("SdfConstantEmitter.setup : at least one output pin is expected");
		}
		
		//Check that each output pin has a specified token rate
		for(Pin p : getPinsOut()){
			moc.getPinRate(p); //Checks that the token rate exists for this pin
		}

	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SdflibPackage.SDF_CONSTANT_EMITTER__VALUE_TO_EMIT:
				return getValueToEmit();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SdflibPackage.SDF_CONSTANT_EMITTER__VALUE_TO_EMIT:
				setValueToEmit(newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_CONSTANT_EMITTER__VALUE_TO_EMIT:
				setValueToEmit(VALUE_TO_EMIT_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_CONSTANT_EMITTER__VALUE_TO_EMIT:
				return VALUE_TO_EMIT_EDEFAULT == null ? getValueToEmit() != null : !VALUE_TO_EMIT_EDEFAULT.equals(getValueToEmit());
		}
		return super.eIsSet(featureID);
	}

	
} //SdfConstantEmitterImpl
