/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfDelayImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.impl;

import java.util.Arrays;
import java.util.Collection;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;

import fr.supelec.modhelx.core.impl.AtomicBlockImpl;

import fr.supelec.modhelx.core.time.InstantValue;

import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfDelay;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;

import org.eclipse.emf.ecore.EClass;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Sdf Delay</b></em>'.
 * An SDF delay delays its input by a given number of samples.
 * The number of samples by which the input is delayed is the
 * number of samples in the initialData parameter of the delay.
 * If this parameter is not set, the integer value 0 is used 
 * and the delay has size 1.
 * This parameter can be set as a value (delay of size 1), or
 * as a Java ollection or array (delay of same size as the collection).
 * The outputs of a delay are marked as not depending instantaneously
 * from its input. Therefore, a delay may be used to break a dependency loop.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.sdflib.impl.SdfDelayImpl#getInitialData <em>Initial Data</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class SdfDelayImpl extends AtomicBlockImpl implements SdfDelay {
	private SdfModelOfComputation myMoC_ = null;
	
	/** Array of delayed data samples (circular buffer) */
	private Object dataSamples_[] = null;
	private Object dataSamples_tmp_[] = null;
	/** Input index into the circular buffer */
	private int inputIndex_ = 0;
	private int inputIndex_tmp_ = 0;
	/** Output index from the circular buffer */
	private int outputIndex_ = 0;
	private int outputIndex_tmp_ = 0;
	/** Has data already been produced on the outputs ? */
	private int dataProduced_ = 0;
	private int dataProduced_tmp_ = 0;	
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SdfDelayImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdflibPackage.Literals.SDF_DELAY;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	@SuppressWarnings("unchecked")
	public EList<Object> getInitialData() {
		//The initial data list is stored as a parameter of the block
		Object value = getValueOfParameter("initialData");
		if (value == null) {
			// No initial data -> set it to 0
			EList<Object> l = new BasicEList<Object>();
			l.add(new Integer(0));
			return l;
		} else if (value instanceof EList<?>) {
			// We already have an EList -> OK
			return (EList<Object>)value;
		} else if (value instanceof Collection<?>) {
			// We have a collection -> build an EList using its elements
			return new BasicEList<Object>((Collection<Object>)value);
		} else if (value instanceof Object[]) {
			// We have an array -> build an EList using its elements
			return new BasicEList<Object>(Arrays.asList((Object[])value));
		} else {
			// We have a raw object -> build an EList containing this object
			EList<Object> l = new BasicEList<Object>();
			l.add(value);
			return l;
		}
	}

	/**
	 * Get a sample of data from the delay buffer
	 * @generated NOT
	 */
	private Object getData() {
		Object value = dataSamples_tmp_[outputIndex_tmp_];
		outputIndex_tmp_ = (outputIndex_tmp_ + 1) % dataSamples_tmp_.length;
		dataProduced_tmp_ ++;
		return value;
	}
	
	/**
	 * Put a sample of data in the delay buffer
	 * @generated NOT
	 */
	private void putData(Object value) {
		dataSamples_tmp_[inputIndex_tmp_] = value;
		inputIndex_tmp_ = (inputIndex_tmp_ + 1) % dataSamples_tmp_.length;
		dataProduced_tmp_ --;
	}
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc) {
		if (dataProduced_tmp_ == 0) {
			// If we have not yet produced an output sample since 
			// we received our last input, it is time to do so
			Object value = getData();
			for (Pin p : getPinsOut()) {
				p.addToken(myMoC_.createDataToken(value));
			}
		}
		Pin in = getPinsIn().get(0); 
		if (in.hasTokens()) {
			// If we have input data, take it and store it in the delay buffer
			Object value = myMoC_.getValueOfToken(in.getToken(true));
			putData(value);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		if (moc instanceof SdfModelOfComputation) {
			myMoC_ = (SdfModelOfComputation)moc;
		} else {
			throw new Error(getClass().getName()+".setup(): MoC is not SDF");
		}
		super.setup(myMoC_, initialTime);
		if (getPinsIn().size() != 1) {
			throw new Error(getClass().getName()+".setup(): I should have EXACTLY ONE input pin");
		}
		if (myMoC_.getPinRate(getPinsIn().get(0)) != 1) {
			throw new Error(getClass().getName()+".setup(): SDF Delays work only at rate 1");
		}
		if (getPinsOut().size() == 0) {
			throw new Error(getClass().getName()+".setup(): I should have at least one output pin");
		}
		for (Pin p : getPinsOut()) {
			// The output pins of a delay do not depend instantaneously on its inputs
			myMoC_.setPinInstantaneousDependency(p, false);
			if (myMoC_.getPinRate(p) != 1) {
				throw new Error(getClass().getName()+".setup(): SDF Delays work only at rate 1");
			}
		}
		dataSamples_ = getInitialData().toArray();
		inputIndex_ = 0;
		outputIndex_ = 0;
		dataProduced_ = 0;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfSnapshot(ModelOfComputation moc) {
		super.endOfSnapshot(moc);
		// Store temporary state into permanent state
		dataSamples_ = dataSamples_tmp_;
		dataSamples_tmp_ = null;
		inputIndex_ = inputIndex_tmp_;
		outputIndex_ = outputIndex_tmp_;
		dataProduced_ = dataProduced_tmp_;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void reset(ModelOfComputation moc, InstantValue time) {
		super.reset(moc, time);
		// Set temporary state to permanent state
		dataSamples_tmp_ = new Object[dataSamples_.length];
		for (int i = 0; i < dataSamples_.length; i++) {
			dataSamples_tmp_[i] = dataSamples_[i];
		}
		inputIndex_tmp_ = inputIndex_;
		outputIndex_tmp_ = outputIndex_;
		dataProduced_tmp_ = dataProduced_;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SdflibPackage.SDF_DELAY__INITIAL_DATA:
				return getInitialData();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SdflibPackage.SDF_DELAY__INITIAL_DATA:
				getInitialData().clear();
				getInitialData().addAll((Collection<? extends Object>)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_DELAY__INITIAL_DATA:
				getInitialData().clear();
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_DELAY__INITIAL_DATA:
				return !getInitialData().isEmpty();
		}
		return super.eIsSet(featureID);
	}

} //SdfDelayImpl
