/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfDuplicatorImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.impl;

import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfDuplicator;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Sdf Duplicator</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SdfDuplicatorImpl extends AtomicBlockImpl implements SdfDuplicator {
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SdfDuplicatorImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdflibPackage.Literals.SDF_DUPLICATOR;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc_) {
		SdfModelOfComputation moc;
		try {
			moc = (SdfModelOfComputation)moc_;
		}
		catch (ClassCastException ex) {
			throw new Error("SdfDuplicator.update : moc is not SDF");
		}
		super.update(moc);
		
		Pin input = getPinsIn().get(0);
		if (! input.hasTokens()) {
			throw new Error("SdfDuplicator.update : no event on input token");
		}
		int nbtoconsume = moc.getPinRate(input);
		for(int i = 0; i < nbtoconsume ; i++){
			Token evIn = input.getToken(true);
			Object valueEvIn = moc.getValueOfToken(evIn);
			for (Pin p : getPinsOut()) {
				Token evOut = moc.createDataToken(valueEvIn);
				p.addToken(evOut);
				//TODO keep the remove of the consumed token from the value table ?
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc_, InstantValue initialTime) {
		SdfModelOfComputation moc;
		try {
			moc = (SdfModelOfComputation)moc_;
		}
		catch (ClassCastException ex) {
			throw new Error("SdfDuplicator.setup : moc is not SDF");
		}
		super.setup(moc, initialTime);
		
		//Check that the block has one and only one input pin and at least one output pin
		if (getPinsIn().size() != 1) {
			throw new Error("SdfDuplicator.setup : one and only one pin is expected");
		}
		if (getPinsOut().size() == 0) {
			throw new Error("SdfDuplicator.setup : at least one output pin is expected");
		}
		
		//Check that the input pin has a specified token rate
		for(Pin p : getPinsIn()){
			moc.getPinRate(p); //Checks that a token rate exists for the pin
		}

		//Check that each output pin has a specified token rate
		//and that this rate is equal to the token rate on the input pin
		for(Pin p : getPinsOut()){
			int rate = moc.getPinRate(p); //Checks that a token rate exists for the pin
			if(rate != moc.getPinRate(getPinsIn().get(0))){
				throw new Error("SdfDuplicator.setup :the token rate of the output pin "+p.getName()+" must be equal to the token rate of the input pin "+getPinsIn().get(0).getName());
			}
		}

	}

} //SdfDuplicatorImpl
