/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfMultiplierImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.impl;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;

import fr.supelec.modhelx.core.impl.AtomicBlockImpl;

import fr.supelec.modhelx.core.time.InstantValue;

import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfMultiplier;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;
import fr.supelec.modhelx.util.Numeric;

import org.eclipse.emf.ecore.EClass;

/**
 * <!-- begin-user-doc -->
 * <p>An implementation of the model object '<em><b>Sdf Multiplier</b></em>'.</p>
 * <p>The Multiplier computes the product of the values from its input pins
 * and outputs this product on each output pin.</p>
 * <p>Each input pin has an exponent ("Exponent" property) to which the
 * data from this input pin are raised.</p>
 * <p>If the rate of an input pin is n, n tokens will be read from
 * this pin and multipled to build the value associated to this pin.</p>
 * <p>If the rate of an output pin is n, n tokens having the value of
 * the product will be produced on it.</p>
 * <p>If no exponent is associate to an input pin, a default exponent
 * with value 1 will be used. Negative exponents are allowed (division),
 * as well as fractional exponents (roots).</p>
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SdfMultiplierImpl extends AtomicBlockImpl implements SdfMultiplier {
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SdfMultiplierImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdflibPackage.Literals.SDF_MULTIPLIER;
	}

	private final String EXPONENT_PARAM_NAME = "Exponent";
	private SdfModelOfComputation myMoC_ = null;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc) {
		double output = 1.0;
		for (Pin p : getPinsIn()) {
			double exp = getExponent(p);
			int rate = myMoC_.getPinRate(p);
			double pval = 1.0;
			for (int i = 0; i < rate; i++) {
				// Weird: token is removed twice ? Once by getToken and once by getValueOfDataToken ?
				pval *= Numeric.doubleValueOfObject(myMoC_.getValueOfToken(p.getToken(true)));
			}
			output *= Math.pow(pval, exp);
		}
		for (Pin p : getPinsOut()) {
			int rate = myMoC_.getPinRate(p);
			for (int i = 0; i < rate; i++) {
				p.addToken(myMoC_.createDataToken(output));
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		super.setup(moc, initialTime);
		
		if (moc instanceof SdfModelOfComputation) {
			myMoC_ = (SdfModelOfComputation)moc;
		} else {
			throw new Error(getClass().getName() + ".setup : moc is not SDF");
		}
		for (Pin p : getPinsIn()) {
			if (!hasExponent(p)) {
				setExponent(p, 1);
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setExponent(Pin pin, double exponent) {
		pin.setValueOfParameter(EXPONENT_PARAM_NAME, exponent);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public double getExponent(Pin pin) {
		if (hasExponent(pin)) {
			return (Double)(pin.getValueOfParameter(EXPONENT_PARAM_NAME));
		} else {
			throw new Error("Pin " + pin + " does not have a parameter " + EXPONENT_PARAM_NAME);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean hasExponent(Pin pin) {
		return (pin.getValueOfParameter(EXPONENT_PARAM_NAME) != null);
	}

} //SdfMultiplierImpl
