/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfPrinterImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.impl;

import java.io.PrintStream;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfPrinter;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Sdf Printer</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.sdflib.impl.SdfPrinterImpl#getPrintStream <em>Print Stream</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class SdfPrinterImpl extends AtomicBlockImpl implements SdfPrinter {
	/**
	 * The default value of the '{@link #getPrintStream() <em>Print Stream</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getPrintStream()
	 * @generated
	 * @ordered
	 */
	protected static final Object PRINT_STREAM_EDEFAULT = null;

	/**
	 * The cached value of the '{@link #getPrintStream() <em>Print Stream</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getPrintStream()
	 * @generated
	 * @ordered
	 */
	protected Object printStream = PRINT_STREAM_EDEFAULT;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SdfPrinterImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdflibPackage.Literals.SDF_PRINTER;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Object getPrintStream() {
		return printStream;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setPrintStream(Object newPrintStream) {
		if (! (newPrintStream instanceof PrintStream)) {
			throw new Error("SdfPrinter.setPrintStream : the argument must be a PrintStream");
		}
		Object oldPrintStream = printStream;
		printStream = newPrintStream;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, SdflibPackage.SDF_PRINTER__PRINT_STREAM, oldPrintStream, printStream));
	}

	private SdfModelOfComputation getMoc(ModelOfComputation moc) {
		try {
			SdfModelOfComputation m = (SdfModelOfComputation)moc;
			return m;
		}
		catch (ClassCastException ex) {
			throw new Error("SdfPrinter : moc is not SDF");
		}
	}
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc_) {
		SdfModelOfComputation moc = getMoc(moc_);
		super.update(moc);
		
		if (getPrintStream() == null) return;
		PrintStream out = (PrintStream) getPrintStream();
		
		//For each input pin, we print the data tokens received
		for(Pin pIn : this.getPinsIn()){
			int nbreceived = pIn.getStoredTokens().size();
			int nbtoconsume;
			//But before we check that a parameter specifying the token rate of the input pin exists
			nbtoconsume = moc.getPinRate(pIn);
			//Then we check that the input pin has received the correct number of tokens
			if(nbreceived < nbtoconsume){
				throw new Error("SdfPrinter.update : not enough tokens on the input pin "+pIn.getName());
			}else{
				//Then we print the data tokens
				int cpt = nbtoconsume;
				while(cpt > 0 && pIn.hasTokens()){
					Token ev = pIn.getToken(true);
					out.println("Data token received on " + pIn.getName()
					+ " with value " + moc.getValueOfToken(ev));
					cpt--;
				}
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc_, InstantValue initialTime) {
		SdfModelOfComputation moc = getMoc(moc_);
		super.setup(moc, initialTime);
		
		//Check that the printer has a printstream
		if(getPrintStream()==null){
			throw new Error("SdfPrinter.setup : must have a printStream");
		}
		
		//Check that the printer has at least one input pin but no output pin
		if (getPinsIn().size() == 0) {
			throw new Error("SdfPrinter.setup : input pins are expected");
		}
		if (getPinsOut().size() != 0) {
			throw new Error("SdfPrinter.setup : output pins are not expected");
		}
		
		//Check that each output pin has a specified number of produced tokens
		for(Pin p : getPinsIn()){
			moc.getPinRate(p);
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SdflibPackage.SDF_PRINTER__PRINT_STREAM:
				return getPrintStream();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SdflibPackage.SDF_PRINTER__PRINT_STREAM:
				setPrintStream(newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_PRINTER__PRINT_STREAM:
				setPrintStream(PRINT_STREAM_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_PRINTER__PRINT_STREAM:
				return PRINT_STREAM_EDEFAULT == null ? printStream != null : !PRINT_STREAM_EDEFAULT.equals(printStream);
		}
		return super.eIsSet(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public String toString() {
		if (eIsProxy()) return super.toString();

		StringBuffer result = new StringBuffer(super.toString());
		result.append(" (printStream: ");
		result.append(printStream);
		result.append(')');
		return result.toString();
	}

} //SdfPrinterImpl
