/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfRampImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.impl;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;

import fr.supelec.modhelx.core.impl.AtomicBlockImpl;

import fr.supelec.modhelx.core.time.InstantValue;

import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfRamp;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;

import org.eclipse.emf.ecore.EClass;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Sdf Ramp</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.sdflib.impl.SdfRampImpl#getInitialValue <em>Initial Value</em>}</li>
 *   <li>{@link fr.supelec.modhelx.mocs.sdflib.impl.SdfRampImpl#getStep <em>Step</em>}</li>
 *   <li>{@link fr.supelec.modhelx.mocs.sdflib.impl.SdfRampImpl#getModulo <em>Modulo</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class SdfRampImpl extends AtomicBlockImpl implements SdfRamp {
	private SdfModelOfComputation myMoC_;
	
	private double init_ = 0;
	private double step_ = 1;
	private double modulo_ = 0;
	private double value_ = 0;
	private double value_tmp_ = 0;
	
	/**
	 * The default value of the '{@link #getInitialValue() <em>Initial Value</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getInitialValue()
	 * @generated
	 * @ordered
	 */
	protected static final Object INITIAL_VALUE_EDEFAULT = null;

	/**
	 * The default value of the '{@link #getStep() <em>Step</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getStep()
	 * @generated
	 * @ordered
	 */
	protected static final Object STEP_EDEFAULT = null;

	/**
	 * The default value of the '{@link #getModulo() <em>Modulo</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getModulo()
	 * @generated
	 * @ordered
	 */
	protected static final Object MODULO_EDEFAULT = null;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SdfRampImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdflibPackage.Literals.SDF_RAMP;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Object getInitialValue() {
		return getValueOfParameter("initialValue");
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setInitialValue(Object newInitialValue) {
		setValueOfParameter("initialValue", newInitialValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Object getStep() {
		return getValueOfParameter("step");
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setStep(Object newStep) {
		setValueOfParameter("step", newStep);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Object getModulo() {
		return getValueOfParameter("modulo");
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setModulo(Object newModulo) {
		setValueOfParameter("modulo", newModulo);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc) {
		for (Pin p : getPinsOut()) {
			for (int i = 0; i < myMoC_.getPinRate(p); i++) {
				p.addToken(myMoC_.createDataToken(value_tmp_));
			}
		}
		value_tmp_ += step_;
		if (modulo_ > 0) {
			value_tmp_ %= modulo_;
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		if (moc instanceof SdfModelOfComputation) {
			myMoC_ = (SdfModelOfComputation) moc;
			super.setup(myMoC_, initialTime);
		} else {
			throw new Error(getClass().getName()+".setup(): MoC is not SDF)");
		}
		Object tmp = getInitialValue();
		if (tmp == null) {
			init_ = 0;
		} else if (tmp instanceof Number) {
			init_ = ((Number) tmp).doubleValue();
		} else {
			throw new Error(getClass().getName()+".setup(): initialValue is not a number)");
		}
		tmp = getStep();
		if (tmp == null) {
			step_ = 1;
		} else if (tmp instanceof Number) {
			step_ = ((Number) tmp).doubleValue();
		} else {
			throw new Error(getClass().getName()+".setup(): step is not a number)");
		}
		tmp = getModulo();
		if (tmp == null) {
			modulo_ = 0;
		} else if (tmp instanceof Number) {
			modulo_ = ((Number) tmp).doubleValue();
		} else {
			throw new Error(getClass().getName()+".setup(): modulo is not a number)");
		}
		value_ = init_;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void reset(ModelOfComputation moc, InstantValue time) {
		value_tmp_ = value_;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfSnapshot(ModelOfComputation moc) {
		value_ = value_tmp_;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case SdflibPackage.SDF_RAMP__INITIAL_VALUE:
				return getInitialValue();
			case SdflibPackage.SDF_RAMP__STEP:
				return getStep();
			case SdflibPackage.SDF_RAMP__MODULO:
				return getModulo();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case SdflibPackage.SDF_RAMP__INITIAL_VALUE:
				setInitialValue(newValue);
				return;
			case SdflibPackage.SDF_RAMP__STEP:
				setStep(newValue);
				return;
			case SdflibPackage.SDF_RAMP__MODULO:
				setModulo(newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_RAMP__INITIAL_VALUE:
				setInitialValue(INITIAL_VALUE_EDEFAULT);
				return;
			case SdflibPackage.SDF_RAMP__STEP:
				setStep(STEP_EDEFAULT);
				return;
			case SdflibPackage.SDF_RAMP__MODULO:
				setModulo(MODULO_EDEFAULT);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case SdflibPackage.SDF_RAMP__INITIAL_VALUE:
				return INITIAL_VALUE_EDEFAULT == null ? getInitialValue() != null : !INITIAL_VALUE_EDEFAULT.equals(getInitialValue());
			case SdflibPackage.SDF_RAMP__STEP:
				return STEP_EDEFAULT == null ? getStep() != null : !STEP_EDEFAULT.equals(getStep());
			case SdflibPackage.SDF_RAMP__MODULO:
				return MODULO_EDEFAULT == null ? getModulo() != null : !MODULO_EDEFAULT.equals(getModulo());
		}
		return super.eIsSet(featureID);
	}

} //SdfRampImpl
