/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdfThresholdImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.impl;

import org.eclipse.emf.ecore.EClass;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.AtomicBlockImpl;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfThreshold;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;
import fr.supelec.modhelx.util.Numeric;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Sdf Threshold</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SdfThresholdImpl extends AtomicBlockImpl implements SdfThreshold {
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SdfThresholdImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return SdflibPackage.Literals.SDF_THRESHOLD;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(ModelOfComputation moc) {
		SdfModelOfComputation sdfMoc = (SdfModelOfComputation) moc;
		
		Pin pIn = getPinsIn().get(0);
		
		for(int tokenCtr = 0; tokenCtr < sdfMoc.getPinRate(pIn); tokenCtr++) {
			Token tIn = pIn.getToken(true);
			double valIn = Numeric.doubleValueOfObject(sdfMoc.getValueOfToken(tIn));
			
			// possibly produce an output token on each output pin
			for(Pin pOut : getPinsOut()) {
				Object paramTGT = pOut.getValueOfParameter(PARAM_THRESHOLD_GT);
				Object paramTGEQ = pOut.getValueOfParameter(PARAM_THRESHOLD_GEQ);
				Object paramTLEQ = pOut.getValueOfParameter(PARAM_THRESHOLD_LEQ);
				Object paramTLT = pOut.getValueOfParameter(PARAM_THRESHOLD_LT);
				
				Token tOut = sdfMoc.createDataToken(
						(paramTGT != null && valIn > Numeric.doubleValueOfObject(paramTGT)) ||
						(paramTGEQ != null && valIn >= Numeric.doubleValueOfObject(paramTGEQ)) ||
						(paramTLEQ != null && valIn <= Numeric.doubleValueOfObject(paramTLEQ)) ||
						(paramTLT != null && valIn < Numeric.doubleValueOfObject(paramTLT))
					? 1 : 0);
				
				pOut.addToken(tOut);
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		if(getPinsIn().size() != 1) throw new Error("SdfAccumulator: must have 1 input.");
		if(getPinsOut().size() < 1) throw new Error("SdfAccumulator: wrong number of outputs.");
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setThresholdGT(Pin pin, double threshold) {
		pin.setValueOfParameter(PARAM_THRESHOLD_GT, threshold);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setThresholdGEQ(Pin pin, double threshold) {
		pin.setValueOfParameter(PARAM_THRESHOLD_GEQ, threshold);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setThresholdLEQ(Pin pin, double threshold) {
		pin.setValueOfParameter(PARAM_THRESHOLD_LEQ, threshold);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setThresholdLT(Pin pin, double threshold) {
		pin.setValueOfParameter(PARAM_THRESHOLD_LT, threshold);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public double getThresholdGT(Pin pin) {
		Object param = pin.getValueOfParameter(PARAM_THRESHOLD_GT);
		return Numeric.doubleValueOfObject(param);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public double getThresholdGEQ(Pin pin) {
		Object param = pin.getValueOfParameter(PARAM_THRESHOLD_GEQ);
		return Numeric.doubleValueOfObject(param);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public double getThresholdLT(Pin pin) {
		Object param = pin.getValueOfParameter(PARAM_THRESHOLD_LT);
		return Numeric.doubleValueOfObject(param);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public double getThresholdLEQ(Pin pin) {
		Object param = pin.getValueOfParameter(PARAM_THRESHOLD_LEQ);
		return Numeric.doubleValueOfObject(param);
	}

} //SdfThresholdImpl
