/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SdflibSwitch.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.sdflib.util;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

import fr.supelec.modhelx.core.AtomicBlock;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.ModHelXObject;
import fr.supelec.modhelx.mocs.sdflib.*;
import fr.supelec.modhelx.mocs.sdflib.SdfAdder;
import fr.supelec.modhelx.mocs.sdflib.SdfConstantEmitter;
import fr.supelec.modhelx.mocs.sdflib.SdfDuplicator;
import fr.supelec.modhelx.mocs.sdflib.SdfPrinter;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;

/**
 * <!-- begin-user-doc -->
 * The <b>Switch</b> for the model's inheritance hierarchy.
 * It supports the call {@link #doSwitch(EObject) doSwitch(object)}
 * to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object
 * and proceeding up the inheritance hierarchy
 * until a non-null result is returned,
 * which is the result of the switch.
 * <!-- end-user-doc -->
 * @see fr.supelec.modhelx.mocs.sdflib.SdflibPackage
 * @generated
 */
public class SdflibSwitch<T> {
	/**
	 * The cached model package
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static SdflibPackage modelPackage;

	/**
	 * Creates an instance of the switch.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public SdflibSwitch() {
		if (modelPackage == null) {
			modelPackage = SdflibPackage.eINSTANCE;
		}
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	public T doSwitch(EObject theEObject) {
		return doSwitch(theEObject.eClass(), theEObject);
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected T doSwitch(EClass theEClass, EObject theEObject) {
		if (theEClass.eContainer() == modelPackage) {
			return doSwitch(theEClass.getClassifierID(), theEObject);
		}
		else {
			List<EClass> eSuperTypes = theEClass.getESuperTypes();
			return
				eSuperTypes.isEmpty() ?
					defaultCase(theEObject) :
					doSwitch(eSuperTypes.get(0), theEObject);
		}
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected T doSwitch(int classifierID, EObject theEObject) {
		switch (classifierID) {
			case SdflibPackage.SDF_CONSTANT_EMITTER: {
				SdfConstantEmitter sdfConstantEmitter = (SdfConstantEmitter)theEObject;
				T result = caseSdfConstantEmitter(sdfConstantEmitter);
				if (result == null) result = caseAtomicBlock(sdfConstantEmitter);
				if (result == null) result = caseBlock(sdfConstantEmitter);
				if (result == null) result = caseModHelXObject(sdfConstantEmitter);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case SdflibPackage.SDF_DUPLICATOR: {
				SdfDuplicator sdfDuplicator = (SdfDuplicator)theEObject;
				T result = caseSdfDuplicator(sdfDuplicator);
				if (result == null) result = caseAtomicBlock(sdfDuplicator);
				if (result == null) result = caseBlock(sdfDuplicator);
				if (result == null) result = caseModHelXObject(sdfDuplicator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case SdflibPackage.SDF_ADDER: {
				SdfAdder sdfAdder = (SdfAdder)theEObject;
				T result = caseSdfAdder(sdfAdder);
				if (result == null) result = caseAtomicBlock(sdfAdder);
				if (result == null) result = caseBlock(sdfAdder);
				if (result == null) result = caseModHelXObject(sdfAdder);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case SdflibPackage.SDF_PRINTER: {
				SdfPrinter sdfPrinter = (SdfPrinter)theEObject;
				T result = caseSdfPrinter(sdfPrinter);
				if (result == null) result = caseAtomicBlock(sdfPrinter);
				if (result == null) result = caseBlock(sdfPrinter);
				if (result == null) result = caseModHelXObject(sdfPrinter);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case SdflibPackage.SDF_ACCUMULATOR: {
				SdfAccumulator sdfAccumulator = (SdfAccumulator)theEObject;
				T result = caseSdfAccumulator(sdfAccumulator);
				if (result == null) result = caseAtomicBlock(sdfAccumulator);
				if (result == null) result = caseBlock(sdfAccumulator);
				if (result == null) result = caseModHelXObject(sdfAccumulator);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case SdflibPackage.SDF_THRESHOLD: {
				SdfThreshold sdfThreshold = (SdfThreshold)theEObject;
				T result = caseSdfThreshold(sdfThreshold);
				if (result == null) result = caseAtomicBlock(sdfThreshold);
				if (result == null) result = caseBlock(sdfThreshold);
				if (result == null) result = caseModHelXObject(sdfThreshold);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case SdflibPackage.SDF_LIMITER: {
				SdfLimiter sdfLimiter = (SdfLimiter)theEObject;
				T result = caseSdfLimiter(sdfLimiter);
				if (result == null) result = caseAtomicBlock(sdfLimiter);
				if (result == null) result = caseBlock(sdfLimiter);
				if (result == null) result = caseModHelXObject(sdfLimiter);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case SdflibPackage.SDF_MULTIPLIER: {
				SdfMultiplier sdfMultiplier = (SdfMultiplier)theEObject;
				T result = caseSdfMultiplier(sdfMultiplier);
				if (result == null) result = caseAtomicBlock(sdfMultiplier);
				if (result == null) result = caseBlock(sdfMultiplier);
				if (result == null) result = caseModHelXObject(sdfMultiplier);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case SdflibPackage.SDF_DELAY: {
				SdfDelay sdfDelay = (SdfDelay)theEObject;
				T result = caseSdfDelay(sdfDelay);
				if (result == null) result = caseAtomicBlock(sdfDelay);
				if (result == null) result = caseBlock(sdfDelay);
				if (result == null) result = caseModHelXObject(sdfDelay);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case SdflibPackage.SDF_RAMP: {
				SdfRamp sdfRamp = (SdfRamp)theEObject;
				T result = caseSdfRamp(sdfRamp);
				if (result == null) result = caseAtomicBlock(sdfRamp);
				if (result == null) result = caseBlock(sdfRamp);
				if (result == null) result = caseModHelXObject(sdfRamp);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			default: return defaultCase(theEObject);
		}
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Constant Emitter</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Constant Emitter</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfConstantEmitter(SdfConstantEmitter object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Duplicator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Duplicator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfDuplicator(SdfDuplicator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Adder</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Adder</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfAdder(SdfAdder object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Printer</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Printer</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfPrinter(SdfPrinter object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Accumulator</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Accumulator</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfAccumulator(SdfAccumulator object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Threshold</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Threshold</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfThreshold(SdfThreshold object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Limiter</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Limiter</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfLimiter(SdfLimiter object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Multiplier</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Multiplier</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfMultiplier(SdfMultiplier object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Delay</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Delay</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfDelay(SdfDelay object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sdf Ramp</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sdf Ramp</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSdfRamp(SdfRamp object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Mod Hel XObject</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Mod Hel XObject</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModHelXObject(ModHelXObject object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Block</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Block</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseBlock(Block object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Atomic Block</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Atomic Block</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseAtomicBlock(AtomicBlock object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>EObject</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch, but this is the last case anyway.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>EObject</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject)
	 * @generated
	 */
	public T defaultCase(EObject object) {
		return null;
	}

} //SdflibSwitch
