/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: SimpleAdapterFactory.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.simple.util;


import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.notify.impl.AdapterFactoryImpl;
import org.eclipse.emf.ecore.EObject;

import fr.supelec.modhelx.core.AtomicBlock;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.ModHelXObject;
import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.mocs.simple.*;
import fr.supelec.modhelx.mocs.simple.SimpleConsumer;
import fr.supelec.modhelx.mocs.simple.SimpleModelOfComputation;
import fr.supelec.modhelx.mocs.simple.SimplePackage;
import fr.supelec.modhelx.mocs.simple.SimpleProducer;

/**
 * <!-- begin-user-doc -->
 * The <b>Adapter Factory</b> for the model.
 * It provides an adapter <code>createXXX</code> method for each class of the model.
 * <!-- end-user-doc -->
 * @see fr.supelec.modhelx.mocs.simple.SimplePackage
 * @generated
 */
public class SimpleAdapterFactory extends AdapterFactoryImpl {
	/**
	 * The cached model package.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static SimplePackage modelPackage;

	/**
	 * Creates an instance of the adapter factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public SimpleAdapterFactory() {
		if (modelPackage == null) {
			modelPackage = SimplePackage.eINSTANCE;
		}
	}

	/**
	 * Returns whether this factory is applicable for the type of the object.
	 * <!-- begin-user-doc -->
	 * This implementation returns <code>true</code> if the object is either the model's package or is an instance object of the model.
	 * <!-- end-user-doc -->
	 * @return whether this factory is applicable for the type of the object.
	 * @generated
	 */
	@Override
	public boolean isFactoryForType(Object object) {
		if (object == modelPackage) {
			return true;
		}
		if (object instanceof EObject) {
			return ((EObject)object).eClass().getEPackage() == modelPackage;
		}
		return false;
	}

	/**
	 * The switch that delegates to the <code>createXXX</code> methods.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SimpleSwitch<Adapter> modelSwitch =
		new SimpleSwitch<Adapter>() {
			@Override
			public Adapter caseSimpleModelOfComputation(SimpleModelOfComputation object) {
				return createSimpleModelOfComputationAdapter();
			}
			@Override
			public Adapter caseSimpleProducer(SimpleProducer object) {
				return createSimpleProducerAdapter();
			}
			@Override
			public Adapter caseSimpleConsumer(SimpleConsumer object) {
				return createSimpleConsumerAdapter();
			}
			@Override
			public Adapter caseModHelXObject(ModHelXObject object) {
				return createModHelXObjectAdapter();
			}
			@Override
			public Adapter caseModelOfComputation(ModelOfComputation object) {
				return createModelOfComputationAdapter();
			}
			@Override
			public Adapter caseBlock(Block object) {
				return createBlockAdapter();
			}
			@Override
			public Adapter caseAtomicBlock(AtomicBlock object) {
				return createAtomicBlockAdapter();
			}
			@Override
			public Adapter defaultCase(EObject object) {
				return createEObjectAdapter();
			}
		};

	/**
	 * Creates an adapter for the <code>target</code>.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param target the object to adapt.
	 * @return the adapter for the <code>target</code>.
	 * @generated
	 */
	@Override
	public Adapter createAdapter(Notifier target) {
		return modelSwitch.doSwitch((EObject)target);
	}


	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.simple.SimpleModelOfComputation <em>Model Of Computation</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.simple.SimpleModelOfComputation
	 * @generated
	 */
	public Adapter createSimpleModelOfComputationAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.simple.SimpleProducer <em>Producer</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.simple.SimpleProducer
	 * @generated
	 */
	public Adapter createSimpleProducerAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.mocs.simple.SimpleConsumer <em>Consumer</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.mocs.simple.SimpleConsumer
	 * @generated
	 */
	public Adapter createSimpleConsumerAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.ModHelXObject <em>Mod Hel XObject</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.ModHelXObject
	 * @generated
	 */
	public Adapter createModHelXObjectAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.ModelOfComputation <em>Model Of Computation</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.ModelOfComputation
	 * @generated
	 */
	public Adapter createModelOfComputationAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.Block <em>Block</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.Block
	 * @generated
	 */
	public Adapter createBlockAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link fr.supelec.modhelx.core.AtomicBlock <em>Atomic Block</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see fr.supelec.modhelx.core.AtomicBlock
	 * @generated
	 */
	public Adapter createAtomicBlockAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for the default case.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @generated
	 */
	public Adapter createEObjectAdapter() {
		return null;
	}

} //SimpleAdapterFactory
