/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TFsmModelOfComputationImpl.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.mocs.tfsm.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.WeakHashMap;
import java.util.logging.Level;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.CorePackage;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.impl.ModelOfComputationImpl;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.DurationValue;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.tfsm.TFsmModelOfComputation;
import fr.supelec.modhelx.mocs.tfsm.TfsmPackage;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>TFsm Model Of Computation</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link fr.supelec.modhelx.mocs.tfsm.impl.TFsmModelOfComputationImpl#getClock <em>Clock</em>}</li>
 *   <li>{@link fr.supelec.modhelx.mocs.tfsm.impl.TFsmModelOfComputationImpl#getTimeOfCurrentSnapshot <em>Time Of Current Snapshot</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class TFsmModelOfComputationImpl extends ModelOfComputationImpl implements TFsmModelOfComputation {
	/**
	 * The cached value of the '{@link #getClock() <em>Clock</em>}' reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getClock()
	 * @generated
	 * @ordered
	 */
	protected Clock clock;

	/**
	 * The cached value of the '{@link #getTimeOfCurrentSnapshot() <em>Time Of Current Snapshot</em>}' reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @see #getTimeOfCurrentSnapshot()
	 * @generated
	 * @ordered
	 */
	protected InstantValue timeOfCurrentSnapshot;

	
	/**
	 * Current event to process in the snapshot
	 */
	private Token currentEvent = null;
	
	/**
	 * Current state of the automaton
	 */
	private Block currentState = null;
	
	/**
	 * Saved current state (for reset)
	 */
	private Block initialCurrentState = null;
	
	/**
	 * Transition which has been fired in the snapshot
	 */
	private Block executedTransition = null;
	
	/**
	 * List of the events to process.
	 * The events which are present on the input pins of the model at the beginning
	 * of a snapshot are put in that list.
	 * Each snapshot corresponds to the processing of one event (stored in the "currentEvent" variable
	 * in the start of snapshot).
	 */
	private ArrayList<Token> eventsToProcess = new ArrayList<Token>();
	
	/**
	 * List of the transitions which are fireable in the current snapshot.
	 * A transition is fireable in a snapshot if it is connected to the current state
	 * and is the target of an incoming event.
	 */
	private ArrayList<Block> fireableTransitions = new ArrayList<Block>();
	
	/**
	 * List of the active timed transitions with the deadline of their guard.
	 * A timed transition is active in a snapshot if it is connected to the current state.
	 */
	private Map<Block,InstantValue> activeTimedTransitions = null;

	/**
	 * Saved the list of active timed transitions (for reset)
	 */
	private Map<Block,InstantValue> initialActiveTimedTransitions = null;
	
	/**
	 * A boolean to help identify which step it is (different from FSM because of the timed transitions)
	 */
	private boolean firststep = true;
	
	/**
	 * List of the pins of the transitions which have received an event
	 */
	private ArrayList<Pin> inputsOfFireableTransitionsWithEventReceived = new ArrayList<Pin>();
	
	/**
	 * List of the received events with their destination pin
	 */
	private WeakHashMap<Token, Pin> incomingPinOfEvents = new WeakHashMap<Token, Pin>();
	
	/**
	 * Roles of the different blocks for FSM
	 */
	private final static String
		TFSM_ROLE_PARAM_NAME = "TFSMRole",
		TFSM_ROLE_STATE = "TFSMState",
		TFSM_ROLE_TRANSITION = "TFSMTransition",
		TFSM_ROLE_GUARD = "TFSMGuard",
		TFSM_ROLE_ACTION = "TFSMAction";

	private final static String
		TFSM_FEATURE_PARAM_NAME = "TFSMFeature",
		TFSM_FEATURE_INITIAL_STATE = "TFSMInitialState",
		TFSM_FEATURE_TIMED_TRANSITION = "TFSMTimedTransition";

	private final static String
	TFSM_DEADLINE_PARAM_NAME = "TFSMDeadline";


	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	protected TFsmModelOfComputationImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return TfsmPackage.Literals.TFSM_MODEL_OF_COMPUTATION;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Clock getClock() {
		if (clock != null && clock.eIsProxy()) {
			InternalEObject oldClock = (InternalEObject)clock;
			clock = (Clock)eResolveProxy(oldClock);
			if (clock != oldClock) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, TfsmPackage.TFSM_MODEL_OF_COMPUTATION__CLOCK, oldClock, clock));
			}
		}
		return clock;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public Clock basicGetClock() {
		return clock;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setClock(Clock newClock) {
		Clock oldClock = clock;
		clock = newClock;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, TfsmPackage.TFSM_MODEL_OF_COMPUTATION__CLOCK, oldClock, clock));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public InstantValue getTimeOfCurrentSnapshot() {
		if (timeOfCurrentSnapshot != null && timeOfCurrentSnapshot.eIsProxy()) {
			InternalEObject oldTimeOfCurrentSnapshot = (InternalEObject)timeOfCurrentSnapshot;
			timeOfCurrentSnapshot = (InstantValue)eResolveProxy(oldTimeOfCurrentSnapshot);
			if (timeOfCurrentSnapshot != oldTimeOfCurrentSnapshot) {
				if (eNotificationRequired())
					eNotify(new ENotificationImpl(this, Notification.RESOLVE, TfsmPackage.TFSM_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT, oldTimeOfCurrentSnapshot, timeOfCurrentSnapshot));
			}
		}
		return timeOfCurrentSnapshot;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public InstantValue basicGetTimeOfCurrentSnapshot() {
		return timeOfCurrentSnapshot;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setup(Model model, InstantValue initialTime) {
		super.setup(model, initialTime);
		
		//Initialize the clock with initialTime
		// TODO check if it is the right manner to use initialtime
		timeOfCurrentSnapshot = initialTime;
		Platform.getController().changeMoCInternalData(this, "timeOfCurrentSnapshot", timeOfCurrentSnapshot);
		
		initialCurrentState = null;
		
		//Check that there is one and only one block with initial state role
		for (Block b : model.getStructure().getBlocks()) {
			if (hasInitialStateFeature(b)) {
				if (initialCurrentState != null) {
					throw new Error("Ill-formed model for FSM : more than 1 init block");					
				}
				initialCurrentState = b;
			}
		}
		
		if (initialCurrentState == null) {
			throw new Error("Ill-formed model for FSM : no init block");					
		}
		
		// determine the timed transitions going out the initial current state
		initialActiveTimedTransitions = determineTimedTransitions(initialCurrentState);

		//TODO Perform other checks on the structure
		
		Platform.getLog().info("**** current state = " +initialCurrentState);

	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void startOfSnapshot(Model model) {
		super.startOfSnapshot(model);

		//Get all the events which are present on the input pins of the model
		// and store them in the list, along with their source pin
		for (Pin pinIn : model.getStructure().getPinsIn()) {
			if (pinIn.hasTokens()) {
				Token event = pinIn.getToken(true);
				eventsToProcess.add(event);
				incomingPinOfEvents.put(event, pinIn);
			}
		}
		
		// set currentState to null, so that we can detect if the reset/update was done or not
		currentState = null;
		
		Platform.getLog().info("**** eventsToProcess = " +eventsToProcess);

	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfSnapshot(Model model) {
		super.endOfSnapshot(model);
		
		//Reinitialize the variables for the next snapshot
		executedTransition = null;

		//Put the events produced during the snapshot on the output pins of the model
		for (Block b : model.getStructure().getBlocks()) {
			for (Pin pinOut : b.getPinsOut()) {
				while (pinOut.hasTokens()) {
					Token t = pinOut.getToken(true);
					for (Relation r : pinOut.getOutgoingRelations()) {
						r.getTarget().addToken(t);
					}
				}
			}
		}
		
		// If the snapshot has been done (computeSnapshot), then
		// the first event of the list has been processed in the current snapshot
		// so it has to be removed from the list of the events to process
		if(eventsToProcess.size() > 0)
			eventsToProcess.remove(0);
		
		// Initialize the current state for the next snapshot
		if(currentState != null)
			initialCurrentState = currentState;
		
		initialActiveTimedTransitions = activeTimedTransitions;
		
		Platform.getLog().info("**** endOfSnapshot : new current state is " + (initialCurrentState == null ? "null /!\\" : initialCurrentState.getName()));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void update(Model model) { 
		// We don't update every block
		//super.update(model);
		
		Platform.getLog().entering(getClass().getName(), "update", model.getName());
		
		//If in step 1 (= find the transition to fire)
		if(executedTransition==null){
			
			//First we update the inside guards of the timed transitions which have a deadline at the current time
			//TODO CAUTION : therefore, timed transitions have the priority on others. change ?
			boolean timedtransitionfired = false;
			for (Block trans : this.activeTimedTransitions.keySet()) {
				//if the deadline is now and no other timed transitions has been taken
				Platform.getLog().finest("update " + trans + "? timeOfSnapshot=" + timeOfCurrentSnapshot 
						                           + ", transition time=" + activeTimedTransitions.get(trans));
//				if(!timedtransitionfired && this.activeTimedTransitions.get(trans).compareTo(this.timeOfCurrentSnapshot)==0) {
				if (activeTimedTransitions.get(trans).compareTo(timeOfCurrentSnapshot) < 0) {
					// TODO Is this an error? WAS: throw new Error
					System.err.println("Missed transition at " + activeTimedTransitions.get(trans)
							        + " (current snapshot is at " + timeOfCurrentSnapshot);
				}
				if (   ! timedtransitionfired 
						&& (activeTimedTransitions.get(trans).compareTo(timeOfCurrentSnapshot) <= 0)
				) {
					if(activeTimedTransitions.get(trans).compareTo(this.timeOfCurrentSnapshot) < 0) {
						System.err.println("Warning: in TFSM, active timed transition " + trans + " was to fire at " + activeTimedTransitions.get(trans) + " firing at " + timeOfCurrentSnapshot);
					}

					Platform.getLog().finest("yes, will update!");
					//Get the inside guard
					// TODO CAUTION we assume that a timed transition always has ONE inside guard
					// TODO: ChJ -> why isn't this always the case
					
					// we need to iterate over the blocks inside the transition
					// to find the guard
					Block insideGuard = null;
					for(Block b : ((CompositeBlock)trans).getBlocks()) {
						if(hasGuardRole(b)) {
							insideGuard = b;
							break;
						}
					}
					
					if(insideGuard != null){
						//put the activation tokens on it
						for(Pin p : insideGuard.getPinsIn()){
							Token t = p.getIncomingRelations().get(0).getSource().getToken(true);
							//TODO check if an activation is really present (assumed because of the prepropagate)
							p.addToken(t);
						}
						
						//and we update the guard
						insideGuard.update(this);
						Platform.getLog().info("**** updated inside guard = " +insideGuard);

						//And we check if it produces a token 
						//CAUTION : we assume that a guard produces a token simultaneously on all of its output pins
						// therefore we only have to check one of them
						if (insideGuard.getPinsOut().get(0).hasTokens()) {
							if (executedTransition != null) {
								throw new Error("FsmModelOfComputation.update : more than one successfull transition");					
							}
							executedTransition = trans;
							Platform.getLog().info("**** positive guard = " +insideGuard);
							Platform.getLog().info("**** executed transition = " + executedTransition);
						}

					} // else no guard found    TODO: is this possible?
					
				}
			}

			//if there was no timed transition to update, we update transitions which are targets of events
			if (executedTransition==null && currentEvent != null) {
				//For each fireable transition, we check if its inside guard is positive (i.e. produces a token)
				int i = 0;
				for (Block b : fireableTransitions){
					if(!hasTransitionRole(b)) throw new Error("Ill-formed model for FSM : block "+b+" is not a transition");
					
					Platform.getLog().info("**** checked fireableTransition = " +b);
	
					//Get the input pin in the list of input pins which have received an event
					//CAUTION : we assume that there is only one pin per transition and transitions 
					// and that pins are stored in the same order (done in initial schedule)
					Pin input = inputsOfFireableTransitionsWithEventReceived.get(i);
					++i;
	
					//If there is an inside guard in the transition (which is a Composite Block in this case)
					//CAUTION : we assume that there is at least one outgoing relation (checked at the setup)
					if(input.getOutgoingRelations().size()>=1){
						Relation a = input.getOutgoingRelations().get(0);
						Pin dest = a.getTarget();
	
						//We put the token produced on pin of the inside guard to activate it 
						dest.addToken(currentEvent);
	
						//Then we update the inside guard
						Block guard = dest.getInputOf();
						if(!hasGuardRole(guard)) throw new Error("Ill-formed model for FSM : block "+guard+" is not a guard");
						guard.update(this);
						Platform.getLog().info("**** updated inside guard = " +guard);
	
						
						//And we check if it produces a token 
						//CAUTION : we assume that a guard produces a token simultaneously on all of its output pins
						// therefore we only have to check one of them
						if (guard.getPinsOut().get(0).hasTokens()) {
							if (executedTransition != null) {
								throw new Error("FsmModelOfComputation.update : more than one successfull transition");					
							}
							executedTransition = b;
							Platform.getLog().info("**** positive guard = " +guard);
							Platform.getLog().info("**** executed transition = " + executedTransition);
						}
					}else{
						throw new Error("Ill-formed model for FSM : no inside guard in transition "+b);
						//TODO In another version of FSM : take into account the case of a transition without inside guard.
						//Two cases :
						// - automatic transition : taken without incoming event
						// - opaque transition : able to receive events but black box (produces an event if taken)
					}
				}
			}
			
			
			return;
		}
		//Else, if in step 2 (= update the actions associated to the positive guard)
		if (executedTransition != null) {
			Platform.getLog().finer("Now, we're executing the transition " + executedTransition.getName());
			if(executedTransition instanceof CompositeBlock) {
				Platform.getLog().finest("OK, the action is a composite.");
				CompositeBlock exectranscompo = (CompositeBlock)executedTransition;
				for (Block b : exectranscompo.getBlocks()) {
					Platform.getLog().finest("Block of the composite: " + b.getName() + " hasActionRole? " + hasActionRole(b));
					if(hasActionRole(b)){
						b.update(this);
						Platform.getLog().info("**** updated action = " +b);
					}
				}
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void reset(Model model, InstantValue time) {
		super.reset(model, time);
		
		//FSM is passive, it receives its current time from the platform
		// TODO check if it is the right manner to use time
		this.timeOfCurrentSnapshot = time;
		Platform.getController().changeMoCInternalData(this, "timeOfCurrentSnapshot", timeOfCurrentSnapshot);

		//Clear all the variables which are used to know which blocks to update in the current snapshot
		fireableTransitions.clear();
		inputsOfFireableTransitionsWithEventReceived.clear();
		
		firststep = true;
		executedTransition = null;
		
		// Reset the collection of active timed transitions
		// We must make a copy here, since 'initialActiveTimedTransition' needs to be preserved
		activeTimedTransitions = new HashMap<Block, InstantValue>(initialActiveTimedTransitions);
		
		//Reinitialize the current state (saved in the end of snapshot)
		currentState = initialCurrentState;
		
		// Logging
		Platform.getController().changeMoCInternalData(this, "currentState", currentState);
		Platform.getLog().finer("currentState=" + currentState + ", activeTimedTransitions=" + activeTimedTransitions);
		
		//The event to process is the first event of the list
		if(eventsToProcess.size()>0){
			currentEvent = eventsToProcess.get(0);
		}
		
		//We put tokens on the outputs of the current state in order to activate
		// the blocks (transitions, guards) which are connected to it
		for (Pin p : currentState.getPinsOut()) {
			p.addToken(CoreFactory.eINSTANCE.createToken());
			Platform.getLog().finest("   put token on current state pin: " + p);
//			p.addToken(currentEvent);
			//CAUTION : pb if currentEvent is null 
			// Explanation : reset is called on the whole hierarachy of the model, so there may be no event to process at this time
			// This is particularly true on embedded FSM models (i.e. inside interface blocks)
		}
		
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean snapDet(Model model) {
		super.snapDet(model);

		Platform.getLog().finest("snapDet returns " + (executedTransition == null));
		
		// If at the end of step 1 (= find the transition to fire) then executedTransition is not null, meaning that the snapshot is not finished 
		// If at the end of step 2 (= update the associated actions) then the snapshot is finished and executedTransition is null (done in the final schedule)
		return executedTransition == null;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean newSnap(Model model) {
		super.newSnap(model);
		
		//If we have not processed all the events, a snapshot is needed
		if (! eventsToProcess.isEmpty()) {
			Platform.getLog().finest("New snapshot because there are events to process: " + eventsToProcess);
			return true;
		}

		//If we have processed all the events in the list, we have to check if no new event
		// has been received on the input pins of the model
		for (Pin pinIn : model.getStructure().getPinsIn()) {
			if (pinIn.hasTokens()) {
				Platform.getLog().finest("New snapshot because a new event is on pin " + pinIn);
				return true;
			}
		}
		
		//If there are active timed transitions, a snapshot is needed
		
		// newSnap uses initialActiveTimedTransitions because it is called
		// before reset (via collectTimeConstraints) and after endOfSnapshot,
		// so the variable relevant at these instants is
		// initialActiveTimedConstraints

		if(! this.initialActiveTimedTransitions.isEmpty()) {
			Platform.getLog().finest("New snapshot because there are active time transitions: " + initialActiveTimedTransitions); 
			return true;
		}
		// TODO check if it would not be better to ask newsnap at the active timed transitions ?
		
		Platform.getLog().finest("No new snapshot necessary.");
		
		//Else no new snapshot is needed
		return false;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void collectTimeConstraints(Model model, EList<InstantValue> timeConstraints) {
		Platform.getLog().entering(getClass().getName(), "collectTimeConstraints", model.getName());

		// If a new snapshot is needed (either events have to be processed or there are active timed transitions)
		// we search for the minimum date of the constraint to post
		if (newSnap(model)) {

			// A priori, the minimum date for a constraint is the current date (events to process or deadline of an active timed transition now)
			InstantValue minConstraintTime;
			// TODO: that seems to be very strange. So the actual time of the next snapshot is not this one in general?
			
			Platform.getLog().finer("eventsToProcess=" + eventsToProcess + ", initialActiveTimedTransitions=" + initialActiveTimedTransitions);
			
			// If there are no events to process, we check the minimum deadline of the active timed transitions
			
			// Note: like newSnap, collectTimeConstraints must work with
			// '_initial_ActiveTimedTransitions' before it's happening
			// before reset
			
			if(eventsToProcess.isEmpty()) {
				minConstraintTime = initialActiveTimedTransitions.values().iterator().next();
				
				//We loop over the deadlines of the active timed transitions and search for the minimum deadline
				for(InstantValue delta : this.initialActiveTimedTransitions.values()) {
					if(delta.compareTo(minConstraintTime)<0) minConstraintTime = delta;
				}
			} else {
				// if there are events to process, then the time of the next
				// snapshot must be the same as that of the previous one
				minConstraintTime = timeOfCurrentSnapshot;
			}
			
			Platform.getLog().finest("Posting constraint: " + minConstraintTime);
			
			// We post a constraint at the minimum date (either the current date or a dealine of an active timed transition)
			timeConstraints.add(minConstraintTime);
			//TODO check if a simpler solution would not be to post all the constraints and let the platform decide
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void initialSchedule(Model model) {
		super.initialSchedule(model);
		
		Platform.getLog().info("FSM MoC test : initialschedule ");
		// Two steps : 
		// - the first to find the transition to fire (positive guard)
		// - the second to update associated action
		// TODO In another version of FSM : implement more steps (exit action, guard action, enter action)
		
		
		//If in step 1 (= find the transition to fire)
		if(executedTransition == null){
			// TODO: previously, we determined the active transitions here. Anything to do?

		
			//If there is an incoming event
			if (currentEvent != null) {
				//We list all the fireable transitions, i.e. all the blocks which have transition role,
				// are the target of an incoming event AND are connected to the current state
	
				//For all the relations which leave the pin which has received the current event
				for (Relation r : incomingPinOfEvents.get(currentEvent).getOutgoingRelations()) {
					//Get the connected block and check its role
					Block b = r.getTarget().getInputOf();
					//If it is a transition
					if(hasTransitionRole(b)){
						for (Pin p : b.getPinsIn()) {
							for (Relation r2 : p.getIncomingRelations()) {
								//And if it is connected to the current state
								if (r2.getSource().getOutputOf() == currentState) {
									//We add it to the list of the fireable transitions
									fireableTransitions.add(b);
									//And we store its input pin in the list of inputs of fireable transitions
									// which have received an event
									inputsOfFireableTransitionsWithEventReceived.add(r.getTarget());
									break;
								}
							}
						}
					}
				}
				Platform.getLog().info("**** fireableTransitions = " +fireableTransitions);
			}
		}
		return;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void prePropagate(Model model) {
		super.prePropagate(model);
		
		//If in step 1 (= find the fireable transition)
		//if (currentEvent != null) {
		if(executedTransition==null){
			//For all the output pins of the current state
			for (Pin p : currentState.getPinsOut()) {
				//If events have been produced
				if (p.hasTokens()) {
					p.getToken(true);
					//We move them on the connected pins (i.e. pins of transitions)
					for (Relation r : p.getOutgoingRelations()) {
						//CAUTION : this check may be annoying if we allow an action to be the target of an event
						if(!hasTransitionRole(r.getTarget().getInputOf())) throw new Error("Ill-formed model for FSM : block "+r.getTarget().getInputOf()+" is not a transition");
						r.getTarget().addToken(createEvent());
						// TODO: ChJ, 2010-04-09: I fixed a bug here: because of containment, we cannot propagate the same
						// token several times, so we need to create a new token each time. This bug is very likely to
						// impact FSM as well, so FSM should be corrected. Note: this bug appears only on states
						// with multiple outgoing transitions.
						Platform.getLog().finest("Propagated token from current state to " + r.getTarget());
					}
					
					/*
					for(Relation r : p.getOutgoingRelations()) {
						Platform.getLog().finest("After propagation. Tokens on " + r.getTarget() + ": " + r.getTarget().getStoredTokens());
					}
					*/
				}
			}
			
			//For all the fireable transitions
			for (Block b : fireableTransitions) {
				
				Platform.getLog().finest("Examining transition " + b + "...");
				
				for (Pin p : b.getPinsIn()) {
					//If events have been put on the input of a fireable transition
					// and this fireable transition has an inside guard
					
					Platform.getLog().finest("   Input pin " + p + " tokens=" + p.getStoredTokens() + ", outgoing relations=" + p.getOutgoingRelations());
					
					if (p.hasTokens() && p.getOutgoingRelations().size()>=1) {
						//We move the events inside the transition on the input of the connected guards
						Token t = p.getToken(true);
						for (Relation r : p.getOutgoingRelations()) {
							//CAUTION : this check may be annoying if we allow an action to be the target of an event
							if(!hasGuardRole(r.getTarget().getInputOf())) throw new Error("Ill-formed model for TFSM : block "+r.getTarget().getInputOf()+" is not a guard");
							r.getTarget().addToken(t);
							
							Platform.getLog().finest("Token added from current state to " + r.getTarget() + " of " + r.getTarget().getInputOf());
						}
					}
				}
			}
			
			return;
		}
		
		//Else, if in step 2 (= update the associated actions) 
		if (executedTransition != null) {
			//If the transition has inside guards and actions, 
			if(executedTransition.eClass().equals(CorePackage.eINSTANCE.getCompositeBlock())){
				CompositeBlock exectranscompo = (CompositeBlock)executedTransition;
				for (Block b : exectranscompo.getBlocks()) {
					for (Pin p : b.getPinsOut()) {
						while (p.hasTokens()) {
							//We propagate the tokens produced by the guard to the connected actions
							Token t = p.getToken(true);
							for (Relation r : p.getOutgoingRelations()) {
								r.getTarget().addToken(t);
							}
						}
					}
				}
			}
			//Else nothing to propagate
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void postPropagate(Model model) {
		super.postPropagate(model);
		
		//If in step 1 (= find the fireable transition)
		if (firststep) {
			//nothing to do
			return;
		}
		
		//Else, if in step 2 (= update the associated actions) 
		if (executedTransition != null) {
			//If the transition has inside guards and actions, 
			if(executedTransition.eClass().equals(CorePackage.eINSTANCE.getCompositeBlock())){
				CompositeBlock exectranscompo = (CompositeBlock)executedTransition;
				for (Block b : exectranscompo.getBlocks()) {
					for (Pin p : b.getPinsOut()) {
						while (p.hasTokens()) {
							//We propagate the tokens produced by the action to the connected output pins of the composite
							Token t = p.getToken(true);
							for (Relation r : p.getOutgoingRelations()) {
								r.getTarget().addToken(t);
							}
						}
					}
				}
			}
			//If the transition has tokens on its output pins (eventually produced by its inside actions)
			for (Pin p : executedTransition.getPinsOut()) {
				while (p.hasTokens()) {
					Token t = p.getToken(true);
					for (Relation r : p.getOutgoingRelations()) {
						//We move the tokens to the connected pins
						r.getTarget().addToken(t);

						//If the connected pin is an input, that means that this block is the new current state
						if (r.getTarget().getOutputOf() == null) {
							Block newstate = r.getTarget().getInputOf();
							
							//If the state changes, then we have to reset the current active timed guards
							if(newstate != currentState)
								activeTimedTransitions.clear();
							
							currentState = newstate;
							Platform.getController().changeMoCInternalData(this, "currentState", currentState);
							Platform.getLog().info("**** new current state = " +currentState);
							
							// Determine the timed transitions going out of the new current state if the current state has changed
							activeTimedTransitions = determineTimedTransitions(currentState);
						}
					}
				}
			}
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void finalSchedule(Model model) {
		super.finalSchedule(model);
		// Two steps : 
		// - the first to find the positive guard
		// - the second to update associated action
		// TODO In another version of FSM : implement more steps (exit action, guard action, enter action)
		
		//If in step 1 (= find the fireable transition)
		if (firststep) {
			if(currentEvent!=null) currentEvent = null; 
			firststep = false;//We move in step 2
			return;
		}
		
		//Else, if in step 2 (= update the associated actions) 
		if (executedTransition != null) {
			executedTransition = null; //We move in step 1 for the next snapshot
		}
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setStateRole(Block block) {
		block.setValueOfParameter(TFSM_ROLE_PARAM_NAME, TFSM_ROLE_STATE);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean hasStateRole(Block block) {
		if(block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME)){
			return block.getValueOfParameter(TFSM_ROLE_PARAM_NAME).equals(TFSM_ROLE_STATE);
		}
		return false;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setGuardRole(Block block) {
		block.setValueOfParameter(TFSM_ROLE_PARAM_NAME, TFSM_ROLE_GUARD);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean hasGuardRole(Block block) {
		if(block == null) {
			Platform.getLog().info("hasGuardRole called with block=null. This is a bug.");
		}
		
		
		if(block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME)){
			return block.getValueOfParameter(TFSM_ROLE_PARAM_NAME).equals(TFSM_ROLE_GUARD);
		}
		return false;
	}


	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setActionRole(Block block) {
		block.setValueOfParameter(TFSM_ROLE_PARAM_NAME, TFSM_ROLE_ACTION);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean hasActionRole(Block block) {
		if(block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME)){
			return block.getValueOfParameter(TFSM_ROLE_PARAM_NAME).equals(TFSM_ROLE_ACTION);
		}
		return false;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setInitialStateFeature(Block block) {
		if(!block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME)){
			block.setValueOfParameter(TFSM_ROLE_PARAM_NAME, TFSM_ROLE_STATE);
		}else if(block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME) && !hasStateRole(block)){
			throw new Error("Ill-formed model for FSM : cannot set the role INIT_STATE to a block which does not have STATE role");
		}		
		block.setValueOfParameter(TFSM_FEATURE_PARAM_NAME, TFSM_FEATURE_INITIAL_STATE);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean hasInitialStateFeature(Block block) {
		return (block.getParameters().containsKey(TFSM_FEATURE_PARAM_NAME) && block.getValueOfParameter(TFSM_FEATURE_PARAM_NAME).equals(TFSM_FEATURE_INITIAL_STATE));
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setTransitionRole(Block block) {
		block.setValueOfParameter(TFSM_ROLE_PARAM_NAME, TFSM_ROLE_TRANSITION);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean hasTransitionRole(Block block) {
		if(block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME)){
			return block.getValueOfParameter(TFSM_ROLE_PARAM_NAME).equals(TFSM_ROLE_TRANSITION);
		}
		return false;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public Token createEvent() {
		return CoreFactory.eINSTANCE.createToken();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void startOfUpdate(Model model) {
		super.startOfUpdate(model);
		
		//TODO modify for TimedFSM
		
		Platform.getLog().info("FSM MoC test : start of update ");
		//Get all the events which are present on the input pins of the model
		// and store them in the list, along with their source pin
		for (Pin pinIn : model.getStructure().getPinsIn()) {
			if (pinIn.hasTokens()) {
				Token event = pinIn.getToken(true);
				eventsToProcess.add(event);
				incomingPinOfEvents.put(event, pinIn);
			}
		}
		
		//The event to process is the first event of the list
		if(eventsToProcess.size()>0) currentEvent = eventsToProcess.get(0);
		
		Platform.getLog().info("**** eventsToProcess = " +eventsToProcess);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void endOfUpdate(Model model) {
		super.endOfUpdate(model);
		
		//TODO modify for TimedFSM

		Platform.getLog().info("FSM MoC test : end of update (empty)");

		//Reinitialize the variables for the next snapshot
		executedTransition = null;

		//Put the events produced during the snapshot on the output pins of the model
		for (Block b : model.getStructure().getBlocks()) {
			for (Pin pinOut : b.getPinsOut()) {
				while (pinOut.hasTokens()) {
					Token t = pinOut.getToken(true);
					for (Relation r : pinOut.getOutgoingRelations()) {
						r.getTarget().addToken(t);
					}
				}
			}
		}
		
		// If the snapshot has been done (computeSnapshot), then
		// the first event of the list has been processed in the current snapshot
		// so it has to be removed from the list of the events to process
		if(eventsToProcess.size() > 0)
			eventsToProcess.remove(0);
		
		//Initialize the current state for the next snapshot
		initialCurrentState = currentState;
		
		if (Platform.getLog().isLoggable(Level.INFO)) {
			Platform.getLog().info("**** endOfSnapshot : new current state is " + initialCurrentState.getName());
		}
	}
	
	
	/**
	 * Returns the timed transitions, going out a given state.
	 * 
	 * @param aState a state
	 * @return a {@code Map} that maps transition blocks to deadline values
	 */
	private Map<Block, InstantValue> determineTimedTransitions(Block aState) {
		Map<Block, InstantValue> timedTransitions = new HashMap<Block, InstantValue>();
		
		// We list the outgoing timed transitions, i.e. all the blocks which
		// have transition role, have the timed transition feature and leave
		// the given state

		//For all the relations which leave the current state
		for(Pin poutstate : aState.getPinsOut()){
			for(Relation r : poutstate.getOutgoingRelations()){
				//Get the connected block and check if it is a timed transition
				Block b = r.getTarget().getInputOf();

				if(hasTimedTransitionFeature(b)){
					//We add it to the list of timed transitions with the deadline computed from its internal guard
					for(Pin pIn : b.getPinsIn()){
						for(Relation rinside : pIn.getOutgoingRelations()){
							Block binside = rinside.getTarget().getInputOf();
							InstantValue delta = this.getWhenDeadline(binside);
							DurationValue dur = clock.createDurationValue(clock.createInstantValue(0.), delta);
							InstantValue absoluteTime = timeOfCurrentSnapshot.add(dur);
							if(hasGuardRole(binside)) timedTransitions.put(b, absoluteTime);
						}
					}
					
				}
				
			}
		}
		
		Platform.getLog().finest("Determined time transitions: " + timedTransitions);
		
		return timedTransitions;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean further(Model model) {
		//Méthode équivalente à !snapDet pour FSM
		Platform.getLog().info("FSM MoC test : further");
		return !this.snapDet(model);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setTimedTransitionFeature(Block block) {
		if(!block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME)){
			block.setValueOfParameter(TFSM_ROLE_PARAM_NAME, TFSM_ROLE_TRANSITION);
		}else if(block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME) && !hasTransitionRole(block)){
			throw new Error("Ill-formed model for TFSM : cannot set the feature TIMED_TRANSITION to a block which does not have TRANSITION role");
		}		
		block.setValueOfParameter(TFSM_FEATURE_PARAM_NAME, TFSM_FEATURE_TIMED_TRANSITION);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public boolean hasTimedTransitionFeature(Block block) {
		return (block.getParameters().containsKey(TFSM_FEATURE_PARAM_NAME) && block.getValueOfParameter(TFSM_FEATURE_PARAM_NAME).equals(TFSM_FEATURE_TIMED_TRANSITION));
	}


	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public void setWhenDeadline(Block block, InstantValue deadline) {
		if(!block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME)){
			block.setValueOfParameter(TFSM_ROLE_PARAM_NAME, TFSM_ROLE_GUARD);
		}else if(block.getParameters().containsKey(TFSM_ROLE_PARAM_NAME) && !hasGuardRole(block)){
			throw new Error("Ill-formed model for TFSM : cannot set a deadline to a block which does not have GUARD role");
		}		
		block.setValueOfParameter(TFSM_DEADLINE_PARAM_NAME, deadline);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated NOT
	 */
	public InstantValue getWhenDeadline(Block block) {
		if(hasGuardRole(block)){
			InstantValue deadline;
			try{
				deadline = (InstantValue)block.getValueOfParameter(TFSM_DEADLINE_PARAM_NAME);
			}catch (ClassCastException ex) {
				throw new Error("TFsmModelOfComputation : the deadline of the block is not an InstantValue");
			}
			
			return deadline;
		}
		throw new Error("Ill-formed model for TFSM : a block which does not have the GUARD role cannot have a deadline");
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case TfsmPackage.TFSM_MODEL_OF_COMPUTATION__CLOCK:
				if (resolve) return getClock();
				return basicGetClock();
			case TfsmPackage.TFSM_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT:
				if (resolve) return getTimeOfCurrentSnapshot();
				return basicGetTimeOfCurrentSnapshot();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case TfsmPackage.TFSM_MODEL_OF_COMPUTATION__CLOCK:
				setClock((Clock)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case TfsmPackage.TFSM_MODEL_OF_COMPUTATION__CLOCK:
				setClock((Clock)null);
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case TfsmPackage.TFSM_MODEL_OF_COMPUTATION__CLOCK:
				return clock != null;
			case TfsmPackage.TFSM_MODEL_OF_COMPUTATION__TIME_OF_CURRENT_SNAPSHOT:
				return timeOfCurrentSnapshot != null;
		}
		return super.eIsSet(featureID);
	}

} //TFsmModelOfComputationImpl
