package fr.supelec.modhelx.qdgui;

import java.awt.Graphics;
import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JMenuItem;

import fr.supelec.graphbench.AtomicVertex;
import fr.supelec.graphbench.GraphElement;
import fr.supelec.graphbench.Socket;
import fr.supelec.graphbench.ui.PropertyDescriptorProvider;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.qdgui.rendering.RendererParameters;
import fr.supelec.modhelx.qdgui.rendering.VertexRenderer;
import fr.supelec.modhelx.qdgui.rendering.VertexRenderers;

public class AtomicVertexAdapter extends AtomicVertex implements VertexAdapter, PropertyDescriptorProvider {
	private final Block block;
	private final List<SocketAdapter> sockets = new ArrayList<SocketAdapter>();
	
	private final int width, height;
	private final VertexRenderer customRenderer;
	private final RendererParameters rendererParams;
	
	public AtomicVertexAdapter(CompositeVertexAdapter parent, Block block) {
		super(parent);
		this.block = block;

		customRenderer = VertexRenderers.forBlock(block);
		// at this point, customRenderer may not be null
		if(customRenderer == null) throw new Error("Null renderer for block " + block);

		Common.setSockets(block, this, sockets, customRenderer.renderPinNames());
		
		
		///System.out.println("VertexAdapter: Block class=" + block.getClass() + ", customRenderer=" + customRenderer);

		// create socket adapters
		// maybe we should not do this but rather generate them on the fly???

		// we need to set the location of this pin
		// => inputs on the left of the parent component, outputs on the right

		rendererParams = customRenderer.calculateRendererParameters(this);
		
		width = rendererParams.width;
		height = rendererParams.height;
	}
	

	@Override
	public GraphElement getSelectionForCoordinates(int x, int y) {
		if(getX() <= x && x <= getX() + width && getY() <= y && y <= getY() + height)
			return this;
		else return null;
	}

	@Override
	public Iterable<SocketAdapter> sockets() {
		return sockets;
	}
	
	public List<SocketAdapter> getSocketList() {
		return sockets;
	}

	@Override
	public void render(Graphics g) {
		int x1 = getX(), y1 = getY();

		customRenderer.render(rendererParams, g, x1, y1, isSelected());

		for(Socket s : sockets()) {
			s.render(g);
		}

	}

	@Override
	public void writeElementAttributes(PrintStream s) {
		// TODO Auto-generated method stub

	}

	public Block getBlock() {
		return block;
	}
	
	public Socket getSocketForPin(Pin pin) {
		for(SocketAdapter s : sockets()) {
			if(s.getPin() == pin) return s;
		}
		return null;
	}
	
	public String toString() {
		return "Atomic Block " + block.getName();
	}


	@Override
	public int getHeight() {
		return height;
	}


	@Override
	public int getWidth() {
		return width;
	}


	public PropertyDescriptor[] getPropertyDescriptors() {
		try {
			BeanInfo bi = Introspector.getBeanInfo(block.getClass());
			return bi.getPropertyDescriptors();
		} catch(IntrospectionException e) {
			// TODO
			return new PropertyDescriptor[] {};
		}
	}
	
	public Object getPropertyHolder() {
		return block;
	}
	
    public List<JMenuItem> getMenuItems() {
    	return customRenderer.getMenuItems(rendererParams);
    }

    @Override
    public String getToolTip() {
    	return Common.getToolTip(block, null);
    }

    public StringBuffer serializePlacement() {
    	return Common.serializeVertexPlacement(this, null);
    }
}
