package fr.supelec.modhelx.qdgui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Semaphore;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.WindowConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import fr.supelec.graphbench.AtomicVertex;
import fr.supelec.graphbench.CompositeVertex;
import fr.supelec.graphbench.ControlPoint;
import fr.supelec.graphbench.Edge;
import fr.supelec.graphbench.GraphPanel;
import fr.supelec.graphbench.Vertex;
import fr.supelec.graphbench.VertexObserver;
import fr.supelec.graphbench.ui.ConfigPanel;
import fr.supelec.graphbench.ui.DndSource;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.engine.instrumentation.Controller;

public class ModelWindow extends JFrame {
	private static final long serialVersionUID = 5562452142492608470L;

	private final Model model;
	private final Clock clock;
	private final Semaphore runSemaphore = new Semaphore(0);
	private final JButton btnStep, btnSnap, btnUpdate, btnRun;
	private final JButton btnSaveXY;
	private final JLabel lblInfo;
	private final GraphPanel pnlGraph;
	private final JTextArea txtVariables;
	private Breakpoint nextStop = Breakpoint.SNAPSHOT;
	private final Map<String, Object> mocVariables = new HashMap<String, Object>();
	private final CompositeVertexAdapter adapter;
	private boolean freeRun = false;
	
	public ModelWindow(Model model, Clock clock) {
		this(model, clock, null);
	}
	
	public ModelWindow(Model model, Clock clock, String locationFileName) {
		this(new DndSource[] {}, model, clock, locationFileName);
	}
	
	private JButton createRunButton(String caption, final Breakpoint breakpoint) {
		final JButton btn = new JButton(caption);
        btn.setEnabled(false);
        btn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btn.setEnabled(false);
				nextStop = breakpoint;
				runSemaphore.release();
			}
		});
        return btn;
	}
	
	public ModelWindow(DndSource[] toolbox, Model model, Clock clock, final String placementFileName) {
		super("ModHel'X GUI");
		
		this.model = model;
		this.clock = clock;
		
		adapter = new CompositeVertexAdapter(null, model.getStructure(), model.getMoc());
		//adapter.setWidth(800);
		//adapter.setHeight(500);
		adapter.setX(10);
		adapter.setY(10);
		
        pnlGraph = new GraphPanel(adapter);
        pnlGraph.calculateSize();

        JScrollPane pnlScroll = new JScrollPane(pnlGraph);
        pnlScroll.setMinimumSize(new Dimension(400, 200));
        pnlScroll.setPreferredSize(new Dimension(600, 200));

        
        JPanel pnlControls = new JPanel(new BorderLayout());
        
        btnStep = createRunButton("Next Step", Breakpoint.STEP);
        btnSnap = createRunButton("Next Snapshot", Breakpoint.SNAPSHOT);
        btnUpdate = createRunButton("Next update", Breakpoint.UPDATE);
        btnRun = createRunButton("Run", Breakpoint.NONE);
        
        
        
        btnSaveXY = new JButton("Save placement");
        btnSaveXY.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(placementFileName == null) {
					System.err.println("Could not save placement: no file name was given.");
					return;
				}
				StringBuffer locations = adapter.serializePlacement();
				try {
					PrintStream out = new PrintStream(placementFileName);
					out.println("<locations>");
					out.print(locations);
					out.println("\n</locations>");
				} catch(Exception exc) {
					System.err.println("Error while saving placement");
					exc.printStackTrace(System.err);
				}
			}
		});
        
        
        JPanel pnlRemote = new JPanel(new FlowLayout());
        pnlRemote.add(btnStep);
        pnlRemote.add(btnSnap);
        pnlRemote.add(btnUpdate);
        pnlRemote.add(btnRun);
        pnlRemote.add(btnSaveXY);
        
        
        pnlControls.add(pnlRemote, BorderLayout.SOUTH);
        
        lblInfo = new JLabel("---");
        pnlControls.add(lblInfo, BorderLayout.NORTH);
        
        final JPanel pnlTop;

        // add the toolbox, if there are any tools defined
        if(toolbox.length > 0) {
        	pnlTop = new JPanel(new GridLayout(2, 1));
        	
	        final JPanel pnlToolbox = new JPanel();
	        pnlToolbox.setLayout(new FlowLayout());
	        for(DndSource ds : toolbox) pnlToolbox.add(ds);
	        
	        pnlTop.add(pnlToolbox);
	        pnlTop.add(pnlControls);
        } else {
        	pnlTop = pnlControls;
        }
        
        txtVariables = new JTextArea();
        
        JPanel pnlGauche = new JPanel(new BorderLayout());
        pnlGauche.add(pnlTop, BorderLayout.NORTH);
        pnlGauche.add(pnlScroll, BorderLayout.CENTER);
        pnlGauche.add(txtVariables, BorderLayout.SOUTH);
        
        final ConfigPanel pnlConfig = new ConfigPanel(pnlGraph, adapter);
        pnlConfig.addGraphElementPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                //if(evt.getPropertyName().equals("output")) update();
            }
            
        });
        pnlConfig.setMaximumSize(new Dimension(200, 0));
        
        LagGauge lagGauge = new LagGauge();
        Platform.addLagListener(lagGauge);
        JPanel pnlRight = new JPanel(new BorderLayout());
        pnlRight.add(pnlConfig, BorderLayout.CENTER);
        pnlRight.add(lagGauge, BorderLayout.SOUTH);
        
        final JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, pnlGauche, pnlRight);
        splitPane.setResizeWeight(1.0);
        
        
        // try to restore the locations
        if(placementFileName == null) {
        	System.err.println("No placement file provided, not restoring block placement.");
        } else {
        	System.out.println("Restoring block placement from " + placementFileName);
        	try {
        		DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        		Document doc = null;
        		try {
        			doc = docBuilder.parse(new File(placementFileName));
        		} catch(FileNotFoundException exc1) {
        			System.err.println("File " + placementFileName + " not found, aborting.");
        		}
        		if(doc != null) {
        			Element rootElt = doc.getDocumentElement();
        			for(Node n = rootElt.getFirstChild(); n != null; n = n.getNextSibling()) {
        				if(n instanceof Element && "block".equals(n.getNodeName())) {
        					final Element elt = (Element)n;
        					final String name = elt.getAttribute("name");
        					final int x = Integer.parseInt(elt.getAttribute("x"));
        					final int y = Integer.parseInt(elt.getAttribute("y"));
        					final int w = elt.hasAttribute("width") ? Integer.parseInt(elt.getAttribute("width")) : -1;
        					final int h = elt.hasAttribute("height") ? Integer.parseInt(elt.getAttribute("height")) : -1;
        					adapter.accept(new VertexObserver() {

        						public void observe(AtomicVertex vertex) {
        							doIt((VertexAdapter)vertex);
        						}

        						public void observe(CompositeVertex vertex) {
        							doIt((VertexAdapter)vertex);
        							
        							// restore control points
        							for(Node nEdge = elt.getFirstChild(); nEdge != null; nEdge = nEdge.getNextSibling()) {
        								if(! (nEdge instanceof Element && "edge".equals(nEdge.getNodeName()))) continue;
        								final Element eEdge = (Element)nEdge;
        								final String eFromBlock = eEdge.getAttribute("fromBlock");
        								final String eFromPin = eEdge.getAttribute("fromPin");
        								final String eToBlock = eEdge.getAttribute("toBlock");
        								final String eToPin = eEdge.getAttribute("toPin");

        								for(Node nPoint = eEdge.getFirstChild(); nPoint != null; nPoint = nPoint.getNextSibling()) {
        									if(! ((nPoint instanceof Element) && "point".equals((nPoint.getNodeName())))) continue;
        									final Element ePoint = (Element)nPoint;
        									final int px = Integer.parseInt(ePoint.getAttribute("x"));
        									final int py = Integer.parseInt(ePoint.getAttribute("y"));
        									for(Edge edge : vertex.edges()) {
        										SocketAdapter saFrom = (SocketAdapter)edge.getFromSocket();
        										SocketAdapter saTo = (SocketAdapter)edge.getToSocket();
        										if(eFromBlock.equals(((VertexAdapter)saFrom.getParent()).getBlock().getName())
        												&& eFromPin.equals(saFrom.getPin().getName())
        												&& eToBlock.equals(((VertexAdapter)saTo.getParent()).getBlock().getName())
        												&& eToPin.equals(saTo.getPin().getName())) {
        											edge.addControlPoint(new ControlPoint(vertex, edge, px, py));
        										}
        									}
        								}
        							}
        						}

        						private void doIt(VertexAdapter v) {
        							if(v.getBlock().getName() != null && v.getBlock().getName().equals(name)) {
        								((Vertex)v).setPosition(x, y);
        								if(w != -1 && v instanceof CompositeVertexAdapter) ((CompositeVertexAdapter)v).setWidth(w);
        								if(h != -1 && v instanceof CompositeVertexAdapter) ((CompositeVertexAdapter)v).setHeight(h);
        							}
        						}
        					});
        				}
        			}
        		}
        	} catch(Exception exc) {
        		System.err.println("Exception while restoring block placement.");
        		exc.printStackTrace(System.err);
        	}
        	
        	pnlGraph.calculateSize();
        }
        
        // now set up the frame itself
        setLayout(new BorderLayout());
        add(splitPane, BorderLayout.CENTER);
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setPreferredSize(new Dimension(1000, 650));
        pack();
        
        // repaint the diagram each time the model is updated
        model.eAdapters().add(new EContentAdapter() {
			public void notifyChanged(Notification notification) {
				super.notifyChanged(notification);
				//System.out.println("Model changed.");
				pnlGraph.repaint();
			}
        });
	}
	
	public void run() {
		Platform.setModel(model, clock);
		StepController controller = new StepController();
		Platform.setController(controller);
		
		while(true) {
			lblInfo.setText("Simulation stopped. Ready to start.");
			controller.waitStep();
			try {
				Platform.run();
			} catch(Exception e) {
				e.printStackTrace();
				return;
			} catch(Error e) {
				e.printStackTrace();
				return;
			}
		}
	}


	private class StepController extends Controller {

		private int currentSnaphot = -1;
		private String latestMethod = null;
		
		public void waitStep() {
			//pnlGraph.repaint();
			// no longer necessary since we catch EMF events and repaint the diagram
			// each time the EMF model changes
			btnStep.setEnabled(true);
			btnSnap.setEnabled(true);
			btnUpdate.setEnabled(true);
			btnRun.setEnabled(true);
			
			if(!freeRun) runSemaphore.acquireUninterruptibly();
		}
		
		@Override
		public void beginSnapshot(int number) {
			currentSnaphot = number;
			
			//lblInfo.setText("Snapshot #" + currentSnaphot + " beginning.");
			//waitStep();
		}

		@Override
		public void changeMoCInternalData(ModelOfComputation moc,
				String variableName, Object variableValue) {
			super.changeMoCInternalData(moc, variableName, variableValue);
			
			mocVariables.put(variableName, variableValue);
			
			StringBuffer list = new StringBuffer();
			// update the list of displayed variables
			for(Map.Entry<String, Object> e : mocVariables.entrySet()) {
				list.append(e.getKey()).append(" = ").append(e.getValue()).append('\n');
			}
			txtVariables.setText(list.toString());
		}

		@Override
		public void endSnapshot() {
			lblInfo.setText("Snapshot #" + currentSnaphot + " ending.");
			
			if(nextStop == Breakpoint.STEP || nextStop == Breakpoint.SNAPSHOT || nextStop == Breakpoint.UPDATE)
				waitStep();
		}

		@Override
		public void enterMethod(Object callee, String methodName, Object... params) {
			if(nextStop == Breakpoint.STEP || nextStop == Breakpoint.UPDATE && "update".equals(methodName)) {
				String name = "";
				if(callee instanceof Model) {
					name = ((Model)callee).getName();
				} else if(callee instanceof ModelOfComputation) {
					name = ((ModelOfComputation)callee).getName();
				} else if(callee instanceof Block) {
					name = ((Block)callee).getName();
				} else name = callee.getClass().getSimpleName();
				
				if(name.length() > 0) methodName = name + "." + methodName;
				
				lblInfo.setText(
						(currentSnaphot>=0 ? "[Within snapshot #" + currentSnaphot + "] " : "") + 
						(latestMethod == null ? "" : "Completed " + latestMethod + ".  ") + 
						"Entering " + methodName);
				latestMethod = methodName;
				waitStep();
			}
		}

		@Override
		public void leaveMethod(Object... returnValues) {	
		}

	}
	

	private static enum Breakpoint {
		STEP, SNAPSHOT, UPDATE, NONE
	}

	
	public void setLocationForBlock(final Block block, final int x, final int y) {
		adapter.accept(new VertexObserver() {
			
			public void observe(AtomicVertex vertex) {
				if(((AtomicVertexAdapter)vertex).getBlock() == block) {
					vertex.setX(x);
					vertex.setY(y);
					return;
				}
			}
			
			public void observe(CompositeVertex vertex) {
				if(((CompositeVertexAdapter)vertex).getBlock() == block) {
					vertex.setX(x);
					vertex.setY(y);
					return;
				}
			}
		});

	}
	
	public void setRootSize(int w, int h) {
		adapter.setWidth(w);
		adapter.setHeight(h);
	}

	public void setSize(final Block block, final int w, final int h) {
		adapter.accept(new VertexObserver() {
			public void observe(AtomicVertex vertex) {
				// do nothing, only applies to composite blocks
			}
			
			public void observe(CompositeVertex vertex) {
				CompositeVertexAdapter cva = (CompositeVertexAdapter) vertex;
				
				if(cva.getBlock() == block) {
					cva.setWidth(w);
					cva.setHeight(h);
				}
			}
		});
	}
	
	public void setFreeRun(boolean freeRun) {
		this.freeRun = freeRun;
	}
}