package fr.supelec.modhelx.qdgui;

import fr.supelec.graphbench.CompositeVertex;
import fr.supelec.graphbench.Vertex;
import fr.supelec.graphbench.ui.DndSource;
import fr.supelec.graphbench.ui.VertexInstantiator;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.mocs.petrinet.PetriNetModelOfComputation;
import fr.supelec.modhelx.mocs.petrinet.PetrinetFactory;
import fr.supelec.modhelx.qdgui.rendering.PetriNetPlaceR;
import fr.supelec.modhelx.qdgui.rendering.PetriNetTransitionR;

public class PetriNetGUI {
	private static PetriNetModelOfComputation dummyMoC = PetrinetFactory.eINSTANCE.createPetriNetModelOfComputation();
	
	public static final DndSource[] TOOLBOX = {
		new DndSource(new VertexInstantiator() { 
			public Vertex newInstance(CompositeVertex parent) {
				Block block = CoreFactory.eINSTANCE.createAtomicBlock();
				dummyMoC.setBlockIsPlace(block);
				initPetriNetBlock(block, "P" + (placeCounter++));
				return new AtomicVertexAdapter((CompositeVertexAdapter)parent, block);
			}
		}, "Place"),
		new DndSource(new VertexInstantiator() {
			public Vertex newInstance(CompositeVertex parent) {
				Block block = CoreFactory.eINSTANCE.createAtomicBlock();
				dummyMoC.setBlockIsTransition(block);
				initPetriNetBlock(block, "T" + (transitionCounter++));
				return new AtomicVertexAdapter((CompositeVertexAdapter)parent, block);
			}
		}, "Transition"),
	};
	
	private static void initPetriNetBlock(Block block, String name) {
		block.setName(name);
		Pin p1out = CoreFactory.eINSTANCE.createPin();
		p1out.setName(name.toLowerCase() + "out"); 
		block.getPinsOut().add(p1out);
		Pin p1in = CoreFactory.eINSTANCE.createPin();
		p1in.setName(name.toLowerCase() + "in");
		block.getPinsIn().add(p1in);
	}
	
	private static int placeCounter = 0, transitionCounter = 0;
	
	public static void register() {
		PetriNetPlaceR.register();
		PetriNetTransitionR.register();
	}
	
	public static void main(String[] args) {
		register();
		
		Model model = CoreFactory.eINSTANCE.createModel();
		
		model.setName("Sample_PetriNet");
		
		PetriNetModelOfComputation moc = PetrinetFactory.eINSTANCE.createPetriNetModelOfComputation();
		moc.setName("Sample_PetriNet_MoC");
		Clock clock = TimeFactory.eINSTANCE.createLogicalClock();
		moc.setClock(clock);
		model.setMoc(moc);
		
		//Structure
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("Sample_PetriNet_Structure");
		model.setStructure(modelStructure);
		
		ModelWindow w = new ModelWindow(TOOLBOX, model, clock, "tests/petrinet.xy");
		w.setVisible(true);
		w.run();

	}
}
