package fr.supelec.modhelx.qdgui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.io.PrintStream;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JMenuItem;

import fr.supelec.graphbench.Socket;
import fr.supelec.graphbench.SocketDirection;
import fr.supelec.graphbench.Vertex;
import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;

public class SocketAdapter extends Socket {
	private int normalRadius = 6;
	private int hoverRadius = 8;
	private final Pin pin;
	private boolean renderName;
	private boolean renderPin = true;
	private String[] text;
	private Color[] textColor = null;
	private final ModelOfComputation moc;
	
	public final static int SPACING = 5*6;
	
	
	public SocketAdapter(Pin pin, Vertex parent, boolean renderName) {
		super(parent, 0, 0);
		
		this.pin = pin;
		this.renderName = renderName;
		text = new String[]{ pin.getName() };
		textColor = new Color[]{ Color.BLACK };
		
		moc = getMoc();
	}
	
	private ModelOfComputation getMoc() {
		Vertex v = parent;
		while(v != null) {
			if(v instanceof CompositeVertexAdapter) {
				ModelOfComputation moc = ((CompositeVertexAdapter)v).getMoc();
				if(moc != null) return moc;
			}
			v = v.getParent();
		}
		
		return null;
	}

	@Override
	public SocketDirection getDirection() {
		if(pin.getInputOf() != null) return SocketDirection.IN;
		else return SocketDirection.OUT;
	}

	@Override
	public boolean isInside(int x, int y) {
        int dx = getParent().getX() + getX() - x;
        int dy = getParent().getY() + getY() - y;
        return -normalRadius <= dx && dx <= normalRadius && -normalRadius <= dy && dy <= normalRadius;
	}

	@Override
	public void render(Graphics g) {
		if(!renderPin) return;
		
		int xc = getParent().getX() + getX();
		int yc = getParent().getY() + getY();
		int radius = isSelected() ? hoverRadius : normalRadius;
        int x1 = xc-radius;
        int y1 = yc-radius;
        int size = 2*radius;
        g.setColor(pin.getStoredTokens().size() == 0 ? Color.YELLOW : Color.RED);
        g.fillRect(x1, y1, size, size);
        g.setColor(Color.BLACK);
        g.drawRect(x1, y1, size, size);
        
        if(renderName) {
        	Font origFont = g.getFont();
        	g.setFont(origFont.deriveFont(9.5f));
        	
        	int lineHeight = g.getFontMetrics().getHeight();
        	int yt = yc + 2*normalRadius - (text.length * lineHeight) / 2;
        	
        	for(int i=0; i<text.length; i++) {
        		g.setColor(textColor[i]);
        	
	        	int xt = (getDirection() == SocketDirection.IN ? 
						xc + 2*normalRadius :
							xc - 2*normalRadius - g.getFontMetrics().stringWidth(text[i]) );

        		g.drawString(text[i], xt, yt);
        		yt += lineHeight;
        	}
        	g.setFont(origFont);
        }
	}

	@Override
	public void writeElementAttributes(PrintStream s) {
		// TODO Auto-generated method stub

	}

	public Pin getPin() {
		return pin;
	}
	
    public List<JMenuItem> getMenuItems() {
    	return new LinkedList<JMenuItem>();
    }
    
    public String toString() {
    	return "SocketAdapter[" + pin.getName() + " @x=" + getX() + ",y=" + getY() + "]";
    }

    
    @Override
    public String getToolTip() {
    	StringBuilder sb = new StringBuilder();
		for(Token t : pin.getStoredTokens()) {
			Object val = moc.getValueOfToken(t);
			if(val != null) sb.append("&lt;").append(val).append("&gt; ");
			else sb.append("&lt;\u2022&gt; ");
		}
		
		String additionalText = sb.length() > 0 ? "<br>" + pin.getStoredTokens().size() + " tokens: " + sb.toString() : null; 
    	
    	return Common.getToolTip(pin, additionalText);
    }
    
    public void setRenderName(boolean renderName) {
    	this.renderName = renderName;
    }
    
    public void setAdditionalLine(String additionalLine, Color additionalLineColor) {
    	text = new String[]{ text[0], additionalLine };
    	textColor = new Color[]{ textColor[0], additionalLineColor == null ? Color.BLACK : additionalLineColor };
    }
    
    public void setRenderPin(boolean renderPin) {
    	this.renderPin = renderPin;
    }
    
    public void setNormalRadius(int normalRadius) {
    	this.normalRadius = normalRadius;
    }
}
