package fr.supelec.modhelx.qdgui.rendering;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;

import fr.supelec.graphbench.Vertex;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.corelib.PluginBlock;

public class DefaultAtomicVertexRenderer extends DefaultGenericVertexRenderer {
	private final static Color BLOCK_COLOR = new Color(.90f, .97f, 1f);
	private final Class<? extends Block> acceptedClass;
	private final String acceptedPluggable;
	private final Image icon;
	private String bottomLineText = null;
	private Color bottomLineColor = null;
	
	public final static VertexRenderer DEFAULT_ATOMIC_RENDERER = new DefaultAtomicVertexRenderer();
	
	public DefaultAtomicVertexRenderer(Class<? extends Block> acceptedClass, Image icon) {
		this.acceptedClass = acceptedClass;
		this.acceptedPluggable = null;
		this.icon = icon;
	}
	
	public DefaultAtomicVertexRenderer(String acceptedPluggable, Image icon) {
		this.acceptedClass = null;
		this.acceptedPluggable = acceptedPluggable;
		this.icon = icon;
	}
	
	/**
	 * Constructs a default renderer that renders any block. Don't add it to
	 * the main list of renderers!
	 */
	private DefaultAtomicVertexRenderer() {
		this.acceptedClass = null;
		this.acceptedPluggable = null;
		this.icon = null;
	}
	
	public boolean accept(Block block) {
		if(acceptedClass != null && block.getClass() != acceptedClass) return false;
		if(acceptedPluggable != null && 
				block instanceof PluginBlock && 
				!((PluginBlock)block).getPluggedInClassName().equals(acceptedPluggable)) return false;
		return true;  // by default
	}

	public void render(RendererParameters params, Graphics g, int x, int y, boolean selected) {
		super.render(params, g, x, y, selected, BLOCK_COLOR);
		
		if(icon != null) {
			int w = icon.getWidth(null);
			int h = icon.getHeight(null);
			
			g.drawImage(icon, x+(params.width-w)/2, y+(params.height-h)/2, null);
		}
		
		//System.out.println(params.vertex.getBlock() + " -> " + bottomLineText);
		
		if(bottomLineText != null) {
			g.setColor(bottomLineColor);
			int sw = g.getFontMetrics().stringWidth(bottomLineText);
			int vertexW = ((Vertex)params.vertex).getWidth();
			int vertexH = ((Vertex)params.vertex).getHeight();
			int vertexX = ((Vertex)params.vertex).getX();
			int vertexY = ((Vertex)params.vertex).getY();
			Font f = g.getFont().deriveFont(9.5f);
			Font oldFont = g.getFont();
			g.setFont(f);
			g.drawString(bottomLineText, vertexX + (vertexW-sw)/2, vertexY + vertexH - 2);
			g.setFont(oldFont);
		}
	}
	
	public void setBottomLine(String bottomLineText, Color bottomLineColor) {
		this.bottomLineText = bottomLineText;
		this.bottomLineColor = bottomLineColor;
	}
}
