package fr.supelec.modhelx.qdgui.rendering;

import java.awt.Color;
import java.awt.Graphics;

import fr.supelec.graphbench.Socket;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.qdgui.SocketAdapter;
import fr.supelec.modhelx.qdgui.VertexAdapter;

public abstract class DefaultGenericVertexRenderer extends AbstractVertexRenderer {
	protected void render(RendererParameters params, Graphics g, int x, int y, boolean selected, Color backgroundColor) {
		int width = params.width;
		int height = params.height;
		
		g.setColor(backgroundColor);
		g.fillRect(x, y, width, height);
		
		drawRectShadow(g, x, y, width, height);
		
		g.setColor(selected ? Color.RED : Color.BLACK);
		g.drawRect(x, y, width, height);
				
		Block block = params.vertex.getBlock();
		
		if(block.getName() != null)
			g.drawString(block.getName(), x, y-1);
		else g.drawString("<null>", x, y-1);
	}
	

	@Override
	public boolean renderPinNames() {
		return true;
	}

	public RendererParameters calculateRendererParameters(VertexAdapter vertex) {
		int height = 30;
		int width = 140;

		RendererParameters params = new RendererParameters(vertex, width, height);
		
		recalculatePortPositions(params);

		return params;
	}
	
	public void recalculatePortPositions(RendererParameters params) {
		// input pins
		int yIn = SocketAdapter.SPACING;
		int yOut = SocketAdapter.SPACING;
		
		// set the position of sockets
		for(Socket socket : params.vertex.getSocketList()) {
			switch(socket.getDirection()) {
			case IN:
				socket.setPosition(0, yIn);
				yIn += SocketAdapter.SPACING;
				break;
			case OUT:
				socket.setPosition(params.width - 1, yOut);
				yOut += SocketAdapter.SPACING;
				break;
			}
			
		}

		// adjust height to contain all pins
		if(yIn > params.height) params.height = yIn;
		if(yOut > params.height) params.height = yOut;
	}
}
