package fr.supelec.modhelx.qdgui.rendering;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JMenuItem;

import fr.supelec.graphbench.Socket;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.mocs.petrinet.PetriNetModelOfComputation;
import fr.supelec.modhelx.qdgui.VertexAdapter;

public class PetriNetPlaceR extends AbstractVertexRenderer {
	public RendererParameters calculateRendererParameters(VertexAdapter vertex) {
		RendererParameters params = new RendererParameters(vertex, 60, 40);
		recalculatePortPositions(params);
		return params;
	}


	public void recalculatePortPositions(RendererParameters params) {
		for(Socket s : params.vertex.getSocketList()) {
			switch(s.getDirection()) {
			case IN: s.setPosition(0, 20); break;
			case OUT: s.setPosition(60, 20); break;
			}
		}
	}

	public void render(RendererParameters params, Graphics g_, int x, int y, boolean selected) {
		int nbTokens = params.vertex.getBlock().getPinsOut().get(0).getStoredTokens().size();
		
		Graphics2D g = (Graphics2D)g_;
		
		g.translate(x, y);
		
		g.setColor(Color.WHITE);
		g.fillOval(10, 0, 40, 40);
		g.setColor(selected ? Color.RED : Color.BLACK);
		g.drawOval(10, 0, 40, 40);
		g.drawLine(0, 20, 10, 20);
		g.drawLine(50, 20, 60, 20);
		
		g.drawString(params.vertex.getBlock().getName(), 23, -1);
		
		g.setColor(Color.BLACK);
		
		switch(nbTokens) {
		case 0: break;
		case 1:
			g.fillOval(26, 16, 8, 8);
			break;
		case 2:
			g.fillOval(16, 16, 8, 8);
			g.fillOval(36, 16, 8, 8);
			break;
		case 3:
			g.fillOval(26, 10, 8, 8);
			g.fillOval(16, 20, 8, 8);
			g.fillOval(36, 20, 8, 8);
			break;
		default:
			g.drawString(Integer.toString(nbTokens), 20, 25);
		}
		
		g.translate(-x, -y);
	}
	
	public boolean renderPinNames() {
		return false;
	}
	
	@Override
    public List<JMenuItem> getMenuItems(final RendererParameters params) {
    	JMenuItem itmAdd = new JMenuItem("Add token");
    	itmAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				params.vertex.getBlock().getPinsOut().get(0).addToken(CoreFactory.eINSTANCE.createToken());
			}
		});
    	
    	JMenuItem itmRem = new JMenuItem("Remove token");
    	itmRem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				params.vertex.getBlock().getPinsOut().get(0).getToken(true);
			}
    	});
    	
    	List<JMenuItem> list = new LinkedList<JMenuItem>();
    	list.add(itmAdd);
    	list.add(itmRem);
    	
    	return list;
    }

	private PetriNetPlaceR() {}
	
	private final static PetriNetPlaceR instance;
	
	static {
		instance = new PetriNetPlaceR();
	}
	
	public static void register() {
		VertexRenderers.register(instance);
	}

	public boolean accept(Block block) {
		return PetriNetModelOfComputation.ROLE_PLACE.equals(
				block.getParameters().get(PetriNetModelOfComputation.ROLE_ATTRIBUTE));
	}
}
