package fr.supelec.modhelx.qdgui.rendering;

import java.awt.Color;
import java.awt.Graphics;

import fr.supelec.graphbench.Socket;
import fr.supelec.graphbench.SocketDirection;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.mocs.sdflib.SdfAccumulator;
import fr.supelec.modhelx.mocs.sdflib.SdfLimiter;
import fr.supelec.modhelx.mocs.sdflib.SdfThreshold;
import fr.supelec.modhelx.mocs.sdflib.impl.SdfAccumulatorImpl;
import fr.supelec.modhelx.mocs.sdflib.impl.SdfAdderImpl;
import fr.supelec.modhelx.mocs.sdflib.impl.SdfLimiterImpl;
import fr.supelec.modhelx.mocs.sdflib.impl.SdfMultiplierImpl;
import fr.supelec.modhelx.mocs.sdflib.impl.SdfThresholdImpl;
import fr.supelec.modhelx.qdgui.SocketAdapter;
import fr.supelec.modhelx.util.Numeric;

public final class SDFRenderingLib {
	private final static Color PARAM_COLOR = new Color(35, 131, 255);
	private final static char[] compOps = {'<', '\u2264', '\u2265', '>'};
	
	public static void register() {
		VertexRenderers.register(new DefaultAtomicVertexRenderer(SdfAccumulatorImpl.class, Icons.SUM) {
			@Override
			public void render(RendererParameters params, Graphics g, int x, int y, boolean selected) {
				setBottomLine("=" + ((SdfAccumulator)params.vertex.getBlock()).getCurrentValue(), Color.BLACK);
				
				super.render(params, g, x, y, selected);
			}
		});
		
		VertexRenderers.register(new DefaultAtomicVertexRenderer(SdfLimiterImpl.class, null) {
			public void render(RendererParameters params, Graphics g, int x, int y, boolean selected) {
				SdfLimiter limiter = (SdfLimiter)params.vertex.getBlock();
				setBottomLine("[" + limiter.getLowerBound() + ", " + limiter.getUpperBound() + "]", PARAM_COLOR);
				super.render(params, g, x, y, selected);
			}
		});
		
		VertexRenderers.register(new DefaultAtomicVertexRenderer(SdfAdderImpl.class, Icons.PLUS) {
			@Override
			public void recalculatePortPositions(RendererParameters params) {
				super.recalculatePortPositions(params);
				
				for(Socket socket : params.vertex.getSocketList()) {
					if(socket.getDirection() == SocketDirection.IN) {
						SocketAdapter sa = (SocketAdapter)socket;
						
						Object multFactor = sa.getPin().getValueOfParameter("MultFactor");
						if(multFactor == null) continue;
						sa.setAdditionalLine("\u00D7 " + multFactor.toString(), PARAM_COLOR);
					}
				}
			}	
		});
		
		VertexRenderers.register(new DefaultAtomicVertexRenderer(SdfMultiplierImpl.class, Icons.TIMES) {

		});
		
		VertexRenderers.register(new DefaultAtomicVertexRenderer(SdfThresholdImpl.class, null) {
			@Override
			public void recalculatePortPositions(RendererParameters params) {
				super.recalculatePortPositions(params);
				
				for(Socket socket : params.vertex.getSocketList()) {
					if(socket.getDirection() == SocketDirection.OUT) {
						SocketAdapter sa = (SocketAdapter)socket;
						
						Pin pOut = sa.getPin();
						
						Object[] paramValues = new Object[] {
								pOut.getValueOfParameter(SdfThreshold.PARAM_THRESHOLD_LT),
								pOut.getValueOfParameter(SdfThreshold.PARAM_THRESHOLD_LEQ),
								pOut.getValueOfParameter(SdfThreshold.PARAM_THRESHOLD_GEQ),
								pOut.getValueOfParameter(SdfThreshold.PARAM_THRESHOLD_GT)
						};

						for(int i=0; i<4; i++) {
							if(paramValues[i] != null) {
								sa.setAdditionalLine(compOps[i] + " " + Numeric.doubleValueOfObject(paramValues[i]), PARAM_COLOR);
							}
						}

						
					}
				}
			}
		});

	}
}
