package fr.supelec.modhelx.qdgui.rendering;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;

import fr.supelec.graphbench.Socket;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.qdgui.SocketAdapter;
import fr.supelec.modhelx.qdgui.VertexAdapter;

public class TFSMStateR extends AbstractVertexRenderer {
	private final static Color INACTIVE_COLOR = new Color(1f, 1f, .8f);
	private final static Color ACTIVE_COLOR = new Color(1f, .8f, .6f);
	
	public boolean accept(Block block) {
		return
			"TFSMState".equals(block.getValueOfParameter("TFSMRole")) ||
			"FSMState".equals(block.getValueOfParameter("FSMRole"));
	}
	
	public RendererParameters calculateRendererParameters(VertexAdapter vertex) {
		RendererParameters params = new RendererParameters(vertex, 140, 60);
		recalculatePortPositions(params);
		return params;
	}


	public void recalculatePortPositions(RendererParameters params) {
		for(Socket s : params.vertex.getSocketList()) {
			switch(s.getDirection()) {
			case IN: s.setPosition(0, params.height/2); break;
			case OUT: s.setPosition(params.width-1, params.height/2); break;
			}
			
			((SocketAdapter)s).setNormalRadius(4);
		}
	}

	
	private boolean isCurrentState(RendererParameters params) {
		ModelOfComputation moc = getMoc(params);
		Object currentState = Platform.getController().getInternalMoCVariable(moc, "currentState");
		return currentState == params.vertex.getBlock();
	}
	

	public void render(RendererParameters params, Graphics g_, int x, int y, boolean selected) {
		Graphics2D g = (Graphics2D) g_;
		int width = params.width;
		int height = params.height;
		
		drawOvalShadow(g, x, y, width, height);
		
		g.setColor(isCurrentState(params) ? ACTIVE_COLOR : INACTIVE_COLOR);
		g.fillOval(x, y, width, height);
		
		g.setColor(selected ? Color.RED : Color.BLACK);
		g.drawOval(x, y, width, height);
		
		if("TFSMInitialState".equals(params.vertex.getBlock().getValueOfParameter("TFSMFeature")))
			g.drawOval(x+3, y+3, width-6, height-6);
		
		g.drawString(
				params.vertex.getBlock().getName(), 
				x + width/2 - g.getFontMetrics().stringWidth(params.vertex.getBlock().getName())/2, 
				y + (height + g.getFontMetrics().getHeight())/2);
		
	}

	private TFSMStateR() {}
	
	private final static TFSMStateR instance;
	
	static {
		instance = new TFSMStateR();
	}
	
	public static void register() {
		VertexRenderers.register(instance);
	}
	
	@Override
	public boolean renderPinNames() {
		return false;
	}
}
