package fr.supelec.modhelx.qdgui.rendering;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;

import fr.supelec.graphbench.Socket;
import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.qdgui.SocketAdapter;
import fr.supelec.modhelx.qdgui.VertexAdapter;

public class TFSMTransitionR extends AbstractVertexRenderer {
	private final static Color COLOR = new Color(.8f, 1f, .8f);
	
	public boolean accept(Block block) {
		return
			"TFSMTransition".equals(block.getValueOfParameter("TFSMRole")) ||
			"FSMTransition".equals(block.getValueOfParameter("FSMRole"));
	}
	
	public RendererParameters calculateRendererParameters(VertexAdapter vertex) {
		RendererParameters params = new TransitionRendererParameters(vertex, 140, 60);
		recalculatePortPositions(params);
		return params;
	}


	public void recalculatePortPositions(RendererParameters params) {
		int inCnt = 0, outCnt = 0;
		List<String> text = new ArrayList<String>();
		String name = params.vertex.getBlock().getName();
		String currentLine = "";
		String currentWord = "";
		for(int i=0; i<name.length(); i++) {
			if(name.charAt(i) != '_') {
				currentWord += name.charAt(i);
			}
			if((name.charAt(i) == '_' && currentWord.length()>0) || (i == name.length()-1)) {
				if(currentLine.length() + 1 + currentWord.length() < 18) {
					currentLine += " " + currentWord;
					currentWord = "";
				} else {
					text.add(currentLine);
					currentLine = currentWord;
				}
			}
			currentLine = currentLine.trim();
			//System.out.println(name + ", " + i + ", cW=" + currentWord + ", cL=" + currentLine + ", txt=" + text);
		}
		if(currentLine.length()>0) text.add(currentLine);
		
		for(Socket s : params.vertex.getSocketList()) {
			switch(s.getDirection()) {
			case IN:
				if(inCnt == 0) s.setPosition(0, params.height/2);
				else {
					s.setPosition(40*inCnt, 0);
					((SocketAdapter)s).setRenderPin(false);
				}
				inCnt++;
				break;
			case OUT:
				if(outCnt == 0) s.setPosition(params.width-1, params.height/2);
				else {
					s.setPosition(40*outCnt, params.height-1);
					((SocketAdapter)s).setRenderPin(false);
					text.add("/" + ((SocketAdapter)s).getPin().getName());
				}
				outCnt++;
				break;
			}
			
			((SocketAdapter)s).setNormalRadius(4);
		}
		
		((TransitionRendererParameters)params).text = text.toArray(new String[] {});
		
		params.height = 16 * text.size() + 4;
	}

	
	public void render(RendererParameters params, Graphics g, int x, int y, boolean selected) {
		int width = params.width;
		int height = params.height;
		
		g.setColor(COLOR);
		g.fillRect(x, y, width, height);
		
		drawRectShadow(g, x, y, width, height);
		
		g.setColor(selected ? Color.RED : Color.BLACK);
		g.drawRect(x, y, width, height);
		
		int lineHeigth = g.getFontMetrics().getHeight();
		String[] text = ((TransitionRendererParameters)params).text;
		for(int i=0; i<text.length; i++) {
			g.drawString(text[i], x+10, y+(i+1)*lineHeigth);
		}
		
		
	}

	private TFSMTransitionR() {}
	
	private final static TFSMTransitionR instance;
	
	static {
		instance = new TFSMTransitionR();
	}
	
	public static void register() {
		VertexRenderers.register(instance);
	}
	
	@Override
	public RenderInternals renderInternals() {
		return RenderInternals.RENDER_NOTHING;
	}
	
	@Override
	public boolean renderPinNames() {
		return false;
	}
	
	private static class TransitionRendererParameters extends RendererParameters {
		public TransitionRendererParameters(VertexAdapter vertex, int width, int height) {
			super(vertex, width, height);
		}

		public String[] text;
	}
}
