/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestCoffeeMachine.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeDelay;
import fr.supelec.modhelx.mocs.delib.DeDuplicator;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DelibFactory;

public class TestCoffeeMachine {
	
	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static Clock clock = TimeFactory.eINSTANCE.createLogicalClock();

	public static void main(String[] args) {
		
		buildModel();
		saveModel();
		model = null;
		loadModel();

		Platform.setLogFile("log/coffee_machine_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		
		Platform.setModel(model, clock);
		
		Platform.run();
	}
	
	/*
	 */
	private static void buildModel() {
		model.setName("CoffeeMachine");

		DeModelOfComputation moc = DeFactory.eINSTANCE.createDeModelOfComputation();
		moc.setName("UserMoC");
		moc.setClock(clock);
		model.setMoc(moc);
		
		CompositeBlock coffeeMachine = CoreFactory.eINSTANCE.createCompositeBlock();
		coffeeMachine.setName("CoffeeMachine");
		model.setStructure(coffeeMachine);

		CompositeBlock user = CoreFactory.eINSTANCE.createCompositeBlock();
		user.setName("User");
		coffeeMachine.getBlocks().add(user);

		Pin put_coin = CoreFactory.eINSTANCE.createPin();
		put_coin.setName("put_coin");
		user.getPinsOut().add(put_coin);

		Pin ask_coffee = CoreFactory.eINSTANCE.createPin();
		ask_coffee.setName("ask_coffee");
		user.getPinsOut().add(ask_coffee);

		Pin take_coffee = CoreFactory.eINSTANCE.createPin();
		take_coffee.setName("take_coffee");
		user.getPinsIn().add(take_coffee);


		DeDuplicator user_dup = DelibFactory.eINSTANCE.createDeDuplicator();
		user_dup.setName("user_dup");
		user.getBlocks().add(user_dup);

		Pin dup_in = CoreFactory.eINSTANCE.createPin();
		dup_in.setName("dup_in");
		user_dup.getPinsIn().add(dup_in);

		Pin dup_out1 = CoreFactory.eINSTANCE.createPin();
		dup_out1.setName("dup_out1");
		user_dup.getPinsOut().add(dup_out1);

		Pin dup_out2 = CoreFactory.eINSTANCE.createPin();
		dup_out2.setName("dup_out2");
		user_dup.getPinsOut().add(dup_out2);

		Relation dup_out1_put_coin = CoreFactory.eINSTANCE.createRelation();
		dup_out1_put_coin.setSource(dup_out1);
		dup_out1_put_coin.setTarget(put_coin);
		user.getRelations().add(dup_out1_put_coin);


		DeDelay delay_coffee = DelibFactory.eINSTANCE.createDeDelay();
		delay_coffee.setName("delay_coffee");
		delay_coffee.setTimeDelay(2);
		user.getBlocks().add(delay_coffee);

		Pin delay1_in = CoreFactory.eINSTANCE.createPin();
		delay1_in.setName("delay1_in");
		delay_coffee.getPinsIn().add(delay1_in);

		Pin delay1_out = CoreFactory.eINSTANCE.createPin();
		delay1_out.setName("delay1_out");
		delay_coffee.getPinsOut().add(delay1_out);

		Relation dup_out2_delay1_in = CoreFactory.eINSTANCE.createRelation();
		dup_out2_delay1_in.setSource(dup_out2);
		dup_out2_delay1_in.setTarget(delay1_in);
		user.getRelations().add(dup_out2_delay1_in);

		Relation delay1_out_ask_coffee = CoreFactory.eINSTANCE.createRelation();
		delay1_out_ask_coffee.setSource(delay1_out);
		delay1_out_ask_coffee.setTarget(ask_coffee);
		user.getRelations().add(delay1_out_ask_coffee);

		
		DeDelay delay_drink = DelibFactory.eINSTANCE.createDeDelay();
		delay_drink.setName("delay_drink");
		delay_drink.setTimeDelay(5);
		user.getBlocks().add(delay_drink);

		Pin delay2_in = CoreFactory.eINSTANCE.createPin();
		delay2_in.setName("delay2_in");
		delay_drink.getPinsIn().add(delay2_in);

		Pin delay2_out = CoreFactory.eINSTANCE.createPin();
		delay2_out.setName("delay2_out");
		delay_drink.getPinsOut().add(delay2_out);

		Relation take_coffee_delay2_in = CoreFactory.eINSTANCE.createRelation();
		take_coffee_delay2_in.setSource(take_coffee);
		take_coffee_delay2_in.setTarget(delay2_in);
		user.getRelations().add(take_coffee_delay2_in);

		Relation delay2_out_dup_in = CoreFactory.eINSTANCE.createRelation();
		delay2_out_dup_in.setSource(delay2_out);
		delay2_out_dup_in.setTarget(dup_in);
		user.getRelations().add(delay2_out_dup_in);

}

	private static void saveModel() {
		// create resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register XML resource factory (before creating the resource)
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File("de_1.modhelx").getAbsolutePath()));
		// add the root object to the resource
		resource.getContents().add(model);
		// serialize resource
		try {
			resource.save(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
		}
	}
	
	private static boolean loadModel() {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File("de_1.modhelx").getAbsolutePath()));
		try {
			resource.load(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
			return false;
		}
		
		if (resource.getContents().size() != 1) {
			System.err.println("Error : there must be 1 element");
			return false;
		}
		
		EObject o = resource.getContents().get(0);
		try {
			model = (Model) o;
		}
		catch(ClassCastException ex) {
			System.err.println("Error : expected element not found : " + ex);
			return false;
		}
		
		// Put back transient values
		for (Block b : model.getStructure().getBlocks() ) {
			if ("PRINT".equals(b.getName())) {
				((DeEventPrinter)b).setPrintStream(System.out);
			}
		}
		
		DeModelOfComputation moc = null;
		try {
			moc = (DeModelOfComputation) model.getMoc();
		}
		catch(ClassCastException ex) {
			System.err.println("Error : bad moc found : " + ex);
			return false;
		}
		
		moc.setClock(clock);
		return true;
	}
}
