package fr.supelec.modhelx.tests;

import java.io.StringReader;
import java.util.logging.Level;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.qdgui.rendering.TFSMStateR;
import fr.supelec.modhelx.qdgui.rendering.TFSMStructureR;
import fr.supelec.modhelx.qdgui.rendering.TFSMTransitionR;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.AdapterlibFactory;
import fr.supelec.modhelx.mocs.adapterlib.DeTfsmAdapter;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUserWithoutInput;

import fr.supelec.modhelx.mocs.delib.DeConstantEmitter;
import fr.supelec.modhelx.mocs.delib.DeDelay;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DeScenarioPlayer;
import fr.supelec.modhelx.mocs.delib.DelibFactory;

import fr.supelec.modhelx.mocs.tfsm.TFsmModelOfComputation;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.util.LogFormatter;
import fr.supelec.modhelx.util.ModelBuilder;
import fr.supelec.modhelx.util.TFSMBuilder;

public class TestCoffeeMachineDETFSM {
	public static void main(String[] args) {

		// modele racine DE
		DeModelOfComputation deMoc = DeFactory.eINSTANCE
				.createDeModelOfComputation();
		deMoc.setName("DE_moc");

		Model deModel = CoreFactory.eINSTANCE.createModel();
		deModel.setName("DE_model");
		deModel.setMoc(deMoc);

		CompositeBlock deStructure = CoreFactory.eINSTANCE
				.createCompositeBlock();
		deStructure.setName("DE_structure");
		deModel.setStructure(deStructure);

		DeConstantEmitter coin = DelibFactory.eINSTANCE
				.createDeConstantEmitter();
		coin.setName("Coin");
		coin.setTimeStep(2);
		coin.setValueToEmit(new Integer(1));
		deStructure.getBlocks().add(coin);

		Pin opCoin = CoreFactory.eINSTANCE.createPin();
		opCoin.setName("Coin");
		coin.getPinsOut().add(opCoin);

		DeConstantEmitter coffee = DelibFactory.eINSTANCE
				.createDeConstantEmitter();
		coffee.setName("Coffee");
		coffee.setTimeStep(2);
		coffee.setValueToEmit(new Integer(2));
		Pin opCoffee = CoreFactory.eINSTANCE.createPin();
		opCoffee.setName("Coffee");
		coffee.getPinsOut().add(opCoffee);
		deStructure.getBlocks().add(coffee);

		DeDelay dl = DelibFactory.eINSTANCE.createDeDelay();
		dl.setName("DELAY");
		dl.setTimeDelay(1);

		Pin ipdl = CoreFactory.eINSTANCE.createPin();
		ipdl.setName("DL_IP");
		dl.getPinsIn().add(ipdl);

		Pin opdl = CoreFactory.eINSTANCE.createPin();
		opdl.setName("DL_OP");
		dl.getPinsOut().add(opdl);
		deStructure.getBlocks().add(dl);

		Relation coffee_dl = CoreFactory.eINSTANCE.createRelation();
		coffee_dl.setSource(opCoffee);
		coffee_dl.setTarget(ipdl);
		deStructure.getRelations().add(coffee_dl);

		// controller
		DeTfsmAdapter tfsm_controller = buildSimpleWindowIBForTFSMSubSystem(deModel);

		// get the TFSM clock
		deMoc.setClock(((TFsmModelOfComputation) tfsm_controller
				.getInternalModel().getMoc()).getClock());

		// connect the user to the controller
		ModelBuilder.addRelation(deStructure, opCoin,
				ModelBuilder.pinByName(tfsm_controller, "cmd_user"));
		ModelBuilder.addRelation(deStructure, opdl,
				ModelBuilder.pinByName(tfsm_controller, "cmd_user"));

		// ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(user,
		// "cmd_user"), ModelBuilder.pinByName(tfsm_controller, "cmd_user"));

		// DE Event printer for controller output
		DeEventPrinter printer = DelibFactory.eINSTANCE.createDeEventPrinter();
		printer.setName("Printer");
		printer.setPrintStream(System.out);
		Pin pInObsPrinter = CoreFactory.eINSTANCE.createPin();
		pInObsPrinter.setName("printerIn");
		printer.getPinsIn().add(pInObsPrinter);
		deStructure.getBlocks().add(printer);

		// //connect the controller to the printer
		ModelBuilder.addRelation(deStructure,
				ModelBuilder.pinByName(tfsm_controller, "opPrinter"),
				pInObsPrinter);

		Platform.setModel(deModel, deMoc.getClock());

		Platform.setLogFile("log/TestCoffeeMachineDETFSM.log");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new LogFormatter());

		// Platform.setRealTime(1000L);
		Platform.setNumberOfSnapshots(20);
		TFSMStructureR.register();
		TFSMStateR.register();
		TFSMTransitionR.register();
		ModelWindow w = new ModelWindow(deModel, deMoc.getClock(),
				"tests/testCoffeeMachineDETFSM.xy");
		// w.setFreeRun(true);

		w.setVisible(true);
		w.run();
	}

	public static DeTfsmAdapter buildSimpleWindowIBForTFSMSubSystem(
			Model deModel) {
		// controller (interface block that embeds the TFSM)
		DeTfsmAdapter ib = AdapterlibFactory.eINSTANCE.createDeTfsmAdapter();
		ib.setName("Coffee Machine");

		// ib.setInternalModel(buildSimpleWindowTFSMSubSystemModel(deModel));
		// ib.setInternalModel(buildSimpleWindowTFSMSubSystemModelWithAutomaticUpAndDown(deModel));
		ib.setInternalModel(buildSimpleWindowTFSMSubSystemModel(deModel));

		// external pins
		Pin cmd_user = ModelBuilder.addPinIn(ib, "cmd_user");
		Pin opPrinter = ModelBuilder.addPinOut(ib, "opPrinter");
		// adaptation relations
		ib.setValueAssociatedWithInterfaceRelation(
				ModelBuilder.addRelation(ib, cmd_user, "coin"), 1);
		ib.setValueAssociatedWithInterfaceRelation(
				ModelBuilder.addRelation(ib, cmd_user, "coffee"), 2);
		ib.setValueAssociatedWithInterfaceRelation(
				ModelBuilder.addRelation(ib, "coffee_prepared", opPrinter),
				"Coffee prepared");
		ib.setValueAssociatedWithInterfaceRelation(
				ModelBuilder.addRelation(ib, "coffee_served", opPrinter),
				"coffee served");
		deModel.getStructure().getBlocks().add(ib);
		return ib;

	}

	public static Model buildSimpleWindowTFSMSubSystemModel(Model deModel) {
		TFSMBuilder b = new TFSMBuilder("TestTFSM_MoC", new StringReader(
				"In coin\n" + "In coffee\n" + "Out coffee_prepared\n"
						+ "Out coffee_served\n" +

						"InitialState idle\n" + "State ready\n"
						+ "State prep\n" +

						"Transition idle ready Or coin\n"
						+ "Transition ready prep Or coffee /coffee_prepared\n"
						+ "Transition prep idle Delay 2 /coffee_served\n"));

		// clock = ((TFsmModelOfComputation)model.getMoc()).getClock();
		Model tfsmModel = b.getModel();

		return tfsmModel;
	}
}
