/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestDEFSM_CoffeeMachine.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.util.logging.Level;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.AdapterlibFactory;
import fr.supelec.modhelx.mocs.adapterlib.DeFsmAdapter;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeCoffeeMachineUserWithoutInput;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DelibFactory;
import fr.supelec.modhelx.mocs.fsm.FsmFactory;
import fr.supelec.modhelx.mocs.fsm.FsmModelOfComputation;
import fr.supelec.modhelx.mocs.fsmlib.FsmAndCondition;
import fr.supelec.modhelx.mocs.fsmlib.FsmlibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.qdgui.rendering.TFSMStateR;
import fr.supelec.modhelx.qdgui.rendering.TFSMTransitionR;
import fr.supelec.modhelx.util.LogFormatter;

public class TestDEFSM_CoffeeMachine {
	
	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static Clock clock = TimeFactory.eINSTANCE.createLogicalClock();

	public static void main(String[] args) {
		
		Platform.setLogFile("log/defsm_coffeemachine_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new LogFormatter());
		Platform.setNumberOfSnapshots(10);
		
		buildModel();
		Platform.setModel(model, clock);
		
		TFSMStateR.register();
		TFSMTransitionR.register();
		
//		Platform.run();
		ModelWindow w = new ModelWindow(model, clock, "tests/defsm_coffeemachine.xy");
		
		w.setVisible(true);
		w.run();
	}
	
	/*
	 *   +------------------------------------------------------------------------+
	 *   | TestDEFSM_CoffeeMachine                                			  DE  |
	 *   |  +-------------------------------------------------------------------+ |
	 *   |  | TestDEFSM_CoffeeMachine_Structure                     			| |
	 *   |  |                                                       			| |
	 *   |  |   +----------+ coin    +----------------+         +-----------+	| |
	 *   |  |   | User     >---------> Coffee Machine |         | DEPrinter |	| |
	 *   |  |   |          |         |       (IB)     |         |   		|	| |
	 *   |  |   |          | coffee  |       FSM      | served  |   		|	| |
	 *   |  |   |          >--------->	              >--------->			|	| |
	 *   |  |   |          |         |                |         |  			|	| |
	 *   |  |   +----------+         +----------------+         +-----------+	| |
	 *   |  |                                                       			| |
	 *   |  |                                                       			| |
	 *   |  +-------------------------------------------------------------------+ |
	 *   +------------------------------------------------------------------------+
	 */
	/*
	 *   +--------------------------------------------------------------------------+
	 *   | machine_internal_model                                              FSM  |
	 *   |  +---------------------------------------------------------------------+ |
	 *   |  | machine_internal_model_structure                                    | |
	 *   |  |                                                                     | |
	 *   |  |      +----------+ trans1:g1(coin?)               +----------+       | |
	 *   |  |      | idle     >--------------------------------> ready    |       | |
	 *   |  |      |          |                                |          |       | |
	 *   |  |      |          |                                |          |       | |
	 *   |  |      |          | trans2:g2(coffee?)/a2(served!) |          |       | |
	 *   |  |      |          <--------------------------------<          |       | |
	 *   |  |      +----------+                                +----------+       | |
	 *   |  |                                                                     | |
	 *   |  |                                                                     | |
	 *   |  +---------------------------------------------------------------------+ |
	 *   +--------------------------------------------------------------------------+
	 *   
	 */

	private static void buildModel() {
		//Create top level model
		model.setName("TestDEFSM_CoffeeMachine");

		DeModelOfComputation democ = DeFactory.eINSTANCE.createDeModelOfComputation();
		democ.setName("DE");
		democ.setClock(clock);
		model.setMoc(democ);
		
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestDEFSM_CoffeeMachine_Structure");
		model.setStructure(modelStructure);
		
//		Pin served_on_model = CoreFactory.eINSTANCE.createPin();
//		served_on_model.setName("served_on_model");
//		modelStructure.getPinsOut().add(served_on_model);

		//Create Coffee machine user
		DeCoffeeMachineUserWithoutInput user = DelibFactory.eINSTANCE.createDeCoffeeMachineUserWithoutInput();
		user.setName("user");
		modelStructure.getBlocks().add(user);
		user.setTimeBetweenCoinAndCoffee(3); // TODO ici spécifier le délai entre coin et coffee

		Pin coin_on_user = user.createCoinPin();
		Pin coffee_on_user = user.createCoffeePin();
		
		//Create Printer
		DeEventPrinter printer = DelibFactory.eINSTANCE.createDeEventPrinter();
		printer.setName("printer");
		printer.setPrintStream(System.out);
		modelStructure.getBlocks().add(printer);
		
		Pin served_on_printer = CoreFactory.eINSTANCE.createPin();
		served_on_printer.setName("served");
		printer.getPinsIn().add(served_on_printer); 

		//Create adapter for coffee machine
		DeFsmAdapter coffeeMachineAdapter = AdapterlibFactory.eINSTANCE.createDeFsmAdapter();
		coffeeMachineAdapter.setName("coffee_machine");
		modelStructure.getBlocks().add(coffeeMachineAdapter);
		coffeeMachineAdapter.setPreparationTime(6); // TODO ici spécifier le délai de service de la machine
		
		Pin coin_on_adapter = coffeeMachineAdapter.createCoinPin();
		Pin coffee_on_adapter = coffeeMachineAdapter.createCoffeePin();
		Pin served_on_adapter = coffeeMachineAdapter.createServedPin();
		
		//Create internal model for coffee machine adapter
		Model machine_internal_model = CoreFactory.eINSTANCE.createModel();
		machine_internal_model.setName("machine_internal_model");
		coffeeMachineAdapter.setInternalModel(machine_internal_model);
		
		//constraints.add(new SizeConstraint(coffeeMachineIn, h))
		
		//Create FSM moc
		FsmModelOfComputation fsmmoc = FsmFactory.eINSTANCE.createFsmModelOfComputation();
		fsmmoc.setName("FSM");
		machine_internal_model.setMoc(fsmmoc);

		//Create structure for internal model
		CompositeBlock machine_internal_model_structure = CoreFactory.eINSTANCE.createCompositeBlock();
		machine_internal_model_structure.setName("machine_internal_model_structure");
		machine_internal_model.setStructure(machine_internal_model_structure);
		
		Pin coin_on_fsmmodel = CoreFactory.eINSTANCE.createPin();
		coin_on_fsmmodel.setName("coin");
		machine_internal_model_structure.getPinsIn().add(coin_on_fsmmodel);
		
		Pin coffee_on_fsmmodel = CoreFactory.eINSTANCE.createPin();
		coffee_on_fsmmodel.setName("coffee");
		machine_internal_model_structure.getPinsIn().add(coffee_on_fsmmodel);
		
		Pin served_on_fsmmodel = CoreFactory.eINSTANCE.createPin();
		served_on_fsmmodel.setName("served");
		machine_internal_model_structure.getPinsOut().add(served_on_fsmmodel);
		
		//Connect the adapter to its internal model
		Relation coin_adapter_to_coin_intmodel = CoreFactory.eINSTANCE.createRelation();
		coin_adapter_to_coin_intmodel.setSource(coin_on_adapter);
		coin_adapter_to_coin_intmodel.setTarget(coin_on_fsmmodel);
		coffeeMachineAdapter.getRelations().add(coin_adapter_to_coin_intmodel);

		Relation coffee_adapter_to_coffee_intmodel = CoreFactory.eINSTANCE.createRelation();
		coffee_adapter_to_coffee_intmodel.setSource(coffee_on_adapter);
		coffee_adapter_to_coffee_intmodel.setTarget(coffee_on_fsmmodel);
		coffeeMachineAdapter.getRelations().add(coffee_adapter_to_coffee_intmodel);
		
		Relation served_intmodel_to_adapter_served = CoreFactory.eINSTANCE.createRelation();
		served_intmodel_to_adapter_served.setSource(served_on_fsmmodel);
		served_intmodel_to_adapter_served.setTarget(served_on_adapter);
		coffeeMachineAdapter.getRelations().add(served_intmodel_to_adapter_served);
		
		//Create the automaton in the internal model
		//Create initial idle state
		Block idle_state = CoreFactory.eINSTANCE.createAtomicBlock();
		idle_state.setName("idle");
		fsmmoc.setInitialStateFeature(idle_state);
		machine_internal_model_structure.getBlocks().add(idle_state);

		Pin from_trans2_on_idle_state = CoreFactory.eINSTANCE.createPin();
		from_trans2_on_idle_state.setName("from_trans");
		idle_state.getPinsIn().add(from_trans2_on_idle_state);

		Pin to_trans1_on_idle_state = CoreFactory.eINSTANCE.createPin();
		to_trans1_on_idle_state.setName("to_trans");
		idle_state.getPinsOut().add(to_trans1_on_idle_state);

		//Create ready state
		Block ready_state = CoreFactory.eINSTANCE.createAtomicBlock();
		ready_state.setName("ready");
		fsmmoc.setStateRole(ready_state);
		machine_internal_model_structure.getBlocks().add(ready_state);

		Pin from_trans1_on_ready_State = CoreFactory.eINSTANCE.createPin();
		from_trans1_on_ready_State.setName("from_trans");
		ready_state.getPinsIn().add(from_trans1_on_ready_State); 

		Pin to_trans2_on_ready_state = CoreFactory.eINSTANCE.createPin();
		to_trans2_on_ready_state.setName("to_trans");
		ready_state.getPinsOut().add(to_trans2_on_ready_state); 
		
		//Create transition trans1
		CompositeBlock trans1 = CoreFactory.eINSTANCE.createCompositeBlock();
		trans1.setName("trans_idle_to_ready");
		fsmmoc.setTransitionRole(trans1);
		machine_internal_model_structure.getBlocks().add(trans1);

		Pin coin_on_trans1 = CoreFactory.eINSTANCE.createPin();
		coin_on_trans1.setName("coin");
		trans1.getPinsIn().add(coin_on_trans1);

		Relation coin_evt_to_trans1 = CoreFactory.eINSTANCE.createRelation();
		coin_evt_to_trans1.setSource(coin_on_fsmmodel);
		coin_evt_to_trans1.setTarget(coin_on_trans1);
		machine_internal_model_structure.getRelations().add(coin_evt_to_trans1);

		Pin from_idle_state_on_trans1 = CoreFactory.eINSTANCE.createPin();
		from_idle_state_on_trans1.setName("from_state");
		trans1.getPinsIn().add(from_idle_state_on_trans1);

		Relation idle_state_to_trans1 = CoreFactory.eINSTANCE.createRelation();
		idle_state_to_trans1.setSource(to_trans1_on_idle_state);
		idle_state_to_trans1.setTarget(from_idle_state_on_trans1);
		machine_internal_model_structure.getRelations().add(idle_state_to_trans1);

		Pin to_ready_state_on_trans1 = CoreFactory.eINSTANCE.createPin();
		to_ready_state_on_trans1.setName("to_state");
		trans1.getPinsOut().add(to_ready_state_on_trans1);
		
		Relation trans1_to_ready_state = CoreFactory.eINSTANCE.createRelation();
		trans1_to_ready_state.setSource(to_ready_state_on_trans1);
		trans1_to_ready_state.setTarget(from_trans1_on_ready_State);
		machine_internal_model_structure.getRelations().add(trans1_to_ready_state);


		//Create inside guard g1 of trans1
		FsmAndCondition g1 = FsmlibFactory.eINSTANCE.createFsmAndCondition();
		//g1.setTokenToEmit(fsmmoc.createEvent());
		g1.setName("coin_guard");
		fsmmoc.setGuardRole(g1);
		trans1.getBlocks().add(g1);

		Pin from_idle_state_on_g1 = CoreFactory.eINSTANCE.createPin();
		from_idle_state_on_g1.setName("from_state");
		g1.getPinsIn().add(from_idle_state_on_g1);

		Relation inside_trans1_to_g1_from_idle_state = CoreFactory.eINSTANCE.createRelation();
		inside_trans1_to_g1_from_idle_state.setSource(from_idle_state_on_trans1);
		inside_trans1_to_g1_from_idle_state.setTarget(from_idle_state_on_g1);
		trans1.getRelations().add(inside_trans1_to_g1_from_idle_state);

		Pin coin_on_g1 = CoreFactory.eINSTANCE.createPin();
		coin_on_g1.setName("from_coin_evt");
		g1.getPinsIn().add(coin_on_g1);

		Relation inside_trans1_to_g1_from_coin_evt = CoreFactory.eINSTANCE.createRelation();
		inside_trans1_to_g1_from_coin_evt.setSource(coin_on_trans1);
		inside_trans1_to_g1_from_coin_evt.setTarget(coin_on_g1);
		trans1.getRelations().add(inside_trans1_to_g1_from_coin_evt);

		Pin to_ready_state_on_g1 = CoreFactory.eINSTANCE.createPin();
		to_ready_state_on_g1.setName("to_state");
		g1.getPinsOut().add(to_ready_state_on_g1);
		
		Relation inside_trans1_to_trans1_ready_state = CoreFactory.eINSTANCE.createRelation();
		inside_trans1_to_trans1_ready_state.setSource(to_ready_state_on_g1);
		inside_trans1_to_trans1_ready_state.setTarget(to_ready_state_on_trans1);
		trans1.getRelations().add(inside_trans1_to_trans1_ready_state);

		//Create transition trans2
		CompositeBlock trans2 = CoreFactory.eINSTANCE.createCompositeBlock();
		trans2.setName("trans_ready_to_idle");
		fsmmoc.setTransitionRole(trans2);
		machine_internal_model_structure.getBlocks().add(trans2);

		Pin coffee_on_trans2 = CoreFactory.eINSTANCE.createPin();
		coffee_on_trans2.setName("from_coffee_evt");
		trans2.getPinsIn().add(coffee_on_trans2);

		Relation coffee_evt_to_trans2 = CoreFactory.eINSTANCE.createRelation();
		coffee_evt_to_trans2.setSource(coffee_on_fsmmodel);
		coffee_evt_to_trans2.setTarget(coffee_on_trans2);
		machine_internal_model_structure.getRelations().add(coffee_evt_to_trans2);

		Pin from_ready_state_on_trans2 = CoreFactory.eINSTANCE.createPin();
		from_ready_state_on_trans2.setName("from_state");
		trans2.getPinsIn().add(from_ready_state_on_trans2);

		Relation ready_state_to_trans2 = CoreFactory.eINSTANCE.createRelation();
		ready_state_to_trans2.setSource(to_trans2_on_ready_state);
		ready_state_to_trans2.setTarget(from_ready_state_on_trans2);
		machine_internal_model_structure.getRelations().add(ready_state_to_trans2);

		Pin to_idle_state_on_trans2 = CoreFactory.eINSTANCE.createPin();
		to_idle_state_on_trans2.setName("to_state");
		trans2.getPinsOut().add(to_idle_state_on_trans2);
		
		Relation trans2_to_idle_state = CoreFactory.eINSTANCE.createRelation();
		trans2_to_idle_state.setSource(to_idle_state_on_trans2);
		trans2_to_idle_state.setTarget(from_trans2_on_idle_state);
		machine_internal_model_structure.getRelations().add(trans2_to_idle_state);

		Pin served_on_trans2 = CoreFactory.eINSTANCE.createPin();
		served_on_trans2.setName("served");
		trans2.getPinsOut().add(served_on_trans2);
		
		Relation trans2_to_served_evt = CoreFactory.eINSTANCE.createRelation();
		trans2_to_served_evt.setSource(served_on_trans2);
		trans2_to_served_evt.setTarget(served_on_fsmmodel);
		machine_internal_model_structure.getRelations().add(trans2_to_served_evt);

		//Create inside guard g2
//		AndConstantEmitter g2 = PolymoclibFactory.eINSTANCE.createAndConstantEmitter();
//		g2.setTokenToEmit(fsmmoc.createEvent());
		FsmAndCondition g2 = FsmlibFactory.eINSTANCE.createFsmAndCondition();
		g2.setName("coffee_guard");
		fsmmoc.setGuardRole(g2);
		trans2.getBlocks().add(g2);

		Pin from_ready_state_on_g2 = CoreFactory.eINSTANCE.createPin();
		from_ready_state_on_g2.setName("from_state");
		g2.getPinsIn().add(from_ready_state_on_g2);

		Relation inside_trans2_to_g2_ready_state = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_g2_ready_state.setSource(from_ready_state_on_trans2);
		inside_trans2_to_g2_ready_state.setTarget(from_ready_state_on_g2);
		trans2.getRelations().add(inside_trans2_to_g2_ready_state);

		Pin coffee_on_g2 = CoreFactory.eINSTANCE.createPin();
		coffee_on_g2.setName("from_coffee_evt");
		g2.getPinsIn().add(coffee_on_g2);

		Relation inside_trans2_to_g2_coffee_evt = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_g2_coffee_evt.setSource(coffee_on_trans2);
		inside_trans2_to_g2_coffee_evt.setTarget(coffee_on_g2);
		trans2.getRelations().add(inside_trans2_to_g2_coffee_evt);

		Pin to_idle_state_on_g2 = CoreFactory.eINSTANCE.createPin();
		to_idle_state_on_g2.setName("to_state");
		g2.getPinsOut().add(to_idle_state_on_g2);
		
		Relation inside_trans2_to_trans2_idle_state = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_trans2_idle_state.setSource(to_idle_state_on_g2);
		inside_trans2_to_trans2_idle_state.setTarget(to_idle_state_on_trans2);
		trans2.getRelations().add(inside_trans2_to_trans2_idle_state);

		//constraints.add(new ManhattanConstraint(inside_trans2_to_trans2_idle_state, -30));
		
		Pin to_a2 = CoreFactory.eINSTANCE.createPin();
		to_a2.setName("to_action");
		g2.getPinsOut().add(to_a2);
		
		//Create inside action a2
//		AndConstantEmitter a2 = PolymoclibFactory.eINSTANCE.createAndConstantEmitter();
//		a2.setTokenToEmit(CoreFactory.eINSTANCE.createToken());
		FsmAndCondition a2 = FsmlibFactory.eINSTANCE.createFsmAndCondition();
		a2.setName("served_action");
		fsmmoc.setActionRole(a2);
		trans2.getBlocks().add(a2);

		Pin from_g2 = CoreFactory.eINSTANCE.createPin();
		from_g2.setName("from_guard");
		a2.getPinsIn().add(from_g2);
		
		Relation g2_to_a2 = CoreFactory.eINSTANCE.createRelation();
		g2_to_a2.setSource(to_a2);
		g2_to_a2.setTarget(from_g2);
		trans2.getRelations().add(g2_to_a2);

		Pin served_on_a2 = CoreFactory.eINSTANCE.createPin();
		served_on_a2.setName("to_served_evt");
		a2.getPinsOut().add(served_on_a2);
		
		Relation inside_trans2_to_trans2_served_evt = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_trans2_served_evt.setSource(served_on_a2);
		inside_trans2_to_trans2_served_evt.setTarget(served_on_trans2);
		trans2.getRelations().add(inside_trans2_to_trans2_served_evt);

		//Connect the user to the coffee machine
		Relation user_coin_to_machine_coin = CoreFactory.eINSTANCE.createRelation();
		user_coin_to_machine_coin.setSource(coin_on_user);
		user_coin_to_machine_coin.setTarget(coin_on_adapter);
		modelStructure.getRelations().add(user_coin_to_machine_coin);
		
		Relation user_coffee_to_machine_coffee = CoreFactory.eINSTANCE.createRelation();
		user_coffee_to_machine_coffee.setSource(coffee_on_user);
		user_coffee_to_machine_coffee.setTarget(coffee_on_adapter);
		modelStructure.getRelations().add(user_coffee_to_machine_coffee);
		
		//Connect coffee machine to output model
		Relation machine_served_to_model_served = CoreFactory.eINSTANCE.createRelation();
		machine_served_to_model_served.setSource(served_on_adapter);
		machine_served_to_model_served.setTarget(served_on_printer);
		modelStructure.getRelations().add(machine_served_to_model_served);
	}
	
}
