/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestDE_1.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeAdder;
import fr.supelec.modhelx.mocs.delib.DeConstantEmitter;
import fr.supelec.modhelx.mocs.delib.DeDuplicator;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DelibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.util.DEBlockDependencyComparator;
import fr.supelec.modhelx.util.LogFormatter;

public class TestDE_1 {
	
	protected static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static Clock clock = TimeFactory.eINSTANCE.createLogicalClock();

	public static void main(String[] args) {
		
		buildModel();
		saveModel("tests/de_1.modhelx");
		model = null;
		loadModel("tests/de_1.modhelx");

		Platform.setLogFile("log/de_1_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new LogFormatter());
		Platform.setNumberOfSnapshots(10);
		
		Platform.setModel(model, clock);

		ModelWindow w = new ModelWindow(model, clock);
		w.setVisible(true);
		w.run();
		
		
		Platform.run();
	}
	
	/*
	 */
	protected static void buildModel() {
		model.setName("DE_Model_1");

		DeModelOfComputation moc = DeFactory.eINSTANCE.createDeModelOfComputation();
		moc.setName("MoC");
		moc.setClock(clock);
		model.setMoc(moc);
		
		CompositeBlock cb = CoreFactory.eINSTANCE.createCompositeBlock();
		cb.setName("CB");
		model.setStructure(cb);

		DeConstantEmitter ce1 = DelibFactory.eINSTANCE.createDeConstantEmitter();
		ce1.setName("CE1");
		ce1.setTimeStep(1);
		ce1.setValueToEmit(new Integer(10));
		cb.getBlocks().add(ce1);

		Pin opce1 = CoreFactory.eINSTANCE.createPin();
		opce1.setName("CE1_OP");
		ce1.getPinsOut().add(opce1); 

		DeConstantEmitter ce2 = DelibFactory.eINSTANCE.createDeConstantEmitter();
		ce2.setName("CE2");
		ce2.setTimeStep(2);
		ce2.setValueToEmit(new Integer(2));
		cb.getBlocks().add(ce2);

		Pin opce2 = CoreFactory.eINSTANCE.createPin();
		opce2.setName("CE2_OP");
		ce2.getPinsOut().add(opce2); 

		DeConstantEmitter ce3 = DelibFactory.eINSTANCE.createDeConstantEmitter();
		ce3.setName("CE3");
		ce3.setTimeStep(3);
		ce3.setValueToEmit(new Integer(4));
		cb.getBlocks().add(ce3);

		Pin opce3 = CoreFactory.eINSTANCE.createPin();
		opce3.setName("CE3_OP");
		ce3.getPinsOut().add(opce3); 

		DeDuplicator dup = DelibFactory.eINSTANCE.createDeDuplicator();
		dup.setName("DUP");
		cb.getBlocks().add(dup);
		
		Pin opdup = CoreFactory.eINSTANCE.createPin();
		opdup.setName("DUP_IN");
		dup.getPinsIn().add(opdup); 

		Pin opdup1 = CoreFactory.eINSTANCE.createPin();
		opdup1.setName("DUP_OP1");
		dup.getPinsOut().add(opdup1); 

		Pin opdup2 = CoreFactory.eINSTANCE.createPin();
		opdup2.setName("DUP_OP2");
		dup.getPinsOut().add(opdup2); 

		DeAdder add = DelibFactory.eINSTANCE.createDeAdder();
		add.setName("ADD");
		cb.getBlocks().add(add);
		
		Pin opadd1 = CoreFactory.eINSTANCE.createPin();
		opadd1.setName("ADD_OP1");
		add.getPinsIn().add(opadd1); 

		Pin opadd2 = CoreFactory.eINSTANCE.createPin();
		opadd2.setName("ADD_OP2");
		add.getPinsIn().add(opadd2); 

		Pin opadd3 = CoreFactory.eINSTANCE.createPin();
		opadd3.setName("ADD_OP3");
		add.getPinsIn().add(opadd3); 

		Pin opout = CoreFactory.eINSTANCE.createPin();
		opout.setName("ADD_OPOUT");
		add.getPinsOut().add(opout); 

		DeEventPrinter print = DelibFactory.eINSTANCE.createDeEventPrinter();
		print.setName("PRINT");
		print.setPrintStream(System.out);
		cb.getBlocks().add(print);
		
		Pin opp = CoreFactory.eINSTANCE.createPin();
		opp.setName("PRINT_OP");
		print.getPinsIn().add(opp); 

		Relation ce1_add = CoreFactory.eINSTANCE.createRelation();
		ce1_add.setSource(opce1);
		ce1_add.setTarget(opadd1);
		cb.getRelations().add(ce1_add);
		
		Relation ce2_add = CoreFactory.eINSTANCE.createRelation();
		ce2_add.setSource(opce2);
		ce2_add.setTarget(opadd2);
		cb.getRelations().add(ce2_add);
		
		Relation ce3_dup = CoreFactory.eINSTANCE.createRelation();
		ce3_dup.setSource(opce3);
		ce3_dup.setTarget(opdup);
		cb.getRelations().add(ce3_dup);
		
		Relation dup_add1 = CoreFactory.eINSTANCE.createRelation();
		dup_add1.setSource(opdup1);
		dup_add1.setTarget(opadd3);
		cb.getRelations().add(dup_add1);
		
		Relation dup_add2 = CoreFactory.eINSTANCE.createRelation();
		dup_add2.setSource(opdup2);
		dup_add2.setTarget(opadd3);
		cb.getRelations().add(dup_add2);
		
		Relation add_pr = CoreFactory.eINSTANCE.createRelation();
		add_pr.setSource(opout);
		add_pr.setTarget(opp);
		cb.getRelations().add(add_pr);
		
		System.out.println("CE1 < ADD = " + DEBlockDependencyComparator.isBefore(ce1, add));
		System.out.println("ADD < CE1 = " + DEBlockDependencyComparator.isBefore(add, ce1));
		
}

	private static void saveModel(String name) {
		// create resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register XML resource factory (before creating the resource)
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		// add the root object to the resource
		resource.getContents().add(model);
		// serialize resource
		try {
			resource.save(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
		}
	}
	
	private static boolean loadModel(String name) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		try {
			resource.load(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
			return false;
		}
		
		if (resource.getContents().size() != 1) {
			System.err.println("Error : there must be 1 element");
			return false;
		}
		
		EObject o = resource.getContents().get(0);
		try {
			model = (Model) o;
		}
		catch(ClassCastException ex) {
			System.err.println("Error : expected element not found : " + ex);
			return false;
		}
		
		// Put back transient values
		for (Block b : model.getStructure().getBlocks() ) {
			if ("PRINT".equals(b.getName())) {
				((DeEventPrinter)b).setPrintStream(System.out);
			}
		}
		
		DeModelOfComputation moc = null;
		try {
			moc = (DeModelOfComputation) model.getMoc();
		}
		catch(ClassCastException ex) {
			System.err.println("Error : bad moc found : " + ex);
			return false;
		}
		
		moc.setClock(clock);
		return true;
	}
}
