/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestDE_3.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeDelay;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DelibFactory;

public class TestDE_3 {
	
	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static Clock clock = TimeFactory.eINSTANCE.createLogicalClock();

	public static void main(String[] args) {
		
		buildModel();
		saveModel("tests/de_3.modhelx");
		if (args.length > 0) {
			model = null;
			loadModel("tests/de_3.modhelx");
		}	
		//Platform.setLogFile("log/de_3_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(10);
		
		Platform.setModel(model, clock);
		
//		//Depose d'un token sur l'entree du mod�le
//		IntergerToken ex = CorelibFactory.eINSTANCE.createIntergerToken();
//		fakeUserInput.addToken(ex);
//
//		System.out.println("Got " + fakeUserInput.getStoredTokens().size() + " events on fakeUserInput Pin");
		
		Platform.run();
	}
	
	/*
	 *   +---------------------------------------------+
	 *   | TestDE_3                                    |
	 *   |  +----------------------------------------+ |
	 *   |  | TestDE_3_Structure                     | |
	 *   |  |                                        | |
	 *   |  |   +----------+                         | |
	 *   |  |   | user     >-------------------------> |
	 *   |  |   |          |         +--------+      | |
	 *   |  | +->          |         | delay1 |      | |
	 *   |  | | |          >--------->	     >---+   | |
	 *   |  | | |          |         |        |  |   | |
	 *   |  | | +----------+         +--------+  |   | |
	 *   |  | |                                  |   | |
	 *   |  | +----------------------------------+   | |
	 *   |  +----------------------------------------+ |
	 *   +---------------------------------------------+
	 */
	private static void buildModel() {
		model.setName("TestDE_3");

		DeModelOfComputation moc = DeFactory.eINSTANCE.createDeModelOfComputation();
		moc.setName("TestDE_3_MoC");
		moc.setClock(clock);
		model.setMoc(moc);
		
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestDE_3_Structure");
		model.setStructure(modelStructure);

//		Pin take_coffee = CoreFactory.eINSTANCE.createPin();
//		take_coffee.setName("take_coffee");
//		modelStructure.getPinsIn().add(take_coffee);
//		fakeUserInput = take_coffee;

//		Pin put_coin = CoreFactory.eINSTANCE.createPin();
//		put_coin.setName("put_coin");
//		modelStructure.getPinsOut().add(put_coin);
		Pin put_coin = modelStructure.newOutputPin("put_coin");
//		Pin ask_coffee = CoreFactory.eINSTANCE.createPin();
//		ask_coffee.setName("ask_coffee");
//		modelStructure.getPinsOut().add(ask_coffee);


		
//		DeCoffeeMachineUser user = DelibFactory.eINSTANCE.createDeCoffeeMachineUser();
//		user.setName("user");
//		modelStructure.getBlocks().add(user);
		Block user = modelStructure.addBlock(DelibFactory.eINSTANCE.createDeCoffeeMachineUser(), "user");
		
//		Pin user_in_takeCoffee = CoreFactory.eINSTANCE.createPin();
//		user_in_takeCoffee.setName("user_in_takeCoffee");
//		user.getPinsIn().add(user_in_takeCoffee);
		Pin user_in_takeCoffee = user.newInputPin("user_in_takeCoffee"); 
			
		Pin user_out1_putCoin = CoreFactory.eINSTANCE.createPin();
		user_out1_putCoin.setName("user_out1_putCoin");
		user.getPinsOut().add(user_out1_putCoin);

		Pin user_out2_askCoffee = CoreFactory.eINSTANCE.createPin();
		user_out2_askCoffee.setName("user_out2_askCoffee");
		user.getPinsOut().add(user_out2_askCoffee);

		Relation user_out1_put_coin = CoreFactory.eINSTANCE.createRelation();
		user_out1_put_coin.setSource(user_out1_putCoin);
		user_out1_put_coin.setTarget(put_coin);
		modelStructure.getRelations().add(user_out1_put_coin);


		DeDelay delay1 = DelibFactory.eINSTANCE.createDeDelay();
		delay1.setName("delay1");
		delay1.setTimeDelay(2);
		modelStructure.getBlocks().add(delay1);

		Pin delay1_in = CoreFactory.eINSTANCE.createPin();
		delay1_in.setName("delay1_in");
		delay1.getPinsIn().add(delay1_in);

		Pin delay1_out = CoreFactory.eINSTANCE.createPin();
		delay1_out.setName("delay1_out");
		delay1.getPinsOut().add(delay1_out);

		Relation dup_out2_delay1_in = CoreFactory.eINSTANCE.createRelation();
		dup_out2_delay1_in.setSource(user_out2_askCoffee);
		dup_out2_delay1_in.setTarget(delay1_in);
		modelStructure.getRelations().add(dup_out2_delay1_in);

		Relation delay1_out_user_in = CoreFactory.eINSTANCE.createRelation();
		delay1_out_user_in.setSource(delay1_out);
		delay1_out_user_in.setTarget(user_in_takeCoffee);
		modelStructure.getRelations().add(delay1_out_user_in);
}

	private static void saveModel(String name) {
		// create resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register XML resource factory (before creating the resource)
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		// add the root object to the resource
		resource.getContents().add(model);
		// serialize resource
		try {
			resource.save(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
		}
	}
	
	private static boolean loadModel(String name) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		try {
			resource.load(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
			return false;
		}
		
		if (resource.getContents().size() != 1) {
			System.err.println("Error : there must be 1 element");
			return false;
		}
		
		EObject o = resource.getContents().get(0);
		try {
			model = (Model) o;
		}
		catch(ClassCastException ex) {
			System.err.println("Error : expected element not found : " + ex);
			return false;
		}
		
		// Put back transient values
		for (Block b : model.getStructure().getBlocks() ) {
			if ("PRINT".equals(b.getName())) {
				((DeEventPrinter)b).setPrintStream(System.out);
			}
		}
		
		DeModelOfComputation moc = null;
		try {
			moc = (DeModelOfComputation) model.getMoc();
		}
		catch(ClassCastException ex) {
			System.err.println("Error : bad moc found : " + ex);
			return false;
		}
		
		moc.setClock(clock);
		return true;
	}
}
