package fr.supelec.modhelx.tests;

import java.util.logging.Level;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.AdapterlibFactory;
import fr.supelec.modhelx.mocs.adapterlib.DeSdfAdapter;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DeRampEmitter;
import fr.supelec.modhelx.mocs.delib.DelibFactory;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfAccumulator;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.qdgui.rendering.DERampEmitterR;
import fr.supelec.modhelx.util.LogFormatter;
import fr.supelec.modhelx.util.ModelIO;

public class TestDE_SDF_1 {
	
	private static Model deModel = CoreFactory.eINSTANCE.createModel();
	private static Clock clock;
	

	public static void main(String[] args) {
		DERampEmitterR.register();
		
		buildModel();
		ModelIO.saveModel(deModel, "tests/de_sdf_1.modhelx");
//		model = null;
//		loadModel("tests/sdf_1.modhelx");

		Platform.setLogFile("log/de_sdf_1_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new LogFormatter());
		Platform.setNumberOfSnapshots(10);
		
		//Platform.setModel(deModel);
		

		
		ModelWindow w = new ModelWindow(deModel, clock);
		w.setVisible(true);
		w.run();
		
		
		
		//Platform.run();
	}
	

	private static void buildModel() {
		//// 1) SDF MODEL (INTERNAL)
		Pin pOutSDF, pInSDF;
		Model sdfModel = CoreFactory.eINSTANCE.createModel();
		sdfModel.setName("mSDF");
		
		clock = TimeFactory.eINSTANCE.createLogicalClock();

		{
			//MoC
			SdfModelOfComputation sdfMoc = SdfFactory.eINSTANCE.createSdfModelOfComputation();
			sdfMoc.setName("mSDF_MoC");
			sdfMoc.setClock(clock);
			sdfModel.setMoc(sdfMoc);
			
			//Structure
			CompositeBlock sdfModelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
			sdfModelStructure.setName("mSDF_Structure");
			sdfModel.setStructure(sdfModelStructure);
	
			pOutSDF = CoreFactory.eINSTANCE.createPin();
			pOutSDF.setName("pOutSDF");
			sdfMoc.setPinRate(pOutSDF, 1);
			sdfModelStructure.getPinsOut().add(pOutSDF);
			
			pInSDF = CoreFactory.eINSTANCE.createPin();
			pInSDF.setName("pInSDF");
			sdfMoc.setPinRate(pInSDF, 1);
			sdfModelStructure.getPinsIn().add(pInSDF);
			
	
			// Accumulator
			SdfAccumulator sdfAccumulator = SdflibFactory.eINSTANCE.createSdfAccumulator();
			sdfAccumulator.setName("SdfAccumulator");
			sdfModelStructure.getBlocks().add(sdfAccumulator);
			
			Pin pOutAcc = CoreFactory.eINSTANCE.createPin();
			pOutAcc.setName("pOutAcc");
			sdfMoc.setPinRate(pOutAcc, 1);
			sdfAccumulator.getPinsOut().add(pOutAcc);
	
			Pin pInAcc = CoreFactory.eINSTANCE.createPin();
			pInAcc.setName("pInAcc");
			sdfMoc.setPinRate(pInAcc, 1);
			sdfAccumulator.getPinsIn().add(pInAcc);
	
	
			// Relation Accumulator -> Output pin
			Relation r_pOutAcc_pOut = CoreFactory.eINSTANCE.createRelation();
			r_pOutAcc_pOut.setSource(pOutAcc);
			r_pOutAcc_pOut.setTarget(pOutSDF);
			sdfModelStructure.getRelations().add(r_pOutAcc_pOut);
	
			// Relation Input pin -> Accumulator
			Relation r_pIn_pInAcc = CoreFactory.eINSTANCE.createRelation();
			r_pIn_pInAcc.setSource(pInSDF);
			r_pIn_pInAcc.setTarget(pInAcc);
			sdfModelStructure.getRelations().add(r_pIn_pInAcc);
		}
		
		
		//// 2) DE MODEL (EXTERNAL)
		{
			deModel.setName("mDE");
			
			DeModelOfComputation deMoc = DeFactory.eINSTANCE.createDeModelOfComputation();
			deMoc.setName("mDE_MoC");
			deMoc.setClock(clock); // TODO same clock???
			deModel.setMoc(deMoc);
			
			//Structure
			CompositeBlock deModelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
			deModelStructure.setName("mDE_Structure");
			deModel.setStructure(deModelStructure);
	
			/* No output pin for DE
			Pin pOutDE = CoreFactory.eINSTANCE.createPin();
			pOutDE.setName("pOutDE");
			deModelStructure.getPinsOut().add(pOutDE);
			*/
			
			/*  No input pin for DE
			Pin pInDE = CoreFactory.eINSTANCE.createPin();
			pInDE.setName("pInDE");
			deModelStructure.getPinsIn().add(pInDE);
			*/
			
			// Contents: interface block
			DeSdfAdapter adapter = AdapterlibFactory.eINSTANCE.createDeSdfAdapter();
			adapter.setName("DE_SDF_Adapter");
			adapter.setInternalModel(sdfModel);
			adapter.setSampleTime(2);
			
			// Input connection for interface block ("adaptIn")
			Pin pAdapterIn = CoreFactory.eINSTANCE.createPin();
			pAdapterIn.setName("pAdapterIn");
			adapter.getPinsIn().add(pAdapterIn);
			Relation r_pAdapterIn_pInSDF = CoreFactory.eINSTANCE.createRelation();
			r_pAdapterIn_pInSDF.setSource(pAdapterIn);
			r_pAdapterIn_pInSDF.setTarget(pInSDF);
			adapter.getRelations().add(r_pAdapterIn_pInSDF);
			adapter.setInitialValue(pInSDF, 0);		// sets the initial value
			
			// Output connection for interface block ("adaptOut")
			Pin pAdapterOut = CoreFactory.eINSTANCE.createPin();
			pAdapterOut.setName("pAdapterOut");
			adapter.getPinsOut().add(pAdapterOut);
			Relation r_pOutSDF_pAdapterOut = CoreFactory.eINSTANCE.createRelation();
			r_pOutSDF_pAdapterOut.setSource(pOutSDF);
			r_pOutSDF_pAdapterOut.setTarget(pAdapterOut);
			adapter.getRelations().add(r_pOutSDF_pAdapterOut);
			
			deModelStructure.getBlocks().add(adapter);
			
			// Contents: ramp emitter
			DeRampEmitter re = DelibFactory.eINSTANCE.createDeRampEmitter();
			re.setName("DERamp");
			re.setTimeStep(3);
			re.setInitialValueToEmit(5);
			re.setIncrement(2);
			deModelStructure.getBlocks().add(re);
			
			Pin pCeOut = CoreFactory.eINSTANCE.createPin();
			pCeOut.setName("pCeOut");
			re.getPinsOut().add(pCeOut);
			
			// Relation CE(out) -> adapter(in)
			Relation r_pCeOut_pAdapterIn = CoreFactory.eINSTANCE.createRelation();
			r_pCeOut_pAdapterIn.setSource(pCeOut);
			r_pCeOut_pAdapterIn.setTarget(pAdapterIn);
			deModelStructure.getRelations().add(r_pCeOut_pAdapterIn);
			
			/*
			// Relation DE input pin -> adapter
			Relation r_pInDE_pAdapterIn = CoreFactory.eINSTANCE.createRelation();
			r_pInDE_pAdapterIn.setSource(pInDE);
			r_pInDE_pAdapterIn.setTarget(pAdapterIn);
			deModelStructure.getRelations().add(r_pInDE_pAdapterIn);
			*/
			
			/*
			// Relation adapter -> DE output pin
			Relation r_pAdapterOut_pOutDE = CoreFactory.eINSTANCE.createRelation();
			r_pAdapterOut_pOutDE.setSource(pAdapterOut);
			r_pAdapterOut_pOutDE.setTarget(pOutDE);
			deModelStructure.getRelations().add(r_pAdapterOut_pOutDE);
			*/
			
			DeEventPrinter ep = DelibFactory.eINSTANCE.createDeEventPrinter();
			ep.setName("DEPrinter");
			ep.setPrintStream(System.out);
			deModelStructure.getBlocks().add(ep);
			
			Pin pEpIn = CoreFactory.eINSTANCE.createPin();
			pEpIn.setName("pEpIn");
			ep.getPinsIn().add(pEpIn);
			
			Relation r_pAdapterOut_pEpIn = CoreFactory.eINSTANCE.createRelation();
			r_pAdapterOut_pEpIn.setSource(pAdapterOut);
			r_pAdapterOut_pEpIn.setTarget(pEpIn);
			deModelStructure.getRelations().add(r_pAdapterOut_pEpIn);
		}
		
	}

}
