/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestFSM_1.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.fsm.FsmFactory;
import fr.supelec.modhelx.mocs.fsm.FsmModelOfComputation;
import fr.supelec.modhelx.mocs.polymoclib.AndConstantEmitter;
import fr.supelec.modhelx.mocs.polymoclib.PolymoclibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;

public class TestFSM_1 {
	
	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static FsmModelOfComputation moc;
	
	private static Pin evtx = null;
	private static Pin evty = null;
	private static Pin evtz = null;
	
	public static void main(String[] args) {
		
		buildModel();
		
//		
		saveModel("tests/fsm_1.modhelx");
		if (args.length > 0) {
			model = null;
			loadModel("tests/fsm_1.modhelx");
		}
		// add tokens in input
		Token ex = moc.createEvent();
		evtx.addToken(ex);
		ex = moc.createEvent();
		evty.addToken(ex);

		Platform.setLogFile("log/fsm_1_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		
		// use one argument to use the graphical UI, no argument to use text mode
//		if(args.length > 0) {
			ModelWindow w = new ModelWindow(model, TimeFactory.eINSTANCE.createLogicalClock());
			w.setVisible(true);
			w.run();
//		} else { 
//			
//			Platform.setModel(model);
//			
//			Platform.run();
//			
//			System.out.println("Got " + evtz.getStoredTokens().size() + " events on evtz Pin");
//		}
	}
	
	/*
	 *   +----------------------------------------------------------------+
	 *   | TestFSM_1                                                 FSM  |
	 *   |  +-----------------------------------------------------------+ |
	 *   |  | TestFSM_1_Structure                                       | |
	 *   |  |                                                           | |
	 *   |  |      +----------+ trans1:g1(x?)        +----------+       | |
	 *   |  |      | state1   >----------------------> state2   |       | |
	 *   |  |      |          |                      |          |       | |
	 *   |  |      |          |                      |          |       | |
	 *   |  |      |          | trans2:g2(y?)/a2(z!) |          |       | |
	 *   |  |      |          <----------------------<          |       | |
	 *   |  |      +----------+                      +----------+       | |
	 *   |  |                                                           | |
	 *   |  |                                                           | |
	 *   |  +-----------------------------------------------------------+ |
	 *   +----------------------------------------------------------------+
	 *   
	 */
	private static void buildModel() {
		model.setName("TestFSM_1");

		/* FsmModelOfComputation */ moc = FsmFactory.eINSTANCE.createFsmModelOfComputation();
		moc.setName("FSM");
		model.setMoc(moc);
		
		//Create model structure
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestFSM_1_Structure");
		model.setStructure(modelStructure);

		/*Pin*/ evtx = CoreFactory.eINSTANCE.createPin();
		evtx.setName("evtx");
		modelStructure.getPinsIn().add(evtx);

		/*Pin*/ evty = CoreFactory.eINSTANCE.createPin();
		evty.setName("evty");
		modelStructure.getPinsIn().add(evty);

		/*Pin*/ evtz = CoreFactory.eINSTANCE.createPin();
		evtz.setName("evtz");
		modelStructure.getPinsOut().add(evtz);

		//Create initial state state1
		Block state1 = CoreFactory.eINSTANCE.createAtomicBlock();
		state1.setName("state1");
		moc.setInitialStateFeature(state1);
		modelStructure.getBlocks().add(state1);

		Pin from_trans2 = CoreFactory.eINSTANCE.createPin();
		from_trans2.setName("from_trans2");
		state1.getPinsIn().add(from_trans2);

		Pin to_trans1 = CoreFactory.eINSTANCE.createPin();
		to_trans1.setName("to_trans1");
		state1.getPinsOut().add(to_trans1);


		//Create transition trans1
		CompositeBlock trans1 = CoreFactory.eINSTANCE.createCompositeBlock();
		trans1.setName("trans1");
		moc.setTransitionRole(trans1);
		modelStructure.getBlocks().add(trans1);

		Pin from_evtx = CoreFactory.eINSTANCE.createPin();
		from_evtx.setName("from_evtx");
		trans1.getPinsIn().add(from_evtx);

		Relation evtx_to_trans1 = CoreFactory.eINSTANCE.createRelation();
		evtx_to_trans1.setSource(evtx);
		evtx_to_trans1.setTarget(from_evtx);
		modelStructure.getRelations().add(evtx_to_trans1);

		Pin from_state1 = CoreFactory.eINSTANCE.createPin();
		from_state1.setName("from_state1");
		trans1.getPinsIn().add(from_state1);

		Relation state1_to_trans1 = CoreFactory.eINSTANCE.createRelation();
		state1_to_trans1.setSource(to_trans1);
		state1_to_trans1.setTarget(from_state1);
		modelStructure.getRelations().add(state1_to_trans1);

		Pin to_state2 = CoreFactory.eINSTANCE.createPin();
		to_state2.setName("to_state2");
		trans1.getPinsOut().add(to_state2);
		

		//Create inside guard g1 of trans1
		AndConstantEmitter g1 = PolymoclibFactory.eINSTANCE.createAndConstantEmitter();
		g1.setTokenToEmit(moc.createEvent());
		g1.setName("g1");
		moc.setGuardRole(g1);
		trans1.getBlocks().add(g1);

		Pin from_state1_on_g1 = CoreFactory.eINSTANCE.createPin();
		from_state1_on_g1.setName("from_state1_on_g1");
		g1.getPinsIn().add(from_state1_on_g1);

		Relation inside_trans1_to_g1_from_state1 = CoreFactory.eINSTANCE.createRelation();
		inside_trans1_to_g1_from_state1.setSource(from_state1);
		inside_trans1_to_g1_from_state1.setTarget(from_state1_on_g1);
		trans1.getRelations().add(inside_trans1_to_g1_from_state1);

		Pin from_evtx_on_g1 = CoreFactory.eINSTANCE.createPin();
		from_evtx_on_g1.setName("from_evtx_on_g1");
		g1.getPinsIn().add(from_evtx_on_g1);

		Relation inside_trans1_to_g1_from_evtx = CoreFactory.eINSTANCE.createRelation();
		inside_trans1_to_g1_from_evtx.setSource(from_evtx);
		inside_trans1_to_g1_from_evtx.setTarget(from_evtx_on_g1);
		trans1.getRelations().add(inside_trans1_to_g1_from_evtx);

		Pin to_state2_on_g1 = CoreFactory.eINSTANCE.createPin();
		to_state2_on_g1.setName("to_state2_on_g1");
		g1.getPinsOut().add(to_state2_on_g1);
		
		Relation inside_trans1_to_trans1_state2 = CoreFactory.eINSTANCE.createRelation();
		inside_trans1_to_trans1_state2.setSource(to_state2_on_g1);
		inside_trans1_to_trans1_state2.setTarget(to_state2);
		trans1.getRelations().add(inside_trans1_to_trans1_state2);

		//Create state state2
		Block state2 = CoreFactory.eINSTANCE.createAtomicBlock();
		state2.setName("state2");
		moc.setStateRole(state2);
		modelStructure.getBlocks().add(state2);

		Pin from_trans1 = CoreFactory.eINSTANCE.createPin();
		from_trans1.setName("from_trans1");
		state2.getPinsIn().add(from_trans1); 

		Relation trans1_to_state2 = CoreFactory.eINSTANCE.createRelation();
		trans1_to_state2.setSource(to_state2);
		trans1_to_state2.setTarget(from_trans1);
		modelStructure.getRelations().add(trans1_to_state2);

		Pin to_trans2 = CoreFactory.eINSTANCE.createPin();
		to_trans2.setName("to_trans2");
		state2.getPinsOut().add(to_trans2); 
		
		//Create transition trans2
		CompositeBlock trans2 = CoreFactory.eINSTANCE.createCompositeBlock();
		trans2.setName("trans2");
		moc.setTransitionRole(trans2);
		modelStructure.getBlocks().add(trans2);

		Pin from_evty = CoreFactory.eINSTANCE.createPin();
		from_evty.setName("from_evty");
		trans2.getPinsIn().add(from_evty);

		Relation evty_to_trans2 = CoreFactory.eINSTANCE.createRelation();
		evty_to_trans2.setSource(evty);
		evty_to_trans2.setTarget(from_evty);
		modelStructure.getRelations().add(evty_to_trans2);

		Pin from_state2 = CoreFactory.eINSTANCE.createPin();
		from_state2.setName("from_state2");
		trans2.getPinsIn().add(from_state2);

		Relation state2_to_trans2 = CoreFactory.eINSTANCE.createRelation();
		state2_to_trans2.setSource(to_trans2);
		state2_to_trans2.setTarget(from_state2);
		modelStructure.getRelations().add(state2_to_trans2);

		Pin to_state1 = CoreFactory.eINSTANCE.createPin();
		to_state1.setName("to_state1");
		trans2.getPinsOut().add(to_state1);
		
		Relation trans2_to_state1 = CoreFactory.eINSTANCE.createRelation();
		trans2_to_state1.setSource(to_state1);
		trans2_to_state1.setTarget(from_trans2);
		modelStructure.getRelations().add(trans2_to_state1);

		Pin to_evtz = CoreFactory.eINSTANCE.createPin();
		to_evtz.setName("to_evtz");
		trans2.getPinsOut().add(to_evtz);
		
		Relation trans2_to_evtz = CoreFactory.eINSTANCE.createRelation();
		trans2_to_evtz.setSource(to_evtz);
		trans2_to_evtz.setTarget(evtz);
		modelStructure.getRelations().add(trans2_to_evtz);

		//Create inside guard g2
		AndConstantEmitter g2 = PolymoclibFactory.eINSTANCE.createAndConstantEmitter();
		g2.setTokenToEmit(moc.createEvent());
		g2.setName("g2");
		moc.setGuardRole(g2);
		trans2.getBlocks().add(g2);

		Pin from_state2_on_g2 = CoreFactory.eINSTANCE.createPin();
		from_state2_on_g2.setName("from_state2_on_g2");
		g2.getPinsIn().add(from_state2_on_g2);

		Relation inside_trans2_to_g2_state2 = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_g2_state2.setSource(from_state2);
		inside_trans2_to_g2_state2.setTarget(from_state2_on_g2);
		trans2.getRelations().add(inside_trans2_to_g2_state2);

		Pin from_evty_on_g2 = CoreFactory.eINSTANCE.createPin();
		from_evty_on_g2.setName("from_evty_on_g2");
		g2.getPinsIn().add(from_evty_on_g2);

		Relation inside_trans2_to_g2_evty = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_g2_evty.setSource(from_evty);
		inside_trans2_to_g2_evty.setTarget(from_evty_on_g2);
		trans2.getRelations().add(inside_trans2_to_g2_evty);

		Pin to_state1_on_g2 = CoreFactory.eINSTANCE.createPin();
		to_state1_on_g2.setName("to_state1_on_g2");
		g2.getPinsOut().add(to_state1_on_g2);
		
		Relation inside_trans2_to_trans2_state1 = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_trans2_state1.setSource(to_state1_on_g2);
		inside_trans2_to_trans2_state1.setTarget(to_state1);
		trans2.getRelations().add(inside_trans2_to_trans2_state1);

		Pin to_a2 = CoreFactory.eINSTANCE.createPin();
		to_a2.setName("to_a2");
		g2.getPinsOut().add(to_a2);
		
		//Create inside action a2
		AndConstantEmitter a2 = PolymoclibFactory.eINSTANCE.createAndConstantEmitter();
		a2.setTokenToEmit(CoreFactory.eINSTANCE.createToken());
		a2.setName("a2");
		moc.setActionRole(a2);
		trans2.getBlocks().add(a2);

		Pin from_g2 = CoreFactory.eINSTANCE.createPin();
		from_g2.setName("from_g2");
		a2.getPinsIn().add(from_g2);
		
		Relation g2_to_a2 = CoreFactory.eINSTANCE.createRelation();
		g2_to_a2.setSource(to_a2);
		g2_to_a2.setTarget(from_g2);
		trans2.getRelations().add(g2_to_a2);

		Pin to_evtz_on_a2 = CoreFactory.eINSTANCE.createPin();
		to_evtz_on_a2.setName("to_evtz_on_a2");
		a2.getPinsOut().add(to_evtz_on_a2);
		
		Relation inside_trans2_to_trans2_evtz = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_trans2_evtz.setSource(to_evtz_on_a2);
		inside_trans2_to_trans2_evtz.setTarget(to_evtz);
		trans2.getRelations().add(inside_trans2_to_trans2_evtz);

}

	private static void saveModel(String name) {
		// create resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register XML resource factory (before creating the resource)
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		// add the root object to the resource
		resource.getContents().add(model);
		// serialize resource
		try {
			resource.save(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
		}
	}
	
	private static boolean loadModel(String name) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		try {
			resource.load(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
			return false;
		}
		
		if (resource.getContents().size() != 1) {
			System.err.println("Error : there must be 1 element");
			return false;
		}
		
		EObject o = resource.getContents().get(0);
		try {
			model = (Model) o;
		}
		catch(ClassCastException ex) {
			System.err.println("Error : expected element not found : " + ex);
			return false;
		}
		
		evtx = model.getStructure().getPinsIn().get(0);
		evty = model.getStructure().getPinsIn().get(1);
		evtz = model.getStructure().getPinsOut().get(0);
		
		return true;
	}
}
