/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestFSM_2.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.fsm.FsmFactory;
import fr.supelec.modhelx.mocs.fsm.FsmModelOfComputation;
import fr.supelec.modhelx.mocs.polymoclib.AndConstantEmitter;
import fr.supelec.modhelx.mocs.polymoclib.PolymoclibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;

public class TestFSM_2 {
	
	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static FsmModelOfComputation moc;
	
	private static Pin t = null;
	private static Pin o = null;
	private static Pin p = null;
	private static Pin ok = null;
	
	public static void main(String[] args) {
		
		buildModel();
		saveModel("tests/fsm_2.modhelx");
		if (args.length > 0) {
			model = null;
			loadModel("tests/fsm_2.modhelx");
		}
		Platform.setLogFile("log/fsm_2_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		
		Clock clock = TimeFactory.eINSTANCE.createLogicalClock();
		
		Platform.setModel(model, clock);
		
		Token ex = moc.createEvent();
		t.addToken(ex);
		ex = moc.createEvent();
		o.addToken(ex);
		ex = moc.createEvent();
		p.addToken(ex);
		
		ModelWindow w = new ModelWindow(model, clock);
		w.setVisible(true);
		w.run();
		
		//Platform.run();
		
		System.out.println("Got " + ok.getStoredTokens().size() + " events on ok Pin");
	}
	
	/*
	 *   +--------------------------------------------------------------------------------------------------+
	 *   | TestFSM_2                                                                                   FSM  |
	 *   |  +---------------------------------------------------------------------------------------------+ |
	 *   |  | TestFSM_2_Structure                                                                         | |
	 *   |  |                                                                                             | |
	 *   |  |      +----------+ trans1:g1(t?)        +----------+ trans2:g2(o?)      +----------+         | |
	 *   |  |      | state1   >----------------------> state2   >--------------------> state3   |         | |
	 *   |  |      |          |                      |          |                    |          |         | |
	 *   |  |      |          |                      |          |                    |          >----+    | |
	 *   |  |      |          |                      |          |                    |          |    |    | |
	 *   |  |      |          <----+                 |          |                    |          |    |    | |
	 *   |  |      +----------+    |                 +----------+                    +----------+    |    | |
	 *   |  |                      |   trans3:g3(p?)/a3(ok!)                                         |    | |
	 *   |  |                      +-----------------------------------------------------------------+    | |
	 *   |  +---------------------------------------------------------------------------------------------+ |
	 *   +--------------------------------------------------------------------------------------------------+
	 *   
	 *   Recognizes "t" "o" "p" and produces "ok"
	 */
	private static void buildModel() {
		model.setName("TestFSM_2");

		/* FsmModelOfComputation */ moc = FsmFactory.eINSTANCE.createFsmModelOfComputation();
		moc.setName("FSM");
		model.setMoc(moc);
		
		//Create model structure
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestFSM_2_Structure");
		model.setStructure(modelStructure);

		/*Pin*/ t = CoreFactory.eINSTANCE.createPin();
		t.setName("t");
		modelStructure.getPinsIn().add(t);

		/*Pin*/ o = CoreFactory.eINSTANCE.createPin();
		o.setName("o");
		modelStructure.getPinsIn().add(o);

		/*Pin*/ p = CoreFactory.eINSTANCE.createPin();
		p.setName("p");
		modelStructure.getPinsIn().add(p);

		/*Pin*/ ok = CoreFactory.eINSTANCE.createPin();
		ok.setName("ok");
		modelStructure.getPinsOut().add(ok);

		//Create initial state state1
		Block state1 = CoreFactory.eINSTANCE.createAtomicBlock();
		state1.setName("state1");
		moc.setInitialStateFeature(state1);
		modelStructure.getBlocks().add(state1);

		Pin from_trans3 = CoreFactory.eINSTANCE.createPin();
		from_trans3.setName("from_trans3");
		state1.getPinsIn().add(from_trans3);

		Pin to_trans1 = CoreFactory.eINSTANCE.createPin();
		to_trans1.setName("to_trans1");
		state1.getPinsOut().add(to_trans1);


		//Create transition trans1
		CompositeBlock trans1 = CoreFactory.eINSTANCE.createCompositeBlock();
		trans1.setName("trans1");
		moc.setTransitionRole(trans1);
		modelStructure.getBlocks().add(trans1);

		Pin from_t = CoreFactory.eINSTANCE.createPin();
		from_t.setName("from_t");
		trans1.getPinsIn().add(from_t);

		Relation tmodel_to_ttrans1 = CoreFactory.eINSTANCE.createRelation();
		tmodel_to_ttrans1.setSource(t);
		tmodel_to_ttrans1.setTarget(from_t);
		modelStructure.getRelations().add(tmodel_to_ttrans1);

		Pin from_state1 = CoreFactory.eINSTANCE.createPin();
		from_state1.setName("from_state1");
		trans1.getPinsIn().add(from_state1);

		Relation state1_to_trans1 = CoreFactory.eINSTANCE.createRelation();
		state1_to_trans1.setSource(to_trans1);
		state1_to_trans1.setTarget(from_state1);
		modelStructure.getRelations().add(state1_to_trans1);

		Pin to_state2 = CoreFactory.eINSTANCE.createPin();
		to_state2.setName("to_state2");
		trans1.getPinsOut().add(to_state2);
		

		//Create inside guard g1 of trans1
		AndConstantEmitter g1 = PolymoclibFactory.eINSTANCE.createAndConstantEmitter();
		g1.setTokenToEmit(moc.createEvent());
		g1.setName("g1");
		moc.setGuardRole(g1);
		trans1.getBlocks().add(g1);

		Pin from_state1_on_g1 = CoreFactory.eINSTANCE.createPin();
		from_state1_on_g1.setName("from_state1_on_g1");
		g1.getPinsIn().add(from_state1_on_g1);

		Relation inside_trans1_to_g1_from_state1 = CoreFactory.eINSTANCE.createRelation();
		inside_trans1_to_g1_from_state1.setSource(from_state1);
		inside_trans1_to_g1_from_state1.setTarget(from_state1_on_g1);
		trans1.getRelations().add(inside_trans1_to_g1_from_state1);

		Pin from_t_on_g1 = CoreFactory.eINSTANCE.createPin();
		from_t_on_g1.setName("from_t_on_g1");
		g1.getPinsIn().add(from_t_on_g1);

		Relation inside_trans1_to_g1_from_t = CoreFactory.eINSTANCE.createRelation();
		inside_trans1_to_g1_from_t.setSource(from_t);
		inside_trans1_to_g1_from_t.setTarget(from_t_on_g1);
		trans1.getRelations().add(inside_trans1_to_g1_from_t);

		Pin to_state2_on_g1 = CoreFactory.eINSTANCE.createPin();
		to_state2_on_g1.setName("to_state2_on_g1");
		g1.getPinsOut().add(to_state2_on_g1);
		
		Relation inside_trans1_to_trans1_state2 = CoreFactory.eINSTANCE.createRelation();
		inside_trans1_to_trans1_state2.setSource(to_state2_on_g1);
		inside_trans1_to_trans1_state2.setTarget(to_state2);
		trans1.getRelations().add(inside_trans1_to_trans1_state2);

		//Create state state2
		Block state2 = CoreFactory.eINSTANCE.createAtomicBlock();
		state2.setName("state2");
		moc.setStateRole(state2);
		modelStructure.getBlocks().add(state2);

		Pin from_trans1 = CoreFactory.eINSTANCE.createPin();
		from_trans1.setName("from_trans1");
		state2.getPinsIn().add(from_trans1); 

		Relation trans1_to_state2 = CoreFactory.eINSTANCE.createRelation();
		trans1_to_state2.setSource(to_state2);
		trans1_to_state2.setTarget(from_trans1);
		modelStructure.getRelations().add(trans1_to_state2);

		Pin to_trans2 = CoreFactory.eINSTANCE.createPin();
		to_trans2.setName("to_trans2");
		state2.getPinsOut().add(to_trans2); 
		
		//Create transition trans2
		CompositeBlock trans2 = CoreFactory.eINSTANCE.createCompositeBlock();
		trans2.setName("trans2");
		moc.setTransitionRole(trans2);
		modelStructure.getBlocks().add(trans2);

		Pin from_o = CoreFactory.eINSTANCE.createPin();
		from_o.setName("from_o");
		trans2.getPinsIn().add(from_o);

		Relation evty_to_trans2 = CoreFactory.eINSTANCE.createRelation();
		evty_to_trans2.setSource(o);
		evty_to_trans2.setTarget(from_o);
		modelStructure.getRelations().add(evty_to_trans2);

		Pin from_state2 = CoreFactory.eINSTANCE.createPin();
		from_state2.setName("from_state2");
		trans2.getPinsIn().add(from_state2);

		Relation state2_to_trans2 = CoreFactory.eINSTANCE.createRelation();
		state2_to_trans2.setSource(to_trans2);
		state2_to_trans2.setTarget(from_state2);
		modelStructure.getRelations().add(state2_to_trans2);

		Pin to_state3 = CoreFactory.eINSTANCE.createPin();
		to_state3.setName("to_state3");
		trans2.getPinsOut().add(to_state3);
		
		//Create inside guard g2
		AndConstantEmitter g2 = PolymoclibFactory.eINSTANCE.createAndConstantEmitter();
		g2.setTokenToEmit(moc.createEvent());
		g2.setName("g2");
		moc.setGuardRole(g2);
		trans2.getBlocks().add(g2);

		Pin from_state2_on_g2 = CoreFactory.eINSTANCE.createPin();
		from_state2_on_g2.setName("from_state2_on_g2");
		g2.getPinsIn().add(from_state2_on_g2);

		Relation inside_trans2_to_g2_state2 = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_g2_state2.setSource(from_state2);
		inside_trans2_to_g2_state2.setTarget(from_state2_on_g2);
		trans2.getRelations().add(inside_trans2_to_g2_state2);

		Pin from_o_on_g2 = CoreFactory.eINSTANCE.createPin();
		from_o_on_g2.setName("from_o_on_g2");
		g2.getPinsIn().add(from_o_on_g2);

		Relation inside_trans2_to_g2_evty = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_g2_evty.setSource(from_o);
		inside_trans2_to_g2_evty.setTarget(from_o_on_g2);
		trans2.getRelations().add(inside_trans2_to_g2_evty);

		Pin to_state3_on_g2 = CoreFactory.eINSTANCE.createPin();
		to_state3_on_g2.setName("to_state3_on_g2");
		g2.getPinsOut().add(to_state3_on_g2);
		
		Relation inside_trans2_to_trans2_state3 = CoreFactory.eINSTANCE.createRelation();
		inside_trans2_to_trans2_state3.setSource(to_state3_on_g2);
		inside_trans2_to_trans2_state3.setTarget(to_state3);
		trans2.getRelations().add(inside_trans2_to_trans2_state3);

		//Create state 3
		Block state3 = CoreFactory.eINSTANCE.createAtomicBlock();
		state3.setName("state3");
		moc.setStateRole(state3);
		modelStructure.getBlocks().add(state3);

		Pin from_trans2 = CoreFactory.eINSTANCE.createPin();
		from_trans2.setName("from_trans2");
		state3.getPinsIn().add(from_trans2); 
		
		Relation trans2_to_state3 = CoreFactory.eINSTANCE.createRelation();
		trans2_to_state3.setSource(to_state3);
		trans2_to_state3.setTarget(from_trans2);
		modelStructure.getRelations().add(trans2_to_state3);

		Pin to_trans3 = CoreFactory.eINSTANCE.createPin();
		to_trans3.setName("to_trans3");
		state3.getPinsOut().add(to_trans3); 

		//Create transition 3
		CompositeBlock trans3 = CoreFactory.eINSTANCE.createCompositeBlock();
		trans3.setName("trans3");
		moc.setTransitionRole(trans3);
		modelStructure.getBlocks().add(trans3);

		Pin from_p = CoreFactory.eINSTANCE.createPin();
		from_p.setName("from_p");
		trans3.getPinsIn().add(from_p);

		Relation p_to_trans3 = CoreFactory.eINSTANCE.createRelation();
		p_to_trans3.setSource(p);
		p_to_trans3.setTarget(from_p);
		modelStructure.getRelations().add(p_to_trans3);

		Pin from_state3 = CoreFactory.eINSTANCE.createPin();
		from_state3.setName("from_state3");
		trans3.getPinsIn().add(from_state3);

		Relation state3_to_trans3 = CoreFactory.eINSTANCE.createRelation();
		state3_to_trans3.setSource(to_trans3);
		state3_to_trans3.setTarget(from_state3);
		modelStructure.getRelations().add(state3_to_trans3);

		Pin to_state1 = CoreFactory.eINSTANCE.createPin();
		to_state1.setName("to_state1");
		trans3.getPinsOut().add(to_state1);

		Relation trans3_to_state1 = CoreFactory.eINSTANCE.createRelation();
		trans3_to_state1.setSource(to_state1);
		trans3_to_state1.setTarget(from_trans3);
		modelStructure.getRelations().add(trans3_to_state1);

		
		Pin to_ok = CoreFactory.eINSTANCE.createPin();
		to_ok.setName("to_ok");
		trans3.getPinsOut().add(to_ok);
		
		Relation trans3_to_ok = CoreFactory.eINSTANCE.createRelation();
		trans3_to_ok.setSource(to_ok);
		trans3_to_ok.setTarget(ok);
		modelStructure.getRelations().add(trans3_to_ok);

		//Create inside guard g3
		AndConstantEmitter g3 = PolymoclibFactory.eINSTANCE.createAndConstantEmitter();
		g3.setTokenToEmit(moc.createEvent());
		g3.setName("g3");
		moc.setGuardRole(g3);
		trans3.getBlocks().add(g3);

		Pin from_state3_on_g3 = CoreFactory.eINSTANCE.createPin();
		from_state3_on_g3.setName("from_state3_on_g3");
		g3.getPinsIn().add(from_state3_on_g3);

		Relation inside_trans3_to_g3_state3 = CoreFactory.eINSTANCE.createRelation();
		inside_trans3_to_g3_state3.setSource(from_state3);
		inside_trans3_to_g3_state3.setTarget(from_state3_on_g3);
		trans3.getRelations().add(inside_trans3_to_g3_state3);

		Pin from_p_on_g3 = CoreFactory.eINSTANCE.createPin();
		from_p_on_g3.setName("from_p_on_g3");
		g3.getPinsIn().add(from_p_on_g3);

		Relation inside_trans3_to_g3_p = CoreFactory.eINSTANCE.createRelation();
		inside_trans3_to_g3_p.setSource(from_p);
		inside_trans3_to_g3_p.setTarget(from_p_on_g3);
		trans3.getRelations().add(inside_trans3_to_g3_p);

		Pin to_state1_on_g3 = CoreFactory.eINSTANCE.createPin();
		to_state1_on_g3.setName("to_state1_on_g3");
		g3.getPinsOut().add(to_state1_on_g3);
		
		Relation inside_trans3_to_trans3_state1 = CoreFactory.eINSTANCE.createRelation();
		inside_trans3_to_trans3_state1.setSource(to_state1_on_g3);
		inside_trans3_to_trans3_state1.setTarget(to_state1);
		trans3.getRelations().add(inside_trans3_to_trans3_state1);

		Pin to_a3 = CoreFactory.eINSTANCE.createPin();
		to_a3.setName("to_a3");
		g3.getPinsOut().add(to_a3);
		
		
		//Create inside action a3
		AndConstantEmitter a3 = PolymoclibFactory.eINSTANCE.createAndConstantEmitter();
		a3.setTokenToEmit(CoreFactory.eINSTANCE.createToken());
		a3.setName("a3");
		moc.setActionRole(a3);
		trans3.getBlocks().add(a3);

		Pin from_g3 = CoreFactory.eINSTANCE.createPin();
		from_g3.setName("from_g3");
		a3.getPinsIn().add(from_g3);
		
		Relation g3_to_a3 = CoreFactory.eINSTANCE.createRelation();
		g3_to_a3.setSource(to_a3);
		g3_to_a3.setTarget(from_g3);
		trans3.getRelations().add(g3_to_a3);

		Pin to_ok_on_a3 = CoreFactory.eINSTANCE.createPin();
		to_ok_on_a3.setName("to_ok_on_a3");
		a3.getPinsOut().add(to_ok_on_a3);
		
		Relation inside_trans3_to_trans3_ok = CoreFactory.eINSTANCE.createRelation();
		inside_trans3_to_trans3_ok.setSource(to_ok_on_a3);
		inside_trans3_to_trans3_ok.setTarget(to_ok);
		trans3.getRelations().add(inside_trans3_to_trans3_ok);

	}

	private static void saveModel(String name) {
		// create resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register XML resource factory (before creating the resource)
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		// add the root object to the resource
		resource.getContents().add(model);
		// serialize resource
		try {
			resource.save(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
		}
	}
	
	private static boolean loadModel(String name) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		try {
			resource.load(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
			return false;
		}
		
		if (resource.getContents().size() != 1) {
			System.err.println("Error : there must be 1 element");
			return false;
		}
		
		EObject o = resource.getContents().get(0);
		try {
			model = (Model) o;
		}
		catch(ClassCastException ex) {
			System.err.println("Error : expected element not found : " + ex);
			return false;
		}
		
		// TODO ?
		
		return true;
	}
}
