/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestGenerique.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.de.DePackage;
import fr.supelec.modhelx.mocs.fsm.FsmPackage;
import fr.supelec.modhelx.mocs.petrinet.PetrinetPackage;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdf.SdfPackage;
import fr.supelec.modhelx.mocs.sdflib.SdfPrinter;
import fr.supelec.modhelx.mocs.sdflib.SdflibPackage;
import fr.supelec.modhelx.qdgui.ModelWindow;

public class TestGenerique {
	
	/**
	 * Parameters to set for choosing the model to execute and the way it has to be executed
	 */
	private static final int NB_OF_SNAPSHOTS = 10; //number of snapshots to do
	
	private static final String MODEL_FILE_NAME = "sdf_5.modhelx"; //name of the model to execute (XMI file) 
	private static final String MODEL_FILE_PATH = "tests/"; //optional file path
	
	private static final boolean DO_SAVE = false; //save or do not save the model after execution
	private static final String NEW_MODEL_FILE_NAME = MODEL_FILE_NAME; //optional new file name for saving
	private static final String NEW_MODEL_FILE_PATH = MODEL_FILE_PATH; //optional file path for saving

	private static final boolean DO_LOG = false; // log or do not log execution
	private static final String LOG_FILE_NAME = MODEL_FILE_NAME+"-log.txt"; //optional file name for logging
	private static final String LOG_FILE_PATH = "log/"; //optional file path for logging

	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	/**
	 * Preparation of the model for execution.
	 * Its main role is to set the properties which cannot be serialized (their value is not present in the XMI file of the model)
	 * @param model
	 */
	private static void prepareModel4Execution(Model model){
		//TODO to complete depending on the model which is loaded to set the values of the properties of elements of the model which cannot be serialized
		
		//for a model with the SDF moc:
		if(model.getMoc().eClass()==SdfPackage.Literals.SDF_MODEL_OF_COMPUTATION){
			//set the clock of the SDF moc
			SdfModelOfComputation moc = (SdfModelOfComputation)model.getMoc();
			moc.setClock(TimeFactory.eINSTANCE.createLogicalClock());
			
			//set de output stream of the printer if the model contains a printer
			for(Block b : model.getStructure().getBlocks()){
				if(b.eClass()==SdflibPackage.Literals.SDF_PRINTER){
					((SdfPrinter)b).setPrintStream(System.out);
				}
			}
		}
		//for a model with the FSM moc:
		else if(model.getMoc().eClass()==FsmPackage.Literals.FSM_MODEL_OF_COMPUTATION){
			//TODO
		}
		//for a model with the DE moc:
		else if(model.getMoc().eClass()==DePackage.Literals.DE_MODEL_OF_COMPUTATION){
			//TODO
		}
		//for a model with the PetriNets moc:
		else if(model.getMoc().eClass()==PetrinetPackage.Literals.PETRI_NET_MODEL_OF_COMPUTATION){
			//TODO
		}
	}

	public static void main(String[] args) {
		
//		CorePackage corepackage = CorePackage.eINSTANCE;//necessary for URI mapping

		model = null;
		loadModel(MODEL_FILE_PATH+MODEL_FILE_NAME);
		
		prepareModel4Execution(model);

		Clock clock = ((SdfModelOfComputation)model.getMoc()).getClock();
		
		Platform.setModel(model, clock);

		if(DO_LOG) Platform.setLogFile(LOG_FILE_PATH+LOG_FILE_NAME);
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(NB_OF_SNAPSHOTS);

		ModelWindow w = new ModelWindow(model, clock);
		w.setVisible(true);
		w.run();

		Platform.run();
		
		if(DO_SAVE) saveModel(NEW_MODEL_FILE_PATH+NEW_MODEL_FILE_NAME);
	}
	
	private static void saveModel(String name) {
		// create resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register XML resource factory (before creating the resource)
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		// add the root object to the resource
		resource.getContents().add(model);
		// serialize resource
		try {
			resource.save(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
		}
	}
	
	private static boolean loadModel(String name) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		try {
			resource.load(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
			return false;
		}
		
		if (resource.getContents().size() != 1) {
			System.err.println("Error : there must be 1 element");
			return false;
		}
		
		EObject o = resource.getContents().get(0);
		try {
			model = (Model) o;
		}
		catch(ClassCastException ex) {
			System.err.println("Error : expected element not found : " + ex);
			return false;
		}
		
		// TODO ?
		
		return true;
	}
}
