package fr.supelec.modhelx.tests;

import java.util.logging.Level;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.corelib.CorelibFactory;
import fr.supelec.modhelx.corelib.PluginBlock;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DelibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.util.LogFormatter;
import fr.supelec.modhelx.util.ModelBuilder;
import fr.supelec.modhelx.util.plugins.DeKeypad;

public class TestKeypad {

	public static void main(String[] args) {
		//modèle racine DE
		DeModelOfComputation deMoc = DeFactory.eINSTANCE.createDeModelOfComputation();
		deMoc.setName("DE_moc");
		
		Model deModel = CoreFactory.eINSTANCE.createModel();
		deModel.setName("PowerWindow");
		deModel.setMoc(deMoc);
		deMoc.setClock(TimeFactory.eINSTANCE.createLogicalClock());
		
		CompositeBlock deStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		deStructure.setName("PowerWindow_structure");
		deModel.setStructure(deStructure);
		
		PluginBlock player = CorelibFactory.eINSTANCE.createPluginBlock();
		player.setPluggedInClassName("fr.supelec.modhelx.tests.plugins.DeSynchronousEmitter");
		DeKeypad keypad = new DeKeypad();
		player.setValueOfParameter("source", keypad);

		keypad.addKey(player, "Up", "cmd_user", 1.);
		keypad.addKey(player, "Stop", "cmd_user", 0.);
		keypad.addKey(player, "Down", "cmd_user", -1.);
		keypad.addKey(player, "Obstacle", "cmd_obstacle", 1.);

		keypad.display();
		
		deStructure.addBlock(player, "SyncEmit_Keypad");
		
		
		DeEventPrinter printer = DelibFactory.eINSTANCE.createDeEventPrinter();
		printer.setPrintStream(System.out);
		
		Pin prUser = ModelBuilder.addPinIn(printer, "user");
		deStructure.connect(ModelBuilder.pinByName(player, "cmd_user"), prUser);
		
		Pin prObstacle = ModelBuilder.addPinIn(printer, "obstacle");
		deStructure.connect(ModelBuilder.pinByName(player, "cmd_obstacle"), prObstacle);
		
		deStructure.addBlock(printer, "EventPrinter");
		
		
		
		Platform.setModel(deModel, deMoc.getClock());
		
		Platform.setLogFile("log/TestKeypad.log");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new LogFormatter());
		
		// simulation in "real time"
		Platform.setRealTime(1000, 5000);

		ModelWindow w = new ModelWindow(deModel, deMoc.getClock());

		w.setVisible(true);
		w.run();
	}

}
