package fr.supelec.modhelx.tests;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.mocs.petrinet.PetriNetModelOfComputation;
import fr.supelec.modhelx.mocs.petrinet.PetrinetFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.qdgui.rendering.PetriNetPlaceR;
import fr.supelec.modhelx.qdgui.rendering.PetriNetTransitionR;

public class TestPetriNet_1 {

	private static Model model = CoreFactory.eINSTANCE.createModel();
	private static Clock clock;

	public static void main(String[] args) {
		PetriNetPlaceR.register();
		PetriNetTransitionR.register();
		
		buildModel();

		ModelWindow w = new ModelWindow(model, clock, "tests/petrinet1.xy");
		
		w.setVisible(true);
		w.run();
		
		/*
		Platform.setLogFile("log/petrinet_1_log.txt");
		Platform.setLogLevel(Level.ALL);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(50);

		Platform.setModel(model);
		
		Platform.run();
		*/
	}
	
	/*
	 *   +------------------------------------------------------------------------+
	 *   | TestPetriNet_1                                               PetriNet  |
	 *   |  +-------------------------------------------------------------------+ |
	 *   |  | TestPetriNet_1_Structure                                          | |
	 *   |  |                                                                   | |
	 *   |  |                                                                   | |
	 *   |  |                                                                   | |
	 *   |  |      +--P1-+      +--T1-+      +--P2-+      +--T2-+      +--P4-+  | |
	 *   |  |      |     |      |     |      |     |      |     |      |     |  | |
	 *   |  |   +-->  *  >------>     >---+-->     >---+-->     >---+-->  *  |  | |
	 *   |  |   |  |     |      |     |   |  |     |   ^  |     |   |  |     |  | |
	 *   |  |   |  +-----+      +-----+   |  +-----+   |  +-----+   |  +-----+  | |
	 *   |  |   |                         |            |            |           | |
	 *   |  |   ^                         V  +--P3-+   |            |           | |
	 *   |  |   |                         |  |     |   |            |           | |
	 *   |  |   |                         +--> * * >---+            V           | |
	 *   |  |   |                            |     |                |           | |
	 *   |  |   |                            +-----+                |           | |
	 *   |  |   |                                                   |           | |   
	 *   |  |   +------<----------------------------<---------------+           | |
	 *   |  |                                                                   | |
	 *   |  |                                                                   | |
	 *   |  +-------------------------------------------------------------------+ |
	 *   +------------------------------------------------------------------------+
	 *   
	 *   See http://commons.wikimedia.org/wiki/File:Detailed_petri_net.png
	 */
	private static void buildModel() {
		model.setName("TestPetriNet_1");

		//MoC
		PetriNetModelOfComputation moc = PetrinetFactory.eINSTANCE.createPetriNetModelOfComputation();
		moc.setName("TestPetriNet_1_MoC");
		clock = TimeFactory.eINSTANCE.createLogicalClock();
		moc.setClock(clock);
		model.setMoc(moc);
		
		//Structure
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestPetriNet_1_Structure");
		model.setStructure(modelStructure);


		// P1
		Triple tP1 = createPetriNetPlace(modelStructure, "P1");
		tP1.out.addToken(CoreFactory.eINSTANCE.createToken());
		moc.setBlockIsPlace(tP1.block);

		// P2
		Triple tP2 = createPetriNetPlace(modelStructure, "P2");
		moc.setBlockIsPlace(tP2.block);

		// P3
		Triple tP3 = createPetriNetPlace(modelStructure, "P3");
		tP3.out.addToken(CoreFactory.eINSTANCE.createToken());
		tP3.out.addToken(CoreFactory.eINSTANCE.createToken());
		moc.setBlockIsPlace(tP3.block);

		// P4
		Triple tP4 = createPetriNetPlace(modelStructure, "P4");
		tP4.out.addToken(CoreFactory.eINSTANCE.createToken());
		moc.setBlockIsPlace(tP4.block);

		// T1
		Triple tT1 = createPetriNetTransition(modelStructure, "T1");
		moc.setBlockIsTransition(tT1.block);

		// T2
		Triple tT2 = createPetriNetTransition(modelStructure, "T2");
		moc.setBlockIsTransition(tT2.block);
		

		// connections
		connect(modelStructure, tP1, tT1);
		connect(modelStructure, tT1, tP2);
		connect(modelStructure, tT1, tP3);
		connect(modelStructure, tP2, tT2);
		connect(modelStructure, tP3, tT2);
		connect(modelStructure, tT2, tP4);
		connect(modelStructure, tT2, tP1);
	}

	private static Triple createPetriNetPlace(CompositeBlock modelStructure, String name) {
		return initPetriNetBlock(CoreFactory.eINSTANCE.createAtomicBlock(), modelStructure, name);
	}
	
	private static Triple createPetriNetTransition(CompositeBlock modelStructure, String name) {
		return initPetriNetBlock(CoreFactory.eINSTANCE.createAtomicBlock(), modelStructure, name);
	}
	
	private static Triple initPetriNetBlock(Block p1, CompositeBlock modelStructure, String name) {
		p1.setName(name);
		Pin p1out = CoreFactory.eINSTANCE.createPin();
		p1out.setName(name.toLowerCase() + "out"); 
		p1.getPinsOut().add(p1out);
		Pin p1in = CoreFactory.eINSTANCE.createPin();
		p1in.setName(name.toLowerCase() + "in");
		p1.getPinsIn().add(p1in);
		modelStructure.getBlocks().add(p1);
		
		return new Triple(p1, p1in, p1out);
	}
	
	private static Relation connect(CompositeBlock modelStructure, Triple t1, Triple t2) {
		Relation r = CoreFactory.eINSTANCE.createRelation();
		r.setSource(t1.out);
		r.setTarget(t2.in);
		modelStructure.getRelations().add(r);
		return r;
	}

}


class Triple {
	public final Block block;
	public final Pin in, out;
	
	public Triple(Block block, Pin in, Pin out) {
		this.block = block;
		this.in = in;
		this.out = out;
	}
}
