/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestSDF_1.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfConstantEmitter;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;

public class TestSDF_1 {
	
	private static Model model = CoreFactory.eINSTANCE.createModel();
	private static Clock clock;
	

	public static void main(String[] args) {
		
		buildModel();
		saveModel("tests/sdf_1.modhelx");
		if (args.length > 0) {
			model = null;
			loadModel("tests/sdf_1.modhelx");
		}
		//Platform.setLogFile("log/sdf_1_log.txt");
		Platform.setLogLevel(Level.FINE);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(10);

		Platform.setModel(model, clock);
		
		Platform.run();
	}
	
	/*
	 *   +---------------------------------------------+
	 *   | TestSDF_1                              SDF  |
	 *   |  +----------------------------------------+ |
	 *   |  | TestSDF_1_Structure                    | |
	 *   |  |                                        | |
	 *   |  |   +----------+                         | |
	 *   |  |   | ConstEm  >-------------------------> |
	 *   |  |   |   10     |(3)                   (3)| |
	 *   |  |   |          |                         | |
	 *   |  |   |          |                         | |
	 *   |  |   |          |                         | |
	 *   |  |   +----------+                         | |
	 *   |  |                                        | |
	 *   |  |                                        | |
	 *   |  +----------------------------------------+ |
	 *   +---------------------------------------------+
	 *   
	 *   Schedule : C
	 */
	private static void buildModel() {
		model.setName("TestSDF_1");

		//MoC
		SdfModelOfComputation moc = SdfFactory.eINSTANCE.createSdfModelOfComputation();
		moc.setName("TestSDF_1_MoC");
		clock = TimeFactory.eINSTANCE.createLogicalClock();
		moc.setClock(clock);
		model.setMoc(moc);
		
		//Structure
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestSDF_1_Structure");
		model.setStructure(modelStructure);

		Pin pOut = CoreFactory.eINSTANCE.createPin();
		pOut.setName("pOut");
		moc.setPinRate(pOut, 3);
		modelStructure.getPinsOut().add(pOut);
		

		//Constant Emitter
		SdfConstantEmitter sdfConstEmit = SdflibFactory.eINSTANCE.createSdfConstantEmitter();
		sdfConstEmit.setName("SdfConstEmit");
		modelStructure.getBlocks().add(sdfConstEmit);
		
		Pin pOutCE = CoreFactory.eINSTANCE.createPin();
		pOutCE.setName("pOutCE");
		moc.setPinRate(pOutCE, 3);
		sdfConstEmit.getPinsOut().add(pOutCE);
		
		sdfConstEmit.setValueToEmit(new Integer(10));

		//Relation
		Relation r_pOutCE_pOut = CoreFactory.eINSTANCE.createRelation();
		r_pOutCE_pOut.setSource(pOutCE);
		r_pOutCE_pOut.setTarget(pOut);
		modelStructure.getRelations().add(r_pOutCE_pOut);

	}

	private static void saveModel(String name) {
		// create resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register XML resource factory (before creating the resource)
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		// add the root object to the resource
		resource.getContents().add(model);
		// serialize resource
		try {
			resource.save(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
		}
	}
	
	private static boolean loadModel(String name) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		try {
			resource.load(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
			return false;
		}
		
		if (resource.getContents().size() != 1) {
			System.err.println("Error : there must be 1 element");
			return false;
		}
		
		EObject o = resource.getContents().get(0);
		try {
			model = (Model) o;
		}
		catch(ClassCastException ex) {
			System.err.println("Error : expected element not found : " + ex);
			return false;
		}
		
		// TODO ?
		
		return true;
	}
}
