/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestSDF_3.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfAdder;
import fr.supelec.modhelx.mocs.sdflib.SdfConstantEmitter;
import fr.supelec.modhelx.mocs.sdflib.SdfDuplicator;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;

public class TestSDF_3 {
	
	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static Clock clock;

	public static void main(String[] args) {
		
		buildModel();
		saveModel("tests/sdf_3.modhelx");
		if (args.length > 0) {
			model = null;
			loadModel("tests/sdf_3.modhelx");
		}
		//Platform.setLogFile("log/sdf_3_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(10);

		Platform.setModel(model, clock);
		
		Platform.run();
	}
	
	/*
	 *   +-------------------------------------------------------------+
	 *   | TestSDF_3                                             SDF   |
	 *   |  +-------------------------------------------------------+  |
	 *   |  | TestSDF_3_Structure                                   |  |
	 *   |  |                                                       |  |
	 *   |  |   +----------+      +----------+      +--------+      |  |
	 *   |  |   | ConstEm  >------>   Dup    >------>  Add   |      |  |
	 *   |  |   |   10  (3)|      |(6)    (6)|      |(3)  (3)>------>  |  
	 *   |  |   |       (3)>--+   |          |  +--->(3)     |      |  |
	 *   |  |   +----------+  |   +----------+  |   +--------+      |  |
	 *   |  |                 |                 |                   |  |
	 *   |  |                 +-----------------+                   |  |
	 *   |  |                                                       |  |
	 *   |  +-------------------------------------------------------+  |
	 *   +-------------------------------------------------------------+
	 *   
	 *   Schedule : CCDAA
	 */
	private static void buildModel() {
		model.setName("TestSDF_3");

		//MoC
		SdfModelOfComputation moc = SdfFactory.eINSTANCE.createSdfModelOfComputation();
		moc.setName("TestSDF_3_MoC");
		clock = TimeFactory.eINSTANCE.createLogicalClock();
		moc.setClock(clock);
		model.setMoc(moc);
		
		//Structure
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestSDF_3_Structure");
		model.setStructure(modelStructure);

		Pin pOutModel = CoreFactory.eINSTANCE.createPin();
		pOutModel.setName("pOutModel");
		moc.setPinRate(pOutModel, 3);
		modelStructure.getPinsOut().add(pOutModel);

		//Constant Emitter
		SdfConstantEmitter sdfConstEmit = SdflibFactory.eINSTANCE.createSdfConstantEmitter();
		sdfConstEmit.setName("SdfConstEmit");
		modelStructure.getBlocks().add(sdfConstEmit);
		sdfConstEmit.setValueToEmit(new Integer(10));

		Pin pOut1CE = CoreFactory.eINSTANCE.createPin();
		pOut1CE.setName("pOut1CE");
		sdfConstEmit.getPinsOut().add(pOut1CE);
		moc.setPinRate(pOut1CE, 3);
		
		Pin pOut2CE = CoreFactory.eINSTANCE.createPin();
		pOut2CE.setName("pOut2CE");
		sdfConstEmit.getPinsOut().add(pOut2CE);
		moc.setPinRate(pOut2CE, 3);

		//Duplicator
		SdfDuplicator sdfDup = SdflibFactory.eINSTANCE.createSdfDuplicator();
		sdfDup.setName("SdfDup");
		modelStructure.getBlocks().add(sdfDup);
		
		Pin pInDup = CoreFactory.eINSTANCE.createPin();
		pInDup.setName("pInDup");
		sdfDup.getPinsIn().add(pInDup);
		moc.setPinRate(pInDup, 6);

		Pin pOutDup = CoreFactory.eINSTANCE.createPin();
		pOutDup.setName("pOutDup");
		sdfDup.getPinsOut().add(pOutDup);
		moc.setPinRate(pOutDup, 6);
		
		//Adder
		SdfAdder sdfAdd = SdflibFactory.eINSTANCE.createSdfAdder();
		sdfAdd.setName("sdfAdd");
		modelStructure.getBlocks().add(sdfAdd);

		Pin pIn1Add = CoreFactory.eINSTANCE.createPin();
		pIn1Add.setName("pIn1Add");
		sdfAdd.getPinsIn().add(pIn1Add);
		moc.setPinRate(pIn1Add, 3);

		Pin pIn2Add = CoreFactory.eINSTANCE.createPin();
		pIn2Add.setName("pIn2Add");
		sdfAdd.getPinsIn().add(pIn2Add);
		moc.setPinRate(pIn2Add, 3);

		Pin pOutAdd = CoreFactory.eINSTANCE.createPin();
		pOutAdd.setName("pOutAdd");
		sdfAdd.getPinsOut().add(pOutAdd);
		moc.setPinRate(pOutAdd, 3);

		//Relations
		Relation r_pOut1CE_pInDup = CoreFactory.eINSTANCE.createRelation();
		r_pOut1CE_pInDup.setSource(pOut1CE);
		r_pOut1CE_pInDup.setTarget(pInDup);
		modelStructure.getRelations().add(r_pOut1CE_pInDup);

		Relation r_pOutDup_pIn1Add = CoreFactory.eINSTANCE.createRelation();
		r_pOutDup_pIn1Add.setSource(pOutDup);
		r_pOutDup_pIn1Add.setTarget(pIn1Add);
		modelStructure.getRelations().add(r_pOutDup_pIn1Add);

		Relation r_pOut2CE_pIn2Add = CoreFactory.eINSTANCE.createRelation();
		r_pOut2CE_pIn2Add.setSource(pOut2CE);
		r_pOut2CE_pIn2Add.setTarget(pIn2Add);
		modelStructure.getRelations().add(r_pOut2CE_pIn2Add);

		Relation r_pOutAdd_pOutModel = CoreFactory.eINSTANCE.createRelation();
		r_pOutAdd_pOutModel.setSource(pOutAdd);
		r_pOutAdd_pOutModel.setTarget(pOutModel);
		modelStructure.getRelations().add(r_pOutAdd_pOutModel);
	}

	private static void saveModel(String name) {
		// create resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register XML resource factory (before creating the resource)
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		// add the root object to the resource
		resource.getContents().add(model);
		// serialize resource
		try {
			resource.save(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
		}
	}
	
	private static boolean loadModel(String name) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		try {
			resource.load(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
			return false;
		}
		
		if (resource.getContents().size() != 1) {
			System.err.println("Error : there must be 1 element");
			return false;
		}
		
		EObject o = resource.getContents().get(0);
		try {
			model = (Model) o;
		}
		catch(ClassCastException ex) {
			System.err.println("Error : expected element not found : " + ex);
			return false;
		}
		
		// TODO ?
		
		return true;
	}
}
