package fr.supelec.modhelx.tests;

import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfDelay;
import fr.supelec.modhelx.mocs.sdflib.SdfPrinter;
import fr.supelec.modhelx.mocs.sdflib.SdfRamp;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;

public class TestSDF_Delay {

	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static Clock clock;

	public static void main(String[] args) {
		buildModel();

		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(10);

		Platform.setModel(model, clock);

		ModelWindow w = new ModelWindow(model, clock, "tests/TestSDF_Delay.xy");
		w.setVisible(true);
		w.run();

		Platform.run();
	}

	private static void buildModel() {
		model.setName("TestSDF_Delay");

		// MoC
		SdfModelOfComputation moc = SdfFactory.eINSTANCE.createSdfModelOfComputation();
		moc.setName("TestSDF_Delay_MoC");
		clock = TimeFactory.eINSTANCE.createLogicalClock();
		moc.setClock(clock);
		model.setMoc(moc);
		
		// Structure
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestSDF_Delay_Structure");
		model.setStructure(modelStructure);

		// Ramp
		SdfRamp ramp = SdflibFactory.eINSTANCE.createSdfRamp();
		ramp.setName("ramp");
		ramp.setValueOfParameter("initialValue", 30);
		ramp.setValueOfParameter("step", -15.5);
		ramp.setValueOfParameter("modulo", 32.5);
		modelStructure.getBlocks().add(ramp);

		Pin pOutRamp = CoreFactory.eINSTANCE.createPin();
		pOutRamp.setName("pOutRamp");
		ramp.getPinsOut().add(pOutRamp);
				
		// Delay
		SdfDelay delay= SdflibFactory.eINSTANCE.createSdfDelay();
		delay.setName("delay");
		modelStructure.getBlocks().add(delay);
		List<Number> data = new LinkedList<Number>();
		data.add(Math.PI);
		data.add(Math.E);
		data.add(42);
		delay.setValueOfParameter("initialData", data);
		
		Pin pInDelay = CoreFactory.eINSTANCE.createPin();
		pInDelay.setName("pInDelay");
		delay.getPinsIn().add(pInDelay);

		Pin pOutDelay = CoreFactory.eINSTANCE.createPin();
		pOutDelay.setName("pOutDelay");
		delay.getPinsOut().add(pOutDelay);

		//Printer
		SdfPrinter sdfPrinter = SdflibFactory.eINSTANCE.createSdfPrinter();
		sdfPrinter.setName("sdfPrinter");
		modelStructure.getBlocks().add(sdfPrinter);
		
		sdfPrinter.setPrintStream(System.out);

		Pin pInPrinter = CoreFactory.eINSTANCE.createPin();
		pInPrinter.setName("pInPrinter");
		sdfPrinter.getPinsIn().add(pInPrinter);

		//Relations
		Relation r1 = CoreFactory.eINSTANCE.createRelation();
		r1.setSource(pOutRamp);
		r1.setTarget(pInDelay);
		modelStructure.getRelations().add(r1);

		Relation r2 = CoreFactory.eINSTANCE.createRelation();
		r2.setSource(pOutDelay);
		r2.setTarget(pInPrinter);
		modelStructure.getRelations().add(r2);
	}
}
