/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestSDF_Limiter.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

//import java.io.File;
//import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

//import org.eclipse.emf.common.util.URI;
//import org.eclipse.emf.ecore.EObject;
//import org.eclipse.emf.ecore.resource.Resource;
//import org.eclipse.emf.ecore.resource.ResourceSet;
//import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
//import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfConstantEmitter;
import fr.supelec.modhelx.mocs.sdflib.SdfLimiter;
import fr.supelec.modhelx.mocs.sdflib.SdfPrinter;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;

public class TestSDF_Limiter {
	
	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static Clock clock;
	
	public static void main(String[] args) {
		
		buildModel();
//		saveModel("tests/sdf_6.modhelx");
//		model = null;
//		loadModel("tests/sdf_2.modhelx");

		//Platform.setLogFile("log/sdf_2_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(100);

		Platform.setModel(model, clock);

		ModelWindow w = new ModelWindow(model, clock, "tests/TestSDF_Limiter.xy");
		w.setVisible(true);
		w.run();

		Platform.run();
	}
	
	private static void buildModel() {
		model.setName("TestSDF_6");

		//MoC
		SdfModelOfComputation moc = SdfFactory.eINSTANCE.createSdfModelOfComputation();
		moc.setName("TestSDF_6_MoC");
		clock = TimeFactory.eINSTANCE.createLogicalClock();
		moc.setClock(clock);
		model.setMoc(moc);
		
		//Structure
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestSDF_6_Structure");
		model.setStructure(modelStructure);

		//Constant Emitter
		SdfConstantEmitter sdfConstEmit = SdflibFactory.eINSTANCE.createSdfConstantEmitter();
		sdfConstEmit.setName("SdfConstEmit");
		modelStructure.getBlocks().add(sdfConstEmit);
		sdfConstEmit.setValueToEmit(new Integer(10));

		Pin pOutCE = CoreFactory.eINSTANCE.createPin();
		pOutCE.setName("pOutCE");
		sdfConstEmit.getPinsOut().add(pOutCE);
		moc.setPinRate(pOutCE, 1);
		
		//Limiter
		SdfLimiter sdfLim = SdflibFactory.eINSTANCE.createSdfLimiter();
		sdfLim.setName("sdfLim");
		modelStructure.getBlocks().add(sdfLim);
		sdfLim.setLowerBound(new Integer(0));
		sdfLim.setUpperBound(new Integer(20));
		
		Pin pInLim = CoreFactory.eINSTANCE.createPin();
		pInLim.setName("pInLim");
		sdfLim.getPinsIn().add(pInLim);
		moc.setPinRate(pInLim, 1);

		Pin pOutLim = CoreFactory.eINSTANCE.createPin();
		pOutLim.setName("pOutLim");
		sdfLim.getPinsOut().add(pOutLim);
		moc.setPinRate(pOutLim, 1);

		//Printer
		SdfPrinter sdfPrinter = SdflibFactory.eINSTANCE.createSdfPrinter();
		sdfPrinter.setName("sdfPrinter");
		modelStructure.getBlocks().add(sdfPrinter);
		
		sdfPrinter.setPrintStream(System.out);

		Pin pInPrinter = CoreFactory.eINSTANCE.createPin();
		pInPrinter.setName("pInPrinter");
		sdfPrinter.getPinsIn().add(pInPrinter);
		moc.setPinRate(pInPrinter, 1);

		//Relations
		Relation r1 = CoreFactory.eINSTANCE.createRelation();
		r1.setSource(pOutCE);
		r1.setTarget(pInLim);
		modelStructure.getRelations().add(r1);

		Relation r2 = CoreFactory.eINSTANCE.createRelation();
		r2.setSource(pOutLim);
		r2.setTarget(pInPrinter);
		modelStructure.getRelations().add(r2);
	}

//	private static void saveModel(String name) {
//		// create resource set
//		ResourceSet resourceSet = new ResourceSetImpl();
//		// Register XML resource factory (before creating the resource)
//		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
//				"modhelx", new XMIResourceFactoryImpl());
//		Resource resource = resourceSet.createResource(URI.createFileURI(
//				new File(name).getAbsolutePath()));
//		// add the root object to the resource
//		resource.getContents().add(model);
//		// serialize resource
//		try {
//			resource.save(null);
//		}
//		catch(IOException ex) {
//			System.err.println("Exception : " + ex);
//		}
//	}
	
//	private static boolean loadModel(String name) {
//		ResourceSet resourceSet = new ResourceSetImpl();
//		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
//				"modhelx", new XMIResourceFactoryImpl());
//		Resource resource = resourceSet.createResource(URI.createFileURI(
//				new File(name).getAbsolutePath()));
//		try {
//			resource.load(null);
//		}
//		catch(IOException ex) {
//			System.err.println("Exception : " + ex);
//			return false;
//		}
//		
//		if (resource.getContents().size() != 1) {
//			System.err.println("Error : there must be 1 element");
//			return false;
//		}
//		
//		EObject o = resource.getContents().get(0);
//		try {
//			model = (Model) o;
//		}
//		catch(ClassCastException ex) {
//			System.err.println("Error : expected element not found : " + ex);
//			return false;
//		}
//		
//		
//		return true;
//	}
}
