package fr.supelec.modhelx.tests;

import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfPrinter;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;

public class TestSDF_Loop {

	private static CoreFactory coreFactory = CoreFactory.eINSTANCE;
	private static TimeFactory timeFactory = TimeFactory.eINSTANCE;
	private static SdfFactory sdfFactory = SdfFactory.eINSTANCE;
	private static SdflibFactory sdflibFactory = SdflibFactory.eINSTANCE;
	
	private static Model model = coreFactory.createModel();
	
	private static Clock clock;

	public static void main(String[] args) {
		buildModel();

		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(10);

		Platform.setModel(model, clock);

		ModelWindow w = new ModelWindow(model, clock, "tests/TestSDF_Loop.xy");
		w.setVisible(true);
		w.run();

		Platform.run();
	}

	private static void buildModel() {
		model.setName("TestSDF_Delay");

		// MoC
		SdfModelOfComputation moc = sdfFactory.createSdfModelOfComputation();
		moc.setName("TestSDF_Delay_MoC");
		clock = TimeFactory.eINSTANCE.createLogicalClock();
		moc.setClock(clock);
		model.setMoc(moc);

		// Structure
		CompositeBlock modelStructure = coreFactory.createCompositeBlock();
		modelStructure.setName("TestSDF_Delay_Structure");
		model.setStructure(modelStructure);

		// Ramp
		Block ramp = modelStructure.addBlock(sdflibFactory.createSdfRamp(), "ramp");
		Pin pOutRamp = ramp.newOutputPin("pOutRamp");

		// Adder
		Block add = modelStructure.addBlock(sdflibFactory.createSdfAdder(), "add");
		Pin fromRamp = add.newInputPin("fromRamp");
		Pin fromDelay= add.newInputPin("fromDelay");
		Pin toPrinter= add.newOutputPin("toPrinter");
		Pin toDelay= add.newOutputPin("toDelay");

		// Delay
		Block delay= modelStructure.addBlock(sdflibFactory.createSdfDelay(), "delay");
		delay.setValueOfParameter("initialData", 0);
		Pin pInDelay = delay.newInputPin("pInDelay");
		Pin pOutDelay = delay.newOutputPin("pOutDelay");

		//Printer
		Block printer = modelStructure.addBlock(sdflibFactory.createSdfPrinter(), "sdfPrinter");
		((SdfPrinter) printer).setPrintStream(System.out);
		Pin pInPrinter = printer.newInputPin("pInPrinter");


		//Relations
		modelStructure.connect(pOutRamp, fromRamp);
		modelStructure.connect(toPrinter, pInPrinter);
		modelStructure.connect(toDelay, pInDelay);
		modelStructure.connect(pOutDelay, fromDelay);

	}
}
