/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestSDF_Multiplier.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

//import java.io.File;
//import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

//import org.eclipse.emf.common.util.URI;
//import org.eclipse.emf.ecore.EObject;
//import org.eclipse.emf.ecore.resource.Resource;
//import org.eclipse.emf.ecore.resource.ResourceSet;
//import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
//import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfConstantEmitter;
import fr.supelec.modhelx.mocs.sdflib.SdfMultiplier;
import fr.supelec.modhelx.mocs.sdflib.SdfPrinter;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;

public class TestSDF_Multiplier {
	
	private static Model model = CoreFactory.eINSTANCE.createModel();
	
	private static Clock clock;
	
	public static void main(String[] args) {
		
		buildModel();
//		saveModel("tests/sdf_6.modhelx");
//		model = null;
//		loadModel("tests/sdf_2.modhelx");

		//Platform.setLogFile("log/sdf_2_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(5);

		Platform.setModel(model, clock);

		ModelWindow w = new ModelWindow(model, clock, "tests/TestSDF_Multiplier.xy");
		w.setVisible(true);
		w.run();

		Platform.run();
	}
	
	private static void buildModel() {
		model.setName("TestSDF_Multiplier");

		//MoC
		SdfModelOfComputation moc = SdfFactory.eINSTANCE.createSdfModelOfComputation();
		moc.setName("TestSDF_Multiplier_MoC");
		clock = TimeFactory.eINSTANCE.createLogicalClock();
		moc.setClock(clock);
		model.setMoc(moc);
		
		//Structure
		CompositeBlock modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		modelStructure.setName("TestSDF_Multiplier_Structure");
		model.setStructure(modelStructure);

		//Constant Emitter 1
		SdfConstantEmitter sdfConstEmit1 = SdflibFactory.eINSTANCE.createSdfConstantEmitter();
		sdfConstEmit1.setName("SdfConstEmit1");
		modelStructure.getBlocks().add(sdfConstEmit1);
		sdfConstEmit1.setValueToEmit(16);

		Pin pOutCE1 = CoreFactory.eINSTANCE.createPin();
		pOutCE1.setName("pOutCE1");
		sdfConstEmit1.getPinsOut().add(pOutCE1);
		// optional: default rate is 1
//		moc.setPinRate(pOutCE1, 1);
		
		//Constant Emitter 2
		SdfConstantEmitter sdfConstEmit2 = SdflibFactory.eINSTANCE.createSdfConstantEmitter();
		sdfConstEmit2.setName("SdfConstEmit2");
		modelStructure.getBlocks().add(sdfConstEmit2);
		sdfConstEmit2.setValueToEmit(4);

		Pin pOutCE2 = CoreFactory.eINSTANCE.createPin();
		pOutCE2.setName("pOutCE2");
		sdfConstEmit2.getPinsOut().add(pOutCE2);
//		moc.setPinRate(pOutCE2, 1);
		
		//Multiplier
		SdfMultiplier sdfMult = SdflibFactory.eINSTANCE.createSdfMultiplier();
		sdfMult.setName("sdfMult");
		modelStructure.getBlocks().add(sdfMult);
		
		Pin pInMult1 = CoreFactory.eINSTANCE.createPin();
		pInMult1.setName("pInMult1");
		// Optional: default exponent is 1
		//sdfMult.setExponent(pInMult1, 1);
		sdfMult.getPinsIn().add(pInMult1);
		moc.setPinRate(pInMult1, 2);

		Pin pInMult2 = CoreFactory.eINSTANCE.createPin();
		pInMult2.setName("pInMult2");
		sdfMult.setExponent(pInMult2, -0.5);
		sdfMult.getPinsIn().add(pInMult2);
//		moc.setPinRate(pInMult2, 1);

		Pin pOutMult = CoreFactory.eINSTANCE.createPin();
		pOutMult.setName("pOutMult");
		sdfMult.getPinsOut().add(pOutMult);
		moc.setPinRate(pOutMult, 3);

		//Printer
		SdfPrinter sdfPrinter = SdflibFactory.eINSTANCE.createSdfPrinter();
		sdfPrinter.setName("sdfPrinter");
		modelStructure.getBlocks().add(sdfPrinter);
		
		sdfPrinter.setPrintStream(System.out);

		Pin pInPrinter = CoreFactory.eINSTANCE.createPin();
		pInPrinter.setName("pInPrinter");
		sdfPrinter.getPinsIn().add(pInPrinter);
//		moc.setPinRate(pInPrinter, 1);

		//Relations
		Relation r1 = CoreFactory.eINSTANCE.createRelation();
		r1.setSource(pOutCE1);
		r1.setTarget(pInMult1);
		modelStructure.getRelations().add(r1);

		Relation r2 = CoreFactory.eINSTANCE.createRelation();
		r2.setSource(pOutCE2);
		r2.setTarget(pInMult2);
		modelStructure.getRelations().add(r2);

		Relation r3 = CoreFactory.eINSTANCE.createRelation();
		r3.setSource(pOutMult);
		r3.setTarget(pInPrinter);
		modelStructure.getRelations().add(r3);
	}

//	private static void saveModel(String name) {
//		// create resource set
//		ResourceSet resourceSet = new ResourceSetImpl();
//		// Register XML resource factory (before creating the resource)
//		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
//				"modhelx", new XMIResourceFactoryImpl());
//		Resource resource = resourceSet.createResource(URI.createFileURI(
//				new File(name).getAbsolutePath()));
//		// add the root object to the resource
//		resource.getContents().add(model);
//		// serialize resource
//		try {
//			resource.save(null);
//		}
//		catch(IOException ex) {
//			System.err.println("Exception : " + ex);
//		}
//	}
	
//	private static boolean loadModel(String name) {
//		ResourceSet resourceSet = new ResourceSetImpl();
//		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
//				"modhelx", new XMIResourceFactoryImpl());
//		Resource resource = resourceSet.createResource(URI.createFileURI(
//				new File(name).getAbsolutePath()));
//		try {
//			resource.load(null);
//		}
//		catch(IOException ex) {
//			System.err.println("Exception : " + ex);
//			return false;
//		}
//		
//		if (resource.getContents().size() != 1) {
//			System.err.println("Error : there must be 1 element");
//			return false;
//		}
//		
//		EObject o = resource.getContents().get(0);
//		try {
//			model = (Model) o;
//		}
//		catch(ClassCastException ex) {
//			System.err.println("Error : expected element not found : " + ex);
//			return false;
//		}
//		
//		
//		return true;
//	}
}
