package fr.supelec.modhelx.tests;

import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.corelib.CorelibFactory;
import fr.supelec.modhelx.corelib.PluginBlock;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;

public class TestSDF_Plugin {

	private static CoreFactory coreFactory = CoreFactory.eINSTANCE;
	private static CorelibFactory corelibFactory = CorelibFactory.eINSTANCE;
	private static TimeFactory timeFactory = TimeFactory.eINSTANCE;
	private static SdfFactory sdfFactory = SdfFactory.eINSTANCE;
	private static SdflibFactory sdflibFactory = SdflibFactory.eINSTANCE;
	
	private static Model model = coreFactory.createModel();
	
	private static Clock clock;

	public static void main(String[] args) {
		buildModel();

		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
		Platform.setNumberOfSnapshots(0);

		Platform.setModel(model, clock);

		ModelWindow w = new ModelWindow(model, clock, "tests/TestSDF_Plugin.xy");
		w.setVisible(true);
		w.run();

		Platform.run();
	}

	private static void buildModel() {
		model.setName("TestSDF_Plugin");

		// MoC
		SdfModelOfComputation moc = sdfFactory.createSdfModelOfComputation();
		moc.setName("TestSDF_Plugin_MoC");
		clock = TimeFactory.eINSTANCE.createLogicalClock();
		moc.setClock(clock);
		model.setMoc(moc);

		// Structure
		CompositeBlock modelStructure = coreFactory.createCompositeBlock();
		modelStructure.setName("TestSDF_Plugin_Structure");
		model.setStructure(modelStructure);

		// Ramp
		Block ramp = modelStructure.addBlock(sdflibFactory.createSdfRamp(), "ramp");
		Pin rampToPrinter = ramp.newOutputPin("rampToPrinter");
		Pin rampToPlotter = ramp.newOutputPin("rampToPlotter");
		
		// Plug-in SineGen
		PluginBlock sinegen = (PluginBlock) modelStructure.addBlock(corelibFactory.createPluginBlock(), "sine");
		Pin sineToPrinter= sinegen.newOutputPin("toPrinter");
		Pin sineToPlotter= sinegen.newOutputPin("toPlotter");
		sinegen.setPluggedInClassName("fr.supelec.modhelx.tests.plugins.SdfSineGen");
		sinegen.setValueOfParameter("frequency", 440);
		sinegen.setValueOfParameter("samplingRate", 8000);
		
		// Plug-in SwingPrinter
		PluginBlock swingprint = (PluginBlock) modelStructure.addBlock(corelibFactory.createPluginBlock(), "printer");
		Pin fromSine= swingprint.newInputPin("fromSine");
		Pin fromRamp= swingprint.newInputPin("fromRamp");
//		moc.setPinRate(fromRamp, 2);
		swingprint.setPluggedInClassName("fr.supelec.modhelx.tests.plugins.SdfSwingPrinter");
		
		// Plug-in SwingPlotter
		PluginBlock swingplot = (PluginBlock) modelStructure.addBlock(corelibFactory.createPluginBlock(), "plotter");
		Pin plotFromSine= swingplot.newInputPin("Sine");
		Pin plotFromRamp= swingplot.newInputPin("Ramp");
		swingplot.setPluggedInClassName("fr.supelec.modhelx.tests.plugins.SdfSwingPlotter");
		swingplot.setValueOfParameter("zoom_y", 40);
		swingplot.setValueOfParameter("zoom_x", 10);
		swingplot.setValueOfParameter("y_min", -1.5);
		swingplot.setValueOfParameter("y_max", 1.5);
		swingplot.setValueOfParameter("history", 0);
		//
		swingplot.setValueOfParameter("Sine_plotColor", "BLUE");
		swingplot.setValueOfParameter("Sine_dotSize", 3);
		swingplot.setValueOfParameter("Sine_lineWidth", 2);
		swingplot.setValueOfParameter("Ramp_plotColor", "RED");
		
		//Relations
		modelStructure.connect(sineToPrinter, fromSine);
		modelStructure.connect(rampToPrinter, fromRamp);
		modelStructure.connect(sineToPlotter, plotFromSine);
		modelStructure.connect(rampToPlotter, plotFromRamp);

	}
}
