/**

 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: TestTFSMAymen.java 331 2011-03-11 12:01:35Z marcadet $
 */

package fr.supelec.modhelx.tests;

import java.io.StringReader;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.time.Clock;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.tfsm.TFsmModelOfComputation;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.qdgui.rendering.TFSMStateR;
import fr.supelec.modhelx.qdgui.rendering.TFSMStructureR;
import fr.supelec.modhelx.qdgui.rendering.TFSMTransitionR;
import fr.supelec.modhelx.util.TFSMBuilder;

public class TestTFSMAymen {
	private static Model model;
	private static Clock clock;
	
	public static void main(String[] args) {
		
		buildModel();
//		saveModel("tests/tfsm.modhelx");
//		if (args.length > 0) {
//			model = null;
//			loadModel("tests/tfsm.modhelx");
//		}
		//Platform.setLogFile("log/sdf_4_log.txt");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new SimpleFormatter());
//		Platform.setNumberOfSnapshots(2);
		Platform.setModel(model, clock);
		
//		// simulation in "real time"
//		Platform.setRealTime(1000, 1000);
		
		TFSMStructureR.register();
		TFSMStateR.register();
		TFSMTransitionR.register();
		ModelWindow w = new ModelWindow(model, clock, "tests/testTFSM.txt");
		w.setVisible(true);

		Platform.setRealTime(1000, 1000);
		
		w.run();		
		Platform.run();
	}
	
	
	private static Pin getPinForName(CompositeBlock comp, String pinName) {
		Pin res = getPinForName(comp.getPinsIn(), pinName);
		if(res == null) {
			res = getPinForName(comp.getPinsOut(), pinName);
		}
		
		return res;
	}
	
	private static Pin getPinForName(List<Pin> pins, String pinName) {
		for(Pin p : pins) {
			if(pinName.equals(p.getName())) return p;
		}
		
		return null;
	}
	
	private static void buildModel() {
		
		TFSMBuilder b = new TFSMBuilder("TestTFSM_MoC", new StringReader(
				"In coin\n" +			
				"In coffee\n" +
				"Out coffee_prepared\n" +
				"Out coffee_served\n" +
				
				"InitialState idle\n" +		
				"State ready\n" +
				"State prep\n" +
				
				"Transition idle ready Or coin\n" +
				"Transition ready prep Or coffee /coffee_prepared\n" + 
				"Transition prep idle Delay 2 /coffee_served\n"
				));
		model = b.getModel();
		model.setName("TestTFSM");
		
		clock = ((TFsmModelOfComputation)model.getMoc()).getClock();
		
		// add tokens in input
		/* pourquoi ces erreurs de compilation ? 
		Token ex = b.getMoc().createEvent();
		b.getPin("coin").addToken(ex);
		ex = b.getMoc().createEvent();
		b.getPin("coffee").addToken(ex);
		*/
		
		Token ex = ((TFsmModelOfComputation)model.getMoc()).createEvent();
		getPinForName(model.getStructure(), "coin").addToken(ex);
		ex = ((TFsmModelOfComputation)model.getMoc()).createEvent();
		getPinForName(model.getStructure(), "coffee").addToken(ex);
		
//		tfsmmoc = TfsmFactory.eINSTANCE.createTFsmModelOfComputation();
//		tfsmmoc.setName("TFSM");
//		
//		tfsmmoc.setClock(TimeFactory.eINSTANCE.createLogicalClock());
//		
//		modelStructure = CoreFactory.eINSTANCE.createCompositeBlock();
//		modelStructure.setName("TFSM");
//		
//		// the following parameter is added as a hint for the GUI to identify
//		// a composite block that contains a TFSM
//		modelStructure.setValueOfParameter("TFSMRole", "TFSMStructure");
//
//		model = CoreFactory.eINSTANCE.createModel();
//		model.setName("TFSM");
//		model.setMoc(tfsmmoc);
//		model.setStructure(modelStructure);
		
		
//		Create the automaton in the internal model
		
		//Create initial idle state
//		Block idle_state = CoreFactory.eINSTANCE.createAtomicBlock();
//		idle_state.setName("idle");
//		tfsmmoc.setInitialStateFeature(idle_state);
//		modelStructure.getBlocks().add(idle_state);
//
//		Pin from_trans3_on_idle_state = CoreFactory.eINSTANCE.createPin();
//		from_trans3_on_idle_state.setName("from_trans");
//		idle_state.getPinsIn().add(from_trans3_on_idle_state);
//
//		Pin to_trans1_on_idle_state = CoreFactory.eINSTANCE.createPin();
//		to_trans1_on_idle_state.setName("to_trans");
//		idle_state.getPinsOut().add(to_trans1_on_idle_state);
//
//		//Create ready state
//		Block ready_state = CoreFactory.eINSTANCE.createAtomicBlock();
//		ready_state.setName("ready");
//		tfsmmoc.setStateRole(ready_state);
//		modelStructure.getBlocks().add(ready_state);
//
//		Pin from_trans1_on_ready_State = CoreFactory.eINSTANCE.createPin();
//		from_trans1_on_ready_State.setName("from_trans");
//		ready_state.getPinsIn().add(from_trans1_on_ready_State); 
//
//		Pin to_trans2_on_ready_state = CoreFactory.eINSTANCE.createPin();
//		to_trans2_on_ready_state.setName("to_trans");
//		ready_state.getPinsOut().add(to_trans2_on_ready_state); 
//		
//		//Create prep state
//		Block prep_state = CoreFactory.eINSTANCE.createAtomicBlock();
//		ready_state.setName("ready");
//		tfsmmoc.setStateRole(ready_state);
//		modelStructure.getBlocks().add(ready_state);
//
//		Pin from_trans2_on_prep_State = CoreFactory.eINSTANCE.createPin();
//		from_trans2_on_prep_State.setName("from_trans");
//		prep_state.getPinsIn().add(from_trans2_on_prep_State); 
//
//		Pin to_trans3_on_prep_state = CoreFactory.eINSTANCE.createPin();
//		to_trans2_on_ready_state.setName("to_trans");
//		prep_state.getPinsOut().add(to_trans3_on_prep_state); 
//		
//		//Create transition trans1
//		CompositeBlock trans1 = CoreFactory.eINSTANCE.createCompositeBlock();
//		trans1.setName("trans_idle_to_ready");
//		tfsmmoc.setTransitionRole(trans1);
//		modelStructure.getBlocks().add(trans1);
//
//		Pin from_idle_state_on_trans1 = CoreFactory.eINSTANCE.createPin();
//		from_idle_state_on_trans1.setName("from_state");
//		trans1.getPinsIn().add(from_idle_state_on_trans1);
//
//		Relation idle_state_to_trans1 = CoreFactory.eINSTANCE.createRelation();
//		idle_state_to_trans1.setSource(to_trans1_on_idle_state);
//		idle_state_to_trans1.setTarget(from_idle_state_on_trans1);
//		modelStructure.getRelations().add(idle_state_to_trans1);
//
//		Pin to_ready_state_on_trans1 = CoreFactory.eINSTANCE.createPin();
//		to_ready_state_on_trans1.setName("to_state");
//		trans1.getPinsOut().add(to_ready_state_on_trans1);
//		
//		Relation trans1_to_ready_state = CoreFactory.eINSTANCE.createRelation();
//		trans1_to_ready_state.setSource(to_ready_state_on_trans1);
//		trans1_to_ready_state.setTarget(from_trans1_on_ready_State);
//		modelStructure.getRelations().add(trans1_to_ready_state);
	}
}
