package fr.supelec.modhelx.tests.powerwindow;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Polygon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.util.Numeric;
import fr.supelec.modhelx.util.plugins.DeSynchronousEmitter;
import fr.supelec.modhelx.util.plugins.SynchronousEventSource;
import fr.supelec.modhelx.util.plugins.ValueDisplay;

public class PowerWindowGUI extends JFrame implements ActionListener, SynchronousEventSource, ValueDisplay {

	private static final long serialVersionUID = 1L;
	
	private static class WindowGUI extends JComponent {
		private static final long serialVersionUID = 1L;
		private double pos_ = 0.0;
		private Color win_color_tint_ = new Color(0.4f, 0.4f, 0.4f, 0.7f);
		private Color win_color_clear_ = new Color(0.6f, 0.6f, 0.6f, 0.4f);
		private Color win_color_ = win_color_clear_;
		private Image door_ = java.awt.Toolkit.getDefaultToolkit().createImage(getClass().getClassLoader().getResource("fr/supelec/modhelx/tests/powerwindow/Portiere.png"));
		private Image hand_ = java.awt.Toolkit.getDefaultToolkit().createImage(getClass().getClassLoader().getResource("fr/supelec/modhelx/tests/powerwindow/main_small.png"));
		private int y_basewin_ = 167;
		private int y_wintop_ = 45;
		private int y_obstacle_ = -1;
		
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			Graphics2D g2d = (Graphics2D) g;
			g2d.drawImage(door_, 0, 0, null);
			int zone_x[] = {180, 150, 122, 102, 102, 328, 328, 354, 350, 340, 320, 272, 180 };
			int zone_y[] = { 45,  80, 120, 167, 300, 300, 167, 136,  98,  84,  74,  60,  45 };
			Polygon zone = new Polygon(zone_x, zone_y, zone_x.length);
			g2d.setClip(zone);
			if (y_obstacle_ >= 0) {
				g2d.drawImage(hand_, 150, y_obstacle_, null);
			}
			int frame_x[] = {180, 150, 122, 102, 328, 354, 350, 340, 320, 272, 180 };
			int frame_y[] = { 45,  80, 120, 167, 167, 136,  98,  84,  74,  60,  45 };
			Polygon window = new Polygon(frame_x, frame_y, frame_x.length);
			window.translate((int)((pos_ - 1.0) * 95),
					         (int)((1.0 - pos_) * (y_basewin_ - y_wintop_)));
			Polygon frame = new Polygon(frame_x, frame_y, frame_x.length);
			g2d.setClip(frame);
			g2d.setColor(win_color_);
			g2d.fillPolygon(window);
		}
	
		public Dimension getPreferredSize() {
			return new Dimension(400, 400);
		}

		public void setTinted(boolean tinted) {
			if (tinted) {
				win_color_ = win_color_tint_;
			} else {
				win_color_ = win_color_clear_;
			}
			repaint();
		}

		public void setObstacle(boolean present) {
			if (present) {
				y_obstacle_ = y_wintop_ + (int)((1 - pos_) * (y_basewin_ - y_wintop_)) - 50;
			} else {
				y_obstacle_ = -1;
			}
			repaint();
		}
		
		public void setPos(double pos) {
			pos_ = pos;
			repaint();
		}
	}
	
	private WindowGUI myWindow_ = null;
	private DeSynchronousEmitter myEmitter_ = null;
	private Pin userCmdPin_ = null;
	private Pin obstaclePin_ = null;
	private static PowerWindowGUI instance_ = null;
	
	private JLabel upButton_ = null;
	private JLabel downButton_ = null;
	private boolean winClosing_ = false;
	private boolean winOpening_ = false;
	private boolean upPushed_ = false;
	private boolean downPushed_ = false;
	
	public static PowerWindowGUI getInstance() {
		if (instance_ == null) {
			instance_ = new PowerWindowGUI();
		}
		return instance_;
	}
	
	private void updateButtons() {
		upButton_.setIcon(triangleUp(upPushed_, winClosing_));
		downButton_.setIcon(triangleDown(downPushed_, winOpening_));
		repaint();
	}
	
	private Icon triangleUp(boolean pushed, boolean on) {
		Color bg = new Color(1.0f, 1.0f, 1.0f, 0.0f); // transparent white
		Color fg = new Color(0.5f, 0.5f, 0.5f, 1.0f); // opaque dark gray
		Color line = new Color(0.2f, 0.2f, 0.2f, 1.0f); // opaque very dark gray
		if (pushed) {
			line = new Color(0.8f, 0.8f, 0.8f, 1.0f); // opaque very light gray
		}
		if (on) {
			fg = new Color(0.5f, 1.0F, 0.5f, 1.0f);
		}
		Image img = new BufferedImage(20, 20, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = (Graphics2D) img.getGraphics();
		g.setBackground(bg);
		g.setColor(fg);
		Polygon p = new Polygon();
		p.addPoint(0, 19);
		p.addPoint(19, 19);
		p.addPoint(10,0);
		p.addPoint(0, 19);
		g.clearRect(0, 0, 20, 20);
		g.fillPolygon(p);
		g.setStroke(new BasicStroke(2.0f));
		g.setColor(line);
		g.drawPolygon(p);
		return new ImageIcon(img);
	}
		
	private Icon triangleDown(boolean pushed, boolean on) {
		Color bg = new Color(1.0f, 1.0f, 1.0f, 0.0f); // transparent white
		Color fg = new Color(0.5f, 0.5f, 0.5f, 1.0f); // opaque dark gray
		Color line = new Color(0.2f, 0.2f, 0.2f, 1.0f); // opaque very dark gray
		if (pushed) {
			line = new Color(0.8f, 0.8f, 0.8f, 1.0f); // opaque very light gray
		}
		if (on) {
			fg = new Color(0.5f, 1.0F, 0.5f, 1.0f);
		}
		Image img = new BufferedImage(20, 20, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = (Graphics2D) img.getGraphics();
		g.setBackground(bg);
		g.setColor(fg);
		Polygon p = new Polygon();
		p.addPoint(1, 1);
		p.addPoint(10, 19);
		p.addPoint(19, 1);
		p.addPoint(1, 1);
		g.clearRect(0, 0, 20, 20);
		g.fillPolygon(p);
		g.setStroke(new BasicStroke(2.0f));
		g.setColor(line);
		g.drawPolygon(p);
		return new ImageIcon(img);
	}
	
	private PowerWindowGUI() {
		getContentPane().setLayout(new BorderLayout());
		JPanel cmd = new JPanel(new GridLayout(6, 1));
		// "Up" button
		upButton_ = new JLabel(triangleUp(false, winClosing_));
		upButton_.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				myEmitter_.emitEventNow(userCmdPin_, 1.0);
				upPushed_ = true;
				updateButtons();
			}
			public void mouseReleased(MouseEvent e) {
				myEmitter_.emitEventNow(userCmdPin_, 0.0);
				upPushed_ = false;
				updateButtons();
			}
		});
		cmd.add(upButton_);

		// Down button
		downButton_ = new JLabel(triangleDown(false, winOpening_));
		downButton_.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				myEmitter_.emitEventNow(userCmdPin_, -1.0);
				downPushed_ = true;
				updateButtons();
			}
			public void mouseReleased(MouseEvent e) {
				myEmitter_.emitEventNow(userCmdPin_, 0.0);
				downPushed_ = false;
				updateButtons();
			}
		});
		cmd.add(downButton_);
		// Obstacle
		JCheckBox obst = new JCheckBox("Obstacle", false);
		obst.setActionCommand("OBSTACLE");
		obst.addActionListener(this);
		cmd.add(obst);
		//
		JCheckBox tint = new JCheckBox("Tinted windows", false);
		tint.setActionCommand("TINT");
		tint.addActionListener(this);
		cmd.add(tint);
		//
		myWindow_ = new WindowGUI();
		JPanel left = new JPanel();
		left.add(cmd);
		getContentPane().add(left, BorderLayout.WEST);
		getContentPane().add(myWindow_, BorderLayout.EAST);
	}
	
	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		if (cmd.equals("TINT")) {
			myWindow_.setTinted(((JCheckBox)e.getSource()).isSelected());
		} else if (cmd.equals("OBSTACLE")) {
			if (((JCheckBox)e.getSource()).isSelected()) {
				myEmitter_.emitEventNow(obstaclePin_, 1);
				myWindow_.setObstacle(true);
			} else {
				myEmitter_.emitEventNow(obstaclePin_, -1);
				myWindow_.setObstacle(false);
			}
		}
	}

	public void setEmitter(DeSynchronousEmitter emitter) {
		myEmitter_  = emitter;
		if (emitter == null) {
			throw new Error(getClass().getName()+": emitter is null");
		}
		for (Pin p : myEmitter_.getPinsOut()) {
			if (p.getName().equals("cmd_user")) {
				userCmdPin_ = p;
			} else if (p.getName().equals("cmd_obstacle")) {
				obstaclePin_ = p;
			}
		}
		if (userCmdPin_== null) {
			throw new Error(getClass().getName()+": emitter has no cmd_user output pin");
		}
	}
	
	public void valueChanged(String name, Object value) {
		if (name.equals("position")) {
			myWindow_.setPos(Numeric.doubleValueOfObject(value) / 40);
		}
		if (name.equals("cmd_motor")) {
			if (value.equals(-1.0)) {
				winOpening_ = true;
				winClosing_ = false;
			} else if (value.equals(0.0)) {
				winOpening_ = false;
				winClosing_ = false;
			} else if (value.equals(1.0)) {
				winOpening_ = false;
				winClosing_ = true;
			}
			updateButtons();
		}
	}
	
	public void setup() {
		pack();
		setVisible(true);
	}

	public void wrapup() {
		setVisible(false);
	}

	@Override
	public String toString() {
		return getClass().getSimpleName();
	}
}
