package fr.supelec.modhelx.tests.powerwindow;

import java.util.logging.Level;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.corelib.CorelibFactory;
import fr.supelec.modhelx.corelib.PluginBlock;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.DeSdfAdapter;
import fr.supelec.modhelx.mocs.adapterlib.DeTfsmAdapter;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.tfsm.TFsmModelOfComputation;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.util.DEBlockDependencyComparator;
import fr.supelec.modhelx.util.LogFormatter;
import fr.supelec.modhelx.util.ModelBuilder;
import fr.supelec.modhelx.util.plugins.DeKeypad;

public class SimpleWindow {
	public static void main(String[] args) {
		
		//modèle racine DE
		DeModelOfComputation deMoc = DeFactory.eINSTANCE.createDeModelOfComputation();
		deMoc.setName("DE_moc");
		
		Model deModel = CoreFactory.eINSTANCE.createModel();
		deModel.setName("PowerWindow");
		deModel.setMoc(deMoc);
		
		CompositeBlock deStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		deStructure.setName("PowerWindow_structure");
		deModel.setStructure(deStructure);
		
		// scenario player
		/*
		DeScenarioPlayer player = DelibFactory.eINSTANCE.createDeScenarioPlayer();
		player.setName("User");
		//scénario : on remonte la fenetre en mode auto, on insere un obstacle, puis on descend la fenetre en mode auto
		ModelBuilder.addDeScenarioEvent(player, 2, "cmd_user", 1.0);//montee
		ModelBuilder.addDeScenarioEvent(player, 3, "cmd_user", 0.0);//en mode automatique
		
		ModelBuilder.addDeScenarioEvent(player, 10, "cmd_obstacle", 1.0);//obstacle
																		//la fenetre redescend pdt 2 tps puis stoppe
		ModelBuilder.addDeScenarioEvent(player, 15, "cmd_obstacle", 0.0);//on retire l'obstacle (juste pour etre propre)
		
		ModelBuilder.addDeScenarioEvent(player, 18, "cmd_user", -1.0);//descente
		ModelBuilder.addDeScenarioEvent(player, 19, "cmd_user", 0.0);//en mode automatique
		*/

		
		PluginBlock player = CorelibFactory.eINSTANCE.createPluginBlock();
		player.setPluggedInClassName("fr.supelec.modhelx.util.plugins.DeSynchronousEmitter");
		player.setName("SyncEmit_Keypad");
		DeKeypad keypad = new DeKeypad();
		player.setValueOfParameter("source", keypad);

		keypad.addKey(player, "Up", "cmd_user", 1.);
		keypad.addKey(player, "Stop", "cmd_user", 0.);
		keypad.addKey(player, "Down", "cmd_user", -1.);
		keypad.addKey(player, "Obstacle", "cmd_obstacle", 1.);

//		keypad.display();
		
		
		/*/////
		DeEventPrinter printer = DelibFactory.eINSTANCE.createDeEventPrinter();
		printer.setPrintStream(System.out);
		
		Pin prUser = ModelBuilder.addPinIn(printer, "user");
		deStructure.connect(ModelBuilder.pinByName(player, "cmd_user"), prUser);
		
		deStructure.addBlock(printer, "EventPrinter");
		////*/
		
		
		deStructure.getBlocks().add(player);
		
		//controller
		DeTfsmAdapter tfsm_controller = SimpleWindowTFSMSubSystem.buildSimpleWindowIBForTFSMSubSystem(deModel);

		//get the TFSM clock
		deMoc.setClock(((TFsmModelOfComputation)tfsm_controller.getInternalModel().getMoc()).getClock());
		
		// connect the player to the controller
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(player, "cmd_user"), ModelBuilder.pinByName(tfsm_controller, "cmd_user"));
		
		// create the window system (interface block that embeds the SDF sub-model)
		DeSdfAdapter deWindowSystem = SimpleWindowSDFSubSystem.buildSimpleWindowIBForSDFSubSystem(deModel);
			
		//connect the controller to the window system
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(tfsm_controller, "cmd_motor"), ModelBuilder.pinByName(deWindowSystem, "cmd_motor"));
		
		//connect the player to the window system (obstacle)
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(player, "cmd_obstacle"), ModelBuilder.pinByName(deWindowSystem, "cmd_obstacle"));

		//connect back (loop) the window system to the controller
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(deWindowSystem, "win_obstacle"), ModelBuilder.pinByName(tfsm_controller, "win_obstacle"));
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(deWindowSystem, "win_endstop"), ModelBuilder.pinByName(tfsm_controller, "win_endstop"));

		DEBlockDependencyComparator.setPinInstantaneousDependency(
				ModelBuilder.pinByName(deWindowSystem, "win_obstacle"), false);
		
		DEBlockDependencyComparator.setPinInstantaneousDependency(
				ModelBuilder.pinByName(deWindowSystem, "win_endstop"), false);

		
		Platform.setModel(deModel, deMoc.getClock());
		
		Platform.setLogFile("log/SimpleWindow.log");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new LogFormatter());
		
		// simulation in "real time"
		Platform.setRealTime(1000, 1000);

		ModelWindow w = new ModelWindow(deModel, deMoc.getClock(), "tests/simplewindow.xy");

		w.setVisible(true);
		w.run();
	}
}
