package fr.supelec.modhelx.tests.powerwindow;

import java.util.logging.Level;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.time.TimeFactory;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.AdapterlibFactory;
import fr.supelec.modhelx.mocs.adapterlib.DeSdfAdapter;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DeScenarioPlayer;
import fr.supelec.modhelx.mocs.delib.DelibFactory;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfAccumulator;
import fr.supelec.modhelx.mocs.sdflib.SdfAdder;
import fr.supelec.modhelx.mocs.sdflib.SdfDuplicator;
import fr.supelec.modhelx.mocs.sdflib.SdfLimiter;
import fr.supelec.modhelx.mocs.sdflib.SdfMultiplier;
import fr.supelec.modhelx.mocs.sdflib.SdfPrinter;
import fr.supelec.modhelx.mocs.sdflib.SdfThreshold;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.util.BlockComparator;
import fr.supelec.modhelx.util.LogFormatter;
import fr.supelec.modhelx.util.ModelBuilder;

public class SimpleWindowSDFSubSystem {
	public static void main(String[] args) {
		//create the DE model
		DeModelOfComputation deMoc = DeFactory.eINSTANCE.createDeModelOfComputation();
		deMoc.setName("DE_moc");
		deMoc.setClock(TimeFactory.eINSTANCE.createLogicalClock());
		
		Model deModel = CoreFactory.eINSTANCE.createModel();
		deModel.setName("DE_model");
		deModel.setMoc(deMoc);
		
		CompositeBlock deStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		deStructure.setName("DE_structure");
		deModel.setStructure(deStructure);
		
		// scenario player
		DeScenarioPlayer player = DelibFactory.eINSTANCE.createDeScenarioPlayer();
		player.setName("Player");
		ModelBuilder.addDeScenarioEvent(player, 1, "cmd_motor", 1);
//		ModelBuilder.addDeScenarioEvent(player, 7, "win_endstop", -1);
//		ModelBuilder.addDeScenarioEvent(player, 12, "cmd_motor", -1);
		ModelBuilder.addDeScenarioEvent(player, 5, "cmd_obstacle", 1);
		ModelBuilder.addDeScenarioEvent(player, 6, "cmd_motor", -1);
		ModelBuilder.addDeScenarioEvent(player, 8, "cmd_motor", 0);
		//ModelBuilder.addDeScenarioEvent(player, 14, "cmd_user", 0);
		deStructure.getBlocks().add(player);

		//IB for SDF window system
		DeSdfAdapter deWindowSystem = buildSimpleWindowIBForSDFSubSystem(deModel);
			
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(player, "cmd_motor"), ModelBuilder.pinByName(deWindowSystem, "cmd_motor"));
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(player, "cmd_obstacle"), ModelBuilder.pinByName(deWindowSystem, "cmd_obstacle"));
			
		// de printer obstacle
		DeEventPrinter obsPrint = DelibFactory.eINSTANCE.createDeEventPrinter();
		obsPrint.setName("obstacle printer");
		obsPrint.setPrintStream(System.out);
		deStructure.getBlocks().add(obsPrint);
		
		Pin pin_obsPrint = CoreFactory.eINSTANCE.createPin();
		pin_obsPrint.setName("obstacle");
		obsPrint.getPinsIn().add(pin_obsPrint); 

		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(deWindowSystem, "win_obstacle"), pin_obsPrint);
//		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(player, "cmd_motor"), pin_obsPrint);
		
		// de printer endstop
		DeEventPrinter esPrint = DelibFactory.eINSTANCE.createDeEventPrinter();
		esPrint.setName("endstop printer");
		esPrint.setPrintStream(System.out);
		deStructure.getBlocks().add(esPrint);
		
		Pin pin_esPrint = CoreFactory.eINSTANCE.createPin();
		pin_esPrint.setName("endstop");
		esPrint.getPinsIn().add(pin_esPrint); 

		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(deWindowSystem, "win_endstop"), pin_esPrint);
//		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(player, "obstacle"), pin_esPrint);
		
		

		Platform.setLogFile("log/SimpleWindowSDFSubSystem_.log");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new LogFormatter());
		Platform.setNumberOfSnapshots(100);

		Platform.setModel(deModel, deMoc.getClock());

		ModelWindow w = new ModelWindow(deModel, deMoc.getClock(), "tests/SimpleWindowSDFSUbsystem.xy");
//		ModelWindow w = new ModelWindow(deModel);
		w.setVisible(true);
		w.run();
	}
	
	public static DeSdfAdapter buildSimpleWindowIBForSDFSubSystem(Model deModel){
		// interface block that embedds the SDF window system
		DeSdfAdapter deWindowSystem = AdapterlibFactory.eINSTANCE.createDeSdfAdapter();
		deWindowSystem.setName("WindowSystem_ib");
		deWindowSystem.setSampleTime(1);
		deModel.getStructure().getBlocks().add(deWindowSystem);
		
		Pin pin_dews_motorIn = ModelBuilder.addPinIn(deWindowSystem, "cmd_motor");
 		Pin pin_dews_obstacleIn = ModelBuilder.addPinIn(deWindowSystem, "cmd_obstacle");
		Pin pin_dews_obstacleOut = ModelBuilder.addPinOut(deWindowSystem, "win_obstacle");
		BlockComparator.setInstantaneous(pin_dews_obstacleOut, false);
		Pin pin_dews_endstopOut = ModelBuilder.addPinOut(deWindowSystem, "win_endstop");
		BlockComparator.setInstantaneous(pin_dews_endstopOut, false);
		
		//internal model
//		Model windowSystem = buildSimpleWindowSDFSubSystemModel(deModel);
		Model windowSystem = buildSimpleWindowSDFSubSystemModelWithoutDUPs(deModel);
		deWindowSystem.setInternalModel(windowSystem);
		
		//internal relations
		ModelBuilder.addRelation(deWindowSystem, pin_dews_motorIn, ModelBuilder.pinByName(windowSystem.getStructure(), "cmd_motor"));
		ModelBuilder.addRelation(deWindowSystem, pin_dews_obstacleIn, ModelBuilder.pinByName(windowSystem.getStructure(), "cmd_obstacle"));
		ModelBuilder.addRelation(deWindowSystem, ModelBuilder.pinByName(windowSystem.getStructure(), "win_obstacle"), pin_dews_obstacleOut);
		ModelBuilder.addRelation(deWindowSystem, ModelBuilder.pinByName(windowSystem.getStructure(), "win_endstop"), pin_dews_endstopOut);
			
		//internal pins initial values
		deWindowSystem.setInitialValue(ModelBuilder.pinByName(windowSystem.getStructure(), "cmd_obstacle"), 0.0);
		deWindowSystem.setInitialValue(ModelBuilder.pinByName(windowSystem.getStructure(), "cmd_motor"), 0.0);

		return deWindowSystem;
	}
	
	public static final int MOTOR_TORQUE = 3;
	public static final double WINDOW_HEIGHT = 40.0;
	public static final double OBSTACLE_FORCE = 2;

	public static Model buildSimpleWindowSDFSubSystemModelWithoutDUPs(Model deModel){
		Model windowSystem = CoreFactory.eINSTANCE.createModel();
		windowSystem.setName("WindowSystem");
		
		SdfModelOfComputation sdfMoc = SdfFactory.eINSTANCE.createSdfModelOfComputation();
		sdfMoc.setName("SDF_moc");
		sdfMoc.setClock(((DeModelOfComputation)deModel.getMoc()).getClock());
		windowSystem.setMoc(sdfMoc);

		CompositeBlock windowSystemStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		windowSystemStructure.setName("WindowSystem_structure");
		windowSystem.setStructure(windowSystemStructure);

		Pin pin_ws_cmdMotor = ModelBuilder.addSDFPinIn(windowSystemStructure, "cmd_motor", sdfMoc, 1);
		Pin pin_ws_cmdObstacle = ModelBuilder.addSDFPinIn(windowSystemStructure, "cmd_obstacle", sdfMoc, 1);
		Pin pin_ws_obstacle = ModelBuilder.addSDFPinOut(windowSystemStructure, "win_obstacle", sdfMoc, 1);
		Pin pin_ws_endstop = ModelBuilder.addSDFPinOut(windowSystemStructure, "win_endstop", sdfMoc, 1);


		//DEBUG
//		SdfConstantEmitter cmdMotorEmitter = SdflibFactory.eINSTANCE.createSdfConstantEmitter();
//		cmdMotorEmitter.setName("cmdMotorEmitter");
//		Pin pin_cmdMotor = ModelBuilder.addSDFPinOut(cmdMotorEmitter, "cmdMotor", sdfMoc, 1);
//		windowSystemStructure.getBlocks().add(cmdMotorEmitter);
//		cmdMotorEmitter.setValueToEmit(1.0);

		// Motor
		SdfAdder motor = SdflibFactory.eINSTANCE.createSdfAdder();
		motor.setName("Motor");
		Pin pin_motor_cmdin = ModelBuilder.addSDFPinIn(motor, "cmd_motor", sdfMoc, 1);
		motor.setMultFactor(pin_motor_cmdin, MOTOR_TORQUE);
		Pin pin_motor_torque = ModelBuilder.addSDFPinOut(motor, "torque", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(motor);

		//DEBUG
//		ModelBuilder.addRelation(windowSystemStructure, pin_cmdMotor, pin_motor_cmdin);
		ModelBuilder.addRelation(windowSystemStructure, pin_ws_cmdMotor, pin_motor_cmdin);

//		// duplicator
//		SdfDuplicator dupMotorTorque = SdflibFactory.eINSTANCE.createSdfDuplicator();
//		dupMotorTorque.setName("dupMotorTorque");
//		Pin pin_dupMTorqueIn = ModelBuilder.addSDFPinIn(dupMotorTorque, "MTorque", sdfMoc, 1);
//		Pin pin_dupMTorqueOut1 = ModelBuilder.addSDFPinOut(dupMotorTorque, "MT1", sdfMoc, 1);
//		Pin pin_dupMTorqueOut2 = ModelBuilder.addSDFPinOut(dupMotorTorque, "MT2", sdfMoc, 1);
//		windowSystemStructure.getBlocks().add(dupMotorTorque);
//
//		ModelBuilder.addRelation(windowSystemStructure, pin_motor_torque, pin_dupMTorqueIn);

		// motor torque threshold
		SdfThreshold mtorqueThreshold = SdflibFactory.eINSTANCE.createSdfThreshold();
		mtorqueThreshold.setName("mtorqueThreshold");
		Pin pin_thresh_mTorque = ModelBuilder.addSDFPinIn(mtorqueThreshold, "MTorque", sdfMoc, 1);
		Pin pin_thresh_mTorqueOut = ModelBuilder.addSDFPinOut(mtorqueThreshold, "MTorqueThreshOut", sdfMoc, 1);
		mtorqueThreshold.setThresholdGT(pin_thresh_mTorqueOut, 0);
//		Pin pin_thresh_mTorqueOut2 = ModelBuilder.addSDFPinOut(mtorqueThreshold, "MTorqueThreshOut2", sdfMoc, 1);
//		mtorqueThreshold.setThresholdGT(pin_thresh_mTorqueOut2, 0);
		windowSystemStructure.getBlocks().add(mtorqueThreshold);

		ModelBuilder.addRelation(windowSystemStructure, pin_motor_torque, pin_thresh_mTorque);

		
		//DEBUG
//		SdfConstantEmitter cmdObstacleEmitter = SdflibFactory.eINSTANCE.createSdfConstantEmitter();
//		cmdObstacleEmitter.setName("cmdObstacleEmitter");
//		Pin pin_cmdObstacle = ModelBuilder.addSDFPinOut(cmdObstacleEmitter, "cmdObstacle", sdfMoc, 1);
//		windowSystemStructure.getBlocks().add(cmdObstacleEmitter);
//		cmdObstacleEmitter.setValueToEmit(0.0);

		// Obstacle effect
		SdfAdder obstacleEffect = SdflibFactory.eINSTANCE.createSdfAdder();
		obstacleEffect.setName("ObstacleEffect");
		Pin pin_obstacleEffect_obstacleIn = ModelBuilder.addSDFPinIn(obstacleEffect, "cmd_obstacle", sdfMoc, 1);
		obstacleEffect.setMultFactor(pin_obstacleEffect_obstacleIn, OBSTACLE_FORCE);
		Pin pin_obstacleEffect_forceOut = ModelBuilder.addSDFPinOut(obstacleEffect, "force", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(obstacleEffect);

		//DEBUG
//		ModelBuilder.addRelation(windowSystemStructure, pin_cmdObstacle, pin_obstacleEffect_obstacleIn);
		ModelBuilder.addRelation(windowSystemStructure, pin_ws_cmdObstacle, pin_obstacleEffect_obstacleIn);

		// obstacle effect control
		SdfMultiplier obstacleEffectControl = SdflibFactory.eINSTANCE.createSdfMultiplier();
		obstacleEffectControl.setName("obstacleEffectControl");
		Pin pin_obstEffCtrl_motorOn = ModelBuilder.addSDFPinIn(obstacleEffectControl, "motorOn", sdfMoc, 1);
		obstacleEffectControl.setExponent(pin_obstEffCtrl_motorOn, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstEffCtrl_obstEffect = ModelBuilder.addSDFPinIn(obstacleEffectControl, "obstEffect", sdfMoc, 1);
		obstacleEffectControl.setExponent(pin_obstEffCtrl_obstEffect, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstEffCtrl_obstEffectRes = ModelBuilder.addSDFPinOut(obstacleEffectControl, "obstEffectRes", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(obstacleEffectControl);
		
		ModelBuilder.addRelation(windowSystemStructure, pin_thresh_mTorqueOut, pin_obstEffCtrl_motorOn);
		ModelBuilder.addRelation(windowSystemStructure, pin_obstacleEffect_forceOut, pin_obstEffCtrl_obstEffect);

		// block that calculates resulting force
		SdfAdder resultingForce = SdflibFactory.eINSTANCE.createSdfAdder();
		resultingForce.setName("ResultingForce");
		Pin pin_resForceTorqueIn = ModelBuilder.addSDFPinIn(resultingForce, "torque", sdfMoc, 1);
		resultingForce.setMultFactor(pin_resForceTorqueIn, 1);
		Pin pin_resForceForceIn = ModelBuilder.addSDFPinIn(resultingForce, "force", sdfMoc, 1);
		resultingForce.setMultFactor(pin_resForceForceIn, -1);
		Pin pin_resForceOut = ModelBuilder.addSDFPinOut(resultingForce, "Rforce", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(resultingForce);

		ModelBuilder.addRelation(windowSystemStructure, pin_motor_torque, pin_resForceTorqueIn);
		ModelBuilder.addRelation(windowSystemStructure, pin_obstEffCtrl_obstEffectRes, pin_resForceForceIn);

//		// duplicator
//		SdfDuplicator dupRforce = SdflibFactory.eINSTANCE.createSdfDuplicator();
//		dupRforce.setName("dupRforce");
//		Pin pin_dupRforceIn = ModelBuilder.addSDFPinIn(dupRforce, "Rforce", sdfMoc, 1);
//		Pin pin_dupRforceOut1 = ModelBuilder.addSDFPinOut(dupRforce, "R1", sdfMoc, 1);
//		Pin pin_dupRforceOut2 = ModelBuilder.addSDFPinOut(dupRforce, "R2", sdfMoc, 1);
//		windowSystemStructure.getBlocks().add(dupRforce);
//
//		ModelBuilder.addRelation(windowSystemStructure, pin_resForceOut, pin_dupRforceIn);

		// obstacle threshold
		SdfThreshold obstacleThreshold = SdflibFactory.eINSTANCE.createSdfThreshold();
		obstacleThreshold.setName("obstacleThreshold");
		Pin pin_thresh_rForce = ModelBuilder.addSDFPinIn(obstacleThreshold, "Rforce", sdfMoc, 1);
		Pin pin_thresh_obstacle = ModelBuilder.addSDFPinOut(obstacleThreshold, "obstacle", sdfMoc, 1);
		obstacleThreshold.setThresholdLT(pin_thresh_obstacle, MOTOR_TORQUE);
		windowSystemStructure.getBlocks().add(obstacleThreshold);

		ModelBuilder.addRelation(windowSystemStructure, pin_resForceOut, pin_thresh_rForce);

		// obstacle multiplier
		SdfMultiplier obstacleMultiplier = SdflibFactory.eINSTANCE.createSdfMultiplier();
		obstacleMultiplier.setName("obstacleMultiplier");
		Pin pin_obstMult_torqueThres = ModelBuilder.addSDFPinIn(obstacleMultiplier, "torqueThres", sdfMoc, 1);
		obstacleMultiplier.setExponent(pin_obstMult_torqueThres, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstMult_obstThres = ModelBuilder.addSDFPinIn(obstacleMultiplier, "obstThres", sdfMoc, 1);
		obstacleMultiplier.setExponent(pin_obstMult_obstThres, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstMult_obstacle = ModelBuilder.addSDFPinOut(obstacleMultiplier, "obstacle", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(obstacleMultiplier);

		ModelBuilder.addRelation(windowSystemStructure, pin_thresh_mTorqueOut, pin_obstMult_torqueThres);
		ModelBuilder.addRelation(windowSystemStructure, pin_thresh_obstacle, pin_obstMult_obstThres);

		
		//DEBUG
//		SdfPrinter obstaclePrinter = SdflibFactory.eINSTANCE.createSdfPrinter();
//		obstaclePrinter.setName("obstaclePrinter");
//		Pin pin_obstaclePrinter = ModelBuilder.addSDFPinIn(obstaclePrinter, "obstacle", sdfMoc, 1);
//		obstaclePrinter.setPrintStream(System.out);
//		windowSystemStructure.getBlocks().add(obstaclePrinter);
		
		//DEBUG
		ModelBuilder.addRelation(windowSystemStructure, pin_obstMult_obstacle, pin_ws_obstacle);
//		ModelBuilder.addRelation(windowSystemStructure, pin_obstMult_obstacle, pin_obstaclePrinter);

		// position accumulator
		SdfAccumulator positionAcc = SdflibFactory.eINSTANCE.createSdfAccumulator();
		positionAcc.setName("positionAcc");
		Pin pin_acc_in = ModelBuilder.addSDFPinIn(positionAcc, "in", sdfMoc, 1);
		Pin pin_acc_posOut = ModelBuilder.addSDFPinOut(positionAcc, "position", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(positionAcc);

		ModelBuilder.addRelation(windowSystemStructure, pin_resForceOut, pin_acc_in);


//		// position duplicator
//		SdfDuplicator dupPosition = SdflibFactory.eINSTANCE.createSdfDuplicator();
//		dupPosition.setName("dupPosition");
//		Pin pin_dup_posIn = ModelBuilder.addSDFPinIn(dupPosition, "posIn", sdfMoc, 1);
//		Pin pin_dup_posOut1 = ModelBuilder.addSDFPinOut(dupPosition, "pos1", sdfMoc, 1);
//		Pin pin_dup_posOut2 = ModelBuilder.addSDFPinOut(dupPosition, "pos2", sdfMoc, 1);
//		windowSystemStructure.getBlocks().add(dupPosition);
//
//		ModelBuilder.addRelation(windowSystemStructure, pin_acc_posOut, pin_dup_posIn);

		// position limiter
		SdfLimiter posLimiter = SdflibFactory.eINSTANCE.createSdfLimiter();
		posLimiter.setName("posLimiter");
		Pin pin_posLim_in = ModelBuilder.addSDFPinIn(posLimiter, "posIn", sdfMoc, 1);
		Pin pin_posLim_position = ModelBuilder.addSDFPinOut(posLimiter, "position", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(posLimiter);
		posLimiter.setLowerBound(0.0);
		posLimiter.setUpperBound(WINDOW_HEIGHT);

		ModelBuilder.addRelation(windowSystemStructure, pin_acc_posOut, pin_posLim_in);

		// position printer
		SdfPrinter positionPrinter = SdflibFactory.eINSTANCE.createSdfPrinter();
		positionPrinter.setName("positionPrinter");
		Pin pin_positionPrinter = ModelBuilder.addSDFPinIn(positionPrinter, "position", sdfMoc, 1);
		positionPrinter.setPrintStream(System.out);
		windowSystemStructure.getBlocks().add(positionPrinter);
		
		ModelBuilder.addRelation(windowSystemStructure, pin_posLim_position, pin_positionPrinter);

		// endstop threshold
		SdfThreshold endstop = SdflibFactory.eINSTANCE.createSdfThreshold();
		endstop.setName("endstopThreshold");
		Pin pin_endstop_position = ModelBuilder.addSDFPinIn(endstop, "position", sdfMoc, 1);
		Pin pin_endstop_closed = ModelBuilder.addSDFPinOut(endstop, "closed", sdfMoc, 1);
		endstop.setThresholdLEQ(pin_endstop_closed, 0);
		Pin pin_endstop_opened = ModelBuilder.addSDFPinOut(endstop, "opened", sdfMoc, 1);
		endstop.setThresholdGEQ(pin_endstop_opened, WINDOW_HEIGHT);
		windowSystemStructure.getBlocks().add(endstop);

		ModelBuilder.addRelation(windowSystemStructure, pin_acc_posOut, pin_endstop_position);

		// multiplexer: combines "closed" and "open" signals in an "endstop" signal
		SdfAdder mux = SdflibFactory.eINSTANCE.createSdfAdder();
		mux.setName("MUX");
		Pin pin_mux_closed = ModelBuilder.addSDFPinIn(mux, "closed", sdfMoc, 1);
		mux.setMultFactor(pin_mux_closed, -1);
		Pin pin_mux_opened = ModelBuilder.addSDFPinIn(mux, "opened", sdfMoc, 1);
		mux.setMultFactor(pin_mux_opened, 1);
		Pin pin_mux_endstop = ModelBuilder.addSDFPinOut(mux, "endstop", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(mux);

		ModelBuilder.addRelation(windowSystemStructure, pin_endstop_closed, pin_mux_closed);
		ModelBuilder.addRelation(windowSystemStructure, pin_endstop_opened, pin_mux_opened);

		
		//DEBUG
//		SdfPrinter endstopPrinter = SdflibFactory.eINSTANCE.createSdfPrinter();
//		endstopPrinter.setName("endstopPrinter");
//		Pin pin_endstopPrinter = ModelBuilder.addSDFPinIn(endstopPrinter, "endstop", sdfMoc, 1);
//		endstopPrinter.setPrintStream(System.out);
//		windowSystemStructure.getBlocks().add(endstopPrinter);
		
		//DEBUG
		ModelBuilder.addRelation(windowSystemStructure, pin_mux_endstop, pin_ws_endstop);
//		ModelBuilder.addRelation(windowSystemStructure, pin_mux_endstop, pin_endstopPrinter);
		
		return windowSystem;

	}
	public static Model buildSimpleWindowSDFSubSystemModel(Model deModel){
		Model windowSystem = CoreFactory.eINSTANCE.createModel();
		windowSystem.setName("WindowSystem");
		
		SdfModelOfComputation sdfMoc = SdfFactory.eINSTANCE.createSdfModelOfComputation();
		sdfMoc.setName("SDF_moc");
		sdfMoc.setClock(((DeModelOfComputation)deModel.getMoc()).getClock());
		windowSystem.setMoc(sdfMoc);

		CompositeBlock windowSystemStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		windowSystemStructure.setName("WindowSystem_structure");
		windowSystem.setStructure(windowSystemStructure);

		Pin pin_ws_cmdMotor = ModelBuilder.addSDFPinIn(windowSystemStructure, "cmd_motor", sdfMoc, 1);
		Pin pin_ws_cmdObstacle = ModelBuilder.addSDFPinIn(windowSystemStructure, "cmd_obstacle", sdfMoc, 1);
		Pin pin_ws_obstacle = ModelBuilder.addSDFPinOut(windowSystemStructure, "win_obstacle", sdfMoc, 1);
		Pin pin_ws_endstop = ModelBuilder.addSDFPinOut(windowSystemStructure, "win_endstop", sdfMoc, 1);


		//DEBUG
//		SdfConstantEmitter cmdMotorEmitter = SdflibFactory.eINSTANCE.createSdfConstantEmitter();
//		cmdMotorEmitter.setName("cmdMotorEmitter");
//		Pin pin_cmdMotor = ModelBuilder.addSDFPinOut(cmdMotorEmitter, "cmdMotor", sdfMoc, 1);
//		windowSystemStructure.getBlocks().add(cmdMotorEmitter);
//		cmdMotorEmitter.setValueToEmit(1.0);

		// Motor
		SdfAdder motor = SdflibFactory.eINSTANCE.createSdfAdder();
		motor.setName("Motor");
		Pin pin_motor_cmdin = ModelBuilder.addSDFPinIn(motor, "cmd_motor", sdfMoc, 1);
		motor.setMultFactor(pin_motor_cmdin, MOTOR_TORQUE);
		Pin pin_motor_torque = ModelBuilder.addSDFPinOut(motor, "torque", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(motor);

		//DEBUG
//		ModelBuilder.addRelation(windowSystemStructure, pin_cmdMotor, pin_motor_cmdin);
		ModelBuilder.addRelation(windowSystemStructure, pin_ws_cmdMotor, pin_motor_cmdin);

		// duplicator
		SdfDuplicator dupMotorTorque = SdflibFactory.eINSTANCE.createSdfDuplicator();
		dupMotorTorque.setName("dupMotorTorque");
		Pin pin_dupMTorqueIn = ModelBuilder.addSDFPinIn(dupMotorTorque, "MTorque", sdfMoc, 1);
		Pin pin_dupMTorqueOut1 = ModelBuilder.addSDFPinOut(dupMotorTorque, "MT1", sdfMoc, 1);
		Pin pin_dupMTorqueOut2 = ModelBuilder.addSDFPinOut(dupMotorTorque, "MT2", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(dupMotorTorque);

		ModelBuilder.addRelation(windowSystemStructure, pin_motor_torque, pin_dupMTorqueIn);

		// motor torque threshold
		SdfThreshold mtorqueThreshold = SdflibFactory.eINSTANCE.createSdfThreshold();
		mtorqueThreshold.setName("mtorqueThreshold");
		Pin pin_thresh_mTorque = ModelBuilder.addSDFPinIn(mtorqueThreshold, "MTorque", sdfMoc, 1);
		Pin pin_thresh_mTorqueOut = ModelBuilder.addSDFPinOut(mtorqueThreshold, "MTorqueThreshOut", sdfMoc, 1);
		mtorqueThreshold.setThresholdGT(pin_thresh_mTorqueOut, 0);
		Pin pin_thresh_mTorqueOut2 = ModelBuilder.addSDFPinOut(mtorqueThreshold, "MTorqueThreshOut2", sdfMoc, 1);
		mtorqueThreshold.setThresholdGT(pin_thresh_mTorqueOut2, 0);
		windowSystemStructure.getBlocks().add(mtorqueThreshold);

		ModelBuilder.addRelation(windowSystemStructure, pin_dupMTorqueOut1, pin_thresh_mTorque);

		
		//DEBUG
//		SdfConstantEmitter cmdObstacleEmitter = SdflibFactory.eINSTANCE.createSdfConstantEmitter();
//		cmdObstacleEmitter.setName("cmdObstacleEmitter");
//		Pin pin_cmdObstacle = ModelBuilder.addSDFPinOut(cmdObstacleEmitter, "cmdObstacle", sdfMoc, 1);
//		windowSystemStructure.getBlocks().add(cmdObstacleEmitter);
//		cmdObstacleEmitter.setValueToEmit(0.0);

		// Obstacle effect
		SdfAdder obstacleEffect = SdflibFactory.eINSTANCE.createSdfAdder();
		obstacleEffect.setName("ObstacleEffect");
		Pin pin_obstacleEffect_obstacleIn = ModelBuilder.addSDFPinIn(obstacleEffect, "cmd_obstacle", sdfMoc, 1);
		obstacleEffect.setMultFactor(pin_obstacleEffect_obstacleIn, OBSTACLE_FORCE);
		Pin pin_obstacleEffect_forceOut = ModelBuilder.addSDFPinOut(obstacleEffect, "force", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(obstacleEffect);

		//DEBUG
//		ModelBuilder.addRelation(windowSystemStructure, pin_cmdObstacle, pin_obstacleEffect_obstacleIn);
		ModelBuilder.addRelation(windowSystemStructure, pin_ws_cmdObstacle, pin_obstacleEffect_obstacleIn);

		// obstacle effect control
		SdfMultiplier obstacleEffectControl = SdflibFactory.eINSTANCE.createSdfMultiplier();
		obstacleEffectControl.setName("obstacleEffectControl");
		Pin pin_obstEffCtrl_motorOn = ModelBuilder.addSDFPinIn(obstacleEffectControl, "motorOn", sdfMoc, 1);
		obstacleEffectControl.setExponent(pin_obstEffCtrl_motorOn, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstEffCtrl_obstEffect = ModelBuilder.addSDFPinIn(obstacleEffectControl, "obstEffect", sdfMoc, 1);
		obstacleEffectControl.setExponent(pin_obstEffCtrl_obstEffect, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstEffCtrl_obstEffectRes = ModelBuilder.addSDFPinOut(obstacleEffectControl, "obstEffectRes", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(obstacleEffectControl);
		
		ModelBuilder.addRelation(windowSystemStructure, pin_thresh_mTorqueOut2, pin_obstEffCtrl_motorOn);
		ModelBuilder.addRelation(windowSystemStructure, pin_obstacleEffect_forceOut, pin_obstEffCtrl_obstEffect);

		// block that calculates resulting force
		SdfAdder resultingForce = SdflibFactory.eINSTANCE.createSdfAdder();
		resultingForce.setName("ResultingForce");
		Pin pin_resForceTorqueIn = ModelBuilder.addSDFPinIn(resultingForce, "torque", sdfMoc, 1);
		resultingForce.setMultFactor(pin_resForceTorqueIn, 1);
		Pin pin_resForceForceIn = ModelBuilder.addSDFPinIn(resultingForce, "force", sdfMoc, 1);
		resultingForce.setMultFactor(pin_resForceForceIn, -1);
		Pin pin_resForceOut = ModelBuilder.addSDFPinOut(resultingForce, "Rforce", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(resultingForce);

		ModelBuilder.addRelation(windowSystemStructure, pin_dupMTorqueOut2, pin_resForceTorqueIn);
		ModelBuilder.addRelation(windowSystemStructure, pin_obstEffCtrl_obstEffectRes, pin_resForceForceIn);

		// duplicator
		SdfDuplicator dupRforce = SdflibFactory.eINSTANCE.createSdfDuplicator();
		dupRforce.setName("dupRforce");
		Pin pin_dupRforceIn = ModelBuilder.addSDFPinIn(dupRforce, "Rforce", sdfMoc, 1);
		Pin pin_dupRforceOut1 = ModelBuilder.addSDFPinOut(dupRforce, "R1", sdfMoc, 1);
		Pin pin_dupRforceOut2 = ModelBuilder.addSDFPinOut(dupRforce, "R2", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(dupRforce);

		ModelBuilder.addRelation(windowSystemStructure, pin_resForceOut, pin_dupRforceIn);

		// obstacle threshold
		SdfThreshold obstacleThreshold = SdflibFactory.eINSTANCE.createSdfThreshold();
		obstacleThreshold.setName("obstacleThreshold");
		Pin pin_thresh_rForce = ModelBuilder.addSDFPinIn(obstacleThreshold, "Rforce", sdfMoc, 1);
		Pin pin_thresh_obstacle = ModelBuilder.addSDFPinOut(obstacleThreshold, "obstacle", sdfMoc, 1);
		obstacleThreshold.setThresholdLT(pin_thresh_obstacle, MOTOR_TORQUE);
		windowSystemStructure.getBlocks().add(obstacleThreshold);

		ModelBuilder.addRelation(windowSystemStructure, pin_dupRforceOut1, pin_thresh_rForce);

		// obstacle multiplier
		SdfMultiplier obstacleMultiplier = SdflibFactory.eINSTANCE.createSdfMultiplier();
		obstacleMultiplier.setName("obstacleMultiplier");
		Pin pin_obstMult_torqueThres = ModelBuilder.addSDFPinIn(obstacleMultiplier, "torqueThres", sdfMoc, 1);
		obstacleMultiplier.setExponent(pin_obstMult_torqueThres, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstMult_obstThres = ModelBuilder.addSDFPinIn(obstacleMultiplier, "obstThres", sdfMoc, 1);
		obstacleMultiplier.setExponent(pin_obstMult_obstThres, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstMult_obstacle = ModelBuilder.addSDFPinOut(obstacleMultiplier, "obstacle", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(obstacleMultiplier);

		ModelBuilder.addRelation(windowSystemStructure, pin_thresh_mTorqueOut, pin_obstMult_torqueThres);
		ModelBuilder.addRelation(windowSystemStructure, pin_thresh_obstacle, pin_obstMult_obstThres);

		
		//DEBUG
//		SdfPrinter obstaclePrinter = SdflibFactory.eINSTANCE.createSdfPrinter();
//		obstaclePrinter.setName("obstaclePrinter");
//		Pin pin_obstaclePrinter = ModelBuilder.addSDFPinIn(obstaclePrinter, "obstacle", sdfMoc, 1);
//		obstaclePrinter.setPrintStream(System.out);
//		windowSystemStructure.getBlocks().add(obstaclePrinter);
		
		//DEBUG
		ModelBuilder.addRelation(windowSystemStructure, pin_obstMult_obstacle, pin_ws_obstacle);
//		ModelBuilder.addRelation(windowSystemStructure, pin_obstMult_obstacle, pin_obstaclePrinter);

		// position accumulator
		SdfAccumulator positionAcc = SdflibFactory.eINSTANCE.createSdfAccumulator();
		positionAcc.setName("positionAcc");
		Pin pin_acc_in = ModelBuilder.addSDFPinIn(positionAcc, "in", sdfMoc, 1);
		Pin pin_acc_posOut = ModelBuilder.addSDFPinOut(positionAcc, "position", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(positionAcc);

		ModelBuilder.addRelation(windowSystemStructure, pin_dupRforceOut2, pin_acc_in);


		// position duplicator
		SdfDuplicator dupPosition = SdflibFactory.eINSTANCE.createSdfDuplicator();
		dupPosition.setName("dupPosition");
		Pin pin_dup_posIn = ModelBuilder.addSDFPinIn(dupPosition, "posIn", sdfMoc, 1);
		Pin pin_dup_posOut1 = ModelBuilder.addSDFPinOut(dupPosition, "pos1", sdfMoc, 1);
		Pin pin_dup_posOut2 = ModelBuilder.addSDFPinOut(dupPosition, "pos2", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(dupPosition);

		ModelBuilder.addRelation(windowSystemStructure, pin_acc_posOut, pin_dup_posIn);

		// position limiter
		SdfLimiter posLimiter = SdflibFactory.eINSTANCE.createSdfLimiter();
		posLimiter.setName("posLimiter");
		Pin pin_posLim_in = ModelBuilder.addSDFPinIn(posLimiter, "posIn", sdfMoc, 1);
		Pin pin_posLim_position = ModelBuilder.addSDFPinOut(posLimiter, "position", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(posLimiter);
		posLimiter.setLowerBound(0.0);
		posLimiter.setUpperBound(WINDOW_HEIGHT);

		ModelBuilder.addRelation(windowSystemStructure, pin_dup_posOut1, pin_posLim_in);

		// position printer
		SdfPrinter positionPrinter = SdflibFactory.eINSTANCE.createSdfPrinter();
		positionPrinter.setName("positionPrinter");
		Pin pin_positionPrinter = ModelBuilder.addSDFPinIn(positionPrinter, "position", sdfMoc, 1);
		positionPrinter.setPrintStream(System.out);
		windowSystemStructure.getBlocks().add(positionPrinter);
		
		ModelBuilder.addRelation(windowSystemStructure, pin_posLim_position, pin_positionPrinter);

		// endstop threshold
		SdfThreshold endstop = SdflibFactory.eINSTANCE.createSdfThreshold();
		endstop.setName("endstopThreshold");
		Pin pin_endstop_position = ModelBuilder.addSDFPinIn(endstop, "position", sdfMoc, 1);
		Pin pin_endstop_closed = ModelBuilder.addSDFPinOut(endstop, "closed", sdfMoc, 1);
		endstop.setThresholdLEQ(pin_endstop_closed, 0);
		Pin pin_endstop_opened = ModelBuilder.addSDFPinOut(endstop, "opened", sdfMoc, 1);
		endstop.setThresholdGEQ(pin_endstop_opened, WINDOW_HEIGHT);
		windowSystemStructure.getBlocks().add(endstop);

		ModelBuilder.addRelation(windowSystemStructure, pin_dup_posOut2, pin_endstop_position);

		// multiplexer: combines "closed" and "open" signals in an "endstop" signal
		SdfAdder mux = SdflibFactory.eINSTANCE.createSdfAdder();
		mux.setName("MUX");
		Pin pin_mux_closed = ModelBuilder.addSDFPinIn(mux, "closed", sdfMoc, 1);
		mux.setMultFactor(pin_mux_closed, -1);
		Pin pin_mux_opened = ModelBuilder.addSDFPinIn(mux, "opened", sdfMoc, 1);
		mux.setMultFactor(pin_mux_opened, 1);
		Pin pin_mux_endstop = ModelBuilder.addSDFPinOut(mux, "endstop", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(mux);

		ModelBuilder.addRelation(windowSystemStructure, pin_endstop_closed, pin_mux_closed);
		ModelBuilder.addRelation(windowSystemStructure, pin_endstop_opened, pin_mux_opened);

		
		//DEBUG
//		SdfPrinter endstopPrinter = SdflibFactory.eINSTANCE.createSdfPrinter();
//		endstopPrinter.setName("endstopPrinter");
//		Pin pin_endstopPrinter = ModelBuilder.addSDFPinIn(endstopPrinter, "endstop", sdfMoc, 1);
//		endstopPrinter.setPrintStream(System.out);
//		windowSystemStructure.getBlocks().add(endstopPrinter);
		
		//DEBUG
		ModelBuilder.addRelation(windowSystemStructure, pin_mux_endstop, pin_ws_endstop);
//		ModelBuilder.addRelation(windowSystemStructure, pin_mux_endstop, pin_endstopPrinter);
		
		return windowSystem;

	}


}
