package fr.supelec.modhelx.tests.powerwindow;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.corelib.CorelibFactory;
import fr.supelec.modhelx.mocs.adapterlib.AdapterlibFactory;
import fr.supelec.modhelx.mocs.adapterlib.DeSdfAdapter;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.sdf.SdfFactory;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.mocs.sdflib.SdfAccumulator;
import fr.supelec.modhelx.mocs.sdflib.SdfAdder;
import fr.supelec.modhelx.mocs.sdflib.SdfLimiter;
import fr.supelec.modhelx.mocs.sdflib.SdfMultiplier;
import fr.supelec.modhelx.mocs.sdflib.SdfThreshold;
import fr.supelec.modhelx.mocs.sdflib.SdflibFactory;
import fr.supelec.modhelx.util.BlockComparator;
import fr.supelec.modhelx.util.ModelBuilder;

public class SimpleWindowSDFSubSystemWithProbe {
	
	public static DeSdfAdapter buildSimpleWindowIBForSDFSubSystem(Model deModel){
		// interface block that embedds the SDF window system
		DeSdfAdapter deWindowSystem = AdapterlibFactory.eINSTANCE.createDeSdfAdapter();
		deWindowSystem.setName("WindowSystem_ib");
		deWindowSystem.setSampleTime(1);
		deModel.getStructure().getBlocks().add(deWindowSystem);
		
		Pin pin_dews_motorIn = ModelBuilder.addPinIn(deWindowSystem, "cmd_motor");
 		Pin pin_dews_obstacleIn = ModelBuilder.addPinIn(deWindowSystem, "cmd_obstacle");
		Pin pin_dews_obstacleOut = ModelBuilder.addPinOut(deWindowSystem, "win_obstacle");
		BlockComparator.setInstantaneous(pin_dews_obstacleOut, false);
		Pin pin_dews_endstopOut = ModelBuilder.addPinOut(deWindowSystem, "win_endstop");
		BlockComparator.setInstantaneous(pin_dews_endstopOut, false);
		
		//internal model
		Model windowSystem = buildSimpleWindowSDFSubSystemModel(deModel);
		deWindowSystem.setInternalModel(windowSystem);
		
		//internal relations
		ModelBuilder.addRelation(deWindowSystem, pin_dews_motorIn, ModelBuilder.pinByName(windowSystem.getStructure(), "cmd_motor"));
		ModelBuilder.addRelation(deWindowSystem, pin_dews_obstacleIn, ModelBuilder.pinByName(windowSystem.getStructure(), "cmd_obstacle"));
		ModelBuilder.addRelation(deWindowSystem, ModelBuilder.pinByName(windowSystem.getStructure(), "win_obstacle"), pin_dews_obstacleOut);
		ModelBuilder.addRelation(deWindowSystem, ModelBuilder.pinByName(windowSystem.getStructure(), "win_endstop"), pin_dews_endstopOut);
			
		//internal pins initial values
		deWindowSystem.setInitialValue(ModelBuilder.pinByName(windowSystem.getStructure(), "cmd_obstacle"), 0.0);
		deWindowSystem.setInitialValue(ModelBuilder.pinByName(windowSystem.getStructure(), "cmd_motor"), 0.0);

		return deWindowSystem;
	}
	
	public static final int MOTOR_TORQUE = 3;
	public static final double INITIAL_WINDOW_POSITION = 10.0;
	public static final double WINDOW_HEIGHT = 40.0;
	public static final double OBSTACLE_FORCE = 2;

	public static Model buildSimpleWindowSDFSubSystemModel(Model deModel){
		Model windowSystem = CoreFactory.eINSTANCE.createModel();
		windowSystem.setName("WindowSystem");
		
		SdfModelOfComputation sdfMoc = SdfFactory.eINSTANCE.createSdfModelOfComputation();
		sdfMoc.setName("SDF_moc");
		sdfMoc.setClock(((DeModelOfComputation)deModel.getMoc()).getClock());
		windowSystem.setMoc(sdfMoc);

		CompositeBlock windowSystemStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		windowSystemStructure.setName("WindowSystem_structure");
		windowSystem.setStructure(windowSystemStructure);

		Pin pin_ws_cmdMotor = ModelBuilder.addSDFPinIn(windowSystemStructure, "cmd_motor", sdfMoc, 1);
		Pin pin_ws_cmdObstacle = ModelBuilder.addSDFPinIn(windowSystemStructure, "cmd_obstacle", sdfMoc, 1);
		Pin pin_ws_obstacle = ModelBuilder.addSDFPinOut(windowSystemStructure, "win_obstacle", sdfMoc, 1);
		Pin pin_ws_endstop = ModelBuilder.addSDFPinOut(windowSystemStructure, "win_endstop", sdfMoc, 1);

		// Motor
		SdfAdder motor = SdflibFactory.eINSTANCE.createSdfAdder();
		motor.setName("Motor");
		Pin pin_motor_cmdin = ModelBuilder.addSDFPinIn(motor, "cmd_motor", sdfMoc, 1);
		motor.setMultFactor(pin_motor_cmdin, MOTOR_TORQUE);
		Pin pin_motor_torque = ModelBuilder.addSDFPinOut(motor, "torque", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(motor);

		//DEBUG
		ModelBuilder.addRelation(windowSystemStructure, pin_ws_cmdMotor, pin_motor_cmdin);

		// motor torque threshold
		SdfThreshold mtorqueThreshold = SdflibFactory.eINSTANCE.createSdfThreshold();
		mtorqueThreshold.setName("TestMotorOn");
		Pin pin_thresh_mTorque = ModelBuilder.addSDFPinIn(mtorqueThreshold, "mTorque", sdfMoc, 1);
		Pin pin_thresh_mTorqueOut = ModelBuilder.addSDFPinOut(mtorqueThreshold, "motorOn", sdfMoc, 1);
		mtorqueThreshold.setThresholdGT(pin_thresh_mTorqueOut, 0);
		windowSystemStructure.getBlocks().add(mtorqueThreshold);

		ModelBuilder.addRelation(windowSystemStructure, pin_motor_torque, pin_thresh_mTorque);

		// Obstacle effect
		SdfAdder obstacleEffect = SdflibFactory.eINSTANCE.createSdfAdder();
		obstacleEffect.setName("ObstacleEffect");
		Pin pin_obstacleEffect_obstacleIn = ModelBuilder.addSDFPinIn(obstacleEffect, "cmd_obstacle", sdfMoc, 1);
		obstacleEffect.setMultFactor(pin_obstacleEffect_obstacleIn, OBSTACLE_FORCE);
		Pin pin_obstacleEffect_forceOut = ModelBuilder.addSDFPinOut(obstacleEffect, "force", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(obstacleEffect);

		//DEBUG
		ModelBuilder.addRelation(windowSystemStructure, pin_ws_cmdObstacle, pin_obstacleEffect_obstacleIn);

		// obstacle effect control
		SdfMultiplier obstacleEffectControl = SdflibFactory.eINSTANCE.createSdfMultiplier();
		obstacleEffectControl.setName("IfMotorOn");
		Pin pin_obstEffCtrl_motorOn = ModelBuilder.addSDFPinIn(obstacleEffectControl, "motorOn", sdfMoc, 1);
		obstacleEffectControl.setExponent(pin_obstEffCtrl_motorOn, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstEffCtrl_obstEffect = ModelBuilder.addSDFPinIn(obstacleEffectControl, "obstForce", sdfMoc, 1);
		obstacleEffectControl.setExponent(pin_obstEffCtrl_obstEffect, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstEffCtrl_obstEffectRes = ModelBuilder.addSDFPinOut(obstacleEffectControl, "obstForce", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(obstacleEffectControl);
		
		ModelBuilder.addRelation(windowSystemStructure, pin_thresh_mTorqueOut, pin_obstEffCtrl_motorOn);
		ModelBuilder.addRelation(windowSystemStructure, pin_obstacleEffect_forceOut, pin_obstEffCtrl_obstEffect);

		// block that calculates resulting force
		SdfAdder resultingForce = SdflibFactory.eINSTANCE.createSdfAdder();
		resultingForce.setName("ResultingForce");
		Pin pin_resForceTorqueIn = ModelBuilder.addSDFPinIn(resultingForce, "mTorque", sdfMoc, 1);
		resultingForce.setMultFactor(pin_resForceTorqueIn, 1);
		Pin pin_resForceForceIn = ModelBuilder.addSDFPinIn(resultingForce, "obstForce", sdfMoc, 1);
		resultingForce.setMultFactor(pin_resForceForceIn, -1);
		Pin pin_resForceOut = ModelBuilder.addSDFPinOut(resultingForce, "resultForce", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(resultingForce);

		ModelBuilder.addRelation(windowSystemStructure, pin_motor_torque, pin_resForceTorqueIn);
		ModelBuilder.addRelation(windowSystemStructure, pin_obstEffCtrl_obstEffectRes, pin_resForceForceIn);

		// obstacle threshold
		SdfThreshold obstacleThreshold = SdflibFactory.eINSTANCE.createSdfThreshold();
		obstacleThreshold.setName("TestObstaclePresence");
		Pin pin_thresh_rForce = ModelBuilder.addSDFPinIn(obstacleThreshold, "resultForce", sdfMoc, 1);
		Pin pin_thresh_obstacle = ModelBuilder.addSDFPinOut(obstacleThreshold, "obstaclePres", sdfMoc, 1);
		obstacleThreshold.setThresholdLT(pin_thresh_obstacle, MOTOR_TORQUE);
		windowSystemStructure.getBlocks().add(obstacleThreshold);

		ModelBuilder.addRelation(windowSystemStructure, pin_resForceOut, pin_thresh_rForce);

		// obstacle multiplier
		SdfMultiplier obstacleMultiplier = SdflibFactory.eINSTANCE.createSdfMultiplier();
		obstacleMultiplier.setName("IfMotorOnAndObstaclePresence");
		Pin pin_obstMult_torqueThres = ModelBuilder.addSDFPinIn(obstacleMultiplier, "motorOn", sdfMoc, 1);
		obstacleMultiplier.setExponent(pin_obstMult_torqueThres, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstMult_obstThres = ModelBuilder.addSDFPinIn(obstacleMultiplier, "obstaclePres", sdfMoc, 1);
		obstacleMultiplier.setExponent(pin_obstMult_obstThres, 1.0);// pas nécessaire (valeur par défaut = 1)
		Pin pin_obstMult_obstacle = ModelBuilder.addSDFPinOut(obstacleMultiplier, "obstaclePres", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(obstacleMultiplier);

		ModelBuilder.addRelation(windowSystemStructure, pin_thresh_mTorqueOut, pin_obstMult_torqueThres);
		ModelBuilder.addRelation(windowSystemStructure, pin_thresh_obstacle, pin_obstMult_obstThres);

		//DEBUG
		ModelBuilder.addRelation(windowSystemStructure, pin_obstMult_obstacle, pin_ws_obstacle);

		// position accumulator
		SdfAccumulator positionAcc = SdflibFactory.eINSTANCE.createSdfAccumulator();
		positionAcc.setName("ComputeWinPosition");
		positionAcc.setInitialAccumulatorValue(INITIAL_WINDOW_POSITION);
		Pin acc_in = ModelBuilder.addSDFPinIn(positionAcc, "resultForce", sdfMoc, 1);
		Pin acc_posOut = ModelBuilder.addSDFPinOut(positionAcc, "winPos", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(positionAcc);

		ModelBuilder.addRelation(windowSystemStructure, pin_resForceOut, acc_in);

		// position limiter
		SdfLimiter posLimiter = SdflibFactory.eINSTANCE.createSdfLimiter();
		posLimiter.setName("LimitWinPosition");
		Pin posLim_in = ModelBuilder.addSDFPinIn(posLimiter, "winPos", sdfMoc, 1);
		Pin posLim_position = ModelBuilder.addSDFPinOut(posLimiter, "realWinPos", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(posLimiter);
		posLimiter.setLowerBound(0.0);
		posLimiter.setUpperBound(WINDOW_HEIGHT);

		ModelBuilder.addRelation(windowSystemStructure, acc_posOut, posLim_in);

		// position printer
		Block pProbe = windowSystemStructure.addBlock(CorelibFactory.eINSTANCE.createPluginBlock(), "PositionProbe");
		Pin pin_positionProbe = pProbe.newInputPin("position");
		pProbe.setValueOfParameter("pluggedInClassName", "fr.supelec.modhelx.util.plugins.SdfProbe");
		pProbe.setValueOfParameter("sink", PowerWindowGUI.getInstance());

		windowSystemStructure.connect(posLim_position, pin_positionProbe);

		// endstop threshold
		SdfThreshold endstop = SdflibFactory.eINSTANCE.createSdfThreshold();
		endstop.setName("TestWindowEndstops");
		Pin pin_endstop_position = ModelBuilder.addSDFPinIn(endstop, "winPosition", sdfMoc, 1);
		Pin pin_endstop_closed = ModelBuilder.addSDFPinOut(endstop, "winClosed", sdfMoc, 1);
		endstop.setThresholdLEQ(pin_endstop_closed, 0);
		Pin pin_endstop_opened = ModelBuilder.addSDFPinOut(endstop, "winOpened", sdfMoc, 1);
		endstop.setThresholdGEQ(pin_endstop_opened, WINDOW_HEIGHT);
		windowSystemStructure.getBlocks().add(endstop);

		ModelBuilder.addRelation(windowSystemStructure, acc_posOut, pin_endstop_position);

		// multiplexer: combines "closed" and "open" signals in an "endstop" signal
		SdfAdder mux = SdflibFactory.eINSTANCE.createSdfAdder();
		mux.setName("EndstopMultiplexer");
		Pin pin_mux_closed = ModelBuilder.addSDFPinIn(mux, "closed", sdfMoc, 1);
		mux.setMultFactor(pin_mux_closed, -1);
		Pin pin_mux_opened = ModelBuilder.addSDFPinIn(mux, "opened", sdfMoc, 1);
		mux.setMultFactor(pin_mux_opened, 1);
		Pin pin_mux_endstop = ModelBuilder.addSDFPinOut(mux, "endstop", sdfMoc, 1);
		windowSystemStructure.getBlocks().add(mux);

		ModelBuilder.addRelation(windowSystemStructure, pin_endstop_closed, pin_mux_closed);
		ModelBuilder.addRelation(windowSystemStructure, pin_endstop_opened, pin_mux_opened);

		
		ModelBuilder.addRelation(windowSystemStructure, pin_mux_endstop, pin_ws_endstop);
		
		return windowSystem;
	}
}
