package fr.supelec.modhelx.tests.powerwindow;

import java.io.StringReader;
import java.util.logging.Level;

import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.AdapterlibFactory;
import fr.supelec.modhelx.mocs.adapterlib.DeTfsmAdapter;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.delib.DeEventPrinter;
import fr.supelec.modhelx.mocs.delib.DeScenarioPlayer;
import fr.supelec.modhelx.mocs.delib.DelibFactory;
import fr.supelec.modhelx.mocs.tfsm.TFsmModelOfComputation;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.util.LogFormatter;
import fr.supelec.modhelx.util.ModelBuilder;
import fr.supelec.modhelx.util.TFSMBuilder;

public class SimpleWindowTFSMSubSystem {
	public static void main(String[] args) {
		
		//modèle racine DE
		DeModelOfComputation deMoc = DeFactory.eINSTANCE.createDeModelOfComputation();
		deMoc.setName("DE_moc");
		
		Model deModel = CoreFactory.eINSTANCE.createModel();
		deModel.setName("DE_model");
		deModel.setMoc(deMoc);
		
		CompositeBlock deStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		deStructure.setName("DE_structure");
		deModel.setStructure(deStructure);
		
		
		// scenario player
		DeScenarioPlayer player = DelibFactory.eINSTANCE.createDeScenarioPlayer();
		player.setName("Player");
		ModelBuilder.addDeScenarioEvent(player, 1, "cmd_user", 1.0);
		ModelBuilder.addDeScenarioEvent(player, 2, "cmd_user", 0.0);
//		ModelBuilder.addDeScenarioEvent(player, 4, "win_obstacle", 1.0);

		deStructure.getBlocks().add(player);
		
		//controller
		DeTfsmAdapter tfsm_controller = buildSimpleWindowIBForTFSMSubSystem(deModel);

		//get the TFSM clock
		deMoc.setClock(((TFsmModelOfComputation)tfsm_controller.getInternalModel().getMoc()).getClock());
		
		// connect the player to the controller
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(player, "cmd_user"), ModelBuilder.pinByName(tfsm_controller, "cmd_user"));
//		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(player, "win_endstop"), ModelBuilder.pinByName(tfsm_controller, "win_endstop"));
//		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(player, "win_obstacle"), ModelBuilder.pinByName(tfsm_controller, "win_obstacle"));

		
		// DE Event printer for controller output
		DeEventPrinter obstaclePrinter = DelibFactory.eINSTANCE.createDeEventPrinter();
		obstaclePrinter.setName("Printer");
		obstaclePrinter.setPrintStream(System.out);
		Pin pInObsPrinter = CoreFactory.eINSTANCE.createPin();
		pInObsPrinter.setName("motor_cmd");
		obstaclePrinter.getPinsIn().add(pInObsPrinter);
		deStructure.getBlocks().add(obstaclePrinter);
		
		//connect the controller to the printer
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(tfsm_controller, "cmd_motor"), pInObsPrinter);
		
		
		Platform.setModel(deModel, deMoc.getClock());
		
		Platform.setLogFile("log/SimpleWindow.log");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new LogFormatter());

		//Platform.setRealTime(1000L);
		
		//CustomTFSMState.register();
		//CustomTFSMTransition.register();
		ModelWindow w = new ModelWindow(deModel, deMoc.getClock(), "tests/simplewindow.xy");
		//w.setFreeRun(true);

		w.setVisible(true);
		w.run();
	}
	
	public static DeTfsmAdapter buildSimpleWindowIBForTFSMSubSystem(Model deModel){
		//controller (interface block that embeds the TFSM)
		DeTfsmAdapter ib = AdapterlibFactory.eINSTANCE.createDeTfsmAdapter();
		ib.setName("WindowController_ib");
		
//		ib.setInternalModel(buildSimpleWindowTFSMSubSystemModel(deModel));
//		ib.setInternalModel(buildSimpleWindowTFSMSubSystemModelWithAutomaticUpAndDown(deModel));
		ib.setInternalModel(buildSimpleWindowTFSMSubSystemModelWithRealAutomaticUpAndDown(deModel));
		
		//external pins
		Pin pWinEndstop = ModelBuilder.addPinIn(ib, "win_endstop");
		Pin pObstacle = ModelBuilder.addPinIn(ib, "win_obstacle");
		Pin pCmdUser = ModelBuilder.addPinIn(ib, "cmd_user");
		Pin pCmdMotor = ModelBuilder.addPinOut(ib, "cmd_motor");
		
		//adaptation relations
		ib.setValueAssociatedWithInterfaceRelation(ModelBuilder.addRelation(ib, pWinEndstop, "win_opened"), -1.0);
		ib.setValueAssociatedWithInterfaceRelation(ModelBuilder.addRelation(ib, pWinEndstop, "win_closed"), 1.0);
		ib.setIgnoredValuesOnPin(pWinEndstop, ModelBuilder.makeEList(0.));
		
		ib.setValueAssociatedWithInterfaceRelation(ModelBuilder.addRelation(ib, pObstacle, "win_obstacle"), 1.0);
		ib.setIgnoredValuesOnPin(pObstacle, ModelBuilder.makeEList(0.));
		
		ib.setValueAssociatedWithInterfaceRelation(ModelBuilder.addRelation(ib, pCmdUser, "cmd_up"), 1.0);
		ib.setValueAssociatedWithInterfaceRelation(ModelBuilder.addRelation(ib, pCmdUser, "cmd_stop"), 0.0);
		ib.setValueAssociatedWithInterfaceRelation(ModelBuilder.addRelation(ib, pCmdUser, "cmd_down"), -1.0);
		
		ib.setValueAssociatedWithInterfaceRelation(ModelBuilder.addRelation(ib, "motor_up", pCmdMotor), 1.0);
		ib.setValueAssociatedWithInterfaceRelation(ModelBuilder.addRelation(ib, "motor_stop", pCmdMotor), 0.0);
		ib.setValueAssociatedWithInterfaceRelation(ModelBuilder.addRelation(ib, "motor_down", pCmdMotor), -1.0);
		
		deModel.getStructure().getBlocks().add(ib);
		return ib;
	
	}
	
	public static Model buildSimpleWindowTFSMSubSystemModel(Model deModel){
		//modèle interne TFSM du controller
		TFSMBuilder b = new TFSMBuilder("WindowController", new StringReader(
				"In win_opened\n" +			// 1
				"In win_closed\n" +
				"In win_obstacle\n" +
				"In cmd_up\n" +
				"In cmd_stop\n" +			// 5
				"In cmd_down\n" +
				"Out motor_up\n" +
				"Out motor_stop\n" +
				"Out motor_down\n" +
				"InitialState stop\n" +		// 10
				"State move_up\n" +
				"State emergency_down\n" +
				"State move_down\n" +
				"Transition stop move_up Or cmd_up /motor_up\n" +
				"Transition move_up stop Or cmd_stop win_closed /motor_stop\n" +		// 15
				"Transition move_up emergency_down Or win_obstacle /motor_down\n" +
				"Transition emergency_down stop Delay 2 /motor_stop\n" +
				"Transition move_down stop Or cmd_stop win_opened /motor_stop\n" +
				"Transition stop move_down Or cmd_down /motor_down\n"					// 19
				));
		Model tfsmModel = b.getModel();
		return tfsmModel;
	}
	
	public static Model buildSimpleWindowTFSMSubSystemModelWithAutomaticUpAndDown(Model deModel){
		//modèle interne TFSM du controller
		TFSMBuilder b = new TFSMBuilder("WindowComplexController", new StringReader(
				"In win_opened\n" +			// 1
				"In win_closed\n" +
				"In win_obstacle\n" +
				"In cmd_up\n" +
				"In cmd_stop\n" +			// 5
				"In cmd_down\n" +
				"Out motor_up\n" +
				"Out motor_stop\n" +
				"Out motor_down\n" +
				"InitialState stop\n" +		// 10
				"State move_up_mode\n" +
				"State move_up_manu\n" +
				"State move_up_auto\n" +
				"State emergency_down\n" +
				"State move_down_mode\n" +
				"State move_down_manu\n" +
				"State move_down_auto\n" +
				
				"Transition stop move_up_mode Or cmd_up /motor_up\n" +
				"Transition move_up_mode move_up_auto Or cmd_stop\n" +
				"Transition move_up_auto stop Or win_closed /motor_stop\n" +
				"Transition move_up_mode move_up_manu Delay 2\n" +
				"Transition move_up_manu stop Or win_closed cmd_stop /motor_stop \n" +

				"Transition move_up_manu emergency_down Or win_obstacle /motor_down \n" +
				"Transition move_up_auto emergency_down Or win_obstacle /motor_down \n" +

				"Transition stop move_down_mode Or cmd_down /motor_down\n" +				
				"Transition move_down_mode move_down_auto Or cmd_stop\n" +				
				"Transition move_down_auto stop Or win_opened /motor_stop\n" +				
				"Transition move_down_mode move_down_manu Delay 2\n" +
				"Transition move_down_manu stop Or win_opened cmd_stop /motor_stop \n" +
				
				"Transition emergency_down stop Delay 2 /motor_stop\n" 
				));
		Model tfsmModel = b.getModel();
		return tfsmModel;
	}

	public static Model buildSimpleWindowTFSMSubSystemModelWithRealAutomaticUpAndDown(Model deModel){
		//modèle interne TFSM du controller
		TFSMBuilder b = new TFSMBuilder("WindowComplexController", new StringReader(
				"In win_opened\n" +			// 1
				"In win_closed\n" +
				"In win_obstacle\n" +
				"In cmd_up\n" +
				"In cmd_stop\n" +			// 5
				"In cmd_down\n" +
				"Out motor_up\n" +
				"Out motor_stop\n" +
				"Out motor_down\n" +
				"InitialState stop\n" +		// 10
				"State move_up_mode\n" +
				"State move_up_manu\n" +
				"State move_up_auto\n" +
				"State emergency_down\n" +
				"State move_down_mode\n" +
				"State move_down_manu\n" +
				"State move_down_auto\n" +
				
				"Transition stop move_up_mode Or cmd_up /motor_up\n" +
				"Transition move_up_mode move_up_auto Or cmd_stop\n" +  // go to auto mode if cmd_stop before Delay 2
				"Transition move_up_auto stop Or win_closed cmd_stop cmd_up cmd_down /motor_stop\n" +
				"Transition move_up_mode move_up_manu Delay 2\n" +
				"Transition move_up_manu stop Or win_closed cmd_stop /motor_stop \n" +

				"Transition move_up_manu emergency_down Or win_obstacle /motor_down \n" +
				"Transition move_up_auto emergency_down Or win_obstacle /motor_down \n" +

				"Transition stop move_down_mode Or cmd_down /motor_down\n" +				
				"Transition move_down_mode move_down_auto Or cmd_stop\n" +				
				"Transition move_down_auto stop Or win_opened cmd_stop cmd_up cmd_down /motor_stop\n" +				
				"Transition move_down_mode move_down_manu Delay 2\n" +
				"Transition move_down_manu stop Or win_opened cmd_stop /motor_stop \n" +
				
				"Transition emergency_down stop Delay 2 /motor_stop\n" 
				));
		Model tfsmModel = b.getModel();
		return tfsmModel;
	}

//	public static Model buildSimpleWindowTFSMSubSystemModelWithRealAutomaticUpAndDown(Model deModel){
//		//modèle interne TFSM du controller
//		TFSMBuilder b = new TFSMBuilder("WindowComplexController", new StringReader(
//				"In win_opened\n" +			// 1
//				"In win_closed\n" +
//				"In win_obstacle\n" +
//				"In cmd_up\n" +
//				"In cmd_stop\n" +			// 5
//				"In cmd_down\n" +
//				"Out motor_up\n" +
//				"Out motor_stop\n" +
//				"Out motor_down\n" +
//				"InitialState stop\n" +		// 10
//				"State move_up_mode\n" +
//				"State move_up_manu\n" +
//				"State move_up_auto\n" +
//				"State emergency_down\n" +
//				"State move_down_mode\n" +
//				"State move_down_manu\n" +
//				"State move_down_auto\n" +
//				
//				"Transition stop move_up_mode Or cmd_up /motor_up\n" +
//				"Transition move_up_mode move_up_auto Or cmd_stop\n" +
//				"Transition move_up_auto stop Or win_closed cmd_up cmd_down /motor_stop\n" +
//				"Transition move_up_mode move_up_manu Delay 2\n" +
//				"Transition move_up_manu stop Or win_closed cmd_stop /motor_stop \n" +
//
//				"Transition move_up_manu emergency_down Or win_obstacle /motor_down \n" +
//				"Transition move_up_auto emergency_down Or win_obstacle /motor_down \n" +
//
//				"Transition stop move_down_mode Or cmd_down /motor_down\n" +				
//				"Transition move_down_mode move_down_auto Or cmd_stop\n" +				
//				"Transition move_down_auto stop Or win_opened cmd_up cmd_down /motor_stop\n" +				
//				"Transition move_down_mode move_down_manu Delay 2\n" +
//				"Transition move_down_manu stop Or win_opened cmd_stop /motor_stop \n" +
//				
//				"Transition emergency_down stop Delay 2 /motor_stop\n" 
//				));
//		Model tfsmModel = b.getModel();
//		return tfsmModel;
//	}
}
