package fr.supelec.modhelx.tests.powerwindow;

import java.util.logging.Level;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.CompositeBlock;
import fr.supelec.modhelx.core.CoreFactory;
import fr.supelec.modhelx.core.Model;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.corelib.CorelibFactory;
import fr.supelec.modhelx.corelib.PluginBlock;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.adapterlib.DeSdfAdapter;
import fr.supelec.modhelx.mocs.adapterlib.DeTfsmAdapter;
import fr.supelec.modhelx.mocs.de.DeFactory;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.mocs.tfsm.TFsmModelOfComputation;
import fr.supelec.modhelx.qdgui.ModelWindow;
import fr.supelec.modhelx.qdgui.rendering.SDFRenderingLib;
import fr.supelec.modhelx.qdgui.rendering.TFSMStateR;
import fr.supelec.modhelx.qdgui.rendering.TFSMStructureR;
import fr.supelec.modhelx.qdgui.rendering.TFSMTransitionR;
import fr.supelec.modhelx.util.DEBlockDependencyComparator;
import fr.supelec.modhelx.util.LogFormatter;
import fr.supelec.modhelx.util.ModelBuilder;

public class SimpleWindowWithGUI {
	public static void main(String[] args) {
		
		//modèle racine DE
		DeModelOfComputation deMoc = DeFactory.eINSTANCE.createDeModelOfComputation();
		deMoc.setName("DE_moc");
		
		Model deModel = CoreFactory.eINSTANCE.createModel();
		deModel.setName("PowerWindow");
		deModel.setMoc(deMoc);
		
		CompositeBlock deStructure = CoreFactory.eINSTANCE.createCompositeBlock();
		deStructure.setName("PowerWindow_structure");
		deModel.setStructure(deStructure);
		
		PluginBlock eventFeeder = CorelibFactory.eINSTANCE.createPluginBlock();
		eventFeeder.setPluggedInClassName("fr.supelec.modhelx.util.plugins.DeSynchronousEmitter");
		deStructure.addBlock(eventFeeder, "SyncEmit_GUI");
		Pin cmd_user = eventFeeder.newOutputPin("cmd_user");
		Pin cmd_obstacle = eventFeeder.newOutputPin("cmd_obstacle");
		PowerWindowGUI gui = PowerWindowGUI.getInstance();
		eventFeeder.setValueOfParameter("source", gui);
				
		//controller
		DeTfsmAdapter tfsm_controller = SimpleWindowTFSMSubSystem.buildSimpleWindowIBForTFSMSubSystem(deModel);

		//get the TFSM clock
		deMoc.setClock(((TFsmModelOfComputation)tfsm_controller.getInternalModel().getMoc()).getClock());
		
		// connect the player to the controller
		ModelBuilder.addRelation(deStructure, cmd_user, ModelBuilder.pinByName(tfsm_controller, "cmd_user"));
		
		// create the window system (interface block that embeds the SDF sub-model)
		DeSdfAdapter deWindowSystem = SimpleWindowSDFSubSystemWithProbe.buildSimpleWindowIBForSDFSubSystem(deModel);
			
		//connect the controller to the window system
//		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(tfsm_controller, "cmd_motor"), ModelBuilder.pinByName(deWindowSystem, "cmd_motor"));
		
		Block probe = deStructure.addBlock(CorelibFactory.eINSTANCE.createPluginBlock(), "cmd_motor_probe");
		probe.setValueOfParameter("pluggedInClassName", "fr.supelec.modhelx.util.plugins.DEProbe");
		probe.setValueOfParameter("sink", gui);
		deStructure.connect(ModelBuilder.pinByName(tfsm_controller, "cmd_motor"), probe.newInputPin("cmd_motor"));
		deStructure.connect(ModelBuilder.pinByName(tfsm_controller, "cmd_motor"),ModelBuilder.pinByName(deWindowSystem, "cmd_motor"));
		
		//connect the player to the window system (obstacle)
		ModelBuilder.addRelation(deStructure, cmd_obstacle, ModelBuilder.pinByName(deWindowSystem, "cmd_obstacle"));

		//connect back (loop) the window system to the controller
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(deWindowSystem, "win_obstacle"), ModelBuilder.pinByName(tfsm_controller, "win_obstacle"));
		ModelBuilder.addRelation(deStructure, ModelBuilder.pinByName(deWindowSystem, "win_endstop"), ModelBuilder.pinByName(tfsm_controller, "win_endstop"));

		DEBlockDependencyComparator.setPinInstantaneousDependency(
				ModelBuilder.pinByName(deWindowSystem, "win_obstacle"), false);
		
		DEBlockDependencyComparator.setPinInstantaneousDependency(
				ModelBuilder.pinByName(deWindowSystem, "win_endstop"), false);

		
		Platform.setModel(deModel, deMoc.getClock());
		
		Platform.setLogFile("log/SimpleWindow.log");
		Platform.setLogLevel(Level.FINEST);
		Platform.setLogFormatter(new LogFormatter());
		
		// simulation in "real time"
		Platform.setRealTime(500, 500);
		
		// Custom TFSM rendering
		TFSMStructureR.register();
		TFSMStateR.register();
		TFSMTransitionR.register();
		SDFRenderingLib.register();

		ModelWindow w = new ModelWindow(deModel, deMoc.getClock(), "tests/simplewindow2.xy");

		w.setVisible(true);
		w.run();
	}
}
