/**
 * Copyright (c) 2008-2011 Supélec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * This file is part of the ModHel'X tool
 * 
 * Contributors:
 *     Computer Science Department, Supélec : initial implementation
 * Contacts:
 *     Cecile.Hardebolle@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *
 * $Id: BlockComparator.java 331 2011-03-11 12:01:35Z marcadet $
 */
package fr.supelec.modhelx.util;

import java.util.ArrayList;
import java.util.Comparator;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Relation;

/**
 * 
 * BlockComparator
 *
 * @author Cecile Hardebolle {cecile.hardebolle@supelec.fr}
 */
public class BlockComparator implements Comparator<Block> {
	
	private ArrayList<Block> order;
	private final static String INSTANTANEOUS_PARAM = "Instantaneous";
	
	public BlockComparator() {
		this.order = null;
	}
	
	public BlockComparator(ArrayList<Block> order) {
		this.order = order;
	}
	
    public int compare(Block b1, Block b2) {
        if (order == null) {
            int result = 0;

        	//rend <0 si b1 est avant b2

        	//prendre les pin out du b1, regarder tous les blocks target
        	//si b2 est dans les targets, alors b1 < b2 (rang inf�rieur car plus pres de la racine)
        	//sinon, si b1 est dans les targets des pins out de b2, alors b2 < b1
        	//sinon, ils peuvent �tre de m�me rang, mais une source sera superieure a un non source
            
            //ATTENTION traitement sp�cial pour les relations qui remplacent les alias

        	for (Pin pOutB1 : b1.getPinsOut()) {
    			// do not follow non-instantaneous relations
        		if(isNotInstantaneous(pOutB1)) continue;
        		
        		for (Relation rOutB1 : pOutB1.getOutgoingRelations()) {
        			if (rOutB1.getTarget().getInputOf() != null){//null cas d'une relation alias qui mene vers une sortie
	        			if (rOutB1.getTarget().getInputOf().equals(b2)) {
	        				result = -10;
	        			}
        			}
        		}
        	}

        	for (Pin pOutB2 : b2.getPinsOut()) {
        		// do not follow non-instantaneous relations
        		if(isNotInstantaneous(pOutB2)) continue;
        		
        		for (Relation rOutB2 : pOutB2.getOutgoingRelations()) {
        			if (rOutB2.getTarget().getInputOf() != null){//null cas d'une relation alias qui mene vers une sortie
	        			if (rOutB2.getTarget().getInputOf().equals(b1)) {
	        				result = 10;
	        			}
        			}
        		}
        	}

        	return result;
        }
        
        for (Block b : order) {
        	if ((b1 == b) && (b2 == b)) return   0;
        	if  (b1 == b)               return -10;
        	if               (b2 == b)  return  10;
        }
		throw new Error("BlockComparator.compare : unable to compare blocks " + b1.getName() + " and " + b2.getName());
    }
    
    public static void setInstantaneous(Pin pin, boolean instantaneous) {
    	pin.setValueOfParameter(INSTANTANEOUS_PARAM, instantaneous);
    }
    
    public static boolean isNotInstantaneous(Pin pin) {
    	Object i = pin.getValueOfParameter(INSTANTANEOUS_PARAM);
    	if(i != null && i instanceof Boolean) return ! ((Boolean)i);
    	else return false;
    }
}

