package fr.supelec.modhelx.util;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;

public class LogFormatter extends Formatter {

	private final static Date dat = new Date();
	private final static DateFormat dateFormat = new SimpleDateFormat("HH:mm:ss.SSS: ");

	/**
	 * Format the given LogRecord.
	 * @param record the log record to be formatted.
	 * @return a formatted log record
	 */
	public synchronized String format(LogRecord record) {
		StringBuffer sb = new StringBuffer();
		
		dat.setTime(record.getMillis());
		sb.append(dateFormat.format(dat));
		
		sb.append("[");
		if (record.getSourceClassName() != null) {
			String className = record.getSourceClassName();
			String[] components = className.split("\\.");
			sb.append(components[components.length - 1]);
		} else {
			sb.append(record.getLoggerName());
		}
		if (record.getSourceMethodName() != null) {	
			sb.append(".");
			sb.append(record.getSourceMethodName());
		}
		
		String message = formatMessage(record);
		
		sb.append("]: ");
		sb.append(message);
		
		if (record.getThrown() != null) {
			try {
				StringWriter sw = new StringWriter();
				PrintWriter pw = new PrintWriter(sw);
				record.getThrown().printStackTrace(pw);
				pw.close();
				sb.append(sw.toString());
			} catch (Exception ex) {
			}
		}
		return sb.append("\n").toString();
	}
}
