package fr.supelec.modhelx.util;

import java.io.File;
import java.io.IOException;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import fr.supelec.modhelx.core.Model;

public class ModelIO {

	public static void saveModel(Model model, String name) {
		// create resource set
		ResourceSet resourceSet = new ResourceSetImpl();
		// Register XML resource factory (before creating the resource)
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		// add the root object to the resource
		resource.getContents().add(model);
		// serialize resource
		try {
			resource.save(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
		}
	}
	
	public static Model loadModel(String name) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(
				"modhelx", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(
				new File(name).getAbsolutePath()));
		try {
			resource.load(null);
		}
		catch(IOException ex) {
			System.err.println("Exception : " + ex);
			return null;
		}
		
		if (resource.getContents().size() != 1) {
			System.err.println("Error : there must be 1 element");
			return null;
		}
		
		EObject o = resource.getContents().get(0);
		try {
			return (Model) o;
		}
		catch(ClassCastException ex) {
			System.err.println("Error : expected element not found : " + ex);
			return null;
		}
	}
}
