package fr.supelec.modhelx.util;

import java.util.Collection;

public class StringUtils {
	/**
	 * Returns a string composed of the string representations of all of the
	 * elements of a collection glued together by a given string.
	 * 
	 * Example: let c be a collection containing {"the President", 
	 * "the United States", "America"}. Then implode(c, " of ") returns
	 * "the President of the United States of America".
	 * 
	 * @param <T> the type of the elements of the collection
	 * @param collection a collection of elements
	 * @param glue the string used to glue together the elements
	 * @return a string containing all the elements, glued together
	 * 
	 * @author Christophe Jacquet
	 * @date 2010-04-08
	 */
	public static <T> String implode(Collection<T> collection, String glue) {
		StringBuffer res = new StringBuffer();
		
		for(T elt : collection) {
			if(res.length() > 0) res.append(glue);
			res.append(elt.toString());
		}
		
		return res.toString();
	}
}
