package fr.supelec.modhelx.util.plugins;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.util.DePluggableBlock;

public class DEProbe extends DePluggableBlock {
	private ValueDisplay myDisplay_ = null;
	private final static List<String> parameterNames = Arrays.asList(new String[] {
			"sink"
	});

	private Map<String, Object> eventData = null;
	private Map<String, InstantValue> eventTime = null;
	
	@Override
	public List<String> parameterNames() {
		return parameterNames;
	}
	
	@Override
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		if(! (moc instanceof DeModelOfComputation)) {
			throw new Error("The surrounding moc must be DE.");
		}
		myDisplay_ = (ValueDisplay)getValueOfParameter("sink");
		if (myDisplay_ == null) {
			throw new Error(getClass().getName() + " no value for parameter \"sink\" of type ValueDisplay");
		}
		eventData = new HashMap<String, Object>();
		eventTime = new HashMap<String, InstantValue>();
		myDisplay_.setup();
	}
	
	@Override
	public void update(ModelOfComputation moc) {
		DeModelOfComputation myMoC = getMoC();
		for (Pin p : getPinsIn()) {
			if (p.hasTokens()) {
				String name = p.getName();
				Token t = p.getToken(true);
				eventData.put(name, myMoC.getValueOfToken(t));
				eventTime.put(name, myMoC.getTimestampOfEvent(t));
			}
		}
	}
	
	@Override
	public void endOfSnapshot(ModelOfComputation moc) {
		for (String name : eventData.keySet()) {
			myDisplay_.valueChanged(name, eventData.get(name));
		}
	}

}
