package fr.supelec.modhelx.util.plugins;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFrame;

import fr.supelec.modhelx.core.Block;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.util.ModelBuilder;

public class DeKeypad extends JFrame implements SynchronousEventSource {
	private static final long serialVersionUID = -4706965021813672531L;
	
	private final List<Key> keys = new ArrayList<Key>();
	private DeSynchronousEmitter emitter = null;

	public DeKeypad() {
		super("Keypad");
		
		setLayout(new FlowLayout());
	}
	
	public void setEmitter(DeSynchronousEmitter emitter) {
		this.emitter = emitter;
	}
	
	public void addKey(String caption, Pin pin, Object value) {
		keys.add(new Key(caption, pin, value));
	}
	
	public void addKey(Block block, String caption, String pinName, Object value) {
		Pin pOut = ModelBuilder.addPinIfNecessary(block, pinName);
		
		keys.add(new Key(caption, pOut, value));
	}
	
	public void display() {
		ClickListener listener = new ClickListener();
		
		for(Key k : keys) {
			JButton button = new JButton(k.caption);
			button.addActionListener(listener);
			add(button);
		}
		
		pack();
		setVisible(true);
	}
	
	private class ClickListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if(emitter == null) {
				System.err.println("DeKeypad: button clicked, but emitter not set.");
				return;
			}
			for(Key k : keys) {
				if( ((JButton)e.getSource()).getText().equals(k.caption)) {
					emitter.emitEventNow(k.pin, k.value);
				}
			}
		}
	}
	
	private final static class Key {
		public String caption;
		public Pin pin;
		public Object value;
		
		public Key(String caption, Pin pin, Object value) {
			this.caption = caption;
			this.pin = pin;
			this.value = value;
		}
	}
	
	@Override
	public String toString() {
		return "DEKeypad";
	}

	public void setup() {
		display();
	}

	public void wrapup() {
		setVisible(false);
	}
}