package fr.supelec.modhelx.util.plugins;

import java.util.Arrays;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.Token;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.engine.Platform;
import fr.supelec.modhelx.mocs.de.DeModelOfComputation;
import fr.supelec.modhelx.util.DePluggableBlock;

public class DeSynchronousEmitter extends DePluggableBlock {
	private Pin eventPin;
	private Object eventValue;
	private boolean updateDone = false;
	
	// no parameter
	private final static List<String> parameterNames;
	static {
		parameterNames = Arrays.asList(new String[] {
				"source"
		});
	}

	@Override
	public List<String> parameterNames() {
		return parameterNames;
	}

	@Override
	public void collectTimeConstraints(ModelOfComputation moc, EList<InstantValue> timeConstraints) {
		if(eventPin != null) {
			timeConstraints.add(getMoC().getClock().createInstantValue(-1));
			Platform.getLog().info("DeSync.collectTimeConstraints: asking for snapshot ASAP");
		}
		
		super.collectTimeConstraints(moc, timeConstraints);
	}
	
	@Override
	public void reset(ModelOfComputation moc, InstantValue time) {
		super.reset(moc, time);
		
		updateDone = false;
	}
	
	@Override
	public void update(ModelOfComputation moc) {
		if(eventPin != null) { // && getMoC().getTimeOfCurrentSnapshot().getDenotedInstant().get(0).getDate() == eventTime) {
			// don't check if it's the "right snapshot" because
			//  1) we want to emit asap
			//  2) what if the constraint we emitted was back in time???
			Token token = createEvent(getMoC().getTimeOfCurrentSnapshot(), eventValue);
			eventPin.addToken(token);
			Platform.getLog().info("DeSync.update: sending token " + token + " (" + getMoC().getValueOfToken(token) + ") on " + eventPin + " at " + getMoC().getTimeOfCurrentSnapshot());
		}
		
		updateDone = true;
	}
	
	@Override
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		if (! (moc instanceof DeModelOfComputation))
			throw new Error("The surrounding moc must be DE.");
		
		SynchronousEventSource src = (SynchronousEventSource)getValueOfParameter("source");
		src.setEmitter(this);
		src.setup();
	}
	
	@Override
	public void endOfSnapshot(ModelOfComputation moc) {
		if (updateDone) eventPin = null;   // no more event to emit for now
	}
	
	@Override
	public boolean newSnap(ModelOfComputation moc) {
		return true;
	}
	
	public void emitEventNow(Pin eventPin, Object eventValue) {
		this.eventPin = eventPin;
		this.eventValue = eventValue;

//		System.err.println("Synchronous event on pin " + eventPin + ", value " + eventValue);
		
		Platform.requestSnapshot();
	}
}
