package fr.supelec.modhelx.util.plugins;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.mocs.sdf.SdfModelOfComputation;
import fr.supelec.modhelx.util.SdfPluggableBlock;

public class SdfProbe extends SdfPluggableBlock {
	private ValueDisplay myDisplay_ = null;
	private final static List<String> parameterNames = Arrays.asList(new String[] {
			"sink"
	});

	private HashMap<Pin, Object> previousValues = null;
	private HashMap<Pin, Object> currentValues = null;
	
	@Override
	public List<String> parameterNames() {
		return parameterNames;
	}
	
	@Override
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		if(! (moc instanceof SdfModelOfComputation)) {
			throw new Error("The surrounding moc must be SDF.");
		}
		for (Pin p : getPinsIn()) {
			if (getMoC().getPinRate(p) != 1) {
				throw new Error(getClass().getName() + " I can work only with pins at rate 1");
			}
		}
		previousValues = new HashMap<Pin, Object>();
		currentValues = new HashMap<Pin, Object>();
		myDisplay_ = (ValueDisplay)getValueOfParameter("sink");
		if (myDisplay_ == null) {
			throw new Error(getClass().getName() + " no value for parameter \"sink\" of type ValueDisplay");
		}
		myDisplay_.setup();
	}
	@Override
	public void update(ModelOfComputation moc) {
		for (Pin p : getPinsIn()) {
			if (p.hasTokens()) {
				currentValues.put(p, getMoC().getValueOfToken(p.getToken(true)));
			}
		}
	}
	
	@Override
	public void endOfSnapshot(ModelOfComputation moc) {
		for (Pin p : currentValues.keySet()) {
			Object val = currentValues.get(p);
			if (!val.equals(previousValues.get(p))) {
				myDisplay_.valueChanged(p.getName(), val);
				previousValues.put(p, val);
			}
		}
	}

}
