package fr.supelec.modhelx.util.plugins;

import java.util.Arrays;
import java.util.List;

import fr.supelec.modhelx.core.ModelOfComputation;
import fr.supelec.modhelx.core.Pin;
import fr.supelec.modhelx.core.time.InstantValue;
import fr.supelec.modhelx.util.Numeric;
import fr.supelec.modhelx.util.SdfPluggableBlock;

public class SdfSineGen extends SdfPluggableBlock {
	private static final String parameterNames_[] = {
		"frequency",
		"samplingRate",
		"phase"
	};
	private double pulsation_ = 0;
	private double samplingrate_ = 1;
	private double phase_ = 0;
	private int sampleNum_ = 0;
	private int samplenum_tmp_ = 0;
	
	@Override
	public void setup(ModelOfComputation moc, InstantValue initialTime) {
		super.setup(moc, initialTime);
		pulsation_ = 2 * Math.PI * Numeric.doubleValueOfObject(getValueOfParameter("frequency"));
		samplingrate_ = Numeric.doubleValueOfObject(getValueOfParameter("samplingRate"));
		if (samplingrate_ == 0) {
			samplingrate_ = 1;
		}
		phase_ = Numeric.doubleValueOfObject(getValueOfParameter("phase"));
		sampleNum_ = 0;
	}
	
	@Override
	public void reset(ModelOfComputation moc, InstantValue time) {
		super.reset(moc, time);
		samplenum_tmp_ = sampleNum_;
	}
	
	@Override
	public void endOfSnapshot(ModelOfComputation moc) {
		super.endOfSnapshot(moc);
		sampleNum_ = samplenum_tmp_;
	}
	
	@Override
	public void update(ModelOfComputation moc) {
		super.update(moc);
		double value = Math.sin(pulsation_ * samplenum_tmp_ / samplingrate_ + phase_);
		samplenum_tmp_ ++;
		for (Pin p : getPinsOut()) {
			int rate = getMoC().getPinRate(p);
			for (int i = 0; i < rate; i++) {
				putToken(p, value);
			}
		}
	}

	@Override
	public List<String> parameterNames() {
		return Arrays.asList(parameterNames_);
	}
}
